@@ -7,7 +7,7 @@ do
 	lang=$(basename $po .po)
 	echo -n "$lang"
 	if $(msgmerge "$SRC/$lang.po" "$SRC/elinks.pot" -o "$lang.new.po"); then
-		mv -f "$lang.new.po" "$SRC/$lanf.po"
+		mv -f "$lang.new.po" "$SRC/$lang.po"
 	else
 		echo "msgmerge failed!"
 		rm -f "$lang.new.po"
