@@ -35,7 +35,7 @@
 /* Current leds allocation:
  * 0 - SSL connection indicator
  * 1 - Insert-mode indicator
- * 2 - JavaScript Error indicator
+ * 2 - JavaScript Error indicator or application mode
  * 3 - JavaScript pop-up blocking indicator
  * 4 - unused, reserved for Lua
  * 5 - download in progress */
@@ -73,6 +73,7 @@ enum led_option {
 
 	LEDS_PANEL_TREE,
 	LEDS_PANEL_ENABLE,
+	LEDS_PANEL_REDRAW_INTERVAL,
 
 	LEDS_OPTIONS,
 };
@@ -123,6 +124,12 @@ static union option_info led_options[] = {
 		N_("Enable LEDs. These visual indicators will inform you "
 		"about various states.")),
 
+	INIT_OPT_INT("ui.leds", N_("Redraw interval"),
+		"redraw_interval", OPT_ZERO, 0, 86400000, 1000,
+		N_("Redraw interval in milliseconds. If you don't have the clock enabled, "
+		"you can set it higher. Default value is 1000. Note that active downloads "
+		"take precedence and enforce interval 100ms.")),
+
 	NULL_OPTION_INFO,
 };
 
@@ -134,10 +141,12 @@ static union option_info led_options[] = {
 #define get_leds_show_mem_enable()	get_opt_leds(LEDS_SHOW_MEM_ENABLE).number
 #define get_leds_temperature_enable()	get_opt_leds(LEDS_TEMPERATURE_ENABLE).number
 #define get_leds_temperature_filename()		get_opt_leds(LEDS_TEMPERATURE_FILENAME).string
+#define get_leds_redraw_interval()	get_opt_leds(LEDS_PANEL_REDRAW_INTERVAL).number
 
 void
 init_leds(struct module *module)
 {
+	ELOG
 	timer_duration_backup = 0;
 
 	/* We can't setup timer here, because we may not manage to startup in
@@ -148,12 +157,14 @@ init_leds(struct module *module)
 void
 done_leds(struct module *module)
 {
+	ELOG
 	kill_timer(&redraw_timer);
 }
 
 void
 set_led_value(struct led *led, unsigned char value)
 {
+	ELOG
 	if (value != led->value__) {
 		led->value__ = value;
 		led->value_changed__ = 1;
@@ -163,12 +174,14 @@ set_led_value(struct led *led, unsigned char value)
 unsigned char
 get_led_value(struct led *led)
 {
+	ELOG
 	return led->value__;
 }
 
 void
 unset_led_value(struct led *led)
 {
+	ELOG
 	set_led_value(led, '-');
 }
 
@@ -176,6 +189,7 @@ unset_led_value(struct led *led)
 void
 init_led_panel(struct led_panel *leds)
 {
+	ELOG
 	int i;
 
 	for (i = 0; i < LEDS_COUNT; i++) {
@@ -187,6 +201,7 @@ init_led_panel(struct led_panel *leds)
 static int
 draw_timer(struct terminal *term, int xpos, int ypos, struct color_pair *color)
 {
+	ELOG
 	char s[64];
 	int i, length;
 
@@ -202,6 +217,7 @@ draw_timer(struct terminal *term, int xpos, int ypos, struct color_pair *color)
 static int
 draw_show_ip(struct session *ses, int xpos, int ypos, struct color_pair *color)
 {
+	ELOG
 	if (ses->doc_view && ses->doc_view->document && ses->doc_view->document->ip) {
 		struct terminal *term = ses->tab->term;
 		char *s = ses->doc_view->document->ip;
@@ -219,6 +235,7 @@ draw_show_ip(struct session *ses, int xpos, int ypos, struct color_pair *color)
 static int
 draw_show_mem(struct session *ses, int xpos, int ypos, struct color_pair *color)
 {
+	ELOG
 	struct terminal *term = ses->tab->term;
 	struct string text;
 	int i;
@@ -241,6 +258,7 @@ draw_show_mem(struct session *ses, int xpos, int ypos, struct color_pair *color)
 static int
 draw_temperature(struct session *ses, int xpos, int ypos, struct color_pair *color)
 {
+	ELOG
 	struct terminal *term = ses->tab->term;
 	int temp = 0;
 	int ret;
@@ -291,6 +309,7 @@ draw_temperature(struct session *ses, int xpos, int ypos, struct color_pair *col
 static int
 draw_clock(struct terminal *term, int xpos, int ypos, struct color_pair *color)
 {
+	ELOG
 	char s[64];
 	time_t curtime = time(NULL);
 	struct tm *loctime = localtime(&curtime);
@@ -308,21 +327,19 @@ draw_clock(struct terminal *term, int xpos, int ypos, struct color_pair *color)
 static milliseconds_T
 compute_redraw_interval(void)
 {
+	ELOG
 	if (are_there_downloads())
 		return 100;
 
-	/* TODO: Check whether the time format includes seconds.  If not,
-	 * return milliseconds to next minute. */
-
-	if (get_leds_clock_enable())
-		return 1000;
-
-	return 0;
+	return (get_leds_clock_enable() || get_leds_temperature_enable()
+		|| get_leds_show_mem_enable() || get_leds_show_ip_enable()
+		) ? get_leds_redraw_interval() : 0;
 }
 
 void
 draw_leds(struct session *ses)
 {
+	ELOG
 	struct terminal *term = ses->tab->term;
 	struct color_pair *led_color = NULL;
 	int i;
@@ -408,6 +425,7 @@ end:
 static int
 sync_leds(struct session *ses)
 {
+	ELOG
 	int i;
 	int timer_duration;
 
@@ -440,6 +458,7 @@ sync_leds(struct session *ses)
 static void
 update_download_led(struct session *ses)
 {
+	ELOG
 	struct session_status *status = &ses->status;
 
 	if (are_there_downloads()) {
@@ -463,6 +482,7 @@ update_download_led(struct session *ses)
 static void
 redraw_leds(void *xxx)
 {
+	ELOG
 	struct terminal *term;
 	milliseconds_T delay;
 
@@ -502,6 +522,7 @@ redraw_leds(void *xxx)
 void
 menu_leds_info(struct terminal *term, void *xxx, void *xxxx)
 {
+	ELOG
 	/* If LEDs ever get more dynamic we might have to change this, but it
 	 * should do for now. --jonas */
 	info_box(term, MSGBOX_FREE_TEXT | MSGBOX_SCROLLABLE,
@@ -510,9 +531,9 @@ menu_leds_info(struct terminal *term, void *xxx, void *xxxx)
 		 	"\n"
 			"[SIJP--]\n"
 			" |||||`- Download in progress\n"
-			" ||||`-- Unused\n"
+			" ||||`-- 'O' means an onion domain, 'o' exists the onion location\n"
 			" |||`--- A JavaScript pop-up window was blocked\n"
-			" ||`---- A JavaScript error has occurred\n"
+			" ||`---- A JavaScript error has occurred or application keyboard mode (A)\n"
 			" |`----- The state of insert mode for text-input form-fields\n"
 			" |       'i' means modeless, 'I' means insert mode is on\n"
 			" `------ Whether an SSL connection was used\n"
@@ -524,6 +545,7 @@ menu_leds_info(struct terminal *term, void *xxx, void *xxxx)
 struct led *
 register_led(struct session *ses, int number)
 {
+	ELOG
 	struct led *led;
 
 	if (number >= LEDS_COUNT || number < 0)
@@ -541,6 +563,7 @@ register_led(struct session *ses, int number)
 void
 unregister_led(struct led *led)
 {
+	ELOG
 	assertm(led->used__, "Attempted to unregister unused led!");
 	led->used__ = 0;
 	unset_led_value(led);
