@@ -92,6 +92,7 @@ struct conf_parsing_state {
 static enum parse_error
 show_parse_error(const struct conf_parsing_state *state, enum parse_error err)
 {
+	ELOG
 	static const  char error_msg[][40] = {
 		"no error",        /* ERROR_NONE */
 		"unknown command", /* ERROR_COMMAND */
@@ -112,6 +113,7 @@ show_parse_error(const struct conf_parsing_state *state, enum parse_error err)
 static void
 skip_white(struct conf_parsing_pos *pos)
 {
+	ELOG
 	char *start = pos->look;
 
 	while (*start) {
@@ -138,6 +140,7 @@ skip_white(struct conf_parsing_pos *pos)
 static void
 skip_quoted(struct conf_parsing_pos *pos)
 {
+	ELOG
 	assert(isquote(*pos->look));
 	if_assert_failed return;
 	pos->look++;
@@ -166,6 +169,7 @@ skip_quoted(struct conf_parsing_pos *pos)
 static void
 skip_option_value(struct conf_parsing_pos *pos)
 {
+	ELOG
 	if (isquote(*pos->look)) {
 		/* Looks like OPT_STRING, OPT_CODEPAGE, OPT_LANGUAGE,
 		 * or OPT_COLOR.  */
@@ -186,6 +190,7 @@ skip_option_value(struct conf_parsing_pos *pos)
 static void
 skip_to_unquoted_newline_or_comment(struct conf_parsing_pos *pos)
 {
+	ELOG
 	while (*pos->look && *pos->look != '#' && *pos->look != '\n') {
 		if (isquote(*pos->look))
 			skip_quoted(pos);
@@ -204,6 +209,7 @@ static enum parse_error
 parse_set_common(struct option *opt_tree, struct conf_parsing_state *state,
 		 struct string *mirror, int is_system_conf, int want_domain)
 {
+	ELOG
 	const char *domain_orig = NULL;
 	size_t domain_len = 0;
 	char *domain_copy = NULL;
@@ -393,6 +399,7 @@ static enum parse_error
 parse_set_domain(struct option *opt_tree, struct conf_parsing_state *state,
                  struct string *mirror, int is_system_conf)
 {
+	ELOG
 	return parse_set_common(opt_tree, state, mirror, is_system_conf, 1);
 }
 
@@ -400,6 +407,7 @@ static enum parse_error
 parse_set(struct option *opt_tree, struct conf_parsing_state *state,
           struct string *mirror, int is_system_conf)
 {
+	ELOG
 	return parse_set_common(opt_tree, state, mirror, is_system_conf, 0);
 }
 
@@ -408,6 +416,7 @@ static enum parse_error
 parse_unset(struct option *opt_tree, struct conf_parsing_state *state,
 	    struct string *mirror, int is_system_conf)
 {
+	ELOG
 	const char *optname_orig;
 	size_t optname_len;
 	char *optname_copy;
@@ -485,6 +494,7 @@ static enum parse_error
 parse_bind(struct option *opt_tree, struct conf_parsing_state *state,
 	   struct string *mirror, int is_system_conf)
 {
+	ELOG
 	char *keymap, *keystroke, *action;
 	enum parse_error err = ERROR_NONE;
 	struct conf_parsing_pos before_error;
@@ -578,6 +588,8 @@ static enum parse_error
 parse_include(struct option *opt_tree, struct conf_parsing_state *state,
 	      struct string *mirror, int is_system_conf)
 {
+	ELOG
+	char *xdg_config_home = get_xdg_config_home();
 	char *fname;
 	struct string dumbstring;
 	struct conf_parsing_pos before_error;
@@ -609,7 +621,7 @@ parse_include(struct option *opt_tree, struct conf_parsing_state *state,
 	/* XXX: We should try CONFDIR/<file> when proceeding
 	 * CONFDIR/<otherfile> ;). --pasky */
 	if (load_config_file(fname[0] == '/' ? (char *) ""
-					     : elinks_home,
+					     : xdg_config_home,
 			     fname, opt_tree, 
 			     mirror ? &dumbstring : NULL, 1)) {
 		done_string(&dumbstring);
@@ -644,6 +656,7 @@ static enum parse_error
 parse_config_command(struct option *options, struct conf_parsing_state *state,
 		     struct string *mirror, int is_system_conf)
 {
+	ELOG
 	const struct parse_handler *handler;
 
 	/* If we're mirroring, then everything up to this point must
@@ -679,6 +692,7 @@ parse_config_command(struct option *options, struct conf_parsing_state *state,
 enum parse_error
 parse_config_exmode_command(char *cmd)
 {
+	ELOG
 	struct conf_parsing_state state = {{ 0 }};
 
 	state.pos.look = cmd;
@@ -695,6 +709,7 @@ parse_config_file(struct option *options, const char *name,
 		  char *file, struct string *mirror,
 		  int is_system_conf)
 {
+	ELOG
 	struct conf_parsing_state state = {{ 0 }};
 	int error_occurred = 0;
 
@@ -756,9 +771,9 @@ parse_config_file(struct option *options, const char *name,
 
 	fputc('\a', stderr);
 
-	if (get_cmd_opt_bool("dump")
-	    || get_cmd_opt_bool("source"))
+	if (get_cmd_opt_bool("dump") || get_cmd_opt_bool("source") || get_cmd_opt_bool("test")) {
 		return;
+	}
 
 	sleep(1);
 }
@@ -768,6 +783,7 @@ parse_config_file(struct option *options, const char *name,
 static char *
 read_config_file(char *name)
 {
+	ELOG
 #define FILE_BUF	1024
 	char cfg_buffer[FILE_BUF];
 	struct string string;
@@ -804,16 +820,22 @@ load_config_file(const char *prefix, const char *name,
 		 struct option *options, struct string *mirror,
 		 int is_system_conf)
 {
+	ELOG
 	char *config_str, *config_file;
 
-	config_file = straconcat(prefix, STRING_DIR_SEP, name,
+	int prefixlen = strlen(prefix);
+	int prefix_has_slash = (prefixlen && dir_sep(prefix[prefixlen - 1]));
+	int name_has_slash = dir_sep(name[0]);
+	const char *slash = name_has_slash || prefix_has_slash ? "" : STRING_DIR_SEP;
+
+	config_file = straconcat(prefix, slash, name,
 				 (char *) NULL);
 	if (!config_file) return 1;
 
 	config_str = read_config_file(config_file);
 	if (!config_str) {
 		mem_free(config_file);
-		config_file = straconcat(prefix, STRING_DIR_SEP, ".", name,
+		config_file = straconcat(prefix, slash, ".", name,
 					 (char *) NULL);
 		if (!config_file) return 2;
 
@@ -836,13 +858,15 @@ load_config_file(const char *prefix, const char *name,
 static void
 load_config_from(char *file, struct option *tree)
 {
+	ELOG
 	load_config_file(CONFDIR, file, tree, NULL, 1);
-	load_config_file(empty_string_or_(elinks_home), file, tree, NULL, 0);
+	load_config_file(empty_string_or_(get_xdg_config_home()), file, tree, NULL, 0);
 }
 
 void
 load_config(void)
 {
+	ELOG
 	load_config_from(get_cmd_opt_str("config-file"),
 			 config_options);
 }
@@ -855,6 +879,7 @@ static int comments = 3;
 static inline const char *
 conf_i18n(const char *s, int i18n)
 {
+	ELOG
 	if (i18n) return gettext(s);
 	return s;
 }
@@ -862,6 +887,7 @@ conf_i18n(const char *s, int i18n)
 static void
 add_indent_to_string(struct string *string, int depth)
 {
+	ELOG
 	if (!depth) return;
 
 	add_xchar_to_string(string, ' ', depth * indentation);
@@ -871,6 +897,7 @@ struct string *
 wrap_option_desc(struct string *out, const char *src,
 		 const struct string *indent, int maxwidth)
 {
+	ELOG
 	const char *last_space = NULL;
 	const char *uncopied = src;
 	int width = 0;
@@ -913,6 +940,7 @@ static void
 output_option_desc_as_comment(struct string *out, const struct option *option,
 			      int i18n, int depth)
 {
+	ELOG
 	const char *desc_i18n = conf_i18n(option->desc, i18n);
 	struct string indent;
 
@@ -934,6 +962,7 @@ smart_config_output_fn(struct string *string, struct option *option,
 		       char *path, int depth, int do_print_comment,
 		       int action, int i18n)
 {
+	ELOG
 	if (option->type == OPT_ALIAS)
 		return;
 
@@ -1006,6 +1035,7 @@ smart_config_output_fn_html(struct string *string, struct option *option,
 		       char *path, int depth, int do_print_comment,
 		       int action, int i18n)
 {
+	ELOG
 	static unsigned int counter;
 	int is_str = 0;
 
@@ -1076,6 +1106,7 @@ smart_config_output_fn_html(struct string *string, struct option *option,
 static void
 add_cfg_header_to_string(struct string *string, const char *text)
 {
+	ELOG
 	int n = strlen(text) + 2;
 
 	int_bounds(&n, 10, 80);
@@ -1090,6 +1121,7 @@ add_cfg_header_to_string(struct string *string, const char *text)
 char *
 create_about_config_string(void)
 {
+	ELOG
 	struct option *options = config_options;
 	struct string config;
 	/* Don't write headers if nothing will be added anyway. */
@@ -1145,6 +1177,7 @@ get_me_out:
 char *
 create_config_string(const char *prefix, const char *name)
 {
+	ELOG
 	struct option *options = config_options;
 	struct string config;
 	/* Don't write headers if nothing will be added anyway. */
@@ -1263,6 +1296,7 @@ static int
 write_config_file(char *prefix, char *name,
                   struct terminal *term)
 {
+	ELOG
 	int ret = -1;
 	struct secure_save_info *ssi;
 	char *config_file = NULL;
@@ -1306,7 +1340,10 @@ free_cfg_str:
 int
 write_config(struct terminal *term)
 {
-	if (!elinks_home) {
+	ELOG
+	char *xdg_config_home = get_xdg_config_home();
+
+	if (!xdg_config_home) {
 		if (term) {
 			write_config_dialog(term, get_cmd_opt_str("config-file"),
 				    SS_ERR_DISABLED, 0);
@@ -1314,13 +1351,14 @@ write_config(struct terminal *term)
 		return -1;
 	}
 
-	return write_config_file(elinks_home, get_cmd_opt_str("config-file"),
+	return write_config_file(xdg_config_home, get_cmd_opt_str("config-file"),
 	                         term);
 }
 
 void
 set_option_or_save(const char *str)
 {
+	ELOG
 #define NUMKVPAIRS 16
 	int i;
 	char *kvpairs[NUMKVPAIRS];
