@@ -45,6 +45,7 @@
 void
 menu_about(struct terminal *term, void *xxx, void *xxxx)
 {
+	ELOG
 	info_box(term, MSGBOX_FREE_TEXT,
 		 N_("About"), ALIGN_CENTER,
 		 get_dyn_full_version(term, 1));
@@ -58,6 +59,7 @@ struct keys_toggle_info {
 static void
 push_toggle_keys_display_button(void *data)
 {
+	ELOG
 	struct keys_toggle_info *info = (struct keys_toggle_info *)data;
 
 	menu_keys(info->term, (void *) (intptr_t) !info->toggle, NULL);
@@ -66,6 +68,7 @@ push_toggle_keys_display_button(void *data)
 void
 menu_keys(struct terminal *term, void *d_, void *xxx)
 {
+	ELOG
 	/* [gettext_accelerator_context(menu_keys)] */
 	int d = (intptr_t) d_;
 
@@ -146,6 +149,7 @@ menu_keys(struct terminal *term, void *d_, void *xxx)
 void
 menu_copying(struct terminal *term, void *xxx, void *xxxx)
 {
+	ELOG
 	info_box(term, MSGBOX_FREE_TEXT,
 		 N_("Copying"), ALIGN_CENTER,
 		 msg_text(term, N_("ELinks %s\n"
@@ -164,6 +168,7 @@ menu_copying(struct terminal *term, void *xxx, void *xxxx)
 static char *
 get_resource_info(struct terminal *term, void *data)
 {
+	ELOG
 	struct string info;
 	long val;
 	unsigned longlong bigval;
@@ -268,7 +273,7 @@ get_resource_info(struct terminal *term, void *data)
 	val = get_number_of_temporary_files();
 	add_format_to_string(&info, ": %ld.", val);
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_MEMCOUNT
 #ifdef CONFIG_BROTLI
 	add_format_to_string(&info, "\nBrotli: calls: %ld active: %ld, size: %ld", get_brotli_total_allocs(), get_brotli_active(), get_brotli_size());
 #endif
@@ -284,9 +289,15 @@ get_resource_info(struct terminal *term, void *data)
 #ifdef CONFIG_LIBEVENT
 	add_format_to_string(&info, "\nLibevent: calls: %ld active: %ld, size: %ld", get_libevent_total_allocs(), get_libevent_active(), get_libevent_size());
 #endif
+#ifdef CONFIG_LIBUV
+	add_format_to_string(&info, "\nLibuv: calls: %ld active: %ld, size: %ld", get_libuv_total_allocs(), get_libuv_active(), get_libuv_size());
+#endif
 #ifdef CONFIG_LIBSIXEL
 	add_format_to_string(&info, "\nSixel: calls: %ld active: %ld, size: %ld", get_sixel_total_allocs(), get_sixel_active(), get_sixel_size());
 #endif
+#if defined(CONFIG_KITTY) || defined(CONFIG_LIBSIXEL)
+	add_format_to_string(&info, "\nStbi: calls: %ld active: %ld, size: %ld", get_stbi_total_allocs(), get_stbi_active(), get_stbi_size());
+#endif
 #ifdef CONFIG_MUJS
 	add_format_to_string(&info, "\nMuJS: calls: %ld active: %ld, size: %ld", get_mujs_total_allocs(), get_mujs_active(), get_mujs_size());
 #endif
@@ -319,6 +330,7 @@ get_resource_info(struct terminal *term, void *data)
 void
 resource_info(struct terminal *term)
 {
+	ELOG
 	refreshed_msg_box(term, 0, N_("Resources"), ALIGN_LEFT,
 			  get_resource_info, NULL);
 }
