@@ -33,6 +33,7 @@
 static void
 display_codepage(struct terminal *term, void *name_, void *xxx)
 {
+	ELOG
 	char *name = (char *)name_;
 	struct option *opt = get_opt_rec(term->spec, "charset");
 	int index = get_cp_index(name);
@@ -50,6 +51,7 @@ display_codepage(struct terminal *term, void *name_, void *xxx)
 void
 charset_list(struct terminal *term, void *xxx, void *ses_)
 {
+	ELOG
 	struct session *ses = (struct session *)ses_;
 	int i, items;
 	int sel = 0;
@@ -97,6 +99,9 @@ enum termopt {
 	TERM_OPT_UNDERLINE,
 	TERM_OPT_ITALIC,
 	TERM_OPT_STRIKE,
+#ifdef CONFIG_KITTY
+	TERM_OPT_KITTY,
+#endif
 #ifdef CONFIG_LIBSIXEL
 	TERM_OPT_SIXEL,
 #endif
@@ -114,6 +119,9 @@ static struct option_resolver resolvers[] = {
 	{ TERM_OPT_UNDERLINE,	 "underline"	},
 	{ TERM_OPT_ITALIC,	 "italic"	},
 	{ TERM_OPT_STRIKE,	 "strike"	},
+#ifdef CONFIG_KITTY
+	{ TERM_OPT_KITTY,	 "kitty"	},
+#endif
 #ifdef CONFIG_LIBSIXEL
 	{ TERM_OPT_SIXEL,	 "sixel"	},
 #endif
@@ -122,6 +130,7 @@ static struct option_resolver resolvers[] = {
 static widget_handler_status_T
 push_ok_button(struct dialog_data *dlg_data, struct widget_data *button)
 {
+	ELOG
 	struct terminal *term = dlg_data->win->term;
 	union option_value *values = (union option_value *)dlg_data->dlg->udata;
 
@@ -138,6 +147,7 @@ push_ok_button(struct dialog_data *dlg_data, struct widget_data *button)
 static widget_handler_status_T
 push_save_button(struct dialog_data *dlg_data, struct widget_data *button)
 {
+	ELOG
 	push_ok_button(dlg_data, button);
 	write_config(dlg_data->win->term);
 
@@ -169,6 +179,7 @@ push_save_button(struct dialog_data *dlg_data, struct widget_data *button)
 void
 terminal_options(struct terminal *term, void *xxx, struct session *ses)
 {
+	ELOG
 	/* [gettext_accelerator_context(terminal_options)] */
 	struct dialog *dlg;
 	union option_value *values;
@@ -240,6 +251,9 @@ terminal_options(struct terminal *term, void *xxx, struct session *ses)
 	add_dlg_checkbox(dlg, _("Underline", term), &values[TERM_OPT_UNDERLINE].number);
 	add_dlg_checkbox(dlg, _("Strikethrough", term), &values[TERM_OPT_STRIKE].number);
 	add_dlg_checkbox(dlg, _("UTF-8 I/O", term), &values[TERM_OPT_UTF_8_IO].number);
+#ifdef CONFIG_KITTY
+	add_dlg_checkbox(dlg, _("Kitty", term), &values[TERM_OPT_KITTY].number);
+#endif
 #ifdef CONFIG_LIBSIXEL
 	add_dlg_checkbox(dlg, _("Sixel", term), &values[TERM_OPT_SIXEL].number);
 #endif
@@ -257,6 +271,7 @@ terminal_options(struct terminal *term, void *xxx, struct session *ses)
 static void
 menu_set_language(struct terminal *term, void *pcp_, void *xxx)
 {
+	ELOG
 	int pcp = (intptr_t) pcp_;
 
 	set_language(pcp);
@@ -267,6 +282,7 @@ menu_set_language(struct terminal *term, void *pcp_, void *xxx)
 void
 menu_language_list(struct terminal *term, void *xxx, void *ses)
 {
+	ELOG
 #ifdef CONFIG_NLS
 	int i;
 	struct menu_item *mi = new_menu(FREE_LIST);
@@ -290,6 +306,7 @@ static char height_str[4];
 static void
 push_resize_button(void *data)
 {
+	ELOG
 	struct terminal *term = (struct terminal *)data;
 	char str[MAX_STR_LEN];
 
@@ -303,6 +320,7 @@ push_resize_button(void *data)
 void
 resize_terminal_dialog(struct terminal *term)
 {
+	ELOG
 	/* [gettext_accelerator_context(resize_terminal_dialog)] */
 	struct dialog *dlg;
 	int width = int_min(term->width, 999);
