@@ -25,9 +25,9 @@
 #include "util/string.h"
 #include "util/time.h"
 
-void add_iframeset_entry(struct iframeset_desc **parent,
-		char *url, char *name, int y, int width, int height, int nlink)
+void add_iframeset_entry(struct document *parent, char *url, char *name, int number, int y, int width, int height, int nlink, int hidden)
 {
+	ELOG
 	struct iframeset_desc *iframeset_desc;
 	struct iframe_desc *iframe_desc;
 	int offset;
@@ -36,25 +36,37 @@ void add_iframeset_entry(struct iframeset_desc **parent,
 	assert(parent);
 	if_assert_failed return;
 
-	if (!*parent) {
-		*parent = (struct iframeset_desc *)mem_calloc(1, sizeof(struct iframeset_desc));
+	if (!parent->iframeset_desc) {
+		parent->iframeset_desc = (struct iframeset_desc *)mem_calloc(1, sizeof(struct iframeset_desc));
 	} else {
-		*parent = (struct iframeset_desc *)mem_realloc(*parent, sizeof(struct iframeset_desc) + ((*parent)->n + 1) * sizeof(struct iframe_desc));
+		parent->iframeset_desc = (struct iframeset_desc *)mem_realloc(parent->iframeset_desc, sizeof(struct iframeset_desc) + (parent->iframeset_desc->n + 1) * sizeof(struct iframe_desc));
 	}
-	if (!*parent) return;
+	if (!parent->iframeset_desc) return;
 
-	iframeset_desc = *parent;
+	iframeset_desc = parent->iframeset_desc;
 	offset = iframeset_desc->n;
 	iframe_desc = &iframeset_desc->iframe_desc[offset];
-	iframe_desc->name = stracpy(name);
-	iframe_desc->uri = get_uri(url, URI_NONE);
+
+	if (!*name) {
+		char text[32] = {0};
+		snprintf(text, 31, "-%d", number);
+		iframe_desc->name = stracpy(text);
+	} else {
+		iframe_desc->name = stracpy(name);
+	}
+	iframe_desc->uri = url ? get_uri(url, URI_NONE) : NULL;
+	iframe_desc->to_load = url ? 1 : 0;
 	iframe_desc->box.x = 1;
 	iframe_desc->box.y = y;
 	iframe_desc->box.width = width;
 	iframe_desc->box.height = height;
 	iframe_desc->nlink = nlink;
-	if (!iframe_desc->uri)
+	iframe_desc->number = number;
+	iframe_desc->inserted = 0;
+	iframe_desc->hidden = hidden;
+	if (!iframe_desc->uri) {
 		iframe_desc->uri = get_uri(about_blank, URI_NONE);
+	}
 
 	iframeset_desc->n++;
 }
@@ -62,6 +74,7 @@ void add_iframeset_entry(struct iframeset_desc **parent,
 static void
 add_iframe_to_list(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct document_view *ses_doc_view;
 
 	assert(ses && doc_view);
@@ -86,6 +99,7 @@ static struct document_view *
 find_ifd(struct session *ses, char *name,
 	int depth, int x, int y)
 {
+	ELOG
 	struct document_view *doc_view;
 
 	assert(ses && name);
@@ -121,12 +135,12 @@ find_ifd(struct session *ses, char *name,
 
 
 static struct document_view *
-format_iframe(struct session *ses, struct iframe_desc *iframe_desc,
+format_iframe(struct session *ses, struct document *document, struct iframe_desc *iframe_desc,
 	     struct document_options *o, int j)
 {
+	ELOG
 	struct view_state *vs;
 	struct document_view *doc_view;
-	struct frame *iframe = NULL;
 	struct cache_entry *cached;
 	int plain;
 
@@ -138,9 +152,14 @@ format_iframe(struct session *ses, struct iframe_desc *iframe_desc,
 	if (!loc) {
 		return NULL;
 	}
+	struct frame *iframe = NULL;
+	struct frame *iframe2 = NULL;
 
-	foreach (iframe, loc->iframes) {
-		if (!c_strcasecmp(iframe->name, iframe_desc->name)) break;
+	foreach (iframe2, loc->iframes) {
+		if (!c_strcasecmp(iframe2->name, iframe_desc->name)) {
+			iframe = iframe2;
+			break;
+		}
 	}
 
 	if (!iframe) {
@@ -149,7 +168,7 @@ format_iframe(struct session *ses, struct iframe_desc *iframe_desc,
 
 	vs = &iframe->vs;
 	cached = find_in_cache(vs->uri);
-	if (!cached) {
+	if (!cached || cached->incomplete) {
 		return NULL;
 	}
 
@@ -170,7 +189,34 @@ format_iframe(struct session *ses, struct iframe_desc *iframe_desc,
 redir:
 	doc_view = find_ifd(ses, iframe_desc->name, j, o->box.x, o->box.y);
 	if (doc_view) {
-		render_document(vs, doc_view, o);
+		doc_view->iframe_number = iframe_desc->number;
+		if (!iframe_desc->inserted) {
+render:
+			render_document(vs, doc_view, o);
+			if (!iframe_desc->hidden) {
+				insert_document_into_document(document, doc_view->document, iframe_desc->box.y);
+			}
+			iframe_desc->inserted = 1;
+			iframe_desc->cache_id = cached->cache_id;
+			int yy;
+
+			for (yy = j + 1; yy < document->iframeset_desc->n; yy++) {
+				document->iframeset_desc->iframe_desc[yy].box.y += doc_view->document->height;
+				document->iframeset_desc->iframe_desc[yy].nlink += doc_view->document->nlinks;
+			}
+		} else if (iframe_desc->cache_id != cached->cache_id) {
+			if (!iframe_desc->hidden) {
+				remove_document_from_document(document, doc_view->document, iframe_desc->box.y);
+			}
+
+			int yy;
+
+			for (yy = j + 1; yy < document->iframeset_desc->n; yy++) {
+				document->iframeset_desc->iframe_desc[yy].box.y -= doc_view->document->height;
+				document->iframeset_desc->iframe_desc[yy].nlink -= doc_view->document->nlinks;
+			}
+			goto render;
+		}
 		///assert(doc_view->document);
 		//doc_view->document->iframe = frame_desc;
 	}
@@ -179,13 +225,13 @@ redir:
 	return doc_view;
 }
 
-
 void
-format_iframes(struct session *ses, struct iframeset_desc *ifsd,
-	      struct document_options *op, int depth)
+format_iframes(struct session *ses, struct document *document, struct document_options *op, int depth)
 {
+	ELOG
 	struct document_options o;
 	int j;
+	struct iframeset_desc *ifsd = document->iframeset_desc;
 
 	assert(ses && ifsd && op);
 	if_assert_failed return;
@@ -196,7 +242,7 @@ format_iframes(struct session *ses, struct iframeset_desc *ifsd,
 
 	copy_struct(&o, op);
 
-	o.margin = !!o.margin;
+	//o.margin = !!o.margin;
 
 	for (j = 0; j < ifsd->n; j++) {
 		struct iframe_desc *iframe_desc = &ifsd->iframe_desc[j];
@@ -208,7 +254,7 @@ format_iframes(struct session *ses, struct iframeset_desc *ifsd,
 		o.box.height = int_min(iframe_desc->box.height, ses->tab->term->height - iframe_desc->box.y - 1);
 		o.framename = iframe_desc->name;
 
-		format_iframe(ses, iframe_desc, &o, j);
+		format_iframe(ses, document, iframe_desc, &o, j);
 		o.box.x += o.box.width + 1;
 		o.box.y += o.box.height + 1;
 #ifdef CONFIG_DEBUG
