@@ -26,9 +26,12 @@
 #include "bfu/listmenu.h"
 #include "bfu/menu.h"
 #include "bookmarks/bookmarks.h"
+#include "cache/cache.h"
 #include "config/options.h"
 #include "config/kbdbind.h"
 #include "document/html/frames.h"
+#include "document/html/iframes.h"
+#include "document/html/parser/general.h"
 #include "document/html/parser/link.h"
 #include "document/html/parser/parse.h"
 #include "document/html/parser/stack.h"
@@ -36,7 +39,16 @@
 #include "document/html/renderer.h"
 #include "globhist/globhist.h"
 #include "mime/mime.h"
+#include "network/connection.h"
 #include "protocol/uri.h"
+#include "terminal/image.h"
+#ifdef CONFIG_KITTY
+#include "terminal/kitty.h"
+#endif
+#ifdef CONFIG_LIBSIXEL
+#include "terminal/sixel.h"
+#endif
+#include "util/base64.h"
 #include "util/conv.h"
 #include "util/error.h"
 #include "util/memdebug.h"
@@ -51,6 +63,7 @@ void
 html_a(struct html_context *html_context, char *a,
        char *xxx3, char *xxx4, char **xxx5)
 {
+	ELOG
 	char *href;
 
 	href = get_url_val(a, "href", html_context->doc_cp);
@@ -122,6 +135,7 @@ html_a(struct html_context *html_context, char *a,
 static char *
 truncate_label(char *label, int max_len)
 {
+	ELOG
 	char *new_label;
 	int len = strlen(label);
 	int left_part_len;
@@ -157,6 +171,7 @@ truncate_label(char *label, int max_len)
 static char *
 get_image_filename_from_src(int max_len, char *src)
 {
+	ELOG
 	char *text = NULL;
 	char *start, *filename;
 	int len;
@@ -190,6 +205,7 @@ get_image_filename_from_src(int max_len, char *src)
 static char *
 get_image_label(int max_len, char *label)
 {
+	ELOG
 	char *formatted_label;
 
 	if (!label) return NULL;
@@ -204,6 +220,7 @@ static void
 put_image_label(char *a, char *label,
                 struct html_context *html_context)
 {
+	ELOG
 	color_T saved_foreground;
 	text_style_format_T saved_attr;
 
@@ -222,10 +239,196 @@ put_image_label(char *a, char *label,
 	elformat.style.attr = saved_attr;
 }
 
+#ifdef CONFIG_KITTY
+
+static unsigned int kitty_image_number;
+
+static void
+html_img_kitty(struct html_context *html_context, char *a,
+        char *html, char *eof, char **end)
+{
+	ELOG
+	if (!html_context->document) {
+		return;
+	}
+	struct el_string *pixels = NULL;
+	int width = 0;
+	int height = 0;
+	int im_number = 0;
+	int compressed = 0;
+	char *url = get_attr_val(a, "src", html_context->doc_cp);
+
+	if (!url) {
+		return;
+	}
+	char *url2 = join_urls(html_context->base_href, url);
+
+	if (url2) {
+		struct uri *uri = get_uri(url2, URI_BASE);
+
+		if (uri) {
+			struct cache_entry *cached = get_redirected_cache_entry(uri);
+again:
+			if (cached && !cached->incomplete) {
+				if (cached->pixels) {
+					pixels = cached->pixels;
+					width = cached->width;
+					height = cached->height;
+					im_number = cached->number;
+					compressed = cached->compressed;
+				} else {
+					struct fragment *fragment = get_cache_fragment(cached);
+
+					if (fragment) {
+						pixels = el_kitty_get_image(fragment->data, fragment->length, &width, &height, &compressed);
+
+						if (pixels) {
+							cached->pixels = pixels;
+							cached->width = width;
+							cached->height = height;
+							cached->number = im_number = ++kitty_image_number;
+							cached->compressed = compressed;
+						}
+					}
+				}
+			}
+			if (!pixels) {
+				if (!cached && (uri->protocol == PROTOCOL_DATA || uri->protocol == PROTOCOL_FILE)) {
+					try_to_load_image(uri);
+					cached = find_in_cache(uri);
+
+					if (cached) {
+						goto again;
+					}
+				} else {
+					html_context->special_f(html_context, SP_IMAGE, uri);
+				}
+			}
+			done_uri(uri);
+		}
+		mem_free(url2);
+	}
+	mem_free(url);
+
+	if (!pixels) {
+		return;
+	}
+	struct document *document = html_context->document;
+	html_linebrk(html_context, a, html, eof, end);
+	put_chrs(html_context, "&nbsp;", 6);
+	ln_break(html_context, 1);
+
+	int lineno = html_context->part->cy + html_context->part->box.y;
+	struct k_image *im = NULL;
+	int how_many = add_kitty_image_to_document(document, pixels, lineno, &im, width, height);
+
+	if (!im) {
+		return;
+	}
+	int xw = (im->width + document->options.cell_width - 1) / document->options.cell_width;
+	int y;
+	im->number = html_top->name - document->text.source;
+	im->ID = im_number;
+	im->compressed = compressed;
+
+	for (y = 0; y < how_many; y++) {
+		int x;
+
+		for (x = 0; x < xw; x++) {
+			put_chrs(html_context, "&nbsp;", 6);
+		}
+		ln_break(html_context, 1);
+	}
+}
+#endif
+
+#ifdef CONFIG_LIBSIXEL
+static void
+html_img_sixel(struct html_context *html_context, char *a,
+        char *html, char *eof, char **end)
+{
+	ELOG
+	if (!html_context->options->html_sixel || !html_context->document) {
+		return;
+	}
+	char *data = NULL;
+	int datalen = 0;
+	char *url = get_attr_val(a, "src", html_context->doc_cp);
+
+	if (!url) {
+		return;
+	}
+	char *url2 = join_urls(html_context->base_href, url);
+
+	if (url2) {
+		struct uri *uri = get_uri(url2, URI_BASE);
+
+		if (uri) {
+			struct cache_entry *cached = get_redirected_cache_entry(uri);
+again:
+			if (cached && !cached->incomplete) {
+				struct fragment *fragment = get_cache_fragment(cached);
+
+				if (fragment) {
+					data = el_sixel_get_image(fragment->data, fragment->length, &datalen);
+				}
+			}
+			if (!data) {
+				if (!cached && (uri->protocol == PROTOCOL_DATA || uri->protocol == PROTOCOL_FILE)) {
+					try_to_load_image(uri);
+					cached = find_in_cache(uri);
+
+					if (cached) {
+						goto again;
+					}
+				} else {
+					html_context->special_f(html_context, SP_IMAGE, uri);
+				}
+			}
+			done_uri(uri);
+		}
+		mem_free(url2);
+	}
+	mem_free(url);
+
+	if (!data) {
+		return;
+	}
+	struct document *document = html_context->document;
+	html_linebrk(html_context, a, html, eof, end);
+	put_chrs(html_context, "&nbsp;", 6);
+	ln_break(html_context, 1);
+
+	int lineno = html_context->part->cy + html_context->part->box.y;
+
+	struct image *im = NULL;
+
+	int how_many = add_image_to_document(document, data, datalen, lineno, &im);
+	mem_free(data);
+
+	if (!im) {
+		return;
+	}
+	int xw = (im->width + document->options.cell_width - 1) / document->options.cell_width;
+	int y;
+	im->image_number = html_top->name - document->text.source;
+
+	for (y = 0; y < how_many; y++) {
+		int x;
+
+		for (x = 0; x < xw; x++) {
+			put_chrs(html_context, "&nbsp;", 6);
+		}
+		ln_break(html_context, 1);
+	}
+}
+#endif
+
 static void
 html_img_do(char *a, char *object_src,
             struct html_context *html_context)
 {
+	ELOG
 	int ismap, usemap = 0;
 	int add_brackets = 0;
 	char *src = NULL;
@@ -258,7 +461,7 @@ html_img_do(char *a, char *object_src,
 		elformat.form = NULL;
 		elformat.style.attr |= AT_BOLD;
 		usemap = 1;
- 	}
+	}
 
 	ismap = elformat.link
 	        && has_attr(a, "ismap", html_context->doc_cp)
@@ -365,8 +568,20 @@ html_img_do(char *a, char *object_src,
 
 void
 html_img(struct html_context *html_context, char *a,
-         char *xxx3, char *xxx4, char **xxx5)
+         char *html, char *eof, char **end)
 {
+	ELOG
+#ifdef CONFIG_KITTY
+	if (html_context->options->html_kitty) {
+		html_img_kitty(html_context, a, html, eof, end);
+	}
+#endif
+#ifdef CONFIG_LIBSIXEL
+#ifdef CONFIG_KITTY
+	else
+#endif
+	html_img_sixel(html_context, a, html, eof, end);
+#endif
 	html_img_do(a, NULL, html_context);
 }
 
@@ -374,6 +589,7 @@ void
 html_source(struct html_context *html_context, char *a,
            char *xxx3, char *xxx4, char **xxx5)
 {
+	ELOG
 	char *src, *title;
 	struct document_options *options = html_context->options;
 	int display_style = options->image_link.display_style;
@@ -408,6 +624,7 @@ put_link_line(const char *prefix, const char *linkname,
 	      char *link, const char *target,
 	      struct html_context *html_context)
 {
+	ELOG
 	html_context->has_link_lines = 1;
 	html_stack_dup(html_context, ELEMENT_KILLABLE);
 	ln_break(html_context, 1);
@@ -433,6 +650,7 @@ void
 html_applet(struct html_context *html_context, char *a,
             char *xxx3, char *xxx4, char **xxx5)
 {
+	ELOG
 	char *code, *alt;
 
 	code = get_url_val(a, "code", html_context->doc_cp);
@@ -458,6 +676,7 @@ void
 html_audio(struct html_context *html_context, char *a,
             char *xxx3, char *xxx4, char **xxx5)
 {
+	ELOG
 	char *url;
 
 	/* This just places a link where a audio element would be. */
@@ -476,23 +695,25 @@ html_audio(struct html_context *html_context, char *a,
 	mem_free(url);
 }
 
+
 static void
-html_iframe_do(char *a, char *object_src,
-               struct html_context *html_context)
+html_iframe_do(struct html_context *html_context, char *a,
+        char *html, char *eof, char **end, char *object_src)
 {
+	ELOG
 	char *name, *url = NULL;
 	int height = 0;
 	int width = 0;
+	int hidden = html_context->hidden;
 
 	url = null_or_stracpy(object_src);
 	if (!url) url = get_url_val(a, "src", html_context->doc_cp);
-	if (!url) return;
 
 	name = get_attr_val(a, "name", html_context->doc_cp);
 	if (!name) name = get_attr_val(a, "id", html_context->doc_cp);
 	if (!name) name = stracpy("");
 	if (!name) {
-		mem_free(url);
+		mem_free_if(url);
 		return;
 	}
 
@@ -519,42 +740,107 @@ html_iframe_do(char *a, char *object_src,
 	}
 
 	if (height > 0) {
-		int y = html_context->part->cy + 2;
-		char *url2;
+		char *url2 = url ? join_urls(html_context->base_href, url) : NULL;
+		int y = html_context->part->cy;
 
-		ln_break(html_context, height + 3);
+		if (!hidden) {
+			html_linebrk(html_context, a, html, eof, end);
+			put_chrs(html_context, "&nbsp;", 6);
+			ln_break(html_context, 1);
 
-		url2 = join_urls(html_context->base_href, url);
+			int doc_width = html_context->options->document_width;
+			int i;
+			struct string frame;
 
-		if (url2) {
-			html_context->special_f(html_context, SP_IFRAME, url2, name, y, width, height);
-			mem_free(url2);
+			if (init_string(&frame)) {
+				add_to_string(&frame, "IFrame: ");
+				add_to_string(&frame, name);
+
+				int both = doc_width - frame.length;
+				int left = int_max((both) / 2, 0);
+				int right = int_max(0, left + (both % 2));
+
+				for (i = 0; i < left; i++) {
+					put_chrs(html_context, "&#x2500;", 7);
+				}
+				put_chrs(html_context, frame.source, frame.length);
+				done_string(&frame);
+				for (i = 0; i < right; i++) {
+					put_chrs(html_context, "&#x2500;", 7);
+				}
+			}
+			ln_break(html_context, 1);
+			y = html_context->part->cy;
+
+			if (init_string(&frame)) {
+				add_to_string(&frame, "/IFrame: ");
+				add_to_string(&frame, name);
+
+				int both = doc_width - frame.length;
+				int right = int_max((both) / 2, 0);
+				int left = int_max(0, right + (both % 2));
+
+				for (i = 0; i < left; i++) {
+					put_chrs(html_context, "&#x2500;", 7);
+				}
+				put_chrs(html_context, frame.source, frame.length);
+				done_string(&frame);
+
+				for (i = 0; i < right; i++) {
+					put_chrs(html_context, "&#x2500;", 7);
+				}
+			}
+			ln_break(html_context, 1);
 		}
-	} else {
+		html_context->special_f(html_context, SP_IFRAME, url2, name, html_context->image_number++, y, width, height, hidden);
+		mem_free_if(url2);
+	} else if (!hidden) {
 		if (*name) {
-			put_link_line("IFrame: ", name, url,
+			put_link_line("IFrame: ", name, url ?: "",
 			      html_context->options->framename, html_context);
 		} else {
-			put_link_line("", "IFrame", url,
+			put_link_line("", "IFrame", url ?: "",
 			      html_context->options->framename, html_context);
 		}
 	}
-
 	mem_free(name);
-	mem_free(url);
+	mem_free_if(url);
+}
+
+void
+html_include_fragment(struct html_context *html_context, char *a,
+                      char *html, char *eof, char **end)
+{
+	ELOG
+	char *url = get_url_val(a, "src", html_context->doc_cp);
+
+	elformat.top_name = html_top->name;
+	html_skip(html_context, a);
+
+	char *url2 = url ? join_urls(html_context->base_href, url) : NULL;
+	html_linebrk(html_context, a, html, eof, end);
+	put_chrs(html_context, "&nbsp;", 6);
+	ln_break(html_context, 1);
+
+	int y = html_context->part->cy;
+	html_context->special_f(html_context, SP_IFRAME, url2, "", html_context->image_number++, y, 0, 0, 0);
+	mem_free_if(url2);
+	mem_free_if(url);
 }
 
 void
 html_iframe(struct html_context *html_context, char *a,
-            char *xxx3, char *xxx4, char **xxx5)
+         char *html, char *eof, char **end)
 {
-	html_iframe_do(a, NULL, html_context);
+	ELOG
+	html_iframe_do(html_context, a, html, eof, end, NULL);
 }
 
 void
 html_object(struct html_context *html_context, char *a,
-            char *xxx3, char *xxx4, char **xxx5)
+         char *html, char *eof, char **end)
 {
+	ELOG
 	char *type, *url;
 
 	/* This is just some dirty wrapper. We emulate various things through
@@ -570,7 +856,7 @@ html_object(struct html_context *html_context, char *a,
 
 	if (!c_strncasecmp(type, "text/", 5)) {
 		/* We will just emulate <iframe>. */
-		html_iframe_do(a, url, html_context);
+		html_iframe_do(html_context, a, html, eof, end, url);
 		html_skip(html_context, a);
 
 	} else if (!c_strncasecmp(type, "image/", 6)) {
@@ -604,8 +890,9 @@ html_object(struct html_context *html_context, char *a,
 
 void
 html_embed(struct html_context *html_context, char *a,
-           char *xxx3, char *xxx4, char **xxx5)
+         char *html, char *eof, char **end)
 {
+	ELOG
 	char *type, *extension;
 	char *object_src;
 
@@ -629,7 +916,7 @@ html_embed(struct html_context *html_context, char *a,
 		html_img_do(a, object_src, html_context);
 	} else {
 		/* We will just emulate <iframe>. */
-		html_iframe_do(a, object_src, html_context);
+		html_iframe_do(html_context, a, html, eof, end, object_src);
 	}
 
 	mem_free_if(type);
@@ -640,6 +927,7 @@ void
 html_video(struct html_context *html_context, char *a,
             char *xxx3, char *xxx4, char **xxx5)
 {
+	ELOG
 	char *url;
 
 	/* This just places a link where a video element would be. */
@@ -823,6 +1111,7 @@ static struct lt_default_name lt_names[] = {
 static const char *
 get_lt_default_name(struct hlink *link)
 {
+	ELOG
 	struct lt_default_name *entry = lt_names;
 
 	assert(link);
@@ -838,6 +1127,7 @@ get_lt_default_name(struct hlink *link)
 static void
 html_link_clear(struct hlink *link)
 {
+	ELOG
 	assert(link);
 
 	mem_free_if(link->content_type);
@@ -857,6 +1147,7 @@ static int
 html_link_parse(struct html_context *html_context, char *a,
                 struct hlink *link)
 {
+	ELOG
 	int i;
 
 	assert(a && link);
@@ -913,6 +1204,7 @@ void
 html_link(struct html_context *html_context, char *a,
           char *xxx3, char *xxx4, char **xxx5)
 {
+	ELOG
 	int link_display = html_context->options->meta_link_display;
 	char *name;
 	struct hlink link;
