@@ -44,6 +44,7 @@
 static inline int
 atchr(unsigned char c)
 {
+	ELOG
 	return (c < 127 && (c > '>' || (c > ' ' && c != '=' && !end_of_tag(c))));
 }
 
@@ -60,6 +61,7 @@ parse_element(char *e, char *eof,
 	      char **name, int *namelen,
 	      char **attr, char **end)
 {
+	ELOG
 #define next_char() if (++e == eof) return -1;
 
 	assert(e && eof);
@@ -144,6 +146,7 @@ char *
 get_attr_value(char *e, const char *name,
 	       int cp, enum html_attr_flags flags)
 {
+	ELOG
 	char *n;
 	char *name_start;
 	char *attr = NULL;
@@ -255,6 +258,7 @@ parse_error:
 int
 get_num(char *a, const char *name, int cp)
 {
+	ELOG
 	char *al = get_attr_val(a, name, cp);
 	int result = -1;
 
@@ -278,6 +282,7 @@ get_num(char *a, const char *name, int cp)
 int
 get_num2(char *al)
 {
+	ELOG
 	int result = -1;
 
 	if (al) {
@@ -301,6 +306,7 @@ int
 get_width(char *a, const char *name, int limited,
           struct html_context *html_context)
 {
+	ELOG
 	char *value = get_attr_val(a, name, html_context->doc_cp);
 	int ret = get_width2(value, limited, html_context);
 
@@ -311,6 +317,7 @@ get_width(char *a, const char *name, int limited,
 int
 get_width2(char *value, int limited, struct html_context *html_context)
 {
+	ELOG
 	char *str = value;
 	char *end;
 	int percentage = 0;
@@ -391,6 +398,7 @@ get_width2(char *value, int limited, struct html_context *html_context)
 char *
 skip_comment(char *html, char *eof)
 {
+	ELOG
 	if (html + 4 <= eof && html[2] == '-' && html[3] == '-') {
 		html += 4;
 		while (html < eof) {
@@ -459,6 +467,7 @@ static struct element_info elements[] = {
  {"ADDRESS",     html_address,     NULL,                 2, ET_NESTABLE    },
  {"APPLET",      html_applet,      NULL,                 1, ET_NON_PAIRABLE},
  {"ARTICLE",     html_article,     NULL,                 0, ET_NESTABLE    },
+ {"ASIDE",       html_linebrk,     NULL,                 1, ET_NESTABLE    },
  {"AUDIO",       html_audio,       NULL,                 1, ET_NON_PAIRABLE},
  {"B",           html_bold,        NULL,                 0, ET_NESTABLE    },
  {"BASE",        html_base,        NULL,                 0, ET_NON_PAIRABLE},
@@ -470,16 +479,19 @@ static struct element_info elements[] = {
  {"CAPTION",     html_center,      NULL,                 1, ET_NESTABLE    },
  {"CENTER",      html_center,      NULL,                 1, ET_NESTABLE    },
  {"CODE",        html_fixed,       NULL,                 0, ET_NESTABLE    },
- {"DD",          html_dd,          NULL,                 1, ET_NON_PAIRABLE},
+ {"DD",          html_dd,          NULL,                 1, ET_NON_NESTABLE},
  {"DFN",         html_bold,        NULL,                 0, ET_NESTABLE    },
  {"DIR",         html_ul,          NULL,                 2, ET_NESTABLE    },
  {"DIV",         html_linebrk,     NULL,                 1, ET_NESTABLE    },
  {"DL",          html_dl,          NULL,                 1, ET_NESTABLE    },
- {"DT",          html_dt,          NULL,                 1, ET_NON_PAIRABLE},
+ {"DT",          html_dt,          NULL,                 1, ET_NON_NESTABLE},
  {"EM",          html_italic,      NULL,                 0, ET_NESTABLE    },
  {"EMBED",       html_embed,       NULL,                 0, ET_NON_PAIRABLE},
+ {"FIGCAPTION",  html_linebrk,     NULL,                 1, ET_NESTABLE    },
+ {"FIGURE",      html_linebrk,     NULL,                 1, ET_NESTABLE    },
  {"FIXED",       html_fixed,       NULL,                 0, ET_NESTABLE    },
  {"FONT",        html_font,        NULL,                 0, ET_NESTABLE    },
+ {"FOOTER",      html_linebrk,     NULL,                 1, ET_NON_NESTABLE},
  {"FORM",        html_form,        NULL,                 1, ET_NESTABLE    },
  {"FRAME",       html_frame,       NULL,                 1, ET_NON_PAIRABLE},
  {"FRAMESET",    html_frameset,    NULL,                 1, ET_NESTABLE    },
@@ -490,16 +502,18 @@ static struct element_info elements[] = {
  {"H5",          html_h5,          NULL,                 2, ET_NON_NESTABLE},
  {"H6",          html_h6,          NULL,                 2, ET_NON_NESTABLE},
  {"HEAD",        html_head,        NULL,                 0, ET_NESTABLE    },
+ {"HEADER",      html_linebrk,     NULL,                 1, ET_NON_NESTABLE},
  {"HR",          html_hr,          NULL,                 2, ET_NON_PAIRABLE},
  {"HTML",        html_html,        html_html_close,      0, ET_NESTABLE    },
  {"I",           html_italic,      NULL,                 0, ET_NESTABLE    },
  {"IFRAME",      html_iframe,      NULL,                 1, ET_NON_PAIRABLE},
  {"IMG",         html_img,         NULL,                 0, ET_NON_PAIRABLE},
+ {"INCLUDE-FRAGMENT", html_include_fragment, NULL,       1, ET_NON_NESTABLE},
  {"INPUT",       html_input,       NULL,                 0, ET_NON_PAIRABLE},
  {"LI",          html_li,          NULL,                 1, ET_LI          },
  {"LINK",        html_link,        NULL,                 1, ET_NON_PAIRABLE},
  {"LISTING",     html_pre,         NULL,                 2, ET_NESTABLE    },
- {"MAIN",        html_main,        NULL,                 0, ET_NON_NESTABLE},
+ {"MAIN",        html_main,        NULL,                 1, ET_NON_NESTABLE},
  {"MENU",        html_ul,          NULL,                 2, ET_NESTABLE    },
  {"META",        html_meta,        NULL,                 0, ET_NON_PAIRABLE},
  {"NAV",         html_linebrk,     NULL,                 1, ET_NESTABLE    },
@@ -513,7 +527,7 @@ static struct element_info elements[] = {
  {"Q",           html_quote,       html_quote_close,     0, ET_NESTABLE    },
  {"S",           html_strike,      NULL,                 0, ET_NESTABLE    },
  {"SCRIPT",      html_script,      NULL,                 0, ET_NESTABLE    },
- {"SECTION",     html_section,     NULL,                 0, ET_NESTABLE    },
+ {"SECTION",     html_section,     NULL,                 1, ET_NESTABLE    },
  {"SELECT",      html_select,      NULL,                 0, ET_NESTABLE    },
  {"SOURCE",      html_source,      NULL,                 1, ET_NON_PAIRABLE},
  {"SPAN",        html_span,        NULL,                 0, ET_NESTABLE    },
@@ -522,6 +536,7 @@ static struct element_info elements[] = {
  {"STYLE",       html_style,       html_style_close,     0, ET_NESTABLE    },
  {"SUB",         html_subscript,   html_subscript_close, 0, ET_NESTABLE    },
  {"SUP",         html_superscript, NULL,                 0, ET_NESTABLE    },
+ {"SVG",         html_svg,         NULL,                 0, ET_NON_NESTABLE},
  {"TABLE",       html_table,       NULL,                 2, ET_NESTABLE    },
  {"TD",          html_td,          NULL,                 0, ET_NESTABLE    },
  {"TEMPLATE",    html_template,    NULL,                 0, ET_NON_NESTABLE},
@@ -545,6 +560,7 @@ static struct element_info elements[] = {
 static int
 compar(const void *a, const void *b)
 {
+	ELOG
 	return c_strcasecmp(((struct element_info *) a)->name,
 			    ((struct element_info *) b)->name);
 }
@@ -557,6 +573,7 @@ static struct element_info *internal_pointer;
 static void
 tags_list_reset(void)
 {
+	ELOG
 	internal_pointer = elements;
 }
 
@@ -567,6 +584,7 @@ tags_list_reset(void)
 static struct fastfind_key_value *
 tags_list_next(void)
 {
+	ELOG
 	static struct fastfind_key_value kv;
 
 	if (!internal_pointer->name) return NULL;
@@ -588,6 +606,7 @@ static struct fastfind_index ff_tags_index
 void
 init_tags_lookup(void)
 {
+	ELOG
 #ifdef USE_FASTFIND
 	fastfind_index(&ff_tags_index, FF_COMPRESS | FF_LOCALE_INDEP);
 #endif
@@ -596,6 +615,7 @@ init_tags_lookup(void)
 void
 free_tags_lookup(void)
 {
+	ELOG
 #ifdef USE_FASTFIND
 	fastfind_done(&ff_tags_index);
 #endif
@@ -618,6 +638,7 @@ const char *
 count_newline_entities(const char *html, const char *eof,
 		       int *newlines_out)
 {
+	ELOG
 	int newlines = 0;
 	int prev_was_cr = 0; /* treat CRLF as one newline, not two */
 
@@ -673,6 +694,7 @@ parse_html(char *html, char *eof,
 	   struct part *part, char *head,
 	   struct html_context *html_context)
 {
+	ELOG
 	char *base_pos = html;
 	int noupdate = 0;
 
@@ -852,6 +874,7 @@ start_element(struct element_info *ei,
               char *eof, char *attr,
               struct html_context *html_context)
 {
+	ELOG
 #define ELEMENT_RENDER_PROLOGUE \
 	ln_break(html_context, ei->linebreak); \
 	a = get_attr_val(attr, "id", html_context->doc_cp); \
@@ -1026,6 +1049,8 @@ start_element(struct element_info *ei,
 							&html_context->css_styles,
 							&html_context->stack);
 
+			html_context->visibility_hidden = 0;
+
 			if (selector) {
 				apply_css_selector_style(html_context, html_top, selector);
 				done_css_selector(selector);
@@ -1047,7 +1072,12 @@ start_element(struct element_info *ei,
 	/* Call the element's open handler. */
 	if (ei->open) {
 		if (has_attr(attr, "hidden", html_context->doc_cp)) {
-			html_skip(html_context, attr);
+			if (ei->open != html_iframe) {
+				html_skip(html_context, attr);
+			} else {
+				html_context->hidden = 1;
+				ei->open(html_context, attr, html, eof, &html);
+			}
 		} else {
 			ei->open(html_context, attr, html, eof, &html);
 		}
@@ -1060,6 +1090,8 @@ start_element(struct element_info *ei,
 	} else
 #endif
 	do {
+		html_context->visibility_hidden = 0;
+
 		if (selector && html_top->options) {
 			/* Call it now to override default colors of the elements. */
 			selector = get_css_selector_for_element(html_context, html_top,
@@ -1101,6 +1133,7 @@ end_element(struct element_info *ei,
             char *eof, char *attr,
             struct html_context *html_context)
 {
+	ELOG
 	struct html_element *e, *elt;
 	int lnb = 0;
 	int kill = 0;
@@ -1200,6 +1233,7 @@ process_element(char *name, int namelen, int endingtag,
                 struct html_context *html_context)
 
 {
+	ELOG
 	struct element_info *ei;
 
 #ifndef USE_FASTFIND
@@ -1239,6 +1273,7 @@ void
 scan_http_equiv(char *s, char *eof, struct string *head,
 		struct string *title, int cp)
 {
+	ELOG
 	char *name, *attr, *he, *c;
 	int namelen;
 
@@ -1333,6 +1368,7 @@ xsp:
 int
 supports_html_media_attr(const char *media)
 {
+	ELOG
 	const char *optstr;
 
 	/* The 1999-12-24 edition of HTML 4.01 is inconsistent on what
