@@ -104,6 +104,7 @@ set_screen_char_color(struct screen_char *schar,
 		      color_flags_T color_flags,
 		      color_mode_T color_mode)
 {
+	ELOG
 	struct color_pair colors = INIT_COLOR_PAIR(bgcolor, fgcolor);
 
 	set_term_color(schar, &colors, color_flags, color_mode);
@@ -113,6 +114,7 @@ static int
 realloc_line(struct html_context *html_context, struct document *document,
              int y, int length)
 {
+	ELOG
 	struct screen_char *pos, *end;
 	struct line *line;
 	int orig_length;
@@ -151,6 +153,7 @@ realloc_line(struct html_context *html_context, struct document *document,
 static inline int
 realloc_spaces(struct part *part, int length)
 {
+	ELOG
 	if (length < part->spaces_len)
 		return 0;
 
@@ -183,6 +186,7 @@ static inline void
 clear_hchars(struct html_context *html_context, int x, int y, int width,
 	     struct screen_char *a)
 {
+	ELOG
 	struct part *part;
 	struct screen_char *pos, *end;
 
@@ -218,6 +222,7 @@ get_frame_char(struct html_context *html_context, struct part *part,
 	       int x, int y, unsigned char data,
                color_T bgcolor, color_T fgcolor)
 {
+	ELOG
 	struct screen_char *template_;
 
 	assert(html_context);
@@ -246,6 +251,7 @@ static inline struct screen_char *
 get_format_screen_char(struct html_context *html_context,
                        link_state_T link_state)
 {
+	ELOG
 	static struct text_style ta_cache = INIT_TEXT_STYLE(-1, 0x0, 0x0);
 	static struct screen_char schar_cache;
 
@@ -275,6 +281,7 @@ draw_frame_hchars(struct part *part, int x, int y, int width,
 		  unsigned char data, color_T bgcolor, color_T fgcolor,
 		  struct html_context *html_context)
 {
+	ELOG
 	struct screen_char *template_;
 
 	assert(width > 0);
@@ -296,6 +303,7 @@ draw_frame_hchars(struct part *part, int x, int y, int width,
 void
 draw_blockquote_chars(struct part *part, int y, struct html_context *html_context)
 {
+	ELOG
 	int i;
 	int x = par_elformat.orig_leftmargin;
 	struct screen_char *const schar = get_format_screen_char(html_context, 0);
@@ -324,6 +332,7 @@ draw_frame_vchars(struct part *part, int x, int y, int height,
 		  unsigned char data, color_T bgcolor, color_T fgcolor,
 		  struct html_context *html_context)
 {
+	ELOG
 	struct screen_char *template_ = get_frame_char(html_context, part, x, y,
 	                                              data, bgcolor, fgcolor);
 
@@ -343,6 +352,7 @@ void
 expand_lines(struct html_context *html_context, struct part *part,
              int x, int y, int lines, color_T bgcolor)
 {
+	ELOG
 	int line;
 
 	assert(part && part->document);
@@ -376,6 +386,7 @@ static inline int
 set_hline(struct html_context *html_context, const char *chars, int charslen,
 	  link_state_T link_state)
 {
+	ELOG
 	struct part *const part = html_context->part;
 	struct screen_char *const schar = get_format_screen_char(html_context,
 								 link_state);
@@ -411,7 +422,9 @@ set_hline(struct html_context *html_context, const char *chars, int charslen,
 
 	if (part->document) {
 		struct document *const document = part->document;
-
+#ifdef CONFIG_LIBDOM
+		html_context->current_number = html_top->name - document->text.source;
+#endif
 		/* Reallocate LINE(y).chars[] to large enough.  The
 		 * last parameter of realloc_line is the index of the
 		 * last element to which we may want to write,
@@ -485,6 +498,7 @@ set_hline(struct html_context *html_context, const char *chars, int charslen,
 						unsigned char attr = schar->attr;
 
 						schar->data = (unsigned char)*chars++;
+						schar->number = html_context->current_number;
 						schar->attr = SCREEN_ATTR_FRAME;
 						copy_screen_chars(&POS(x, y), schar, 1);
 						schar->attr = attr;
@@ -518,6 +532,7 @@ good_char:
 
 					for(i = 0; i < unicode_to_cell(data); i++) {
 						schar->data = encode_utf8(data)[i];
+						schar->number = html_context->current_number;
 						part->char_width[x] = 1;
 						copy_screen_chars(&POS(x++, y), schar, 1);
 					}
@@ -527,14 +542,17 @@ good_char:
 
 				if (unicode_to_cell(data) == 2) {
 					schar->data = (unicode_val_T)data;
+					schar->number = html_context->current_number;
 					part->char_width[x] = 2;
 					copy_screen_chars(&POS(x++, y), schar, 1);
 					schar->data = UCS_NO_CHAR;
+					schar->number = html_context->current_number;
 					part->spaces[x] = 0;
 					part->char_width[x] = 0;
 				} else {
 					part->char_width[x] = unicode_to_cell(data);
 					schar->data = (unicode_val_T)data;
+					schar->number = html_context->current_number;
 				}
 				copy_screen_chars(&POS(x++, y), schar, 1);
 			} /* while chars < end */
@@ -543,10 +561,12 @@ good_char:
 				part->char_width[x] = 1;
 				if (*chars == NBSP_CHAR) {
 					schar->data = ' ';
+					schar->number = html_context->current_number;
 					part->spaces[x] = html_context->options->wrap_nbsp;
 				} else {
 					part->spaces[x] = (*chars == ' ');
 					schar->data = *chars;
+					schar->number = html_context->current_number;
 				}
 				copy_screen_chars(&POS(x, y), schar, 1);
 			}
@@ -619,6 +639,7 @@ static inline void
 set_hline(struct html_context *html_context, const char *chars, int charslen,
 	  link_state_T link_state)
 {
+	ELOG
 	struct part *part = html_context->part;
 	struct screen_char *schar = get_format_screen_char(html_context,
 	                                                   link_state);
@@ -632,6 +653,9 @@ set_hline(struct html_context *html_context, const char *chars, int charslen,
 		return;
 
 	if (part->document) {
+#ifdef CONFIG_LIBDOM
+		html_context->current_number = html_top->name - part->document->text.source;
+#endif
 		if (realloc_line(html_context, part->document,
 		                 Y(y), X(x) + charslen - 1) < 0)
 			return;
@@ -646,10 +670,12 @@ set_hline(struct html_context *html_context, const char *chars, int charslen,
 		for (; charslen > 0; charslen--, x++, chars++) {
 			if (*chars == NBSP_CHAR) {
 				schar->data = ' ';
+				schar->number = html_context->current_number;
 				part->spaces[x] = html_context->options->wrap_nbsp;
 			} else {
 				part->spaces[x] = (*chars == ' ');
 				schar->data = *chars;
+				schar->number = html_context->current_number;
 			}
 			copy_screen_chars(&POS(x, y), schar, 1);
 		}
@@ -668,6 +694,7 @@ set_hline(struct html_context *html_context, const char *chars, int charslen,
 static void
 move_links(struct html_context *html_context, int xf, int yf, int xt, int yt)
 {
+	ELOG
 	struct part *part;
 	struct tag *tag;
 	int nlink = renderer_context.last_link_to_move;
@@ -785,6 +812,7 @@ move_links(struct html_context *html_context, int xf, int yf, int xt, int yt)
 static inline void
 copy_chars(struct html_context *html_context, int x, int y, int width, struct screen_char *d)
 {
+	ELOG
 	struct part *part;
 
 	assert(html_context);
@@ -804,6 +832,7 @@ copy_chars(struct html_context *html_context, int x, int y, int width, struct sc
 static inline void
 move_chars(struct html_context *html_context, int x, int y, int nx, int ny)
 {
+	ELOG
 	struct part *part;
 
 	assert(html_context);
@@ -831,6 +860,7 @@ move_chars(struct html_context *html_context, int x, int y, int nx, int ny)
 static inline void
 shift_chars(struct html_context *html_context, int y, int shift)
 {
+	ELOG
 	struct part *part;
 	struct screen_char *a;
 	int len;
@@ -860,6 +890,7 @@ shift_chars(struct html_context *html_context, int y, int shift)
 static inline void
 del_chars(struct html_context *html_context, int x, int y)
 {
+	ELOG
 	struct part *part;
 
 	assert(html_context);
@@ -885,6 +916,7 @@ del_chars(struct html_context *html_context, int x, int y)
 static int inline
 split_line_at(struct html_context *html_context, int width)
 {
+	ELOG
 	struct part *part;
 	int tmp;
 	int new_width = width + par_elformat.rightmargin;
@@ -976,6 +1008,7 @@ split_line_at(struct html_context *html_context, int width)
 static int
 split_line(struct html_context *html_context)
 {
+	ELOG
 	struct part *part;
 	int x;
 
@@ -1033,6 +1066,7 @@ split_line(struct html_context *html_context)
 static void
 insert_spaces_in_link(struct part *part, int x, int y, int new_spaces)
 {
+	ELOG
 	int i = part->document->nlinks;
 
 	x = X(x);
@@ -1070,6 +1104,7 @@ insert_spaces_in_link(struct part *part, int x, int y, int new_spaces)
 static void
 justify_line(struct html_context *html_context, int y)
 {
+	ELOG
 	struct part *part;
 	struct screen_char *line; /* we save original line here */
 	int len;
@@ -1213,6 +1248,7 @@ out_of_memory:
 static void
 align_line(struct html_context *html_context, int y, int last)
 {
+	ELOG
 	struct part *part;
 	int shift;
 	int len;
@@ -1246,6 +1282,8 @@ align_line(struct html_context *html_context, int y, int last)
 static inline void
 init_link_event_hooks(struct html_context *html_context, struct link *link)
 {
+	ELOG
+#ifdef CONFIG_ECMASCRIPT
 	link->event_hooks = (LIST_OF(struct script_event_hook) *)mem_calloc(1, sizeof(*link->event_hooks));
 	if (!link->event_hooks) return;
 
@@ -1275,7 +1313,6 @@ init_link_event_hooks(struct html_context *html_context, struct link *link)
 	add_evhook(link->event_hooks, SEVHOOK_ONKEYUP, elformat.onkeyup);
 	add_evhook(link->event_hooks, SEVHOOK_ONKEYPRESS, elformat.onkeypress);
 
-#ifdef CONFIG_ECMASCRIPT
 	add_evhook(link->event_hooks, SEVHOOK_ONKEYPRESS_BODY, html_context->document->body_onkeypress);
 #endif
 #undef add_evhook
@@ -1284,6 +1321,7 @@ init_link_event_hooks(struct html_context *html_context, struct link *link)
 static struct link *
 new_link(struct html_context *html_context, const char *name, int namelen)
 {
+	ELOG
 	struct document *document;
 	struct part *part;
 	int link_number;
@@ -1309,7 +1347,7 @@ new_link(struct html_context *html_context, const char *name, int namelen)
 
 	link = &document->links[document->nlinks++];
 	link->number = link_number - 1;
-	if (document->options.use_tabindex) link->number += elformat.tabindex;
+	link->tabindex = document->options.use_tabindex ? elformat.tabindex : 0;
 	link->accesskey = elformat.accesskey;
 	link->title = null_or_stracpy(elformat.title);
 	link->where_img = null_or_stracpy(elformat.image);
@@ -1391,6 +1429,7 @@ new_link(struct html_context *html_context, const char *name, int namelen)
 static void
 html_special_tag(struct document *document, char *t, int x, int y)
 {
+	ELOG
 	struct tag *tag;
 	int tag_len;
 
@@ -1410,11 +1449,23 @@ html_special_tag(struct document *document, char *t, int x, int y)
 		renderer_context.last_tag_for_newline = tag;
 }
 
+static void
+put_chars_invisible(struct html_context *html_context, const char *chars, int charslen)
+{
+	ELOG
+	const unsigned char *data = (const unsigned char *)chars;
+	int i;
+
+	for (i = 0; i < charslen; i++) {
+		put_chars(html_context, (data[i] < 33 ? " " : NBSP_CHAR_STRING), 1);
+	}
+}
 
 void
 put_chars_conv(struct html_context *html_context,
                const char *chars, int charslen)
 {
+	ELOG
 	assert(html_context);
 	if_assert_failed return;
 
@@ -1425,11 +1476,12 @@ put_chars_conv(struct html_context *html_context,
 		put_chars(html_context, chars, charslen);
 		return;
 	}
+	void *func = html_context->visibility_hidden ? put_chars_invisible : put_chars;
 
 	convert_string(renderer_context.convert_table, chars, charslen,
 	               html_context->options->cp,
 	               (elformat.style.attr & AT_NO_ENTITIES) ? CSM_NONE : CSM_DEFAULT,
-		       NULL, (void (*)(void *, char *, int)) put_chars, html_context);
+		       NULL, (void (*)(void *, char *, int)) func, html_context);
 }
 
 /*
@@ -1440,6 +1492,7 @@ put_chars_conv(struct html_context *html_context,
 int
 dec2qwerty(int num, char *link_sym, const char *key, int base)
 {
+	ELOG
 	int newlen, i, pow;
 
 	if (base < 2) return 0;
@@ -1461,6 +1514,7 @@ dec2qwerty(int num, char *link_sym, const char *key, int base)
 int
 qwerty2dec(const char *link_sym, const char *key, int base)
 {
+	ELOG
 	int z = 0;
 	int symlen = strlen(link_sym);
 	int i;
@@ -1477,6 +1531,7 @@ qwerty2dec(const char *link_sym, const char *key, int base)
 static inline void
 put_link_number(struct html_context *html_context)
 {
+	ELOG
 	char *symkey = get_opt_str("document.browse.links.label_key", NULL);
 	struct part *part = html_context->part;
 	char s[64];
@@ -1527,6 +1582,7 @@ static inline void
 init_link_state_info(char *link, char *target,
 		     char *image, struct el_form_control *form)
 {
+	ELOG
 	assert_link_variable(renderer_context.link_state_info.image, image);
 	assert_link_variable(renderer_context.link_state_info.target, target);
 	assert_link_variable(renderer_context.link_state_info.link, link);
@@ -1540,6 +1596,7 @@ init_link_state_info(char *link, char *target,
 static inline void
 done_link_state_info(void)
 {
+	ELOG
 	mem_free_if(renderer_context.link_state_info.link);
 	mem_free_if(renderer_context.link_state_info.target);
 	mem_free_if(renderer_context.link_state_info.image);
@@ -1557,6 +1614,7 @@ process_link(struct html_context *html_context, link_state_T link_state,
 		   const char *chars, int charslen)
 #endif /* CONFIG_UTF8 */
 {
+	ELOG
 	struct part *part = html_context->part;
 	struct link *link;
 	int x_offset = 0;
@@ -1654,6 +1712,7 @@ process_link(struct html_context *html_context, link_state_T link_state,
 static inline link_state_T
 get_link_state(struct html_context *html_context)
 {
+	ELOG
 	link_state_T state;
 
 	if (!(elformat.link || elformat.image || elformat.form)) {
@@ -1681,6 +1740,7 @@ get_link_state(struct html_context *html_context)
 static inline int
 html_has_non_space_chars(const char *chars, int charslen)
 {
+	ELOG
 	int pos = 0;
 
 	while (pos < charslen)
@@ -1690,9 +1750,11 @@ html_has_non_space_chars(const char *chars, int charslen)
 	return 0;
 }
 
+
 static void
 put_chars(struct html_context *html_context, const char *chars, int charslen)
 {
+	ELOG
 	link_state_T link_state;
 	struct part *part;
 #ifdef CONFIG_UTF8
@@ -1816,6 +1878,7 @@ put_chars(struct html_context *html_context, const char *chars, int charslen)
 void
 line_break(struct html_context *html_context)
 {
+	ELOG
 	struct part *part;
 	struct tag *tag;
 
@@ -1869,6 +1932,7 @@ end:
 static void
 html_special_form(struct part *part, struct form *form)
 {
+	ELOG
 	struct form *nform;
 
 	assert(part && form);
@@ -1932,7 +1996,7 @@ html_special_form(struct part *part, struct form *form)
 		return;
 	}
 
-	ERROR("hole between forms");
+	EL_ERROR("hole between forms");
 	done_form(form);
 	return;
 }
@@ -1940,6 +2004,7 @@ html_special_form(struct part *part, struct form *form)
 static void
 html_special_form_control(struct part *part, struct el_form_control *fc)
 {
+	ELOG
 	struct form *form;
 
 	assert(part && fc);
@@ -1979,6 +2044,7 @@ html_special_form_control(struct part *part, struct el_form_control *fc)
 static void
 assert_forms_list_ok(LIST_OF(struct form) *forms)
 {
+	ELOG
 	int saw_form_num_0 = 0;
 	struct form *outer;
 
@@ -2016,6 +2082,7 @@ assert_forms_list_ok(LIST_OF(struct form) *forms)
 void
 check_html_form_hierarchy(struct part *part)
 {
+	ELOG
 	struct document *document = part->document;
 	INIT_LIST_OF(struct el_form_control, form_controls);
 	struct form *form;
@@ -2064,6 +2131,7 @@ check_html_form_hierarchy(struct part *part)
 static inline void
 color_link_lines(struct html_context *html_context)
 {
+	ELOG
 	struct document *document = html_context->part->document;
 	struct color_pair colors = INIT_COLOR_PAIR(par_elformat.color.background, 0x0);
 	color_mode_T color_mode = document->options.color_mode;
@@ -2091,6 +2159,7 @@ color_link_lines(struct html_context *html_context)
 void *
 html_special(struct html_context *html_context, html_special_type_T c, ...)
 {
+	ELOG
 	va_list l;
 	struct part *part;
 	struct document *document;
@@ -2218,18 +2287,31 @@ html_special(struct html_context *html_context, html_special_type_T c, ...)
 			break;
 
 		case SP_IFRAME:
-		{
 			if (document) {
 				char *url = va_arg(l, char *);
 				char *name = va_arg(l, char *);
+				int x = va_arg(l, int);
 				int y = va_arg(l, int);
 				int width = va_arg(l, int);
 				int height = va_arg(l, int);
+				int hidden = va_arg(l, int);
 
-				add_iframeset_entry(&document->iframe_desc, url, name, y, width, height, document->nlinks);
+				add_iframeset_entry(document, url, name, x, y, width, height, document->nlinks, hidden);
 			}
 			break;
+
+		case SP_IMAGE:
+#if defined(CONFIG_KITTY) || defined(CONFIG_LIBSIXEL)
+		{
+			if (document) {
+				struct uri *uri = va_arg(l, struct uri *);
+
+				add_to_uri_list(&document->image_uris, uri);
+			}
 		}
+#endif
+			break;
+
 		default:
 			break;
 
@@ -2243,6 +2325,7 @@ html_special(struct html_context *html_context, html_special_type_T c, ...)
 void
 free_table_cache(void)
 {
+	ELOG
 	if (table_cache) {
 		struct hash_item *item;
 		int i;
@@ -2264,6 +2347,7 @@ format_html_part(struct html_context *html_context,
 		 int x, int y, char *head,
 		 int link_num)
 {
+	ELOG
 	struct part *part;
 	void *html_state;
 	struct tag *saved_last_tag_to_move = renderer_context.last_tag_to_move;
@@ -2405,6 +2489,7 @@ ret:
 static void
 subst_frame_chars(struct document *document)
 {
+	ELOG
 	int y;
 
 	for (y = 0; y < document->height; y++) {
@@ -2525,6 +2610,7 @@ void
 render_html_document(struct cache_entry *cached, struct document *document,
 		     struct string *buffer)
 {
+	ELOG
 	struct html_context *html_context;
 	struct part *part;
 	char *start;
