@@ -30,6 +30,7 @@ static int in_script = 0;
 static bool
 dump_dom_element_closing(struct string *buf, dom_node *node)
 {
+	ELOG
 	dom_exception exc;
 	dom_string *node_name = NULL;
 	dom_node_type type;
@@ -80,6 +81,7 @@ dump_dom_element_closing(struct string *buf, dom_node *node)
 static bool
 dump_node_element_attribute(struct string *buf, dom_node *node)
 {
+	ELOG
 	dom_exception exc;
 	dom_string *attr = NULL;
 	dom_string *attr_value = NULL;
@@ -120,6 +122,7 @@ dump_node_element_attribute(struct string *buf, dom_node *node)
 static bool
 dump_dom_element(void *mapa, void *mapa_rev, struct string *buf, dom_node *node, int depth)
 {
+	ELOG
 	dom_exception exc;
 	dom_string *node_name = NULL;
 	dom_node_type type;
@@ -145,6 +148,8 @@ dump_dom_element(void *mapa, void *mapa_rev, struct string *buf, dom_node *node,
 					add_to_string(buf, "&lt;");
 				} else if (!in_script && length == 1 && *string_text == '>') {
 					add_to_string(buf, "&gt;");
+				} else if (!in_script && length == 1 && *string_text == '&') {
+					add_to_string(buf, "&amp;");
 				} else {
 					add_bytes_to_string(buf, string_text, length);
 				}
@@ -218,6 +223,7 @@ dump_dom_element(void *mapa, void *mapa_rev, struct string *buf, dom_node *node,
 static bool
 walk_tree(void *mapa, void *mapa_rev, struct string *buf, dom_node *node, bool start, int depth)
 {
+	ELOG
 	dom_exception exc;
 	dom_node *child;
 
@@ -269,6 +275,7 @@ walk_tree(void *mapa, void *mapa_rev, struct string *buf, dom_node *node, bool s
 void
 render_xhtml_document(struct cache_entry *cached, struct document *document, struct string *buffer)
 {
+	ELOG
 	if (!libdom_initialised) {
 		corestrings_init();
 		keybstrings_init();
@@ -305,16 +312,17 @@ render_xhtml_document(struct cache_entry *cached, struct document *document, str
 void
 free_libdom(void)
 {
+	ELOG
 	if (libdom_initialised) {
 		keybstrings_fini();
 		corestrings_fini();
 	}
 }
 
-#if 0
 static void
 walk_tree2(struct document *document, dom_node *node)
 {
+	ELOG
 	dom_node *child = NULL;
 	/* Get the node's first child */
 	dom_exception exc = dom_node_get_first_child(node, &child);
@@ -403,12 +411,12 @@ next:
 		child = next_child;
 	}
 }
-#endif
 
 #if 0
 static void
 walk_tree2_color(struct terminal *term, struct el_box *box, struct document *document, int vx, int vy, dom_node *node)
 {
+	ELOG
 	dom_node *child = NULL;
 	/* Get the node's first child */
 	dom_exception exc = dom_node_get_first_child(node, &child);
@@ -482,6 +490,7 @@ walk_tree2_color(struct terminal *term, struct el_box *box, struct document *doc
 void
 debug_dump_xhtml(void *d)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	dom_html_document *doc = (dom_html_document *)d;
 
@@ -521,9 +530,35 @@ debug_dump_xhtml(void *d)
 #endif
 }
 
+void
+debug_dump_xhtml2(void *r)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	dom_node *root = (dom_node *)r;
+
+	struct string text;
+
+	if (!init_string(&text)) {
+		return;
+	}
+
+	if (walk_tree(NULL, NULL, &text, root, true, 0) == false) {
+		fprintf(stderr, "Failed to complete DOM structure dump.\n");
+		done_string(&text);
+		//dom_node_unref(doc);
+		return;
+	}
+
+	fprintf(stderr, "\n---%s\n", text.source);
+	done_string(&text);
+#endif
+}
+
 void
 dump_xhtml(struct cache_entry *cached, struct document *document, int parse)
 {
+	ELOG
 	dom_exception exc; /* returned by libdom functions */
 	dom_html_document *doc = NULL; /* document, loaded into libdom */
 	dom_node *root = NULL; /* root element of document */
@@ -587,7 +622,7 @@ dump_xhtml(struct cache_entry *cached, struct document *document, int parse)
 			if (!cached2) {
 				return;
 			}
-			cached2->head = cached->head;
+			mem_free_set(&cached2->head, cached->head);
 			cached->head = NULL;
 
 			add_fragment(cached2, 0, document->text.source, document->text.length);
@@ -597,7 +632,6 @@ dump_xhtml(struct cache_entry *cached, struct document *document, int parse)
 			document->cache_id = cached2->cache_id;
 			document->cached = cached2;
 			render_xhtml_document(cached2, document, &document->text);
-			object_unlock(cached);
 			return;
 		}
 		render_html_document(cached, document, &document->text);
@@ -607,6 +641,7 @@ dump_xhtml(struct cache_entry *cached, struct document *document, int parse)
 static bool
 fire_dom_event(dom_event *event, dom_node *target)
 {
+	ELOG
 	dom_exception exc;
 	bool result;
 
@@ -622,6 +657,7 @@ fire_dom_event(dom_event *event, dom_node *target)
 int
 fire_generic_dom_event(void *t, void *tar, int bubbles, int cancelable)
 {
+	ELOG
 	dom_string *typ = (dom_string *)t;
 	dom_node *target = (dom_node *)tar;
 	dom_exception exc;
@@ -645,6 +681,7 @@ fire_generic_dom_event(void *t, void *tar, int bubbles, int cancelable)
 int
 fire_onload(void *doc)
 {
+	ELOG
 	dom_node *root = NULL;
 
 	if (!doc) {
@@ -661,10 +698,10 @@ fire_onload(void *doc)
 	return res;
 }
 
-#if 0
 void
 walk2(struct document *document)
 {
+	ELOG
 	dom_exception exc; /* returned by libdom functions */
 	dom_html_document *doc = NULL; /* document, loaded into libdom */
 	dom_node *root = NULL; /* root element of document */
@@ -688,15 +725,14 @@ walk2(struct document *document)
 	walk_tree2(document, root);
 	dom_node_unref(root);
 }
-#endif
 
-#if 0
 static int prev_offset = 0;
 static struct node_rect *prev_element = NULL;
 
 struct node_rect *
 get_element_rect(struct document *document, int offset)
 {
+	ELOG
 	if (offset == prev_offset) {
 		return prev_element;
 	}
@@ -733,38 +769,39 @@ get_element_rect(struct document *document, int offset)
 
 	return n;
 }
-#endif
-//static void dump_results(struct document *document);
-
 #if 0
+static void dump_results(struct document *document);
+#endif
+
 void
-scan_document(struct document_view *doc_view)
+scan_document(struct document *document)
 {
+	ELOG
 	int y, x;
 
-	if (!doc_view || !doc_view->document) {
+	if (document->scanned) {
 		return;
 	}
 
-	if (doc_view->document->hh) {
-		free_hash(&doc_view->document->hh);
+	if (document->hh) {
+		free_hash(&document->hh);
 	}
-	doc_view->document->hh = init_hash8();
+	document->hh = init_hash8();
 
-	if (!doc_view->document->hh) {
+	if (!document->hh) {
 		return;
 	}
 	prev_offset = 0;
 	prev_element = NULL;
 
-	for (y = 0; y < doc_view->document->height; y++) {
-		for (x = 0; x < doc_view->document->data[y].length; x++) {
-			int offset = doc_view->document->data[y].ch.chars[x].element_offset;
+	for (y = 0; y < document->height; y++) {
+		for (x = 0; x < document->data[y].length; x++) {
+			int offset = document->data[y].ch.chars[x].number;
 
 			if (!offset) {
 				continue;
 			}
-			struct node_rect *tab = get_element_rect(doc_view->document, offset);
+			struct node_rect *tab = get_element_rect(document, offset);
 
 			if (!tab) {
 				continue;
@@ -780,22 +817,23 @@ scan_document(struct document_view *doc_view)
 				tab->x1 = x;
 			}
 			if (tab->y1 < y) {
-				tab->y1 = x;
+				tab->y1 = y;
 			}
 		}
 	}
-//	dump_results(doc_view->document);
+//	dump_results(document);
 	prev_offset = 0;
 	prev_element = NULL;
-	walk2(doc_view->document);
+	document->scanned = 1;
+	walk2(document);
 //	dump_results(doc_view->document);
 }
-#endif
 
 #if 0
 static void
 dump_results(struct document *document)
 {
+	ELOG
 	struct hash_item *item;
 	int i;
 
@@ -812,6 +850,7 @@ dump_results(struct document *document)
 void
 try_to_color(struct terminal *term, struct el_box *box, struct document *document, int vx, int vy)
 {
+	ELOG
 	dom_exception exc; /* returned by libdom functions */
 	dom_html_document *doc = NULL; /* document, loaded into libdom */
 	dom_node *root = NULL; /* root element of document */
