@@ -21,17 +21,19 @@ void dump_xhtml(struct cache_entry *cached, struct document *document, int parse
 
 void free_libdom(void);
 void debug_dump_xhtml(void *doc);
+void debug_dump_xhtml2(void *node);
 
 int fire_generic_dom_event(void *typ, void *target, int bubbles, int cancelable);
 int fire_onload(void *doc);
 
 #if 0
 void walk2(struct document *document);
-void scan_document(struct document_view *doc_view);
 void try_to_color(struct terminal *term, struct el_box *box, struct document *document, int vx, int vy);
-struct node_rect *get_element_rect(struct document *document, int offset);
 #endif
 
+void scan_document(struct document *document);
+struct node_rect *get_element_rect(struct document *document, int offset);
+
 #ifdef __cplusplus
 }
 #endif
