@@ -47,12 +47,14 @@ extern int ecmascript_enabled;
 int
 ecmascript_get_interpreter_count(void)
 {
+	ELOG
 	return interpreter_count;
 }
 
 void
 toggle_ecmascript(struct session *ses)
 {
+	ELOG
 	ecmascript_enabled = !ecmascript_enabled;
 
 	if (ecmascript_enabled) {
@@ -66,6 +68,7 @@ toggle_ecmascript(struct session *ses)
 void
 ecmascript_protocol_handler(struct session *ses, struct uri *uri)
 {
+	ELOG
 	struct document_view *doc_view = current_frame(ses);
 	struct string current_url = INIT_STRING(struri(uri), (int)strlen(struri(uri)));
 	char *redirect_url, *redirect_abs_url;
@@ -112,6 +115,7 @@ add_snippets(struct ecmascript_interpreter *interpreter,
              LIST_OF(struct ecmascript_string_list_item) *doc_snippets,
              LIST_OF(struct ecmascript_string_list_item) *queued_snippets)
 {
+	ELOG
 	struct ecmascript_string_list_item *doc_current = (struct ecmascript_string_list_item *)doc_snippets->next;
 
 #ifdef CONFIG_LEDS
@@ -120,8 +124,9 @@ add_snippets(struct ecmascript_interpreter *interpreter,
 #endif
 
 	if (list_empty(*doc_snippets)
-	    || !get_opt_bool("ecmascript.enable", NULL))
+	    || !is_ecmascript_enabled_for_interpreter(interpreter)) {
 		return;
+	}
 
 	/* We do this all only once per view_state now. */
 	if (!list_empty(*queued_snippets)) {
@@ -173,6 +178,7 @@ process_snippets(struct ecmascript_interpreter *interpreter,
                  LIST_OF(struct ecmascript_string_list_item) *snippets,
                  struct ecmascript_string_list_item **current)
 {
+	ELOG
 	if (!*current)
 		*current = (struct ecmascript_string_list_item *)snippets->next;
 	for (; *current != (struct ecmascript_string_list_item *) snippets;
@@ -271,7 +277,7 @@ process_snippets(struct ecmascript_interpreter *interpreter,
 			 * without gradual_loading set. I'm not sure about
 			 * the implications though so I won't do it before
 			 * 0.10.0. --pasky */
-			ERROR("The script of %s was lost in too full a cache!",
+			EL_ERROR("The script of %s was lost in too full a cache!",
 			      uristring);
 #endif
 			return;
@@ -291,6 +297,7 @@ process_snippets(struct ecmascript_interpreter *interpreter,
 void
 check_for_snippets(struct view_state *vs, struct document_options *options, struct document *document)
 {
+	ELOG
 	if (!vs->ecmascript_fragile) {
 		assert(vs->ecmascript);
 	}
@@ -345,6 +352,7 @@ check_for_snippets(struct view_state *vs, struct document_options *options, stru
 void
 ecmascript_put_interpreter(struct ecmascript_interpreter *interpreter)
 {
+	ELOG
 	assert(interpreter);
 	assert(interpreter->backend_nesting == 0);
 	/* If the assertion fails, it is better to leak the
@@ -391,6 +399,8 @@ ecmascript_put_interpreter(struct ecmascript_interpreter *interpreter)
 #endif
 	interpreter->vs->ecmascript = NULL;
 	interpreter->vs->ecmascript_fragile = 1;
+	kill_timer(&interpreter->ani);
+	del_from_list(interpreter);
 	mem_free(interpreter);
 	--interpreter_count;
 }
@@ -398,6 +408,7 @@ ecmascript_put_interpreter(struct ecmascript_interpreter *interpreter)
 void
 check_events_for_element(struct ecmascript_interpreter *ecmascript, dom_node *element, struct term_event *ev)
 {
+	ELOG
 	const char *event_name = script_event_hook_name[SEVHOOK_ONKEYDOWN];
 
 	if (!ecmascript) {
@@ -414,6 +425,7 @@ check_events_for_element(struct ecmascript_interpreter *ecmascript, dom_node *el
 void
 ecmascript_reset_state(struct view_state *vs)
 {
+	ELOG
 	struct form_view *fv;
 	int i;
 
@@ -443,11 +455,12 @@ int
 ecmascript_eval_boolback(struct ecmascript_interpreter *interpreter,
 			 struct string *code)
 {
+	ELOG
 	int result;
 
-	if (!get_ecmascript_enable(interpreter))
+	if (!is_ecmascript_enabled_for_interpreter(interpreter)) {
 		return -1;
-	assert(interpreter);
+	}
 	interpreter->backend_nesting++;
 #ifdef CONFIG_MUJS
 	result = mujs_eval_boolback(interpreter, code);
@@ -466,6 +479,7 @@ ecmascript_eval_boolback(struct ecmascript_interpreter *interpreter,
 int
 ecmascript_current_link_evhook(struct document_view *doc_view, enum script_event_hook_type type)
 {
+	ELOG
 	struct link *link;
 	struct script_event_hook *evhook;
 
@@ -508,6 +522,7 @@ ecmascript_current_link_evhook(struct document_view *doc_view, enum script_event
 void
 ecmascript_detach_form_view(struct form_view *fv)
 {
+	ELOG
 #ifdef CONFIG_MUJS
 #elif defined(CONFIG_QUICKJS)
 	quickjs_detach_form_view(fv);
@@ -518,6 +533,7 @@ ecmascript_detach_form_view(struct form_view *fv)
 
 void ecmascript_detach_form_state(struct form_state *fs)
 {
+	ELOG
 #ifdef CONFIG_MUJS
 #elif defined(CONFIG_QUICKJS)
 	quickjs_detach_form_state(fs);
@@ -528,6 +544,7 @@ void ecmascript_detach_form_state(struct form_state *fs)
 
 void ecmascript_moved_form_state(struct form_state *fs)
 {
+	ELOG
 #ifdef CONFIG_MUJS
 #elif defined(CONFIG_QUICKJS)
 	quickjs_moved_form_state(fs);
@@ -536,9 +553,39 @@ void ecmascript_moved_form_state(struct form_state *fs)
 #endif
 }
 
+int
+document_fire_onkeydown(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+#ifdef CONFIG_ECMASCRIPT_SMJS
+	return spidermonkey_document_fire_onkeydown(interpreter, ev);
+#endif
+#ifdef CONFIG_QUICKJS
+	return quickjs_document_fire_onkeydown(interpreter, ev);
+#endif
+#ifdef CONFIG_MUJS
+	return mujs_document_fire_onkeydown(interpreter, ev);
+#endif
+}
+
+int
+document_fire_onkeyup(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+#ifdef CONFIG_ECMASCRIPT_SMJS
+	return spidermonkey_document_fire_onkeyup(interpreter, ev);
+#endif
+#ifdef CONFIG_QUICKJS
+	return quickjs_document_fire_onkeyup(interpreter, ev);
+#endif
+#ifdef CONFIG_MUJS
+	return mujs_document_fire_onkeyup(interpreter, ev);
+#endif
+}
+
+
 void *
 walk_tree_query(dom_node *node, const char *selector, int depth)
 {
+	ELOG
 	dom_exception exc;
 	dom_node *child = NULL;
 	void *res = NULL;
@@ -592,6 +639,7 @@ walk_tree_query(dom_node *node, const char *selector, int depth)
 void
 walk_tree_query_append(dom_node *node, const char *selector, int depth, LIST_OF(struct selector_node) *result_list)
 {
+	ELOG
 	dom_exception exc;
 	dom_node *child;
 	void *res = NULL;
@@ -644,6 +692,7 @@ walk_tree_query_append(dom_node *node, const char *selector, int depth, LIST_OF(
 static bool
 node_has_classes(struct dom_node *node, void *ctx)
 {
+	ELOG
 	LIST_OF(struct class_string) *list = (LIST_OF(struct class_string) *)ctx;
 	struct class_string *st;
 
@@ -666,6 +715,7 @@ node_has_classes(struct dom_node *node, void *ctx)
 static LIST_OF(struct class_string) *
 prepare_strings(const char *text)
 {
+	ELOG
 	if (!text) {
 		return NULL;
 	}
@@ -706,6 +756,7 @@ prepare_strings(const char *text)
 void
 free_el_dom_collection(void *ctx)
 {
+	ELOG
 	if (!ctx) {
 		return;
 	}
@@ -761,6 +812,7 @@ el_dom_html_collection_initialise(dom_html_document *doc,
                 dom_node *root,
                 dom_callback_is_in_collection ic, void *ctx)
 {
+	ELOG
 	assert(doc != NULL);
 	assert(ic != NULL);
 	assert(root != NULL);
@@ -784,6 +836,7 @@ el_dom_html_collection_create(dom_html_document *doc,
                 void *ctx,
                 struct el_dom_html_collection **col)
 {
+	ELOG
 	*col = (struct el_dom_html_collection *)malloc(sizeof(struct el_dom_html_collection));
 
 	if (*col == NULL) {
@@ -796,6 +849,7 @@ el_dom_html_collection_create(dom_html_document *doc,
 void *
 get_elements_by_class_name(dom_html_document *doc, dom_node *node, const char *classes)
 {
+	ELOG
 	if (!node || !classes) {
 		return NULL;
 	}
@@ -814,9 +868,34 @@ get_elements_by_class_name(dom_html_document *doc, dom_node *node, const char *c
 	return col;
 }
 
+static bool
+scripts_callback(struct dom_node *node, void *ctx)
+{
+	ELOG
+	dom_html_element_type ty;
+	dom_exception exc = dom_html_element_get_tag_type(node, &ty);
+
+	return (exc == DOM_NO_ERR) && (ty == DOM_HTML_ELEMENT_TYPE_SCRIPT);
+}
+
+void *
+get_scripts(dom_html_document *doc, dom_node *node)
+{
+	ELOG
+	struct el_dom_html_collection *col = NULL;
+	dom_exception exc = el_dom_html_collection_create(doc, node, scripts_callback, NULL, &col);
+
+	if (exc != DOM_NO_ERR || !col) {
+		return NULL;
+	}
+
+	return col;
+}
+
 void
 camel_to_html(const char *text, struct string *result)
 {
+	ELOG
 	add_to_string(result, "data-");
 
 	for (; *text; text++) {
@@ -832,6 +911,7 @@ camel_to_html(const char *text, struct string *result)
 static bool
 el_dump_node_element_attribute(struct string *buf, dom_node *node)
 {
+	ELOG
 	dom_exception exc;
 	dom_string *attr = NULL;
 	dom_string *attr_value = NULL;
@@ -872,6 +952,7 @@ el_dump_node_element_attribute(struct string *buf, dom_node *node)
 static bool
 el_dump_element(struct string *buf, dom_node *node, bool toSortAttrs)
 {
+	ELOG
 // TODO toSortAttrs
 	dom_exception exc;
 	dom_string *node_name = NULL;
@@ -956,6 +1037,7 @@ el_dump_element(struct string *buf, dom_node *node, bool toSortAttrs)
 void
 ecmascript_walk_tree(struct string *buf, void *nod, bool start, bool toSortAttrs)
 {
+	ELOG
 	dom_node *node = (dom_node *)(nod);
 	dom_nodelist *children = NULL;
 	dom_node_type type;
