@@ -25,6 +25,7 @@
 #include "document/html/frames.h"
 #include "document/document.h"
 #include "document/forms.h"
+#include "document/libdom/corestrings.h"
 #include "document/libdom/doc.h"
 #include "document/libdom/mapa.h"
 #include "document/view.h"
@@ -39,6 +40,7 @@
 #include "js/mujs/forms.h"
 #include "js/mujs/fragment.h"
 #include "js/mujs/implementation.h"
+#include "js/mujs/keyboard.h"
 #include "js/mujs/location.h"
 #include "js/mujs/document.h"
 #include "js/mujs/element.h"
@@ -69,33 +71,12 @@
 
 //static xmlpp::Document emptyDoc;
 
-struct el_listener {
-	LIST_HEAD_EL(struct el_listener);
-	char *typ;
-	const char *fun;
-};
-
-enum readyState {
-	LOADING = 0,
-	INTERACTIVE,
-	COMPLETE
-};
-
-struct mjs_document_private {
-	struct ecmascript_interpreter *interpreter;
-	const char *thisval;
-	LIST_OF(struct el_listener) listeners;
-	dom_event_listener *listener;
-	void *node;
-	int ref_count;
-	enum readyState state;
-};
-
 static void document_event_handler(dom_event *event, void *pw);
 
 void *
 mjs_doc_getprivate(js_State *J, int idx)
 {
+	ELOG
 	struct mjs_document_private *priv = (struct mjs_document_private *)js_touserdata(J, idx, "document");
 
 	if (!priv) {
@@ -110,10 +91,12 @@ static void mjs_document_dispatchEvent(js_State *J);
 static void
 mjs_document_get_property_anchors(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -132,6 +115,7 @@ mjs_document_get_property_anchors(js_State *J)
 static void
 mjs_document_get_property_baseURI(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -163,10 +147,12 @@ mjs_document_get_property_baseURI(js_State *J)
 static void
 mjs_document_get_property_body(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -189,6 +175,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_set_property_body(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -200,6 +187,7 @@ mjs_document_set_property_body(js_State *J)
 static void
 mjs_document_get_property_cookie(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -234,6 +222,7 @@ mjs_document_get_property_cookie(js_State *J)
 static void
 mjs_document_set_property_cookie(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -267,6 +256,7 @@ mjs_document_set_property_cookie(js_State *J)
 static void
 mjs_document_get_property_charset(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -277,10 +267,12 @@ mjs_document_get_property_charset(js_State *J)
 static void
 mjs_document_get_property_childNodes(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -311,6 +303,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_get_property_currentScript(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -321,17 +314,17 @@ mjs_document_get_property_currentScript(js_State *J)
 
 	if (mapa) {
 		dom_node *elem = (dom_node *)find_in_map(mapa, interpreter->element_offset);
+		NODEINFO(elem);
 
 		if (elem) {
-			dom_string *tag_name = NULL;
-			dom_exception exc = dom_node_get_node_name(elem, &tag_name);
+			dom_html_element_type ty;
+			dom_exception exc = dom_html_element_get_tag_type(elem, &ty);
 
-			if (exc != DOM_NO_ERR || !tag_name) {
+			if (exc != DOM_NO_ERR) {
 				js_pushnull(J);
 				return;
 			}
-			bool isScript = !strcmp("SCRIPT", dom_string_data(tag_name));
-			dom_string_unref(tag_name);
+			bool isScript = (ty == DOM_HTML_ELEMENT_TYPE_SCRIPT);
 
 			if (isScript) {
 				mjs_push_node(J, elem);
@@ -345,6 +338,7 @@ mjs_document_get_property_currentScript(js_State *J)
 static void
 mjs_document_get_property_defaultView(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -354,10 +348,12 @@ mjs_document_get_property_defaultView(js_State *J)
 static void
 mjs_document_get_property_doctype(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushundefined(J);
@@ -371,10 +367,12 @@ mjs_document_get_property_doctype(js_State *J)
 static void
 mjs_document_get_property_documentElement(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -397,6 +395,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_get_property_documentURI(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -424,6 +423,7 @@ mjs_document_get_property_documentURI(js_State *J)
 static void
 mjs_document_get_property_domain(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -455,10 +455,12 @@ mjs_document_get_property_domain(js_State *J)
 static void
 mjs_document_get_property_forms(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -477,42 +479,44 @@ mjs_document_get_property_forms(js_State *J)
 static void
 mjs_document_get_property_head(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
-	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
-	struct document_view *doc_view = interpreter->vs->doc_view;
-	struct document *document = doc_view->document;
+	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
-	if (!document->dom) {
+	if (!doc) {
 		js_pushnull(J);
 		return;
 	}
-// TODO
-#if 0
-	xmlpp::Document *docu = (xmlpp::Document *)document->dom;
-	xmlpp::Element* root = (xmlpp::Element *)docu->get_root_node();
+	dom_nodelist *nodes = NULL;
+	dom_exception exc = dom_document_get_elements_by_tag_name(doc, corestring_dom_HEAD, &nodes);
 
-	xmlpp::ustring xpath = "//head";
-	xmlpp::Node::NodeSet elements = root->find(xpath);
-
-	if (elements.size() == 0) {
+	if (exc != DOM_NO_ERR || !nodes) {
 		js_pushnull(J);
 		return;
 	}
-	auto element = elements[0];
-	mjs_push_node(J, element);
-#endif
-	js_pushnull(J);
+	dom_node *head = NULL;
+	exc = dom_nodelist_item(nodes, 0, &head);
+	dom_nodelist_unref(nodes);
+
+	if (exc != DOM_NO_ERR || !head) {
+		js_pushnull(J);
+		return;
+	}
+	mjs_push_node(J, head);
 }
 
 static void
 mjs_document_get_property_images(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -531,6 +535,7 @@ mjs_document_get_property_images(js_State *J)
 static void
 mjs_document_get_property_implementation(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -540,10 +545,12 @@ mjs_document_get_property_implementation(js_State *J)
 static void
 mjs_document_get_property_links(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -562,10 +569,133 @@ mjs_document_get_property_links(js_State *J)
 static void
 mjs_document_get_property_location(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
-	mjs_push_location(J);
+	mjs_push_location(J, NULL);
+}
+
+static void
+mjs_document_get_property_onkeydown(js_State *J)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	struct mjs_document_private *doc_private = (struct mjs_document_private *)js_touserdata(J, 0, "document");
+
+	if (!doc_private || !doc_private->onkeydown) {
+		js_pushnull(J);
+		return;
+	}
+	js_getregistry(J, doc_private->onkeydown);
+}
+
+static void
+mjs_document_get_property_onkeyup(js_State *J)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	struct mjs_document_private *doc_private = (struct mjs_document_private *)js_touserdata(J, 0, "document");
+
+	if (!doc_private || !doc_private->onkeyup) {
+		js_pushnull(J);
+		return;
+	}
+	js_getregistry(J, doc_private->onkeyup);
+}
+
+static void
+mjs_document_set_property_onkeydown(js_State *J)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	struct mjs_document_private *doc_private = (struct mjs_document_private *)js_touserdata(J, 0, "document");
+
+	if (!doc_private) {
+		js_pushundefined(J);
+		return;
+	}
+	js_copy(J, 1);
+	doc_private->onkeydown = js_ref(J);
+	js_pushundefined(J);
+}
+
+static void
+mjs_document_set_property_onkeyup(js_State *J)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	struct mjs_document_private *doc_private = (struct mjs_document_private *)js_touserdata(J, 0, "document");
+
+	if (!doc_private) {
+		js_pushundefined(J);
+		return;
+	}
+	js_copy(J, 1);
+	doc_private->onkeyup = js_ref(J);
+	js_pushundefined(J);
+}
+
+int
+mujs_document_fire_onkeydown(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	js_State *J = (js_State *)interpreter->backend_data;
+
+	if (!J) {
+		return 0;
+	}
+	struct mjs_document_private *doc_private = (struct mjs_document_private *)interpreter->doc_private;
+
+	if (!doc_private || !doc_private->onkeydown) {
+		return 0;
+	}
+	js_getregistry(J, doc_private->onkeydown); /* retrieve the js function from the registry */
+	js_getregistry(J, doc_private->thisval);
+	mjs_push_keyboardEvent(J, ev, "onkeydown");
+	js_pcall(J, 1);
+	js_pop(J, 1);
+
+	check_for_rerender(interpreter, "onkeydown");
+	return 1;
+}
+
+int
+mujs_document_fire_onkeyup(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	js_State *J = (js_State *)interpreter->backend_data;
+
+	if (!J) {
+		return 0;
+	}
+	struct mjs_document_private *doc_private = (struct mjs_document_private *)interpreter->doc_private;
+
+	if (!doc_private || !doc_private->onkeyup) {
+		return 0;
+	}
+	js_getregistry(J, doc_private->onkeyup); /* retrieve the js function from the registry */
+	js_getregistry(J, doc_private->thisval);
+	mjs_push_keyboardEvent(J, ev, "onkeyup");
+	js_pcall(J, 1);
+	js_pop(J, 1);
+
+	check_for_rerender(interpreter, "onkeyup");
+	return 1;
 }
 
 static void mjs_document_set_property_url(js_State *J);
@@ -573,6 +703,7 @@ static void mjs_document_set_property_url(js_State *J);
 static void
 mjs_document_set_property_location(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -582,6 +713,7 @@ mjs_document_set_property_location(js_State *J)
 static void
 mjs_document_get_property_nodeType(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -591,6 +723,7 @@ mjs_document_get_property_nodeType(js_State *J)
 static void
 mjs_document_get_property_readyState(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -617,6 +750,7 @@ mjs_document_get_property_readyState(js_State *J)
 static void
 mjs_document_get_property_referrer(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -642,7 +776,7 @@ mjs_document_get_property_referrer(js_State *J)
 	switch (get_opt_int("protocol.http.referer.policy", NULL)) {
 	case REFERER_NONE:
 		/* oh well */
-		js_pushundefined(J);
+		js_pushstring(J, "");
 		return;
 
 	case REFERER_FAKE:
@@ -659,7 +793,7 @@ mjs_document_get_property_referrer(js_State *J)
 				mem_free(str);
 				return;
 			} else {
-				js_pushundefined(J);
+				js_pushstring(J, "");
 				return;
 			}
 		}
@@ -672,57 +806,39 @@ mjs_document_get_property_referrer(js_State *J)
 			js_pushstring(J, str);
 			mem_free(str);
 			return;
-		} else {
-			js_pushundefined(J);
-			return;
 		}
 		break;
 	}
-	js_pushundefined(J);
+	js_pushstring(J, "");
 }
 
 static void
 mjs_document_get_property_scripts(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
-	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
-	struct document_view *doc_view = interpreter->vs->doc_view;
-	struct document *document = doc_view->document;
+	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
-	if (!document->dom) {
+	if (!doc) {
 		js_pushnull(J);
 		return;
 	}
-// TODO
-#if 0
-	xmlpp::Document *docu = (xmlpp::Document *)document->dom;
-	xmlpp::Element* root = (xmlpp::Element *)docu->get_root_node();
+	dom_html_collection *col = (dom_html_collection *)get_scripts(doc, (dom_node *)doc);
 
-	xmlpp::ustring xpath = "//script";
-	xmlpp::Node::NodeSet *elements = new(std::nothrow) xmlpp::Node::NodeSet;
-
-	if (!elements) {
+	if (!col) {
 		js_pushnull(J);
 		return;
 	}
-
-	*elements = root->find(xpath);
-
-	if (elements->size() == 0) {
-		js_pushnull(J);
-		return;
-	}
-	mjs_push_collection(J, elements);
-#endif
-	js_pushnull(J);
+	mjs_push_collection2(J, col);
 }
 
-
 static void
 mjs_document_get_property_title(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -748,6 +864,7 @@ mjs_document_get_property_title(js_State *J)
 static void
 mjs_document_set_property_title(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -783,6 +900,7 @@ mjs_document_set_property_title(js_State *J)
 static void
 mjs_document_get_property_url(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -816,6 +934,7 @@ mjs_document_get_property_url(js_State *J)
 static void
 mjs_document_set_property_url(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -845,6 +964,7 @@ mjs_document_set_property_url(js_State *J)
 static void
 mjs_document_write_do(js_State *J, int newline)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -891,6 +1011,7 @@ mjs_document_write_do(js_State *J, int newline)
 static void
 mjs_document_write(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -901,6 +1022,7 @@ mjs_document_write(js_State *J)
 static void
 mjs_document_writeln(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -911,6 +1033,7 @@ mjs_document_writeln(js_State *J)
 static void
 mjs_document_replace(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -979,6 +1102,7 @@ mjs_document_replace(js_State *J)
 static void
 mjs_document_addEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -989,6 +1113,7 @@ mjs_document_addEventListener(js_State *J)
 		return;
 	}
 	dom_document *doc = (dom_document *)doc_private->node;
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1055,9 +1180,7 @@ mjs_document_addEventListener(js_State *J)
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(doc_private->listener);
 
 	js_pushundefined(J);
@@ -1066,6 +1189,7 @@ ex:
 static void
 mjs_document_removeEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1076,6 +1200,7 @@ mjs_document_removeEventListener(js_State *J)
 		return;
 	}
 	dom_document *doc = (dom_document *)doc_private->node;
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1128,10 +1253,12 @@ mjs_document_removeEventListener(js_State *J)
 static void
 mjs_document_createComment(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1169,10 +1296,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_createDocumentFragment(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1195,10 +1324,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_createElement(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
+
 	dom_string *tag_name = NULL;
 	dom_exception exc;
 	const char *str = js_tostring(J, 1);
@@ -1231,10 +1363,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_createTextNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
+
 	dom_string *data = NULL;
 	dom_exception exc;
 	const char *str = js_tostring(J, 1);
@@ -1267,10 +1402,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_getElementById(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1308,10 +1445,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_getElementsByClassName(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_html_document *doc = (dom_html_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1331,6 +1470,7 @@ mjs_document_getElementsByClassName(js_State *J)
 static void
 mjs_document_getElementsByName(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1375,10 +1515,12 @@ mjs_document_getElementsByName(js_State *J)
 static void
 mjs_document_getElementsByTagName(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1413,10 +1555,12 @@ mjs_document_getElementsByTagName(js_State *J)
 static void
 mjs_document_querySelector(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1460,10 +1604,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_document_querySelectorAll(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1509,10 +1655,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_doctype_get_property_name(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document_type *dtd = (dom_document_type *)(js_touserdata(J, 0, "doctype"));
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		js_pushnull(J);
@@ -1532,10 +1680,12 @@ mjs_doctype_get_property_name(js_State *J)
 static void
 mjs_doctype_get_property_publicId(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document_type *dtd = (dom_document_type *)(js_touserdata(J, 0, "doctype"));
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		js_pushnull(J);
@@ -1555,10 +1705,12 @@ mjs_doctype_get_property_publicId(js_State *J)
 static void
 mjs_doctype_get_property_systemId(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document_type *dtd = (dom_document_type *)(js_touserdata(J, 0, "doctype"));
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		js_pushnull(J);
@@ -1578,6 +1730,7 @@ mjs_doctype_get_property_systemId(js_State *J)
 static void
 mjs_document_toString(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1587,6 +1740,7 @@ mjs_document_toString(js_State *J)
 static void
 mjs_doctype_toString(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1598,19 +1752,20 @@ void *map_doctypes;
 static void
 mjs_doctype_finalizer(js_State *J, void *node)
 {
+	ELOG
+	NODEINFO(node);
 	attr_erase_from_map(map_doctypes, node);
 
-	if (node) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-		dom_node_unref((dom_node *)node);
-	}
+	dom_node_unref((dom_node *)node);
 }
 
 void
 mjs_push_doctype(js_State *J, void *node)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1628,6 +1783,7 @@ mjs_push_doctype(js_State *J, void *node)
 static void
 mjs_doc_private_finalizer(js_State *J, void *priv)
 {
+	ELOG
 	struct mjs_document_private *doc_private = (struct mjs_document_private *)priv;
 
 	if (doc_private) {
@@ -1641,13 +1797,19 @@ mjs_doc_private_finalizer(js_State *J, void *priv)
 			mem_free_set(&l->typ, NULL);
 			if (l->fun) js_unref(J, l->fun);
 		}
+		if (doc_private->onkeydown) {
+			js_unref(J, doc_private->onkeydown);
+		}
+		if (doc_private->onkeyup) {
+			js_unref(J, doc_private->onkeyup);
+		}
 		free_list(doc_private->listeners);
-		if (doc_private->node) {
+		NODEINFO(doc_private->node);
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(doc_private->node);
-		}
+		dom_node_unref(doc_private->node);
+
 		if (doc_private->thisval) {
 			js_unref(J, doc_private->thisval);
 		}
@@ -1658,6 +1820,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 void
 mjs_push_document(js_State *J, void *doc)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1710,6 +1873,8 @@ mjs_push_document(js_State *J, void *doc)
 		addproperty(J, "links", mjs_document_get_property_links, NULL);
 		addproperty(J, "location", mjs_document_get_property_location, mjs_document_set_property_location);
 		addproperty(J, "nodeType", mjs_document_get_property_nodeType, NULL);
+		addproperty(J, "onkeydown", mjs_document_get_property_onkeydown, mjs_document_set_property_onkeydown);
+		addproperty(J, "onkeyup", mjs_document_get_property_onkeyup, mjs_document_set_property_onkeyup);
 		addproperty(J, "readyState", mjs_document_get_property_readyState, NULL);
 		addproperty(J, "referrer", mjs_document_get_property_referrer, NULL);
 		addproperty(J, "scripts", mjs_document_get_property_scripts, NULL);
@@ -1721,7 +1886,8 @@ mjs_push_document(js_State *J, void *doc)
 	init_list(doc_private->listeners);
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	doc_private->interpreter = interpreter;
-	interpreter->doc = doc_private->node = doc;
+	doc_private->node = doc;
+	interpreter->doc_private = doc_private;
 	doc_private->ref_count = 1;
 	doc_private->thisval = js_ref(J);
 	if (doc) {
@@ -1735,9 +1901,11 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 void
 mjs_push_document2(js_State *J, void *doc)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
+	NODEINFO(doc);
 	struct mjs_document_private *doc_private = (struct mjs_document_private *)mem_calloc(1, sizeof(*doc_private));
 
 	if (!doc_private) {
@@ -1787,6 +1955,8 @@ mjs_push_document2(js_State *J, void *doc)
 		addproperty(J, "links", mjs_document_get_property_links, NULL);
 		addproperty(J, "location", mjs_document_get_property_location, mjs_document_set_property_location);
 		addproperty(J, "nodeType", mjs_document_get_property_nodeType, NULL);
+		addproperty(J, "onkeydown", mjs_document_get_property_onkeydown, mjs_document_set_property_onkeydown);
+		addproperty(J, "onkeyup", mjs_document_get_property_onkeyup, mjs_document_set_property_onkeyup);
 		addproperty(J, "readyState", mjs_document_get_property_readyState, NULL);
 		addproperty(J, "referrer", mjs_document_get_property_referrer, NULL);
 		addproperty(J, "scripts", mjs_document_get_property_scripts, NULL);
@@ -1812,6 +1982,7 @@ mjs_push_document2(js_State *J, void *doc)
 static void
 mjs_document_fun(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1821,6 +1992,7 @@ mjs_document_fun(js_State *J)
 static void
 mjs_document_constructor(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1836,28 +2008,30 @@ mjs_document_constructor(js_State *J)
 	}
 	add_to_string(&str, "<!doctype html>\n<html><head></head><body></body></html>");
 	void *doc = document_parse_text("utf-8", str.source, str.length);
+	NODEINFO(doc);
+
 	done_string(&str);
 
 	js_newobject(J);
 	{
 		js_newuserdata(J, "document", doc_private, mjs_doc_private_finalizer);
-		addmethod(J, "addEventListener", mjs_document_addEventListener, 3);
-		addmethod(J, "createComment",	mjs_document_createComment, 1);
-		addmethod(J, "createDocumentFragment",mjs_document_createDocumentFragment, 0);
-		addmethod(J, "createElement",	mjs_document_createElement, 1);
-		addmethod(J, "createTextNode",	mjs_document_createTextNode, 1);
-		addmethod(J, "dispatchEvent",	mjs_document_dispatchEvent, 1);
-		addmethod(J, "write",		mjs_document_write, 1);
-		addmethod(J, "writeln",		mjs_document_writeln, 1);
-		addmethod(J, "replace",		mjs_document_replace, 2);
-		addmethod(J, "getElementById",	mjs_document_getElementById, 1);
-		addmethod(J, "getElementsByClassName",	mjs_document_getElementsByClassName, 1);
-//		addmethod(J, "getElementsByName",	mjs_document_getElementsByName, 1);
-		addmethod(J, "getElementsByTagName",	mjs_document_getElementsByTagName, 1);
-		addmethod(J, "querySelector",	mjs_document_querySelector, 1);
-		addmethod(J, "querySelectorAll",	mjs_document_querySelectorAll, 1);
-		addmethod(J, "removeEventListener", mjs_document_removeEventListener, 3);
-		addmethod(J, "toString", mjs_document_toString, 0);
+		addmethod(J, "Document.property.addEventListener", mjs_document_addEventListener, 3);
+		addmethod(J, "Document.property.createComment",	mjs_document_createComment, 1);
+		addmethod(J, "Document.property.createDocumentFragment",mjs_document_createDocumentFragment, 0);
+		addmethod(J, "Document.property.createElement",	mjs_document_createElement, 1);
+		addmethod(J, "Document.property.createTextNode",	mjs_document_createTextNode, 1);
+		addmethod(J, "Document.property.dispatchEvent",	mjs_document_dispatchEvent, 1);
+		addmethod(J, "Document.property.write",		mjs_document_write, 1);
+		addmethod(J, "Document.property.writeln",		mjs_document_writeln, 1);
+		addmethod(J, "Document.property.replace",		mjs_document_replace, 2);
+		addmethod(J, "Document.property.getElementById",	mjs_document_getElementById, 1);
+		addmethod(J, "Document.property.getElementsByClassName",	mjs_document_getElementsByClassName, 1);
+//		addmethod(J, "Document.property.getElementsByName",	mjs_document_getElementsByName, 1);
+		addmethod(J, "Document.property.getElementsByTagName",	mjs_document_getElementsByTagName, 1);
+		addmethod(J, "Document.property.querySelector",	mjs_document_querySelector, 1);
+		addmethod(J, "Document.property.querySelectorAll",	mjs_document_querySelectorAll, 1);
+		addmethod(J, "Document.property.removeEventListener", mjs_document_removeEventListener, 3);
+		addmethod(J, "Document.property.toString", mjs_document_toString, 0);
 
 		addproperty(J, "anchors", mjs_document_get_property_anchors, NULL);
 		addproperty(J, "baseURI", mjs_document_get_property_baseURI, NULL);
@@ -1882,6 +2056,8 @@ mjs_document_constructor(js_State *J)
 		addproperty(J, "links", mjs_document_get_property_links, NULL);
 		addproperty(J, "location", mjs_document_get_property_location, mjs_document_set_property_location);
 		addproperty(J, "nodeType", mjs_document_get_property_nodeType, NULL);
+		addproperty(J, "onkeydown", mjs_document_get_property_onkeydown, mjs_document_set_property_onkeydown);
+		addproperty(J, "onkeyup", mjs_document_get_property_onkeyup, mjs_document_set_property_onkeyup);
 		addproperty(J, "readyState", mjs_document_get_property_readyState, NULL);
 		addproperty(J, "referrer", mjs_document_get_property_referrer, NULL);
 		addproperty(J, "scripts", mjs_document_get_property_scripts, NULL);
@@ -1907,6 +2083,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 int
 mjs_document_init(js_State *J)
 {
+	ELOG
 	js_pushglobal(J);
 	js_newcconstructor(J, mjs_document_fun, mjs_document_constructor, "Document", 0);
 	js_defglobal(J, "Document", JS_DONTENUM);
@@ -1916,10 +2093,12 @@ mjs_document_init(js_State *J)
 static void
 mjs_document_dispatchEvent(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = (dom_document *)mjs_doc_getprivate(J, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		js_pushboolean(J, 0);
@@ -1939,6 +2118,7 @@ mjs_document_dispatchEvent(js_State *J)
 static void
 document_event_handler(dom_event *event, void *pw)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
