@@ -90,6 +90,7 @@ static void mjs_fragment_set_property_textContent(js_State *J);
 void *
 mjs_getprivate_fragment(js_State *J, int idx)
 {
+	ELOG
 	struct mjs_fragment_private *priv = (struct mjs_fragment_private *)js_touserdata(J, idx, "fragment");
 
 	if (!priv) {
@@ -102,10 +103,13 @@ mjs_getprivate_fragment(js_State *J, int idx)
 static void
 mjs_fragment_get_property_children(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -126,10 +130,13 @@ mjs_fragment_get_property_children(js_State *J)
 static void
 mjs_fragment_get_property_childElementCount(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t res = 0;
@@ -152,10 +159,13 @@ mjs_fragment_get_property_childElementCount(js_State *J)
 static void
 mjs_fragment_get_property_childNodes(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -176,10 +186,13 @@ mjs_fragment_get_property_childNodes(js_State *J)
 static void
 mjs_fragment_get_property_firstChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -203,10 +216,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_firstElementChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -262,10 +278,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_lastChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *last_child = NULL;
 	dom_exception exc;
 
@@ -289,10 +308,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_lastElementChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -348,10 +370,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_nextElementSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_next = NULL;
 
@@ -366,12 +391,10 @@ mjs_fragment_get_property_nextElementSibling(js_State *J)
 		dom_exception exc = dom_node_get_next_sibling(node, &next);
 		dom_node_type type;
 
-		if (prev_next) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
 
 		if (exc != DOM_NO_ERR || !next) {
 			js_pushnull(J);
@@ -397,10 +420,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_nodeName(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *node = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(node);
+
 	dom_string *name = NULL;
 	dom_exception exc;
 
@@ -421,10 +447,12 @@ mjs_fragment_get_property_nodeName(js_State *J)
 static void
 mjs_fragment_get_property_nodeType(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *node = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(node);
 	dom_node_type type;
 	dom_exception exc;
 
@@ -444,10 +472,13 @@ mjs_fragment_get_property_nodeType(js_State *J)
 static void
 mjs_fragment_get_property_nodeValue(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *node = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(node);
+
 	dom_string *content = NULL;
 	dom_exception exc;
 
@@ -468,11 +499,13 @@ mjs_fragment_get_property_nodeValue(js_State *J)
 static void
 mjs_fragment_set_property_nodeValue(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *node = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(node);
 
 	if (!node) {
 		js_error(J, "error");
@@ -501,10 +534,13 @@ mjs_fragment_set_property_nodeValue(js_State *J)
 static void
 mjs_fragment_get_property_nextSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -528,6 +564,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_ownerDocument(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -539,10 +576,13 @@ mjs_fragment_get_property_ownerDocument(js_State *J)
 static void
 mjs_fragment_get_property_parentElement(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -566,10 +606,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_parentNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -594,10 +637,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_previousElementSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_prev = NULL;
 
@@ -612,12 +658,10 @@ mjs_fragment_get_property_previousElementSibling(js_State *J)
 		dom_exception exc = dom_node_get_previous_sibling(node, &prev);
 		dom_node_type type;
 
-		if (prev_prev) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_prev);
-		}
+		dom_node_unref(prev_prev);
 
 		if (exc != DOM_NO_ERR || !prev) {
 			js_pushnull(J);
@@ -643,10 +687,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_previousSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -670,10 +717,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_get_property_textContent(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -693,10 +742,12 @@ mjs_fragment_get_property_textContent(js_State *J)
 static void
 mjs_fragment_set_property_textContent(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushundefined(J);
@@ -726,6 +777,7 @@ mjs_fragment_set_property_textContent(js_State *J)
 static xmlpp::Element*
 el_add_child_element_common(xmlNode* child, xmlNode* node)
 {
+	ELOG
 	if (!node) {
 		xmlFreeNode(child);
 		throw xmlpp::internal_error("Could not add child element node");
@@ -739,6 +791,7 @@ el_add_child_element_common(xmlNode* child, xmlNode* node)
 static void
 mjs_fragment_addEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -749,6 +802,7 @@ mjs_fragment_addEventListener(js_State *J)
 		return;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -816,9 +870,7 @@ mjs_fragment_addEventListener(js_State *J)
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(el_private->listener);
 	js_pushundefined(J);
 }
@@ -826,6 +878,7 @@ ex:
 static void
 mjs_fragment_removeEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -836,6 +889,7 @@ mjs_fragment_removeEventListener(js_State *J)
 		return;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -889,11 +943,14 @@ mjs_fragment_removeEventListener(js_State *J)
 static void
 mjs_fragment_appendChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *res = NULL;
 
 	if (!el) {
@@ -923,10 +980,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_cloneNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -952,16 +1011,17 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static bool
 isAncestor(dom_node *el, dom_node *node)
 {
+	ELOG
 	dom_node *prev_next = NULL;
 	while (node) {
 		dom_exception exc;
 		dom_node *next = NULL;
-		if (prev_next) {
+
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
+
 		if (el == node) {
 			return true;
 		}
@@ -981,10 +1041,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_contains(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1041,14 +1103,16 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 	}
 }
 
-
 static void
 mjs_fragment_hasChildNodes(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_exception exc;
 	bool res;
 
@@ -1068,11 +1132,13 @@ mjs_fragment_hasChildNodes(js_State *J)
 static void
 mjs_fragment_insertBefore(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1103,10 +1169,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_isEqualNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1142,10 +1210,12 @@ mjs_fragment_isEqualNode(js_State *J)
 static void
 mjs_fragment_isSameNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1159,10 +1229,12 @@ mjs_fragment_isSameNode(js_State *J)
 static void
 mjs_fragment_querySelector(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -1190,6 +1262,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_querySelectorAll(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1202,6 +1275,7 @@ mjs_fragment_querySelectorAll(js_State *J)
 		return;
 	}
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -1227,11 +1301,14 @@ mjs_fragment_querySelectorAll(js_State *J)
 static void
 mjs_fragment_removeChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
+
 	dom_node *el2 = (dom_node *)(mjs_getprivate_fragment(J, 1));
 	dom_exception exc;
 	dom_node *spare;
@@ -1252,6 +1329,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_fragment_toString(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1261,17 +1339,16 @@ mjs_fragment_toString(js_State *J)
 static void
 mjs_fragment_finalizer(js_State *J, void *priv)
 {
+	ELOG
 	struct mjs_fragment_private *el_private = (struct mjs_fragment_private *)priv;
 
 	if (el_private) {
 		dom_node *node = (dom_node *)(el_private->node);
-
-		if (node && (node->refcnt > 0)) {
+		NODEINFO(node);
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(node);
-		}
+		dom_node_unref(node);
 		mem_free(el_private);
 	}
 }
@@ -1279,6 +1356,8 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 void
 mjs_push_fragment(js_State *J, void *node)
 {
+	ELOG
+	NODEINFO(node);
 	struct mjs_fragment_private *el_private = NULL;
 
 #if 0
@@ -1353,6 +1432,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_DocumentFragment_fun(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1362,11 +1442,18 @@ mjs_DocumentFragment_fun(js_State *J)
 static void
 mjs_DocumentFragment_constructor(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
-	dom_document *doc = (dom_document *)interpreter->doc;
+	struct mjs_document_private *doc_private = (struct mjs_document_private *)interpreter->doc_private;
+
+	if (!doc_private) {
+		js_pushnull(J);
+		return;
+	}
+	dom_document *doc = (dom_document *)doc_private->node;
 
 	if (!doc) {
 		js_pushnull(J);
@@ -1374,6 +1461,7 @@ mjs_DocumentFragment_constructor(js_State *J)
 	}
 	dom_document_fragment *fragment = NULL;
 	dom_exception exc = dom_document_create_document_fragment(doc, &fragment);
+	NODEINFO(fragment);
 
 	if (exc != DOM_NO_ERR || !fragment) {
 		js_pushnull(J);
@@ -1393,20 +1481,20 @@ mjs_DocumentFragment_constructor(js_State *J)
 		js_copy(J, 0);
 		el_private->thisval = js_ref(J);
 		js_newuserdata(J, "fragment", el_private, mjs_fragment_finalizer);
-		addmethod(J, "addEventListener", mjs_fragment_addEventListener, 3);
-		addmethod(J, "appendChild",mjs_fragment_appendChild, 1);
-		addmethod(J, "cloneNode",	mjs_fragment_cloneNode, 1);
-		addmethod(J, "contains",	mjs_fragment_contains, 1);
-		addmethod(J, "dispatchEvent",	mjs_fragment_dispatchEvent, 1);
-		addmethod(J, "hasChildNodes",	mjs_fragment_hasChildNodes, 0);
-		addmethod(J, "insertBefore",	mjs_fragment_insertBefore, 2);
-		addmethod(J, "isEqualNode",	mjs_fragment_isEqualNode, 1);
-		addmethod(J, "isSameNode",		mjs_fragment_isSameNode, 1);
-		addmethod(J, "querySelector",	mjs_fragment_querySelector, 1);
-		addmethod(J, "querySelectorAll",	mjs_fragment_querySelectorAll, 1);
-		addmethod(J, "removeChild",	mjs_fragment_removeChild, 1);
-		addmethod(J, "removeEventListener", mjs_fragment_removeEventListener, 3);
-		addmethod(J, "toString",		mjs_fragment_toString, 0);
+		addmethod(J, "DocumentFragment.prototype.addEventListener", mjs_fragment_addEventListener, 3);
+		addmethod(J, "DocumentFragment.prototype.appendChild",mjs_fragment_appendChild, 1);
+		addmethod(J, "DocumentFragment.prototype.cloneNode",	mjs_fragment_cloneNode, 1);
+		addmethod(J, "DocumentFragment.prototype.contains",	mjs_fragment_contains, 1);
+		addmethod(J, "DocumentFragment.prototype.dispatchEvent",	mjs_fragment_dispatchEvent, 1);
+		addmethod(J, "DocumentFragment.prototype.hasChildNodes",	mjs_fragment_hasChildNodes, 0);
+		addmethod(J, "DocumentFragment.prototype.insertBefore",	mjs_fragment_insertBefore, 2);
+		addmethod(J, "DocumentFragment.prototype.isEqualNode",	mjs_fragment_isEqualNode, 1);
+		addmethod(J, "DocumentFragment.prototype.isSameNode",		mjs_fragment_isSameNode, 1);
+		addmethod(J, "DocumentFragment.prototype.querySelector",	mjs_fragment_querySelector, 1);
+		addmethod(J, "DocumentFragment.prototype.querySelectorAll",	mjs_fragment_querySelectorAll, 1);
+		addmethod(J, "DocumentFragment.prototype.removeChild",	mjs_fragment_removeChild, 1);
+		addmethod(J, "DocumentFragment.prototype.removeEventListener", mjs_fragment_removeEventListener, 3);
+		addmethod(J, "DocumentFragment.prototype.toString",		mjs_fragment_toString, 0);
 
 		addproperty(J, "children",	mjs_fragment_get_property_children, NULL);
 		addproperty(J, "childElementCount",	mjs_fragment_get_property_childElementCount, NULL);
@@ -1432,6 +1520,7 @@ mjs_DocumentFragment_constructor(js_State *J)
 int
 mjs_fragment_init(js_State *J)
 {
+	ELOG
 	js_pushglobal(J);
 	js_newcconstructor(J, mjs_DocumentFragment_fun, mjs_DocumentFragment_constructor, "DocumentFragment", 0);
 	js_defglobal(J, "DocumentFragment", JS_DONTENUM);
@@ -1442,6 +1531,7 @@ mjs_fragment_init(js_State *J)
 void
 check_element_event(void *interp, void *elem, const char *event_name, struct term_event *ev)
 {
+	ELOG
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)interp;
 	js_State *J = (js_State *)interpreter->backend_data;
 	void *second = attr_find_in_map(map_privates, elem);
@@ -1479,10 +1569,12 @@ check_element_event(void *interp, void *elem, const char *event_name, struct ter
 static void
 mjs_fragment_dispatchEvent(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_fragment(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushboolean(J, 0);
@@ -1502,6 +1594,7 @@ mjs_fragment_dispatchEvent(js_State *J)
 static void
 fragment_event_handler(dom_event *event, void *pw)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
