@@ -7,8 +7,10 @@
 extern "C" {
 #endif
 
-int mjs_location_init(js_State *J);
-void mjs_push_location(js_State *J);
+struct view_state;
+
+int mjs_location_init(js_State *J, struct view_state *vs);
+void mjs_push_location(js_State *J, struct view_state *vs);
 
 #ifdef __cplusplus
 }
