@@ -8,7 +8,9 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef CONFIG_LIBCURL
 #include <curl/curl.h>
+#endif
 
 #include "elinks.h"
 
@@ -97,6 +99,7 @@ static void ontimeout_run(void *data);
 char *
 normalize(char *value)
 {
+	ELOG
 	char *ret = value;
 	size_t index = strspn(ret, "\r\n\t ");
 	ret += index;
@@ -118,6 +121,7 @@ normalize(char *value)
 static void
 mjs_xhr_finalizer(js_State *J, void *data)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -162,6 +166,7 @@ mjs_xhr_finalizer(js_State *J, void *data)
 static void
 mjs_xhr_static_get_property_UNSENT(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -171,6 +176,7 @@ mjs_xhr_static_get_property_UNSENT(js_State *J)
 static void
 mjs_xhr_static_get_property_OPENED(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -180,6 +186,7 @@ mjs_xhr_static_get_property_OPENED(js_State *J)
 static void
 mjs_xhr_static_get_property_HEADERS_RECEIVED(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -189,6 +196,7 @@ mjs_xhr_static_get_property_HEADERS_RECEIVED(js_State *J)
 static void
 mjs_xhr_static_get_property_LOADING(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -199,6 +207,7 @@ mjs_xhr_static_get_property_LOADING(js_State *J)
 static void
 mjs_xhr_static_get_property_DONE(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -218,6 +227,7 @@ static void mjs_xhr_setRequestHeader(js_State *J);
 static void
 mjs_xhr_abort(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -232,6 +242,7 @@ mjs_xhr_abort(js_State *J)
 static void
 mjs_xhr_addEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -281,6 +292,7 @@ mjs_xhr_addEventListener(js_State *J)
 static void
 mjs_xhr_removeEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -327,6 +339,7 @@ mjs_xhr_removeEventListener(js_State *J)
 static void
 mjs_xhr_getAllResponseHeaders(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -349,6 +362,7 @@ mjs_xhr_getAllResponseHeaders(js_State *J)
 static void
 mjs_xhr_getResponseHeader(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -375,6 +389,7 @@ mjs_xhr_getResponseHeader(js_State *J)
 static void
 mjs_xhr_open(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -507,6 +522,7 @@ mjs_xhr_open(js_State *J)
 static void
 mjs_xhr_overrideMimeType(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -517,9 +533,10 @@ mjs_xhr_overrideMimeType(js_State *J)
 static void
 onload_run(void *data)
 {
+	ELOG
 	struct mjs_xhr *xhr = (struct mjs_xhr *)data;
 
-	if (xhr) {
+	if (xhr && ecmascript_found(xhr->interpreter)) {
 		struct ecmascript_interpreter *interpreter = xhr->interpreter;
 		js_State *J = (js_State *)interpreter->backend_data;
 
@@ -548,9 +565,10 @@ onload_run(void *data)
 static void
 onloadend_run(void *data)
 {
+	ELOG
 	struct mjs_xhr *xhr = (struct mjs_xhr *)data;
 
-	if (xhr) {
+	if (xhr && ecmascript_found(xhr->interpreter)) {
 		struct ecmascript_interpreter *interpreter = xhr->interpreter;
 		js_State *J = (js_State *)interpreter->backend_data;
 
@@ -579,9 +597,10 @@ onloadend_run(void *data)
 static void
 onreadystatechange_run(void *data)
 {
+	ELOG
 	struct mjs_xhr *xhr = (struct mjs_xhr *)data;
 
-	if (xhr) {
+	if (xhr && ecmascript_found(xhr->interpreter)) {
 		struct ecmascript_interpreter *interpreter = xhr->interpreter;
 		js_State *J = (js_State *)interpreter->backend_data;
 
@@ -610,9 +629,10 @@ onreadystatechange_run(void *data)
 static void
 ontimeout_run(void *data)
 {
+	ELOG
 	struct mjs_xhr *xhr = (struct mjs_xhr *)data;
 
-	if (xhr) {
+	if (xhr && ecmascript_found(xhr->interpreter)) {
 		struct ecmascript_interpreter *interpreter = xhr->interpreter;
 		js_State *J = (js_State *)interpreter->backend_data;
 
@@ -641,6 +661,7 @@ ontimeout_run(void *data)
 static void
 mjs_xhr_loading_callback(struct download *download, struct mjs_xhr *xhr)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -680,6 +701,7 @@ mjs_xhr_loading_callback(struct download *download, struct mjs_xhr *xhr)
 static size_t
 write_data(void *ptr, size_t size, size_t nmemb, void *stream)
 {
+	ELOG
 	struct mjs_xhr *xhr = (struct mjs_xhr *)stream;
 
 	size_t length = xhr->responseLength;
@@ -701,6 +723,7 @@ write_data(void *ptr, size_t size, size_t nmemb, void *stream)
 static void
 mjs_xhr_send(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -766,6 +789,7 @@ mjs_xhr_send(js_State *J)
 		}
 
 		if (!xhr->async) {
+#ifdef CONFIG_LIBCURL
 			char *url = get_uri_string(xhr->uri, URI_DIR_LOCATION | URI_PATH | URI_USER | URI_PASSWORD);
 
 			if (!url) {
@@ -798,6 +822,10 @@ mjs_xhr_send(js_State *J)
 
 			js_pushundefined(J);
 			return;
+#else
+			js_error(J, "unimplemented");
+			return;
+#endif
 		}
 		xhr->download.data = xhr;
 		xhr->download.callback = (download_callback_T *)mjs_xhr_loading_callback;
@@ -812,6 +840,7 @@ mjs_xhr_send(js_State *J)
 static bool
 valid_header(const char *header)
 {
+	ELOG
 	if (!*header) {
 		return false;
 	}
@@ -827,6 +856,7 @@ valid_header(const char *header)
 static bool
 forbidden_header(const char *header)
 {
+	ELOG
 	const char *bad[] = {
 		"Accept-Charset"
 		"Accept-Encoding",
@@ -872,6 +902,7 @@ forbidden_header(const char *header)
 static void
 mjs_xhr_setRequestHeader(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -917,6 +948,7 @@ mjs_xhr_setRequestHeader(js_State *J)
 static void
 mjs_xhr_get_property_onabort(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -932,6 +964,7 @@ mjs_xhr_get_property_onabort(js_State *J)
 static void
 mjs_xhr_set_property_onabort(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -953,6 +986,7 @@ mjs_xhr_set_property_onabort(js_State *J)
 static void
 mjs_xhr_get_property_onerror(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -968,6 +1002,7 @@ mjs_xhr_get_property_onerror(js_State *J)
 static void
 mjs_xhr_set_property_onerror(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -989,6 +1024,7 @@ mjs_xhr_set_property_onerror(js_State *J)
 static void
 mjs_xhr_get_property_onload(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1004,6 +1040,7 @@ mjs_xhr_get_property_onload(js_State *J)
 static void
 mjs_xhr_set_property_onload(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1025,6 +1062,7 @@ mjs_xhr_set_property_onload(js_State *J)
 static void
 mjs_xhr_get_property_onloadend(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1040,6 +1078,7 @@ mjs_xhr_get_property_onloadend(js_State *J)
 static void
 mjs_xhr_set_property_onloadend(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1061,6 +1100,7 @@ mjs_xhr_set_property_onloadend(js_State *J)
 static void
 mjs_xhr_get_property_onloadstart(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1076,6 +1116,7 @@ mjs_xhr_get_property_onloadstart(js_State *J)
 static void
 mjs_xhr_set_property_onloadstart(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1097,6 +1138,7 @@ mjs_xhr_set_property_onloadstart(js_State *J)
 static void
 mjs_xhr_get_property_onprogress(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1112,6 +1154,7 @@ mjs_xhr_get_property_onprogress(js_State *J)
 static void
 mjs_xhr_set_property_onprogress(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1133,6 +1176,7 @@ mjs_xhr_set_property_onprogress(js_State *J)
 static void
 mjs_xhr_get_property_onreadystatechange(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1148,6 +1192,7 @@ mjs_xhr_get_property_onreadystatechange(js_State *J)
 static void
 mjs_xhr_set_property_onreadystatechange(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1169,6 +1214,7 @@ mjs_xhr_set_property_onreadystatechange(js_State *J)
 static void
 mjs_xhr_get_property_ontimeout(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1184,6 +1230,7 @@ mjs_xhr_get_property_ontimeout(js_State *J)
 static void
 mjs_xhr_set_property_ontimeout(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1205,6 +1252,7 @@ mjs_xhr_set_property_ontimeout(js_State *J)
 static void
 mjs_xhr_get_property_readyState(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1220,6 +1268,7 @@ mjs_xhr_get_property_readyState(js_State *J)
 static void
 mjs_xhr_get_property_response(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1235,6 +1284,7 @@ mjs_xhr_get_property_response(js_State *J)
 static void
 mjs_xhr_get_property_responseText(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1255,6 +1305,7 @@ mjs_xhr_get_property_responseText(js_State *J)
 static void
 mjs_xhr_get_property_responseType(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1270,6 +1321,7 @@ mjs_xhr_get_property_responseType(js_State *J)
 static void
 mjs_xhr_get_property_responseURL(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1285,6 +1337,7 @@ mjs_xhr_get_property_responseURL(js_State *J)
 static void
 mjs_xhr_get_property_status(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1300,6 +1353,7 @@ mjs_xhr_get_property_status(js_State *J)
 static void
 mjs_xhr_get_property_statusText(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1315,6 +1369,7 @@ mjs_xhr_get_property_statusText(js_State *J)
 static void
 mjs_xhr_get_property_upload(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1330,6 +1385,7 @@ mjs_xhr_get_property_upload(js_State *J)
 static void
 mjs_xhr_get_property_timeout(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1345,6 +1401,7 @@ mjs_xhr_get_property_timeout(js_State *J)
 static void
 mjs_xhr_get_property_withCredentials(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1360,6 +1417,7 @@ mjs_xhr_get_property_withCredentials(js_State *J)
 static void
 mjs_xhr_set_property_responseType(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1377,6 +1435,7 @@ mjs_xhr_set_property_responseType(js_State *J)
 static void
 mjs_xhr_set_property_timeout(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1394,6 +1453,7 @@ mjs_xhr_set_property_timeout(js_State *J)
 static void
 mjs_xhr_set_property_withCredentials(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1422,6 +1482,7 @@ mjs_xhr_set_property_withCredentials(js_State *J)
 static void
 mjs_xhr_fun(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1431,6 +1492,7 @@ mjs_xhr_fun(js_State *J)
 static void
 mjs_xhr_constructor(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1453,15 +1515,15 @@ mjs_xhr_constructor(js_State *J)
 	{
 		xhr->thisval = js_ref(J);
 		js_newuserdata(J, "xhr", xhr, mjs_xhr_finalizer);
-		addmethod(J, "abort", mjs_xhr_abort, 0);
-		addmethod(J, "addEventListener", mjs_xhr_addEventListener, 3);
-		addmethod(J, "getAllResponseHeaders", mjs_xhr_getAllResponseHeaders, 0);
-		addmethod(J, "getResponseHeader", mjs_xhr_getResponseHeader, 1);
-		addmethod(J, "open", mjs_xhr_open, 5);
-		addmethod(J, "overrideMimeType", mjs_xhr_overrideMimeType, 1);
-		addmethod(J, "removeEventListener", mjs_xhr_removeEventListener, 3);
-		addmethod(J, "send", mjs_xhr_send, 1);
-		addmethod(J, "setRequestHeader", mjs_xhr_setRequestHeader, 2);
+		addmethod(J, "XMLHttpRequest.prototype.abort", mjs_xhr_abort, 0);
+		addmethod(J, "XMLHttpRequest.prototype.addEventListener", mjs_xhr_addEventListener, 3);
+		addmethod(J, "XMLHttpRequest.prototype.getAllResponseHeaders", mjs_xhr_getAllResponseHeaders, 0);
+		addmethod(J, "XMLHttpRequest.prototype.getResponseHeader", mjs_xhr_getResponseHeader, 1);
+		addmethod(J, "XMLHttpRequest.prototype.open", mjs_xhr_open, 5);
+		addmethod(J, "XMLHttpRequest.prototype.overrideMimeType", mjs_xhr_overrideMimeType, 1);
+		addmethod(J, "XMLHttpRequest.prototype.removeEventListener", mjs_xhr_removeEventListener, 3);
+		addmethod(J, "XMLHttpRequest.prototype.send", mjs_xhr_send, 1);
+		addmethod(J, "XMLHttpRequest.prototype.setRequestHeader", mjs_xhr_setRequestHeader, 2);
 
 		addproperty(J, "UNSENT", mjs_xhr_static_get_property_UNSENT, NULL);
 		addproperty(J, "OPENED", mjs_xhr_static_get_property_OPENED, NULL);
@@ -1493,6 +1555,7 @@ mjs_xhr_constructor(js_State *J)
 int
 mjs_xhr_init(js_State *J)
 {
+	ELOG
 	js_pushglobal(J);
 	js_newcconstructor(J, mjs_xhr_fun, mjs_xhr_constructor, "XMLHttpRequest", 0);
 	js_defglobal(J, "XMLHttpRequest", JS_DONTENUM);
