@@ -18,6 +18,7 @@
 #include "cookies/cookies.h"
 #include "dialogs/status.h"
 #include "document/document.h"
+#include "document/libdom/corestrings.h"
 #include "document/libdom/doc.h"
 #include "document/libdom/mapa.h"
 #include "document/view.h"
@@ -28,6 +29,7 @@
 #include "js/quickjs.h"
 #include "js/quickjs/collection.h"
 #include "js/quickjs/event.h"
+#include "js/quickjs/keyboard.h"
 #include "js/quickjs/form.h"
 #include "js/quickjs/forms.h"
 #include "js/quickjs/fragment.h"
@@ -65,6 +67,8 @@ struct js_document_private {
 	struct ecmascript_interpreter *interpreter;
 	JSValue thisval;
 	JSValue images;
+	JSValue onkeydown;
+	JSValue onkeyup;
 	dom_event_listener *listener;
 	void *node;
 	enum readyState state;
@@ -75,6 +79,7 @@ static void document_event_handler(dom_event *event, void *pw);
 void *
 js_doc_getopaque(JSValueConst obj)
 {
+	ELOG
 	REF_JS(obj);
 
 	struct js_document_private *res = (struct js_document_private *)JS_GetOpaque(obj, js_document_class_id);
@@ -89,12 +94,14 @@ js_doc_getopaque(JSValueConst obj)
 void *
 document_get_node(JSValueConst obj)
 {
+	ELOG
 	return js_doc_getopaque(obj);
 }
 
 static void
 js_document_mark(JSRuntime *rt, JSValueConst val, JS_MarkFunc *mark_func)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -108,18 +115,22 @@ js_document_mark(JSRuntime *rt, JSValueConst val, JS_MarkFunc *mark_func)
 		foreach(l, doc_private->listeners) {
 			JS_MarkValue(rt, l->fun, mark_func);
 		}
+		JS_MarkValue(rt, doc_private->onkeydown, mark_func);
+		JS_MarkValue(rt, doc_private->onkeyup, mark_func);
 	}
 }
 
 static JSValue
 js_document_get_property_anchors(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -141,6 +152,7 @@ js_document_get_property_anchors(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_baseURI(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -175,12 +187,14 @@ js_document_get_property_baseURI(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_body(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -195,7 +209,7 @@ js_document_get_property_body(JSContext *ctx, JSValueConst this_val)
 		return JS_NULL;
 	}
 	//dom_node_unref(doc);
-	JSValue rr = getNode(ctx, body);
+	JSValue rr = getElement(ctx, body);
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
@@ -207,6 +221,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_document_set_property_body(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -220,6 +235,7 @@ js_document_set_property_body(JSContext *ctx, JSValueConst this_val, JSValue val
 static JSValue
 js_document_get_property_cookie(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -257,6 +273,7 @@ js_document_get_property_cookie(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_set_property_cookie(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -296,6 +313,7 @@ js_document_set_property_cookie(JSContext *ctx, JSValueConst this_val, JSValue v
 static JSValue
 js_document_get_property_charset(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -310,12 +328,14 @@ js_document_get_property_charset(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_childNodes(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -348,6 +368,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_document_get_property_currentScript(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -371,16 +392,16 @@ js_document_get_property_currentScript(JSContext *ctx, JSValueConst this_val)
 
 	if (mapa) {
 		dom_node *elem = (dom_node *)find_in_map(mapa, interpreter->element_offset);
+		NODEINFO(elem);
 
 		if (elem) {
-			dom_string *tag_name = NULL;
-			dom_exception exc = dom_node_get_node_name(elem, &tag_name);
+			dom_html_element_type ty;
+			dom_exception exc = dom_html_element_get_tag_type(elem, &ty);
 
-			if (exc != DOM_NO_ERR || !tag_name) {
+			if (exc != DOM_NO_ERR) {
 				return JS_NULL;
 			}
-			bool isScript = !strcmp("SCRIPT", dom_string_data(tag_name));
-			dom_string_unref(tag_name);
+			bool isScript = (ty == DOM_HTML_ELEMENT_TYPE_SCRIPT);
 
 			if (isScript) {
 				return getNode(ctx, elem);
@@ -395,6 +416,7 @@ js_document_get_property_currentScript(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_defaultView(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -407,12 +429,14 @@ js_document_get_property_defaultView(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_doctype(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -429,12 +453,14 @@ js_document_get_property_doctype(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_documentElement(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -461,6 +487,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_document_get_property_documentURI(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -492,6 +519,7 @@ js_document_get_property_documentURI(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_domain(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -526,12 +554,14 @@ js_document_get_property_domain(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_forms(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -555,20 +585,40 @@ js_document_get_property_forms(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_head(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
-// TODO
-	return JS_NULL;
+	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
-//	return getNode(ctx, element);
+	if (!doc) {
+		return JS_NULL;
+	}
+	dom_nodelist *nodes = NULL;
+	dom_exception exc = dom_document_get_elements_by_tag_name(doc, corestring_dom_HEAD, &nodes);
+
+	if (exc != DOM_NO_ERR || !nodes) {
+		return JS_NULL;
+	}
+	dom_node *head = NULL;
+	exc = dom_nodelist_item(nodes, 0, &head);
+	dom_nodelist_unref(nodes);
+
+	if (exc != DOM_NO_ERR || !head) {
+		return JS_NULL;
+	}
+	JSValue rr = getElement(ctx, head);
+
+	RETURN_JS(rr);
 }
 
 static JSValue
 js_document_get_property_images(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -584,6 +634,7 @@ js_document_get_property_images(JSContext *ctx, JSValueConst this_val)
 		return JS_DupValue(ctx, res->images);
 	}
 	dom_html_document *doc = (struct dom_html_document *)res->node;
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -605,6 +656,7 @@ js_document_get_property_images(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_implementation(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -616,12 +668,14 @@ js_document_get_property_implementation(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_links(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -643,6 +697,7 @@ js_document_get_property_links(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_location(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -659,6 +714,7 @@ js_document_get_property_location(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_nodeType(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -667,9 +723,143 @@ js_document_get_property_nodeType(JSContext *ctx, JSValueConst this_val)
 	return JS_NewInt32(ctx, 9);
 }
 
+static JSValue
+js_document_get_property_onkeydown(JSContext *ctx, JSValueConst this_val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct js_document_private *doc_private = (struct js_document_private *)(JS_GetOpaque(this_val, js_document_class_id));
+
+	if (!doc_private) {
+		return JS_NULL;
+	}
+	JSValue ret = JS_DupValue(ctx, doc_private->onkeydown);
+
+	RETURN_JS(ret);
+}
+
+static JSValue
+js_document_set_property_onkeydown(JSContext *ctx, JSValueConst this_val, JSValue val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct js_document_private *doc_private = (struct js_document_private *)(JS_GetOpaque(this_val, js_document_class_id));
+
+	if (!doc_private) {
+		return JS_UNDEFINED;
+	}
+	JS_FreeValue(ctx, doc_private->onkeydown);
+	doc_private->onkeydown = JS_DupValue(ctx, val);
+
+	return JS_UNDEFINED;
+}
+
+
+static JSValue
+js_document_get_property_onkeyup(JSContext *ctx, JSValueConst this_val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct js_document_private *doc_private = (struct js_document_private *)(JS_GetOpaque(this_val, js_document_class_id));
+
+	if (!doc_private) {
+		return JS_NULL;
+	}
+	JSValue ret = JS_DupValue(ctx, doc_private->onkeyup);
+
+	RETURN_JS(ret);
+}
+
+int
+quickjs_document_fire_onkeydown(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JSContext *ctx = (JSContext *)interpreter->backend_data;
+	JSValue obj = interpreter->document_obj;
+
+	REF_JS(obj);
+	struct js_document_private *doc_private = (struct js_document_private *)(JS_GetOpaque(obj, js_document_class_id));
+
+	if (!doc_private) {
+		return 0;
+	}
+	if (JS_IsNull(doc_private->onkeydown)) {
+		return 0;
+	}
+	JSValue func = JS_DupValue(ctx, doc_private->onkeydown);
+	JSValue arg = get_keyboardEvent(ctx, ev);
+	JSValue ret = JS_Call(ctx, func, doc_private->thisval, 1, (JSValueConst *) &arg);
+	JS_FreeValue(ctx, ret);
+	JS_FreeValue(ctx, func);
+	JS_FreeValue(ctx, arg);
+	check_for_rerender(interpreter, "onkeydown");
+	return 1;
+}
+
+int
+quickjs_document_fire_onkeyup(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JSContext *ctx = (JSContext *)interpreter->backend_data;
+	JSValue obj = interpreter->document_obj;
+
+	REF_JS(obj);
+	struct js_document_private *doc_private = (struct js_document_private *)(JS_GetOpaque(obj, js_document_class_id));
+
+	if (!doc_private) {
+		return 0;
+	}
+	if (JS_IsNull(doc_private->onkeyup)) {
+		return 0;
+	}
+	JSValue func = JS_DupValue(ctx, doc_private->onkeyup);
+	JSValue arg = get_keyboardEvent(ctx, ev);
+	JSValue ret = JS_Call(ctx, func, doc_private->thisval, 1, (JSValueConst *) &arg);
+	JS_FreeValue(ctx, ret);
+	JS_FreeValue(ctx, func);
+	JS_FreeValue(ctx, arg);
+	check_for_rerender(interpreter, "onkeyup");
+	return 1;
+}
+
+static JSValue
+js_document_set_property_onkeyup(JSContext *ctx, JSValueConst this_val, JSValue val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct js_document_private *doc_private = (struct js_document_private *)(JS_GetOpaque(this_val, js_document_class_id));
+
+	if (!doc_private) {
+		return JS_UNDEFINED;
+	}
+	JS_FreeValue(ctx, doc_private->onkeyup);
+	doc_private->onkeyup = JS_DupValue(ctx, val);
+
+	return JS_UNDEFINED;
+}
+
 static JSValue
 js_document_set_property_location(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -706,6 +896,7 @@ js_document_set_property_location(JSContext *ctx, JSValueConst this_val, JSValue
 static JSValue
 js_document_get_property_readyState(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -735,6 +926,7 @@ js_document_get_property_readyState(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_get_property_referrer(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -761,7 +953,7 @@ js_document_get_property_referrer(JSContext *ctx, JSValueConst this_val)
 	switch (get_opt_int("protocol.http.referer.policy", NULL)) {
 	case REFERER_NONE:
 		/* oh well */
-		return JS_UNDEFINED;
+		return JS_NewString(ctx, "");
 
 	case REFERER_FAKE:
 		return JS_NewString(ctx, get_opt_str("protocol.http.referer.fake", NULL));
@@ -777,8 +969,10 @@ js_document_get_property_referrer(JSContext *ctx, JSValueConst this_val)
 
 				RETURN_JS(ret);
 			} else {
-				return JS_UNDEFINED;
+				return JS_NewString(ctx, "");
 			}
+		} else {
+			return JS_NewString(ctx, "");
 		}
 		break;
 
@@ -791,36 +985,41 @@ js_document_get_property_referrer(JSContext *ctx, JSValueConst this_val)
 
 			RETURN_JS(ret);
 		} else {
-			return JS_UNDEFINED;
+			return JS_NewString(ctx, "");
 		}
 		break;
 	}
 
-	return JS_UNDEFINED;
+	return JS_NewString(ctx, "");
 }
 
 static JSValue
 js_document_get_property_scripts(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
+	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
-// TODO
-	//dom_html_document *doc = (dom_html_document *)document->dom;
+	if (!doc) {
+		return JS_NULL;
+	}
+	dom_html_collection *col = (dom_html_collection *)get_scripts(doc, (dom_node *)doc);
 
-	return JS_NULL;
-
-//	JSValue rr = getCollection(ctx, elements);
-//	JS_FreeValue(ctx, rr);
-
-//	RETURN_JS(rr);
+	if (!col) {
+		return JS_NULL;
+	}
+	JSValue ret = getCollection2(ctx, col);
+	RETURN_JS(ret);
 }
 
 static JSValue
 js_document_get_property_title(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -848,6 +1047,7 @@ js_document_get_property_title(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_set_property_title(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -891,6 +1091,7 @@ js_document_set_property_title(JSContext *ctx, JSValueConst this_val, JSValue va
 static JSValue
 js_document_get_property_url(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -925,6 +1126,7 @@ js_document_get_property_url(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_set_property_url(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -961,6 +1163,7 @@ js_document_set_property_url(JSContext *ctx, JSValueConst this_val, JSValue val)
 static JSValue
 js_document_write_do(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv, int newline)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1014,6 +1217,7 @@ js_document_write_do(JSContext *ctx, JSValueConst this_val, int argc, JSValueCon
 static JSValue
 js_document_write(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1026,6 +1230,7 @@ js_document_write(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst
 static JSValue
 js_document_writeln(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1038,6 +1243,7 @@ js_document_writeln(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 static JSValue
 js_document_replace(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1115,6 +1321,7 @@ js_document_replace(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 static JSValue
 js_document_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1126,6 +1333,7 @@ js_document_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JS
 		return JS_NULL;
 	}
 	dom_html_document *doc = doc_private->node;
+	NODEINFO(doc);
 
 	if (argc < 2) {
 		return JS_UNDEFINED;
@@ -1188,9 +1396,7 @@ js_document_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JS
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(doc_private->listener);
 
 	return JS_UNDEFINED;
@@ -1199,6 +1405,7 @@ ex:
 static JSValue
 js_document_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1210,6 +1417,7 @@ js_document_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 		return JS_FALSE;
 	}
 	dom_html_document *doc = doc_private->node;
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_FALSE;
@@ -1240,6 +1448,7 @@ js_document_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 static JSValue
 js_document_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1307,6 +1516,7 @@ js_document_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc,
 static JSValue
 js_document_createComment(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1316,6 +1526,7 @@ js_document_createComment(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1356,6 +1567,7 @@ js_document_createComment(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 static JSValue
 js_document_createDocumentFragment(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1365,6 +1577,7 @@ js_document_createDocumentFragment(JSContext *ctx, JSValueConst this_val, int ar
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1386,6 +1599,7 @@ js_document_createDocumentFragment(JSContext *ctx, JSValueConst this_val, int ar
 static JSValue
 js_document_createElement(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1395,6 +1609,7 @@ js_document_createElement(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1440,6 +1655,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_document_createTextNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1449,6 +1665,7 @@ js_document_createTextNode(JSContext *ctx, JSValueConst this_val, int argc, JSVa
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1489,6 +1706,7 @@ js_document_createTextNode(JSContext *ctx, JSValueConst this_val, int argc, JSVa
 static JSValue
 js_document_getElementById(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1498,6 +1716,7 @@ js_document_getElementById(JSContext *ctx, JSValueConst this_val, int argc, JSVa
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1543,11 +1762,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_document_getElementsByClassName(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1574,6 +1795,7 @@ js_document_getElementsByClassName(JSContext *ctx, JSValueConst this_val, int ar
 static JSValue
 js_document_getElementsByName(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1622,6 +1844,7 @@ js_document_getElementsByName(JSContext *ctx, JSValueConst this_val, int argc, J
 static JSValue
 js_document_getElementsByTagName(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1631,6 +1854,7 @@ js_document_getElementsByTagName(JSContext *ctx, JSValueConst this_val, int argc
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1672,6 +1896,7 @@ js_document_getElementsByTagName(JSContext *ctx, JSValueConst this_val, int argc
 static JSValue
 js_document_querySelector(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1681,6 +1906,7 @@ js_document_querySelector(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1725,6 +1951,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_document_querySelectorAll(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1734,6 +1961,7 @@ js_document_querySelectorAll(JSContext *ctx, JSValueConst this_val, int argc, JS
 		return JS_FALSE;
 	}
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(this_val);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1797,12 +2025,14 @@ JSClass doctype_class = {
 static JSValue
 js_doctype_get_property_name(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_document_type *dtd = (dom_document_type *)(JS_GetOpaque(this_val, js_doctype_class_id));
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		return JS_NULL;
@@ -1822,12 +2052,14 @@ js_doctype_get_property_name(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_doctype_get_property_publicId(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_document_type *dtd = (dom_document_type *)(JS_GetOpaque(this_val, js_doctype_class_id));
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		return JS_NULL;
@@ -1847,12 +2079,14 @@ js_doctype_get_property_publicId(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_doctype_get_property_systemId(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_document_type *dtd = (dom_document_type *)(JS_GetOpaque(this_val, js_doctype_class_id));
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		return JS_NULL;
@@ -1872,6 +2106,7 @@ js_doctype_get_property_systemId(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_document_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1904,6 +2139,8 @@ static const JSCFunctionListEntry js_document_proto_funcs[] = {
 	JS_CGETSET_DEF("links", js_document_get_property_links, NULL),
 	JS_CGETSET_DEF("location",	js_document_get_property_location, js_document_set_property_location),
 	JS_CGETSET_DEF("nodeType", js_document_get_property_nodeType, NULL),
+	JS_CGETSET_DEF("onkeydown",	js_document_get_property_onkeydown, js_document_set_property_onkeydown),
+	JS_CGETSET_DEF("onkeyup",	js_document_get_property_onkeyup, js_document_set_property_onkeyup),
 	JS_CGETSET_DEF("readyState", js_document_get_property_readyState, NULL),
 	JS_CGETSET_DEF("referrer", js_document_get_property_referrer, NULL),
 	JS_CGETSET_DEF("scripts", js_document_get_property_scripts, NULL),
@@ -1933,6 +2170,7 @@ static const JSCFunctionListEntry js_document_proto_funcs[] = {
 static void
 js_document_finalizer(JSRuntime *rt, JSValue val)
 {
+	ELOG
 	REF_JS(val);
 
 	struct js_document_private *doc_private = (struct js_document_private *)JS_GetOpaque(val, js_document_class_id);
@@ -1948,14 +2186,14 @@ js_document_finalizer(JSRuntime *rt, JSValue val)
 			mem_free_set(&l->typ, NULL);
 			JS_FreeValueRT(rt, l->fun);
 		}
+		JS_FreeValueRT(rt, doc_private->onkeydown);
+		JS_FreeValueRT(rt, doc_private->onkeyup);
 		free_list(doc_private->listeners);
-
-		if (doc_private->node) {
+		NODEINFO(doc_private->node);
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref((dom_node *)doc_private->node);
-		}
+		dom_node_unref((dom_node *)doc_private->node);
 		mem_free(doc_private);
 	}
 }
@@ -1969,6 +2207,7 @@ static JSClassDef js_document_class = {
 static JSValue
 js_doctype_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1989,16 +2228,15 @@ void *map_doctypes;
 static void
 js_doctype_finalizer(JSRuntime *rt, JSValue val)
 {
+	ELOG
 	REF_JS(val);
 
 	dom_node *node = (dom_node *)JS_GetOpaque(val, js_doctype_class_id);
 
-	if (node) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-		dom_node_unref(node);
-	}
+	dom_node_unref(node);
 }
 
 static JSClassDef js_doctype_class = {
@@ -2009,6 +2247,7 @@ static JSClassDef js_doctype_class = {
 int
 js_doctype_init(JSContext *ctx)
 {
+	ELOG
 	JSValue doctype_proto;
 
 	/* create the doctype class */
@@ -2033,9 +2272,11 @@ js_doctype_init(JSContext *ctx)
 JSValue
 getDoctype(JSContext *ctx, void *node)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
+	NODEINFO(node);
 	static int initialized;
 	/* create the element class */
 	if (!initialized) {
@@ -2055,6 +2296,7 @@ getDoctype(JSContext *ctx, void *node)
 JSValue
 getDocument(JSContext *ctx, void *doc)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -2064,6 +2306,7 @@ getDocument(JSContext *ctx, void *doc)
 		return JS_NULL;
 	}
 	init_list(doc_private->listeners);
+	NODEINFO(doc);
 
 	if (doc) {
 #ifdef ECMASCRIPT_DEBUG
@@ -2096,6 +2339,8 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 	JS_FreeValue(ctx, global_obj);
 	doc_private->thisval = document_obj;
 	doc_private->images = JS_NULL;
+	doc_private->onkeydown = JS_NULL;
+	doc_private->onkeyup = JS_NULL;
 
 	RETURN_JS(document_obj);
 }
@@ -2103,9 +2348,11 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 JSValue
 getDocument2(JSContext *ctx, void *doc)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
+	NODEINFO(doc);
 	struct js_document_private *doc_private = (struct js_document_private *)mem_calloc(1, sizeof(*doc_private));
 
 	if (!doc_private) {
@@ -2113,9 +2360,9 @@ getDocument2(JSContext *ctx, void *doc)
 	}
 	init_list(doc_private->listeners);
 
-//	if (doc) {
-//		dom_node_ref((dom_node *)doc);
-//	}
+	if (doc) {
+		dom_node_ref((dom_node *)doc);
+	}
 	doc_private->node = doc;
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	doc_private->interpreter = interpreter;
@@ -2125,6 +2372,8 @@ getDocument2(JSContext *ctx, void *doc)
 	JS_SetOpaque(document_obj, doc_private);
 	doc_private->thisval = document_obj;
 	doc_private->images = JS_NULL;
+	doc_private->onkeydown = JS_NULL;
+	doc_private->onkeyup = JS_NULL;
 
 	RETURN_JS(document_obj);
 }
@@ -2132,6 +2381,7 @@ getDocument2(JSContext *ctx, void *doc)
 static JSValue
 js_document_constructor(JSContext *ctx, JSValueConst new_target, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -2156,6 +2406,7 @@ js_document_constructor(JSContext *ctx, JSValueConst new_target, int argc, JSVal
 	}
 	add_to_string(&str, "<!doctype html>\n<html><head></head><body></body></html>");
 	void *doc = document_parse_text("utf-8", str.source, str.length);
+	NODEINFO(doc);
 	done_string(&str);
 
 	if (doc) {
@@ -2168,6 +2419,8 @@ js_document_constructor(JSContext *ctx, JSValueConst new_target, int argc, JSVal
 	JS_SetPropertyFunctionList(ctx, document_obj, js_document_proto_funcs, countof(js_document_proto_funcs));
 	JS_SetOpaque(document_obj, doc_private);
 	doc_private->thisval = document_obj;
+	doc_private->onkeydown = JS_NULL;
+	doc_private->onkeyup = JS_NULL;
 
 	RETURN_JS(document_obj);
 }
@@ -2175,6 +2428,7 @@ js_document_constructor(JSContext *ctx, JSValueConst new_target, int argc, JSVal
 static void
 JS_NewGlobalCConstructor2(JSContext *ctx, JSValue func_obj, const char *name, JSValueConst proto)
 {
+	ELOG
 	REF_JS(func_obj);
 	REF_JS(proto);
 
@@ -2190,6 +2444,7 @@ JS_NewGlobalCConstructor2(JSContext *ctx, JSValue func_obj, const char *name, JS
 static JSValueConst
 JS_NewGlobalCConstructor(JSContext *ctx, const char *name, JSCFunction *func, int length, JSValueConst proto)
 {
+	ELOG
 	JSValue func_obj;
 	func_obj = JS_NewCFunction2(ctx, func, name, length, JS_CFUNC_constructor_or_func, 0);
 	REF_JS(func_obj);
@@ -2203,6 +2458,7 @@ JS_NewGlobalCConstructor(JSContext *ctx, const char *name, JSCFunction *func, in
 int
 js_document_init(JSContext *ctx)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -2224,6 +2480,7 @@ js_document_init(JSContext *ctx)
 static void
 document_event_handler(dom_event *event, void *pw)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
