@@ -74,6 +74,7 @@ static JSValue js_fragment_set_property_textContent(JSContext *ctx, JSValueConst
 void *
 js_getopaque_fragment(JSValueConst obj, JSClassID class_id)
 {
+	ELOG
 	REF_JS(obj);
 	struct js_fragment_private *res = (struct js_fragment_private *)JS_GetOpaque(obj, class_id);
 
@@ -94,18 +95,22 @@ js_getopaque_fragment(JSValueConst obj, JSClassID class_id)
 void *
 fragment_get_node(JSValueConst obj)
 {
+	ELOG
 	return js_getopaque_fragment(obj, js_fragment_class_id);
 }
 
 static JSValue
 js_fragment_get_property_children(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -128,12 +133,15 @@ js_fragment_get_property_children(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_fragment_get_property_childElementCount(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t res = 0;
@@ -158,12 +166,15 @@ js_fragment_get_property_childElementCount(JSContext *ctx, JSValueConst this_val
 static JSValue
 js_fragment_get_property_childNodes(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -183,11 +194,10 @@ js_fragment_get_property_childNodes(JSContext *ctx, JSValueConst this_val)
 	RETURN_JS(rr);
 }
 
-
-
 static JSValue
 js_fragment_get_property_firstChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -196,6 +206,7 @@ js_fragment_get_property_firstChild(JSContext *ctx, JSValueConst this_val)
 	dom_exception exc;
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -219,12 +230,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_firstElementChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -283,12 +297,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_lastChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_node *last_child = NULL;
 	dom_exception exc;
 
@@ -314,12 +331,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_lastElementChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -377,12 +397,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_nextElementSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_next = NULL;
 
@@ -397,12 +420,10 @@ js_fragment_get_property_nextElementSibling(JSContext *ctx, JSValueConst this_va
 		dom_exception exc = dom_node_get_next_sibling(node, &next);
 		dom_node_type type;
 
-		if (prev_next) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
 
 		if (exc != DOM_NO_ERR || !next) {
 			return JS_NULL;
@@ -428,12 +449,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_nodeName(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *node = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(node);
+
 	dom_string *name = NULL;
 	dom_exception exc;
 	JSValue r;
@@ -461,12 +485,15 @@ js_fragment_get_property_nodeName(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_fragment_get_property_nodeType(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *node = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(node);
+
 	dom_node_type type;
 	dom_exception exc;
 
@@ -488,12 +515,15 @@ js_fragment_get_property_nodeType(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_fragment_get_property_nodeValue(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *node = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(node);
+
 	dom_string *content = NULL;
 	dom_exception exc;
 	JSValue r;
@@ -518,6 +548,7 @@ js_fragment_get_property_nodeValue(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_fragment_set_property_nodeValue(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -525,6 +556,7 @@ js_fragment_set_property_nodeValue(JSContext *ctx, JSValueConst this_val, JSValu
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *node = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(node);
 
 	if (!node) {
 		return JS_UNDEFINED;
@@ -555,12 +587,15 @@ js_fragment_set_property_nodeValue(JSContext *ctx, JSValueConst this_val, JSValu
 static JSValue
 js_fragment_get_property_nextSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -589,6 +624,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_ownerDocument(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -603,12 +639,15 @@ js_fragment_get_property_ownerDocument(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_fragment_get_property_parentElement(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)js_getopaque_fragment(this_val, js_fragment_class_id);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -634,11 +673,14 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_parentNode(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 	dom_node *el = (dom_node *)js_getopaque_fragment(this_val, js_fragment_class_id);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -665,12 +707,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_previousElementSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_prev = NULL;
 
@@ -685,12 +730,10 @@ js_fragment_get_property_previousElementSibling(JSContext *ctx, JSValueConst thi
 		dom_exception exc = dom_node_get_previous_sibling(node, &prev);
 		dom_node_type type;
 
-		if (prev_prev) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_prev);
-		}
+		dom_node_unref(prev_prev);
 
 		if (exc != DOM_NO_ERR || !prev) {
 			return JS_NULL;
@@ -717,11 +760,14 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_previousSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -749,12 +795,14 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_get_property_textContent(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -778,6 +826,7 @@ js_fragment_get_property_textContent(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_fragment_set_property_textContent(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -785,6 +834,7 @@ js_fragment_set_property_textContent(JSContext *ctx, JSValueConst this_val, JSVa
 	REF_JS(val);
 
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_UNDEFINED;
@@ -818,6 +868,7 @@ js_fragment_set_property_textContent(JSContext *ctx, JSValueConst this_val, JSVa
 static xmlpp::Element*
 el_add_child_element_common(xmlNode* child, xmlNode* node)
 {
+	ELOG
 	if (!node) {
 		xmlFreeNode(child);
 		throw xmlpp::internal_error("Could not add child element node");
@@ -831,6 +882,7 @@ el_add_child_element_common(xmlNode* child, xmlNode* node)
 static JSValue
 js_fragment_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -842,6 +894,7 @@ js_fragment_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 		return JS_FALSE;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_FALSE;
@@ -873,6 +926,7 @@ js_fragment_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 static JSValue
 js_fragment_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -884,6 +938,7 @@ js_fragment_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JS
 		return JS_NULL;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -956,9 +1011,7 @@ js_fragment_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JS
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(el_private->listener);
 	//dom_node_unref(el);
 
@@ -968,6 +1021,7 @@ ex:
 static JSValue
 js_fragment_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -979,6 +1033,7 @@ js_fragment_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc,
 		return JS_NULL;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -1042,6 +1097,7 @@ js_fragment_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc,
 static JSValue
 js_fragment_appendChild(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1049,6 +1105,8 @@ js_fragment_appendChild(JSContext *ctx, JSValueConst this_val, int argc, JSValue
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_node *res = NULL;
 	dom_exception exc;
 
@@ -1091,6 +1149,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_cloneNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1103,6 +1162,7 @@ js_fragment_cloneNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueCo
 		return JS_NULL;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -1132,16 +1192,17 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static bool
 isAncestor(dom_node *el, dom_node *node)
 {
+	ELOG
 	dom_node *prev_next = NULL;
 	while (node) {
 		dom_exception exc;
 		dom_node *next = NULL;
-		if (prev_next) {
+
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
+
 		if (el == node) {
 			return true;
 		}
@@ -1160,6 +1221,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_contains(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1172,6 +1234,7 @@ js_fragment_contains(JSContext *ctx, JSValueConst this_val, int argc, JSValueCon
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1223,6 +1286,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_hasChildNodes(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1235,6 +1299,8 @@ js_fragment_hasChildNodes(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
+
 	dom_exception exc;
 	bool res;
 
@@ -1256,6 +1322,7 @@ js_fragment_hasChildNodes(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 static JSValue
 js_fragment_insertBefore(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1269,6 +1336,7 @@ js_fragment_insertBefore(JSContext *ctx, JSValueConst this_val, int argc, JSValu
 	}
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1310,6 +1378,7 @@ js_fragment_insertBefore(JSContext *ctx, JSValueConst this_val, int argc, JSValu
 static JSValue
 js_fragment_isEqualNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1322,6 +1391,7 @@ js_fragment_isEqualNode(JSContext *ctx, JSValueConst this_val, int argc, JSValue
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1365,6 +1435,7 @@ js_fragment_isEqualNode(JSContext *ctx, JSValueConst this_val, int argc, JSValue
 static JSValue
 js_fragment_isSameNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1377,6 +1448,7 @@ js_fragment_isSameNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueC
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1396,6 +1468,7 @@ js_fragment_isSameNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueC
 static JSValue
 js_fragment_querySelector(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1408,6 +1481,7 @@ js_fragment_querySelector(JSContext *ctx, JSValueConst this_val, int argc, JSVal
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -1440,6 +1514,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_fragment_querySelectorAll(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1456,6 +1531,7 @@ js_fragment_querySelectorAll(JSContext *ctx, JSValueConst this_val, int argc, JS
 		return JS_NULL;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -1491,6 +1567,7 @@ js_fragment_querySelectorAll(JSContext *ctx, JSValueConst this_val, int argc, JS
 static JSValue
 js_fragment_removeChild(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1504,6 +1581,7 @@ js_fragment_removeChild(JSContext *ctx, JSValueConst this_val, int argc, JSValue
 	}
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *el = (dom_node *)(js_getopaque_fragment(this_val, js_fragment_class_id));
+	NODEINFO(el);
 
 	if (!el || !JS_IsObject(argv[0])) {
 		return JS_NULL;
@@ -1534,6 +1612,7 @@ js_fragment_removeChild(JSContext *ctx, JSValueConst this_val, int argc, JSValue
 static JSValue
 js_fragment_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1581,6 +1660,7 @@ static const JSCFunctionListEntry js_fragment_proto_funcs[] = {
 static
 void js_fragment_finalizer(JSRuntime *rt, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1591,23 +1671,20 @@ void js_fragment_finalizer(JSRuntime *rt, JSValue val)
 	if (el_private) {
 		struct fragment_listener *l;
 		dom_node *el = (dom_node *)el_private->node;
+		NODEINFO(el);
 
 		if (el_private->listener) {
 			dom_event_listener_unref(el_private->listener);
 		}
 
-		if (el) {
 //			void *old_node_data = NULL;
 //			dom_node_set_user_data(el, corestring_dom___ns_key_html_content_data, NULL, js_html_document_user_data_handler,
 //				(void *) &old_node_data);
 
-			if (el->refcnt > 0) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-				dom_node_unref(el);
-			}
-		}
+		dom_node_unref(el);
 
 		foreach(l, el_private->listeners) {
 			mem_free_set(&l->typ, NULL);
@@ -1623,6 +1700,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 js_fragment_mark(JSRuntime *rt, JSValueConst val, JS_MarkFunc *mark_func)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1650,6 +1728,7 @@ static JSClassDef js_fragment_class = {
 static JSValue
 js_DocumentFragment_constructor(JSContext *ctx, JSValueConst new_target, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1663,6 +1742,7 @@ js_DocumentFragment_constructor(JSContext *ctx, JSValueConst new_target, int arg
 	}
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_html_document *doc = (struct dom_html_document *)js_doc_getopaque(interpreter->document_obj);
+	NODEINFO(doc);
 
 	if (!doc) {
 		return JS_NULL;
@@ -1696,6 +1776,7 @@ js_DocumentFragment_constructor(JSContext *ctx, JSValueConst new_target, int arg
 static void
 JS_NewGlobalCConstructor2(JSContext *ctx, JSValue func_obj, const char *name, JSValueConst proto)
 {
+	ELOG
 	REF_JS(func_obj);
 	REF_JS(proto);
 
@@ -1711,6 +1792,7 @@ JS_NewGlobalCConstructor2(JSContext *ctx, JSValue func_obj, const char *name, JS
 static JSValueConst
 JS_NewGlobalCConstructor(JSContext *ctx, const char *name, JSCFunction *func, int length, JSValueConst proto)
 {
+	ELOG
 	JSValue func_obj;
 	func_obj = JS_NewCFunction2(ctx, func, name, length, JS_CFUNC_constructor_or_func, 0);
 	REF_JS(func_obj);
@@ -1724,6 +1806,7 @@ JS_NewGlobalCConstructor(JSContext *ctx, const char *name, JSCFunction *func, in
 int
 js_fragment_init(JSContext *ctx)
 {
+	ELOG
 	JSValue fragment_proto;
 
 	/* create the fragment class */
@@ -1752,9 +1835,11 @@ js_fragment_init(JSContext *ctx)
 JSValue
 getDocumentFragment(JSContext *ctx, void *node)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
+	NODEINFO(node);
 	struct js_fragment_private *el_private = (struct js_fragment_private *)mem_calloc(1, sizeof(*el_private));
 
 	if (!el_private) {
@@ -1781,6 +1866,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 fragment_event_handler(dom_event *event, void *pw)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
