@@ -28,13 +28,18 @@ static JSClassID js_location_class_id;
 static JSValue
 js_location_get_property_hash(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -48,7 +53,8 @@ js_location_get_property_hash(JSContext *ctx, JSValueConst this_val)
 		return JS_EXCEPTION;
 	}
 
-	if (vs->uri->fragmentlen) {
+	if (vs->uri && vs->uri->fragment && vs->uri->fragmentlen) {
+		add_char_to_string(&fragment, '#');
 		add_bytes_to_string(&fragment, vs->uri->fragment, vs->uri->fragmentlen);
 	}
 
@@ -61,13 +67,18 @@ js_location_get_property_hash(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_host(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -94,13 +105,18 @@ js_location_get_property_host(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_hostname(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -127,13 +143,18 @@ js_location_get_property_hostname(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_href(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -160,13 +181,18 @@ js_location_get_property_href(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_origin(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -193,13 +219,18 @@ js_location_get_property_origin(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_pathname(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -227,13 +258,18 @@ js_location_get_property_pathname(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_port(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -259,13 +295,18 @@ js_location_get_property_port(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_protocol(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -296,13 +337,18 @@ js_location_get_property_protocol(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_get_property_search(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -331,13 +377,18 @@ js_location_get_property_search(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_location_set_property_hash(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -362,6 +413,7 @@ js_location_set_property_hash(JSContext *ctx, JSValueConst this_val, JSValue val
 static JSValue
 js_location_set_property_host(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -369,7 +421,11 @@ js_location_set_property_host(JSContext *ctx, JSValueConst this_val, JSValue val
 	REF_JS(val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -393,6 +449,7 @@ js_location_set_property_host(JSContext *ctx, JSValueConst this_val, JSValue val
 static JSValue
 js_location_set_property_hostname(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -400,7 +457,11 @@ js_location_set_property_hostname(JSContext *ctx, JSValueConst this_val, JSValue
 	REF_JS(val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -424,6 +485,7 @@ js_location_set_property_hostname(JSContext *ctx, JSValueConst this_val, JSValue
 static JSValue
 js_location_set_property_href(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -431,7 +493,11 @@ js_location_set_property_href(JSContext *ctx, JSValueConst this_val, JSValue val
 	REF_JS(val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -455,6 +521,7 @@ js_location_set_property_href(JSContext *ctx, JSValueConst this_val, JSValue val
 static JSValue
 js_location_set_property_pathname(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -462,7 +529,11 @@ js_location_set_property_pathname(JSContext *ctx, JSValueConst this_val, JSValue
 	REF_JS(val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -486,6 +557,7 @@ js_location_set_property_pathname(JSContext *ctx, JSValueConst this_val, JSValue
 static JSValue
 js_location_set_property_port(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -493,7 +565,11 @@ js_location_set_property_port(JSContext *ctx, JSValueConst this_val, JSValue val
 	REF_JS(val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -517,6 +593,7 @@ js_location_set_property_port(JSContext *ctx, JSValueConst this_val, JSValue val
 static JSValue
 js_location_set_property_protocol(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -524,7 +601,11 @@ js_location_set_property_protocol(JSContext *ctx, JSValueConst this_val, JSValue
 	REF_JS(val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -548,6 +629,7 @@ js_location_set_property_protocol(JSContext *ctx, JSValueConst this_val, JSValue
 static JSValue
 js_location_set_property_search(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -555,7 +637,11 @@ js_location_set_property_search(JSContext *ctx, JSValueConst this_val, JSValue v
 	REF_JS(val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -579,13 +665,18 @@ js_location_set_property_search(JSContext *ctx, JSValueConst this_val, JSValue v
 static JSValue
 js_location_assign(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -607,13 +698,18 @@ js_location_assign(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst
 static JSValue
 js_location_reload(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -629,13 +725,18 @@ js_location_reload(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst
 static JSValue
 js_location_replace(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
-	struct view_state *vs = interpreter->vs;
+	struct view_state *vs = JS_GetOpaque(this_val, js_location_class_id);
+
+	if (!vs) {
+		vs = interpreter->vs;
+	}
 
 	if (!vs) {
 #ifdef ECMASCRIPT_DEBUG
@@ -667,6 +768,7 @@ js_location_replace(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 static JSValue
 js_location_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -692,13 +794,32 @@ static const JSCFunctionListEntry js_location_proto_funcs[] = {
 	JS_CFUNC_DEF("toLocaleString", 0, js_location_toString),
 };
 
+static void
+js_location_finalize(JSRuntime *rt, JSValue val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(val);
+
+	struct view_state *vs = (struct view_state *)JS_GetOpaque(val, js_location_class_id);
+
+	if (vs) {
+		vs->location_obj = JS_NULL;
+	}
+}
+
+
 static JSClassDef js_location_class = {
 	"location",
+	.finalizer = js_location_finalize,
 };
 
 JSValue
 js_location_init(JSContext *ctx)
 {
+	ELOG
 	JSValue location_proto;
 
 	/* create the location class */
@@ -721,8 +842,9 @@ js_location_init(JSContext *ctx)
 }
 
 JSValue
-getLocation(JSContext *ctx)
+getLocation(JSContext *ctx, struct view_state *vs)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -737,7 +859,11 @@ getLocation(JSContext *ctx)
 	REF_JS(location_obj);
 	JS_SetPropertyFunctionList(ctx, location_obj, js_location_proto_funcs, countof(js_location_proto_funcs));
 	JS_SetClassProto(ctx, js_location_class_id, location_obj);
+	JS_SetOpaque(location_obj, vs);
 
+	if (vs) {
+		vs->location_obj = location_obj;
+	}
 	JSValue rr = JS_DupValue(ctx, location_obj);
 	RETURN_JS(rr);
 }
