@@ -28,6 +28,7 @@
 #include "js/quickjs/keyboard.h"
 #include "js/quickjs/location.h"
 #include "js/quickjs/message.h"
+#include "js/quickjs/performance.h"
 #include "js/quickjs/window.h"
 #include "js/timer.h"
 #include "intl/libintl.h"
@@ -37,6 +38,7 @@
 #include "session/task.h"
 #include "terminal/tab.h"
 #include "util/conv.h"
+#include "viewer/text/view.h"
 #include "viewer/text/vs.h"
 
 #define countof(x) (sizeof(x) / sizeof((x)[0]))
@@ -54,6 +56,7 @@ struct el_window {
 	JSValue thisval;
 	LIST_OF(struct win_listener) listeners;
 	JSValue onmessage;
+	struct view_state *vs;
 };
 
 struct el_message {
@@ -66,6 +69,7 @@ extern struct term_event last_event;
 static void
 js_window_finalize(JSRuntime *rt, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -80,13 +84,40 @@ js_window_finalize(JSRuntime *rt, JSValue val)
 			mem_free_set(&l->typ, NULL);
 		}
 		free_list(elwin->listeners);
+
+		if (elwin->vs) {
+			elwin->vs->win_obj = JS_NULL;
+		}
 		mem_free(elwin);
 	}
 }
 
+void
+detach_js_view_state(struct view_state *vs)
+{
+	ELOG
+	assert(vs);
+	if_assert_failed return;
+
+	if (!JS_IsNull(vs->win_obj)) {
+		struct el_window *elwin = (struct el_window *)JS_GetOpaque(vs->win_obj, js_window_class_id);
+
+		if (elwin) {
+			elwin->vs = NULL;
+		}
+		vs->win_obj = JS_NULL;
+	}
+
+	if (!JS_IsNull(vs->location_obj)) {
+		JS_SetOpaque(vs->location_obj, NULL);
+		vs->location_obj = JS_NULL;
+	}
+}
+
 static void
 js_window_mark(JSRuntime *rt, JSValueConst val, JS_MarkFunc *mark_func)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -118,6 +149,7 @@ js_window_mark(JSRuntime *rt, JSValueConst val, JS_MarkFunc *mark_func)
 JSValue
 js_window_open(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -137,7 +169,7 @@ js_window_open(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *ar
 	doc_view = vs->doc_view;
 	ses = doc_view->session;
 
-	if (get_opt_bool("ecmascript.block_window_opening", ses)) {
+	if (get_opt_bool("ecmascript.block_window_opening", NULL)) {
 #ifdef CONFIG_LEDS
 		set_led_value(ses->status.popup_led, 'P');
 #endif
@@ -245,15 +277,197 @@ end:
 	return ret;
 }
 
-/* @window_funcs{"setInterval"} */
+/* @window_funcs{"scrollBy"} */
 JSValue
-js_window_setInterval(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+js_window_scrollBy(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
-	int timeout = 0;
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	if (argc == 2) {
+		int32_t dx = 0;
+		int32_t dy = 0;
+
+		JS_ToInt32(ctx, &dx, argv[0]);
+		JS_ToInt32(ctx, &dy, argv[1]);
+		int sx = (dx + term->cell_width - 1) / term->cell_width;
+		int sy = (dy + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy);
+		horizontal_scroll(ses, doc_view, sx);
+
+		return JS_UNDEFINED;
+	}
+
+	if (argc == 1) {
+		int32_t top = 0;
+		int32_t left = 0;
+		JSValue r = JS_GetPropertyStr(ctx, argv[0], "top");
+		JS_ToInt32(ctx, &top, r);
+		r = JS_GetPropertyStr(ctx, argv[0], "left");
+		JS_ToInt32(ctx, &left, r);
+
+		int sx = (left + term->cell_width - 1) / term->cell_width;
+		int sy = (top + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy);
+		horizontal_scroll(ses, doc_view, sx);
+	}
+
+	return JS_UNDEFINED;
+}
+
+/* @window_funcs{"scrollByLines"} */
+JSValue
+js_window_scrollByLines(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+
+	if (argc > 0) {
+		int32_t steps = 0;
+
+		JS_ToInt32(ctx, &steps, argv[0]);
+		vertical_scroll(ses, doc_view, steps);
+	}
+
+	return JS_UNDEFINED;
+}
+
+/* @window_funcs{"scrollByPages"} */
+JSValue
+js_window_scrollByPages(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+
+	if (argc > 0) {
+		int32_t steps = 0;
+
+		JS_ToInt32(ctx, &steps, argv[0]);
+		vertical_scroll(ses, doc_view, steps * doc_view->box.height);
+	}
+
+	return JS_UNDEFINED;
+}
+
+/* @window_funcs{"scrollTo"} */
+JSValue
+js_window_scrollTo(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	if (argc == 2) {
+		int32_t x = 0;
+		int32_t y = 0;
+
+		JS_ToInt32(ctx, &x, argv[0]);
+		JS_ToInt32(ctx, &y, argv[1]);
+
+		int sx = (x + term->cell_width - 1) / term->cell_width;
+		int sy = (y + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy - doc_view->vs->y);
+		horizontal_scroll(ses, doc_view, sx - doc_view->vs->x);
+
+		return JS_UNDEFINED;
+	}
+
+	if (argc == 1) {
+		int32_t top = 0;
+		int32_t left = 0;
+		JSValue r = JS_GetPropertyStr(ctx, argv[0], "top");
+		JS_ToInt32(ctx, &top, r);
+		r = JS_GetPropertyStr(ctx, argv[0], "left");
+		JS_ToInt32(ctx, &left, r);
+
+		int sx = (left + term->cell_width - 1) / term->cell_width;
+		int sy = (top + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy - doc_view->vs->y);
+		horizontal_scroll(ses, doc_view, sx - doc_view->vs->x);
+
+	}
+
+	return JS_UNDEFINED;
+}
+
+/* @window_funcs{"scroll"} */
+JSValue
+js_window_scroll(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	return js_window_scrollTo(ctx, this_val, argc, argv);
+}
+
+/* @window_funcs{"requestAninationFrame"} */
+JSValue
+js_window_requestAnimationFrame(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	JSValueConst func;
+
+	if (argc != 1) {
+		return JS_UNDEFINED;
+	}
+	func = argv[0];
+
+	if (JS_IsFunction(ctx, func)) {
+		int id = ecmascript_set_request2(ctx, func);
+		return JS_NewInt32(ctx, id);
+	}
+	return JS_UNDEFINED;
+}
+
+
+/* @window_funcs{"setInterval"} */
+JSValue
+js_window_setInterval(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	int32_t timeout = 0;
 	JSValueConst func;
 
 	if (argc != 2) {
@@ -308,11 +522,12 @@ js_window_setInterval(JSContext *ctx, JSValueConst this_val, int argc, JSValueCo
 JSValue
 js_window_setTimeout(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
-	int timeout = 0;
+	int32_t timeout = 0;
 	JSValueConst func;
 
 	if (argc < 1) {
@@ -332,7 +547,7 @@ js_window_setTimeout(JSContext *ctx, JSValueConst this_val, int argc, JSValueCon
 	func = argv[0];
 
 	if (JS_IsFunction(ctx, func)) {
-		struct ecmascript_timeout *id = ecmascript_set_timeout2q(ctx, func, timeout, -1);
+		struct ecmascript_timeout *id = ecmascript_set_timeout2q(ctx, func, timeout, 0);
 #if SIZEOF_INTPTR_T == 4
 		return JS_NewInt32(ctx, (intptr_t)(id));
 #else
@@ -350,7 +565,7 @@ js_window_setTimeout(JSContext *ctx, JSValueConst this_val, int argc, JSValueCon
 		JS_FreeCString(ctx, code);
 
 		if (code2) {
-			struct ecmascript_timeout *id = ecmascript_set_timeout(ctx, code2, timeout, -1);
+			struct ecmascript_timeout *id = ecmascript_set_timeout(ctx, code2, timeout, 0);
 #if SIZEOF_INTPTR_T == 4
 			return JS_NewInt32(ctx, (intptr_t)(id));
 #else
@@ -366,12 +581,13 @@ js_window_setTimeout(JSContext *ctx, JSValueConst this_val, int argc, JSValueCon
 JSValue
 js_window_clearInterval(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
-	if (argc != 1) {
+	if (argc != 1 || JS_IsNull(argv[0])) {
 		return JS_UNDEFINED;
 	}
 	intptr_t number;
@@ -387,11 +603,41 @@ js_window_clearInterval(JSContext *ctx, JSValueConst this_val, int argc, JSValue
 #endif
 	struct ecmascript_timeout *t = (struct ecmascript_timeout *)(number);
 
-	if (t && found_in_map_timer(t->tid)) {
-		kill_timer(&t->tid);
-		done_string(&t->code);
-		del_from_list(t);
-		mem_free(t);
+	if (found_in_map_timer(t)) {
+		t->timeout_next = -1;
+	}
+
+	return JS_UNDEFINED;
+}
+
+/* @window_funcs{"clearRequestAnimationFrame"} */
+JSValue
+js_window_clearRequestAnimationFrame(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+
+	if (argc != 1 || JS_IsNull(argv[0])) {
+		return JS_UNDEFINED;
+	}
+	int32_t id;
+
+	if (JS_ToInt32(ctx, &id, argv[0])) {
+		return JS_UNDEFINED;
+	}
+
+	if (!id) {
+		return JS_UNDEFINED;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+
+	if (id == interpreter->request) {
+		interpreter->request = 0;
+		JS_FreeValue(ctx, interpreter->request_func);
+		interpreter->request_func = JS_NULL;
 	}
 
 	return JS_UNDEFINED;
@@ -402,12 +648,13 @@ js_window_clearInterval(JSContext *ctx, JSValueConst this_val, int argc, JSValue
 JSValue
 js_window_clearTimeout(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
-	if (argc != 1) {
+	if (argc != 1 || JS_IsNull(argv[0])) {
 		return JS_UNDEFINED;
 	}
 	intptr_t number;
@@ -423,11 +670,8 @@ js_window_clearTimeout(JSContext *ctx, JSValueConst this_val, int argc, JSValueC
 #endif
 	struct ecmascript_timeout *t = (struct ecmascript_timeout *)(number);
 
-	if (t && found_in_map_timer(t->tid)) {
-		kill_timer(&t->tid);
-		done_string(&t->code);
-		del_from_list(t);
-		mem_free(t);
+	if (found_in_map_timer(t)) {
+		t->timeout_next = -1;
 	}
 
 	return JS_UNDEFINED;
@@ -437,6 +681,7 @@ js_window_clearTimeout(JSContext *ctx, JSValueConst this_val, int argc, JSValueC
 JSValue
 js_window_getComputedStyle(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -456,6 +701,7 @@ js_window_getComputedStyle(JSContext *ctx, JSValueConst this_val, int argc, JSVa
 static JSValue
 js_window_get_property_closed(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -467,6 +713,7 @@ js_window_get_property_closed(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_window_get_property_event(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -478,6 +725,7 @@ js_window_get_property_event(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_window_get_property_innerHeight(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -506,6 +754,7 @@ js_window_get_property_innerHeight(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_window_get_property_innerWidth(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -534,12 +783,17 @@ js_window_get_property_innerWidth(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_window_get_property_location(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+	struct el_window *elwin = (struct el_window *)JS_GetOpaque(this_val, js_window_class_id);
 
+	if (elwin && elwin->vs) {
+		return getLocation(ctx, elwin->vs);
+	}
 	JSValue ret = interpreter->location_obj;
 
 	JS_DupValue(ctx, ret);
@@ -550,6 +804,7 @@ js_window_get_property_location(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_window_set_property_location(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -583,15 +838,15 @@ js_window_set_property_location(JSContext *ctx, JSValueConst this_val, JSValue v
 	return JS_UNDEFINED;
 }
 
+static JSValue js_window_get_property_top(JSContext *ctx, JSValueConst this_val);
 
 static JSValue
 js_window_get_property_parent(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
-	REF_JS(this_val);
-
 	/* XXX: It would be nice if the following worked, yes.
 	 * The problem is that we get called at the point where
 	 * document.frame properties are going to be mostly NULL.
@@ -603,12 +858,59 @@ js_window_get_property_parent(JSContext *ctx, JSValueConst this_val)
 	 * INCORRECT but works for the most common cases of just two
 	 * frames. Better something than nothing. */
 
-	return JS_UNDEFINED;
+	return js_window_get_property_top(ctx, this_val);
+}
+
+static JSValue
+js_window_get_property_performance(JSContext *ctx, JSValueConst this_val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+
+	return getPerformance(ctx);
+}
+
+static JSValue
+js_window_get_property_scrollX(JSContext *ctx, JSValueConst this_val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	return JS_NewInt32(ctx, doc_view->vs->x * term->cell_width);
+}
+
+static JSValue
+js_window_get_property_scrollY(JSContext *ctx, JSValueConst this_val)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	REF_JS(this_val);
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	return JS_NewInt32(ctx, doc_view->vs->y * term->cell_height);
 }
 
 static JSValue
 js_window_get_property_self(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -622,6 +924,7 @@ js_window_get_property_self(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_window_get_property_status(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -633,6 +936,7 @@ js_window_get_property_status(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_window_set_property_status(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -663,6 +967,7 @@ js_window_set_property_status(JSContext *ctx, JSValueConst this_val, JSValue val
 static JSValue
 js_window_get_property_top(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -710,6 +1015,7 @@ js_window_get_property_top(JSContext *ctx, JSValueConst this_val)
 JSValue
 js_window_alert(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -746,6 +1052,7 @@ js_window_alert(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *a
 static JSValue
 js_window_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -757,6 +1064,7 @@ js_window_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst
 static JSValue
 js_window_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -819,6 +1127,7 @@ js_window_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSVa
 static JSValue
 js_window_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -868,6 +1177,7 @@ js_window_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc, J
 static void
 onmessage_run(void *data)
 {
+	ELOG
 	struct el_message *mess = (struct el_message *)data;
 
 	if (mess) {
@@ -911,6 +1221,7 @@ onmessage_run(void *data)
 static JSValue
 js_window_postMessage(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -967,6 +1278,9 @@ static const JSCFunctionListEntry js_window_proto_funcs[] = {
 	JS_CGETSET_DEF("innerWidth", js_window_get_property_innerWidth, NULL),
 	JS_CGETSET_DEF("location", js_window_get_property_location, js_window_set_property_location),
 	JS_CGETSET_DEF("parent", js_window_get_property_parent, NULL),
+	JS_CGETSET_DEF("performance", js_window_get_property_performance, NULL),
+	JS_CGETSET_DEF("scrollX", js_window_get_property_scrollX, NULL),
+	JS_CGETSET_DEF("scrollY", js_window_get_property_scrollY, NULL),
 	JS_CGETSET_DEF("self", js_window_get_property_self, NULL),
 	JS_CGETSET_DEF("status", js_window_get_property_status, js_window_set_property_status),
 	JS_CGETSET_DEF("top", js_window_get_property_top, NULL),
@@ -974,11 +1288,18 @@ static const JSCFunctionListEntry js_window_proto_funcs[] = {
 	JS_CFUNC_DEF("addEventListener", 3, js_window_addEventListener),
 	JS_CFUNC_DEF("alert", 1, js_window_alert),
 	JS_CFUNC_DEF("clearInterval", 1, js_window_clearInterval),
+	JS_CFUNC_DEF("clearRequestAnimationFrame", 1, js_window_clearRequestAnimationFrame),
 	JS_CFUNC_DEF("clearTimeout", 1, js_window_clearTimeout),
 	JS_CFUNC_DEF("getComputedStyle", 2, js_window_getComputedStyle),
 	JS_CFUNC_DEF("open", 3, js_window_open),
 	JS_CFUNC_DEF("postMessage", 3, js_window_postMessage),
 	JS_CFUNC_DEF("removeEventListener", 3, js_window_removeEventListener),
+	JS_CFUNC_DEF("requestAnimationFrame", 1, js_window_requestAnimationFrame),
+	JS_CFUNC_DEF("scroll", 2, js_window_scroll),
+	JS_CFUNC_DEF("scrollBy", 2, js_window_scrollBy),
+	JS_CFUNC_DEF("scrollByLines", 1, js_window_scrollByLines),
+	JS_CFUNC_DEF("scrollByPages", 1, js_window_scrollByPages),
+	JS_CFUNC_DEF("scrollTo", 2, js_window_scrollTo),
 	JS_CFUNC_DEF("setInterval", 2, js_window_setInterval),
 	JS_CFUNC_DEF("setTimeout", 2, js_window_setTimeout),
 	JS_CFUNC_DEF("toString", 0, js_window_toString)
@@ -993,6 +1314,7 @@ static JSClassDef js_window_class = {
 int
 js_window_init(JSContext *ctx)
 {
+	ELOG
 	JSValue window_proto;
 
 	/* create the window class */
@@ -1015,3 +1337,32 @@ js_window_init(JSContext *ctx)
 
 	return 0;
 }
+
+JSValue
+getWindow(JSContext *ctx, struct view_state *vs)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	struct el_window *elwin = (struct el_window *)mem_calloc(1, sizeof(*elwin));
+
+	if (!elwin) {
+		return JS_NULL;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
+
+	JSValue win_obj = JS_NewObjectClass(ctx, js_window_class_id);
+	JS_SetPropertyFunctionList(ctx, win_obj, js_window_proto_funcs, countof(js_window_proto_funcs));
+	init_list(elwin->listeners);
+	elwin->interpreter = interpreter;
+	elwin->thisval = win_obj;
+	elwin->vs = vs;
+
+	if (vs) {
+		vs->win_obj = win_obj;
+	}
+	JS_SetOpaque(win_obj, elwin);
+
+	return win_obj;
+}
\ No newline at end of file
