@@ -11,6 +11,8 @@
 
 #include "elinks.h"
 
+#include "js/libdom/dom.h"
+
 #include "js/spidermonkey/util.h"
 
 #include "bfu/dialog.h"
@@ -20,6 +22,7 @@
 #include "dialogs/menu.h"
 #include "dialogs/status.h"
 #include "document/html/frames.h"
+#include "document/libdom/mapa.h"
 #include "document/libdom/renderer.h"
 #include "document/libdom/renderer2.h"
 #include "document/document.h"
@@ -32,6 +35,7 @@
 #include "js/spidermonkey/customevent.h"
 #include "js/spidermonkey/document.h"
 #include "js/spidermonkey/domparser.h"
+#include "js/spidermonkey/element.h"
 #include "js/spidermonkey/event.h"
 #include "js/spidermonkey/form.h"
 #include "js/spidermonkey/fragment.h"
@@ -72,6 +76,7 @@
 
 #include <jsapi.h>
 #include <js/CompilationAndEvaluation.h>
+#include <js/friend/DumpFunctions.h>
 #include <js/Printf.h>
 #include <js/SourceText.h>
 #include <js/Warnings.h>
@@ -115,12 +120,13 @@ error_reporter(JSContext *ctx, JSErrorReport *report)
 	set_led_value(ses->status.ecmascript_led, 'J');
 #endif
 
-	if (!get_opt_bool("ecmascript.error_reporting", ses))
+	if (!get_opt_bool("ecmascript.error_reporting", NULL))
 		goto reported;
 
 	f = open_memstream(&ptr, &size);
 
 	if (f) {
+		js::DumpBacktrace(ctx, f);
 		JS::PrintError(f, report, true/*reportWarnings*/);
 		fclose(f);
 
@@ -262,7 +268,7 @@ spidermonkey_get_interpreter(struct ecmascript_interpreter *interpreter)
 	JSContext *ctx;
 	JSObject *console_obj, *document_obj, *Document_obj,  *fragment_obj, /* *forms_obj,*/ *history_obj,
 	         *statusbar_obj, *menubar_obj, *navigator_obj, *node_obj, *localstorage_obj, *screen_obj,
-	         *xhr_obj, *event_obj, *keyboardEvent_obj, *messageEvent_obj, *customEvent_obj,
+	         *xhr_obj, *element_obj, *event_obj, *keyboardEvent_obj, *messageEvent_obj, *customEvent_obj,
 	         *url_obj, *urlSearchParams_obj, *domparser_obj, *image_obj;
 
 	assert(interpreter);
@@ -403,6 +409,15 @@ spidermonkey_get_interpreter(struct ecmascript_interpreter *interpreter)
 		goto release_and_fail;
 	}
 
+	element_obj = spidermonkey_InitClass(ctx, global, NULL,
+					      &element_class, Element_constructor, 0,
+					      element_props,
+					      element_funcs,
+					      NULL, NULL, "Element");
+	if (!element_obj) {
+		goto release_and_fail;
+	}
+
 	xhr_obj = spidermonkey_InitClass(ctx, global, NULL,
 					&xhr_class, xhr_constructor, 0,
 					xhr_props,
@@ -621,39 +636,76 @@ spidermonkey_eval(struct ecmascript_interpreter *interpreter,
 	interpreter->ret = ret;
 
 	JS::RootedObject cg(ctx, JS::CurrentGlobalOrNull(ctx));
+
+	struct view_state *vs = interpreter->vs;
+	JSObject *th = NULL;
+
+	if (vs) {
+		struct document *document = vs->doc_view->document;
+		void *mapa = (void *)document->element_map;
+
+		if (mapa) {
+			dom_node *elem = (dom_node *)find_in_map(mapa, interpreter->element_offset);
+
+			if (elem) {
+				dom_html_element_type ty;
+				dom_exception exc = dom_html_element_get_tag_type(elem, &ty);
+
+				if (exc == DOM_NO_ERR && ty != DOM_HTML_ELEMENT_TYPE_SCRIPT) {
+					th = getElement(ctx, elem);
+				}
+			}
+		}
+	}
+	JS::RootedObjectVector envChain(ctx);
+
+	if (th) {
+		JS::RootedObject thisobj(ctx, th);
+
+		if (envChain.append(thisobj)) {
+			JSAutoRealm ar(ctx, cg);
+			JS_WrapObject(ctx, envChain[0]);
+		}
+	}
 	JS::RootedValue r_val(ctx, rval);
 	JS::CompileOptions options(ctx);
 
-	char *utf8_data = (char *)mem_alloc(code->length);
+	char *utf16_data = (char *)mem_calloc(sizeof(char16_t), (code->length + 1));
 
-	if (!utf8_data) {
+	if (!utf16_data) {
 		return;
 	}
-	iconv_t cd = iconv_open("UTF-8//IGNORE", "UTF-8");
+	iconv_t cd = iconv_open("UTF-16//IGNORE", "UTF-8");
 
 	if (cd == (iconv_t)-1) {
-		mem_free(utf8_data);
+		mem_free(utf16_data);
 		return;
 	}
 	char *inbuf = code->source;
-	char *outbuf = utf8_data;
+	char *outbuf = utf16_data;
 	size_t inbytes_len = code->length;
-	size_t outbytes_len = code->length;
+	size_t outbytes_left = (code->length + 1) * sizeof(char16_t);
 
-	if (iconv(cd, &inbuf, &inbytes_len, &outbuf, &outbytes_len) < 0) {
-		mem_free(utf8_data);
+	if (iconv(cd, &inbuf, &inbytes_len, &outbuf, &outbytes_left) < 0) {
+		mem_free(utf16_data);
 		iconv_close(cd);
 		return;
 	}
-	JS::SourceText<mozilla::Utf8Unit> srcBuf;
-	if (!srcBuf.init(ctx, utf8_data, code->length - outbytes_len, JS::SourceOwnership::Borrowed)) {
+	JS::SourceText<char16_t> srcBuf;
+
+	if (!srcBuf.init(ctx, (char16_t *)utf16_data, ((code->length + 1) * sizeof(char16_t) - outbytes_left) / sizeof(char16_t), JS::SourceOwnership::Borrowed)) {
 		iconv_close(cd);
-		mem_free(utf8_data);
+		mem_free(utf16_data);
 		return;
 	}
 	iconv_close(cd);
-	JS::Evaluate(ctx, options, srcBuf, &r_val);
-	mem_free(utf8_data);
+
+	if (envChain.empty()) {
+		JS::Evaluate(ctx, options, srcBuf, &r_val);
+	} else {
+		JS::Evaluate(ctx, envChain, options, srcBuf, &r_val);
+	}
+	mem_free(utf16_data);
 
 	spidermonkey_check_for_exception(ctx);
 
@@ -683,6 +735,31 @@ spidermonkey_call_function(struct ecmascript_interpreter *interpreter,
 	done_heartbeat(interpreter->heartbeat);
 }
 
+void
+spidermonkey_call_function_timestamp(struct ecmascript_interpreter *interpreter,
+                  JS::HandleValue fun, struct string *ret)
+{
+	JSContext *ctx;
+	JS::Value rval;
+
+	assert(interpreter);
+	if (!js_module_init_ok) {
+		return;
+	}
+	ctx = (JSContext *)interpreter->backend_data;
+	JSAutoRealm ar(ctx, (JSObject *)interpreter->ac->get());
+
+	interpreter->heartbeat = add_heartbeat(interpreter);
+	interpreter->ret = ret;
+
+	JS::RootedValue r_val(ctx, rval);
+	JS::RootedObject cg(ctx, JS::CurrentGlobalOrNull(ctx));
+
+	JS::RootedValueArray<1> argv(ctx);
+	argv[0].setNumber(interpreter->timestamp);
+	JS_CallFunctionValue(ctx, cg, fun, argv, &r_val);
+	done_heartbeat(interpreter->heartbeat);
+}
 
 char *
 spidermonkey_eval_stringback(struct ecmascript_interpreter *interpreter,
