@@ -24,6 +24,7 @@
 #include "document/html/frames.h"
 #include "document/document.h"
 #include "document/forms.h"
+#include "document/libdom/corestrings.h"
 #include "document/libdom/doc.h"
 #include "document/libdom/mapa.h"
 #include "document/libdom/renderer2.h"
@@ -37,6 +38,7 @@
 #include "js/spidermonkey/form.h"
 #include "js/spidermonkey/forms.h"
 #include "js/spidermonkey/implementation.h"
+#include "js/spidermonkey/keyboard.h"
 #include "js/spidermonkey/location.h"
 #include "js/spidermonkey/document.h"
 #include "js/spidermonkey/element.h"
@@ -93,6 +95,8 @@ struct document_private {
 	JS::Heap<JSObject *> thisval;
 	JS::Heap<JSObject *> *images;
 	dom_event_listener *listener;
+	JS::Heap<JS::Value> *onkeydown;
+	JS::Heap<JS::Value> *onkeyup;
 	int ref_count;
 	enum readyState state;
 };
@@ -105,6 +109,7 @@ static void document_finalize(JS::GCContext *op, JSObject *obj)
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(obj, 0);
+	NODEINFO(doc);
 	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(obj, 1);
 
 	if (doc_private) {
@@ -128,14 +133,18 @@ static void document_finalize(JS::GCContext *op, JSObject *obj)
 		if (doc_private->images) {
 			delete (doc_private->images);
 		}
+		if (doc_private->onkeyup) {
+			delete (doc_private->onkeyup);
+		}
+		if (doc_private->onkeydown) {
+			delete (doc_private->onkeydown);
+		}
 		mem_free(doc_private);
 		JS::SetReservedSlot(obj, 1, JS::UndefinedValue());
 	}
 
-	if (doc) {
-		dom_node_unref(doc);
-		JS::SetReservedSlot(obj, 0, JS::UndefinedValue());
-	}
+	dom_node_unref(doc);
+	JS::SetReservedSlot(obj, 0, JS::UndefinedValue());
 }
 
 JSClassOps document_ops = {
@@ -176,6 +185,7 @@ document_get_property_anchors(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -264,6 +274,7 @@ document_get_property_body(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -276,7 +287,7 @@ document_get_property_body(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		args.rval().setNull();
 		return true;
 	}
-	JSObject *obj = getNode(ctx, body);
+	JSObject *obj = getElement(ctx, body);
 	args.rval().setObject(*obj);
 
 	return true;
@@ -409,6 +420,7 @@ document_get_property_childNodes(JSContext *ctx, unsigned int argc, JS::Value *v
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -475,15 +487,15 @@ document_get_property_currentScript(JSContext *ctx, unsigned int argc, JS::Value
 		dom_node *elem = (dom_node *)find_in_map(mapa, interpreter->element_offset);
 
 		if (elem) {
-			dom_string *tag_name = NULL;
-			dom_exception exc = dom_node_get_node_name(elem, &tag_name);
+			NODEINFO(elem);
+			dom_html_element_type ty;
+			dom_exception exc = dom_html_element_get_tag_type(elem, &ty);
 
-			if (exc != DOM_NO_ERR || !tag_name) {
+			if (exc != DOM_NO_ERR) {
 				args.rval().setNull();
 				return true;
 			}
-			bool isScript = !strcmp("SCRIPT", dom_string_data(tag_name));
-			dom_string_unref(tag_name);
+			bool isScript = (ty == DOM_HTML_ELEMENT_TYPE_SCRIPT);
 
 			if (isScript) {
 				JSObject *obj = getNode(ctx, elem);
@@ -533,6 +545,7 @@ document_get_property_doctype(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -563,6 +576,7 @@ document_get_property_documentElement(JSContext *ctx, unsigned int argc, JS::Val
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -702,6 +716,7 @@ document_get_property_forms(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -727,8 +742,39 @@ document_get_property_head(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	JS::CallArgs args = CallArgsFromVp(argc, vp);
-// TODO
-	args.rval().setNull();
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+
+	if (!JS_InstanceOf(ctx, hobj, &document_class, NULL)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
+
+	if (!doc) {
+		args.rval().setNull();
+		return true;
+	}
+	dom_nodelist *nodes = NULL;
+	dom_exception exc = dom_document_get_elements_by_tag_name(doc, corestring_dom_HEAD, &nodes);
+
+	if (exc != DOM_NO_ERR || !nodes) {
+		args.rval().setNull();
+		return true;
+	}
+	dom_node *head = NULL;
+	exc = dom_nodelist_item(nodes, 0, &head);
+	dom_nodelist_unref(nodes);
+
+	if (exc != DOM_NO_ERR || !head) {
+		args.rval().setNull();
+		return true;
+	}
+	JSObject *obj = getElement(ctx, head);
+	args.rval().setObject(*obj);
+
 	return true;
 }
 
@@ -759,6 +805,7 @@ document_get_property_images(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -812,6 +859,7 @@ document_get_property_links(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -854,7 +902,7 @@ document_get_property_location(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
 
 	if (!interpreter->location_obj) {
-		interpreter->location_obj = getLocation(ctx);
+		interpreter->location_obj = getLocation(ctx, NULL);
 	}
 	args.rval().setObject(*(JSObject *)(interpreter->location_obj));
 	return true;
@@ -882,6 +930,168 @@ document_get_property_nodeType(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	return true;
 }
 
+static bool
+document_get_property_onkeydown(JSContext *ctx, unsigned int argc, JS::Value *vp)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
+	args.rval().set(*(doc_private->onkeydown));
+
+	return true;
+}
+
+static bool
+document_set_property_onkeydown(JSContext *ctx, unsigned int argc, JS::Value *vp)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
+
+	if (doc_private->onkeydown) {
+		delete doc_private->onkeydown;
+	}
+	doc_private->onkeydown = new JS::Heap<JS::Value>(args[0]);
+
+	return true;
+}
+
+static bool
+document_get_property_onkeyup(JSContext *ctx, unsigned int argc, JS::Value *vp)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
+	args.rval().set(*(doc_private->onkeyup));
+
+	return true;
+}
+
+int
+spidermonkey_document_fire_onkeydown(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JSContext *ctx = (JSContext *)interpreter->backend_data;
+	JSObject *obj = interpreter->document_obj;
+	JS::RootedObject hobj(ctx, obj);
+
+	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
+
+	if (!doc_private->onkeydown) {
+		return 0;
+	}
+	JSObject *obj_ev = get_keyboardEvent(ctx, ev);
+	interpreter->heartbeat = add_heartbeat(interpreter);
+	JS::RootedValueArray<1> argv(ctx);
+	argv[0].setObject(*obj_ev);
+	JS::RootedValue r_val(ctx);
+	JS::RootedObject thisv(ctx, doc_private->thisval);
+	JS::RootedValue vfun(ctx, *(doc_private->onkeydown));
+	JS_CallFunctionValue(ctx, thisv, vfun, argv, &r_val);
+	done_heartbeat(interpreter->heartbeat);
+	check_for_rerender(interpreter, "onkeydown");
+	return 1;
+}
+
+int
+spidermonkey_document_fire_onkeyup(struct ecmascript_interpreter *interpreter, struct term_event *ev)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JSContext *ctx = (JSContext *)interpreter->backend_data;
+	JSObject *obj = interpreter->document_obj;
+	JS::RootedObject hobj(ctx, obj);
+
+	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
+
+	if (!doc_private->onkeyup) {
+		return 0;
+	}
+	JSObject *obj_ev = get_keyboardEvent(ctx, ev);
+	interpreter->heartbeat = add_heartbeat(interpreter);
+	JS::RootedValueArray<1> argv(ctx);
+	argv[0].setObject(*obj_ev);
+	JS::RootedValue r_val(ctx);
+	JS::RootedObject thisv(ctx, doc_private->thisval);
+	JS::RootedValue vfun(ctx, *(doc_private->onkeyup));
+	JS_CallFunctionValue(ctx, thisv, vfun, argv, &r_val);
+	done_heartbeat(interpreter->heartbeat);
+	check_for_rerender(interpreter, "onkeyup");
+	return 1;
+}
+
+static bool
+document_set_property_onkeyup(JSContext *ctx, unsigned int argc, JS::Value *vp)
+{
+	ELOG
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
+
+	if (doc_private->onkeyup) {
+		delete doc_private->onkeyup;
+	}
+	doc_private->onkeyup = new JS::Heap<JS::Value>(args[0]);
+
+	return true;
+}
+
 static bool
 document_get_property_readyState(JSContext *ctx, unsigned int argc, JS::Value *vp)
 {
@@ -1006,7 +1216,7 @@ document_get_property_referrer(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	switch (get_opt_int("protocol.http.referer.policy", NULL)) {
 	case REFERER_NONE:
 		/* oh well */
-		args.rval().setUndefined();
+		args.rval().setString(JS_NewStringCopyZ(ctx, ""));
 		break;
 
 	case REFERER_FAKE:
@@ -1022,8 +1232,10 @@ document_get_property_referrer(JSContext *ctx, unsigned int argc, JS::Value *vp)
 				args.rval().setString(JS_NewStringCopyZ(ctx, str));
 				mem_free(str);
 			} else {
-				args.rval().setUndefined();
+				args.rval().setString(JS_NewStringCopyZ(ctx, ""));
 			}
+		} else {
+			args.rval().setString(JS_NewStringCopyZ(ctx, ""));
 		}
 		break;
 
@@ -1034,7 +1246,7 @@ document_get_property_referrer(JSContext *ctx, unsigned int argc, JS::Value *vp)
 			args.rval().setString(JS_NewStringCopyZ(ctx, str));
 			mem_free(str);
 		} else {
-			args.rval().setUndefined();
+			args.rval().setString(JS_NewStringCopyZ(ctx, ""));
 		}
 		break;
 	}
@@ -1049,14 +1261,33 @@ document_get_property_scripts(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	JS::CallArgs args = CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
 
-// TODO
-	args.rval().setNull();
+	if (!JS_InstanceOf(ctx, hobj, &document_class, NULL)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_html_document>(hobj, 0);
+	NODEINFO(doc);
+
+	if (!doc) {
+		args.rval().setNull();
+		return true;
+	}
+	dom_html_collection *col = (dom_html_collection *)get_scripts(doc, (dom_node *)doc);
+
+	if (!col) {
+		args.rval().setNull();
+		return true;
+	}
+	JSObject *obj = getCollection(ctx, col);
+	args.rval().setObject(*obj);
 
 	return true;
 }
 
-
 static bool
 document_get_property_title(JSContext *ctx, unsigned int argc, JS::Value *vp)
 {
@@ -1248,6 +1479,8 @@ JSPropertySpec document_props[] = {
 	JS_PSG("links", document_get_property_links, JSPROP_ENUMERATE),
 	JS_PSGS("location",	document_get_property_location, document_set_property_location, JSPROP_ENUMERATE),
 	JS_PSG("nodeType", document_get_property_nodeType, JSPROP_ENUMERATE),
+	JS_PSGS("onkeydown", document_get_property_onkeydown, document_set_property_onkeydown, JSPROP_ENUMERATE),
+	JS_PSGS("onkeyup", document_get_property_onkeyup, document_set_property_onkeyup, JSPROP_ENUMERATE),
 	JS_PSG("readyState", document_get_property_readyState, JSPROP_ENUMERATE),
 	JS_PSG("referrer",	document_get_property_referrer, JSPROP_ENUMERATE),
 	JS_PSG("scripts",	document_get_property_scripts, JSPROP_ENUMERATE),
@@ -1473,6 +1706,8 @@ document_addEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
+
 	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
 
 	if (!doc || !doc_private) {
@@ -1538,9 +1773,7 @@ document_addEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(doc_private->listener);
 	args.rval().setUndefined();
 
@@ -1573,6 +1806,8 @@ document_removeEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
+
 	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
 
 	if (!doc || !doc_private) {
@@ -1641,6 +1876,7 @@ document_createComment(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -1699,6 +1935,7 @@ document_createDocumentFragment(JSContext *ctx, unsigned int argc, JS::Value *vp
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -1739,6 +1976,8 @@ document_createElement(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
+
 	dom_string *tag_name = NULL;
 	dom_exception exc;
 	char *str;
@@ -1797,6 +2036,7 @@ document_dispatchEvent(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 	struct document_private *doc_private = JS::GetMaybePtrFromReservedSlot<struct document_private>(hobj, 1);
 
 	if (!doc || !doc_private) {
@@ -1897,6 +2137,7 @@ document_createTextNode(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 	dom_string *data = NULL;
 	dom_exception exc;
 	char *str;
@@ -1950,6 +2191,7 @@ document_getElementById(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -2003,6 +2245,7 @@ document_getElementsByClassName(JSContext *ctx, unsigned int argc, JS::Value *vp
 		return false;
 	}
 	dom_html_document *doc = (dom_html_document *)JS::GetMaybePtrFromReservedSlot<dom_html_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -2072,6 +2315,7 @@ document_getElementsByTagName(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -2130,6 +2374,7 @@ document_querySelector(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -2185,6 +2430,7 @@ document_querySelectorAll(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_document *doc = JS::GetMaybePtrFromReservedSlot<dom_document>(hobj, 0);
+	NODEINFO(doc);
 
 	if (!doc) {
 		args.rval().setNull();
@@ -2232,9 +2478,7 @@ static void doctype_finalize(JS::GCContext *op, JSObject *obj)
 #endif
 	dom_document_type *dtd = JS::GetMaybePtrFromReservedSlot<dom_document_type>(obj, 0);
 
-	if (dtd) {
-		dom_node_unref(dtd);
-	}
+	dom_node_unref(dtd);
 }
 
 JSClassOps doctype_ops = {
@@ -2283,6 +2527,7 @@ doctype_get_property_name(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_document_type *dtd = (dom_document_type *)JS::GetMaybePtrFromReservedSlot<dom_document_type>(hobj, 0);
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		args.rval().setNull();
@@ -2328,6 +2573,7 @@ doctype_get_property_publicId(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_document_type *dtd = (dom_document_type *)JS::GetMaybePtrFromReservedSlot<dom_document_type>(hobj, 0);
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		args.rval().setNull();
@@ -2373,6 +2619,7 @@ doctype_get_property_systemId(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_document_type *dtd = (dom_document_type *)JS::GetMaybePtrFromReservedSlot<dom_document_type>(hobj, 0);
+	NODEINFO(dtd);
 
 	if (!dtd) {
 		args.rval().setNull();
@@ -2409,6 +2656,7 @@ getDoctype(JSContext *ctx, void *node)
 	if (!el) {
 		return NULL;
 	}
+	NODEINFO(node);
 
 	JS::RootedObject r_el(ctx, el);
 	JS_DefineProperties(ctx, r_el, (JSPropertySpec *) doctype_props);
@@ -2423,6 +2671,8 @@ getDocument(JSContext *ctx, void *doc)
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
+	NODEINFO(doc);
+
 	struct document_private *doc_private = (struct document_private *)mem_calloc(1, sizeof(*doc_private));
 
 	if (!doc_private) {
@@ -2460,6 +2710,8 @@ initDocument(JSContext *ctx, struct ecmascript_interpreter *interpreter, JSObjec
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
+	NODEINFO(doc);
+
 	struct document_private *doc_private = (struct document_private *)mem_calloc(1, sizeof(*doc_private));
 
 	if (!doc_private) {
@@ -2499,6 +2751,7 @@ Document_constructor(JSContext* ctx, unsigned argc, JS::Value* vp)
 	}
 	add_to_string(&str, "<!doctype html>\n<html><head></head><body></body></html>");
 	void *doc = document_parse_text("utf-8", str.source, str.length);
+	NODEINFO(doc);
 	done_string(&str);
 
 	struct document_private *doc_private = (struct document_private *)mem_calloc(1, sizeof(*doc_private));
