@@ -5,6 +5,6 @@
 
 extern JSClass domRect_class;
 extern JSPropertySpec domRect_props[];
-JSObject *getDomRect(JSContext *ctx);
+JSObject *getDomRect(JSContext *ctx, int x, int y, int width, int height, int top, int right, int bottom, int left);
 
 #endif
