@@ -562,6 +562,7 @@ style_style(JSContext *ctx, unsigned int argc, JS::Value *vp, const char *proper
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 	dom_exception exc;
 	dom_string *style = NULL;
 	char *res = NULL;
@@ -579,10 +580,7 @@ style_style(JSContext *ctx, unsigned int argc, JS::Value *vp, const char *proper
 
 	if (!style || !dom_string_length(style)) {
 		args.rval().setString(JS_NewStringCopyZ(ctx, ""));
-
-		if (style) {
-			dom_string_unref(style);
-		}
+		dom_string_unref(style);
 		return true;
 	}
 	res = get_css_value(dom_string_data(style), property);
@@ -625,6 +623,8 @@ style_get_property_cssText(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_exception exc;
 	dom_string *style = NULL;
 
@@ -674,6 +674,8 @@ style_set_property_cssText(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_exception exc;
 	char *res = NULL;
 	args.rval().setUndefined();
@@ -742,6 +744,8 @@ style_set_style(JSContext *ctx, unsigned int argc, JS::Value *vp, const char *pr
 	}
 	dom_exception exc;
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_string *style = NULL;
 	dom_string *stylestr = NULL;
 	char *res = NULL;
@@ -765,10 +769,7 @@ style_set_style(JSContext *ctx, unsigned int argc, JS::Value *vp, const char *pr
 
 	if (!style || !dom_string_length(style)) {
 		res = set_css_value("", property, value);
-
-		if (style) {
-			dom_string_unref(style);
-		}
+		dom_string_unref(style);
 	} else {
 		res = set_css_value(dom_string_data(style), property, value);
 		dom_string_unref(style);
