@@ -31,6 +31,7 @@
 #include "js/spidermonkey/keyboard.h"
 #include "js/spidermonkey/location.h"
 #include "js/spidermonkey/message.h"
+#include "js/spidermonkey/performance.h"
 #include "js/spidermonkey/window.h"
 #include "js/spidermonkey/xhr.h"
 #include "js/timer.h"
@@ -53,6 +54,7 @@
 #include "viewer/text/draw.h"
 #include "viewer/text/form.h"
 #include "viewer/text/link.h"
+#include "viewer/text/view.h"
 #include "viewer/text/vs.h"
 
 #include <map>
@@ -64,6 +66,9 @@ static bool window_get_property_innerWidth(JSContext *cx, unsigned int argc, JS:
 static bool window_get_property_location(JSContext *cx, unsigned int argc, JS::Value *vp);
 static bool window_set_property_location(JSContext *cx, unsigned int argc, JS::Value *vp);
 static bool window_get_property_parent(JSContext *ctx, unsigned int argc, JS::Value *vp);
+static bool window_get_property_performance(JSContext *ctx, unsigned int argc, JS::Value *vp);
+static bool window_get_property_scrollX(JSContext *ctx, unsigned int argc, JS::Value *vp);
+static bool window_get_property_scrollY(JSContext *ctx, unsigned int argc, JS::Value *vp);
 static bool window_get_property_self(JSContext *ctx, unsigned int argc, JS::Value *vp);
 static bool window_get_property_status(JSContext *ctx, unsigned int argc, JS::Value *vp);
 static bool window_set_property_status(JSContext *ctx, unsigned int argc, JS::Value *vp);
@@ -82,6 +87,7 @@ struct el_window {
 	JS::RootedObject thisval;
 	LIST_OF(struct listener) listeners;
 	JS::RootedValue onmessage;
+	struct view_state *vs;
 };
 
 struct el_message {
@@ -106,6 +112,10 @@ window_finalize(JS::GCContext *op, JSObject *obj)
 			mem_free_set(&l->typ, NULL);
 		}
 		free_list(elwin->listeners);
+
+		if (elwin->vs) {
+			elwin->vs->winobject = NULL;
+		}
 		mem_free(elwin);
 	}
 	moduleRegistry.erase(obj);
@@ -130,6 +140,44 @@ JSClass window_class = {
 	&window_ops
 };
 
+JSClass content_window_class = {
+	"contentWindow",
+	JSCLASS_HAS_RESERVED_SLOTS(1),	/* struct view_state * */
+	&window_ops
+};
+
+void
+detach_js_view_state(struct view_state *vs)
+{
+	assert(main_ctx);
+	assert(vs);
+	if_assert_failed return;
+
+	if (vs->winobject) {
+		JS::RootedObject robj(main_ctx, vs->winobject);
+
+		if (JS_InstanceOf(main_ctx, robj, (JSClass *) &window_class, NULL)
+			|| JS_InstanceOf(main_ctx, robj, (JSClass *) &content_window_class, NULL)) {
+			struct el_window *elwin = JS::GetMaybePtrFromReservedSlot<struct el_window>(robj, 0);
+
+			if (elwin) {
+				elwin->vs = NULL;
+			}
+			vs->winobject = NULL;
+		}
+	}
+
+	if (vs->locobject) {
+		JS::RootedObject lobj(main_ctx, vs->locobject);
+
+		if (JS_InstanceOf(main_ctx, lobj, (JSClass *) &location_class, NULL)) {
+			JS::SetReservedSlot(lobj, 0, JS::UndefinedValue());
+			vs->locobject = NULL;
+		}
+	}
+}
+
+
 /* "location" is special because we need to simulate "location.href"
  * when the code is asking directly for "location". We do not register
  * it as a "known" property since that was yielding strange bugs
@@ -143,6 +191,9 @@ JSPropertySpec window_props[] = {
 	JS_PSG("innerWidth",	window_get_property_innerWidth, JSPROP_ENUMERATE),
 	JS_PSGS("location",	window_get_property_location, window_set_property_location, JSPROP_ENUMERATE),
 	JS_PSG("parent",	window_get_property_parent, JSPROP_ENUMERATE),
+	JS_PSG("performance",	window_get_property_performance, JSPROP_ENUMERATE),
+	JS_PSG("scrollX",	window_get_property_scrollX, JSPROP_ENUMERATE),
+	JS_PSG("scrollY",	window_get_property_scrollY, JSPROP_ENUMERATE),
 	JS_PSG("self",	window_get_property_self, JSPROP_ENUMERATE),
 	JS_PSGS("status",	window_get_property_status, window_set_property_status, 0),
 	JS_PSG("top",	window_get_property_top, JSPROP_ENUMERATE),
@@ -178,25 +229,41 @@ find_child_frame(struct document_view *doc_view, struct frame_desc *tframe)
 static bool window_addEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_alert(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_clearInterval(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_clearRequestAnimationFrame(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_clearTimeout(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_getComputedStyle(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_open(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_postMessage(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_removeEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_requestAnimationFrame(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_scroll(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_scrollBy(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_scrollByLines(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_scrollByPages(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_scrollTo(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_setInterval(JSContext *ctx, unsigned int argc, JS::Value *rval);
 static bool window_setTimeout(JSContext *ctx, unsigned int argc, JS::Value *rval);
+static bool window_toString(JSContext *ctx, unsigned int argc, JS::Value *rval);
 
 const spidermonkeyFunctionSpec window_funcs[] = {
 	{ "addEventListener", window_addEventListener, 3 },
 	{ "alert",	window_alert,		1 },
 	{ "clearInterval",	window_clearInterval,	1 },
+	{ "clearRequestAnimationFrame",	window_clearRequestAnimationFrame,	1 },
 	{ "clearTimeout",	window_clearTimeout,	1 },
 	{ "getComputedStyle",	window_getComputedStyle,	2 },
 	{ "open",	window_open,		3 },
 	{ "postMessage",	window_postMessage,	3 },
 	{ "removeEventListener", window_removeEventListener, 3 },
+	{ "requestAnimationFrame",	window_requestAnimationFrame, 1 },
+	{ "scroll",	window_scroll, 2 },
+	{ "scrollBy",	window_scrollBy, 2 },
+	{ "scrollByLines",	window_scrollByLines, 1 },
+	{ "scrollByPages",	window_scrollByPages, 1 },
+	{ "scrollTo",	window_scrollTo, 2 },
 	{ "setInterval",	window_setInterval,	2 },
 	{ "setTimeout",	window_setTimeout,	2 },
+	{ "toString",	window_toString,	0 },
 	{ "XMLHttpRequest", xhr_constructor, 0 },
 	{ NULL }
 };
@@ -485,7 +552,7 @@ window_open(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	doc_view = vs->doc_view;
 	ses = doc_view->session;
 
-	if (get_opt_bool("ecmascript.block_window_opening", ses)) {
+	if (get_opt_bool("ecmascript.block_window_opening", NULL)) {
 #ifdef CONFIG_LEDS
 		set_led_value(ses->status.popup_led, 'P');
 #endif
@@ -576,6 +643,227 @@ end:
 	return true;
 }
 
+/* @window_funcs{"scroll"} */
+static bool
+window_scroll(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	return window_scrollTo(ctx, argc, rval);
+}
+
+/* @window_funcs{"scrollBy"} */
+static bool
+window_scrollBy(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+
+	if (!JS_InstanceOf(ctx, hobj, &window_class, &args)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	args.rval().setUndefined();
+
+	if (argc == 2) {
+		int dx = args[0].toInt32();
+		int dy = args[1].toInt32();
+		int sx = (dx + term->cell_width - 1) / term->cell_width;
+		int sy = (dy + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy);
+		horizontal_scroll(ses, doc_view, sx);
+
+		return true;
+	}
+
+	if (argc == 1) {
+		int top = 0;
+		int left = 0;
+		JS::RootedValue v(ctx);
+		JS::RootedObject v_obj(ctx, &args[0].toObject());
+
+		if (JS_GetProperty(ctx, v_obj, "top", &v)) {
+			top = v.toInt32();
+		}
+		if (JS_GetProperty(ctx, v_obj, "left", &v)) {
+			left = v.toInt32();
+		}
+		int sx = (left + term->cell_width - 1) / term->cell_width;
+		int sy = (top + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy);
+		horizontal_scroll(ses, doc_view, sx);
+	}
+	return true;
+}
+
+/* @window_funcs{"scrollByLines"} */
+static bool
+window_scrollByLines(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+
+	if (!JS_InstanceOf(ctx, hobj, &window_class, &args)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+
+	args.rval().setUndefined();
+
+	if (argc != 1)
+		return true;
+
+	int steps = args[0].toInt32();
+	vertical_scroll(ses, doc_view, steps);
+
+	return true;
+}
+
+/* @window_funcs{"scrollByPages"} */
+static bool
+window_scrollByPages(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+
+	if (!JS_InstanceOf(ctx, hobj, &window_class, &args)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+
+	args.rval().setUndefined();
+
+	if (argc != 1)
+		return true;
+
+	int steps = args[0].toInt32();
+	vertical_scroll(ses, doc_view, steps * doc_view->box.height);
+
+	return true;
+}
+
+/* @window_funcs{"scrollTo"} */
+static bool
+window_scrollTo(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+
+	if (!JS_InstanceOf(ctx, hobj, &window_class, &args)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	args.rval().setUndefined();
+
+	if (argc == 2) {
+		int x = args[0].toInt32();
+		int y = args[1].toInt32();
+		int sx = (x + term->cell_width - 1) / term->cell_width;
+		int sy = (y + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy - doc_view->vs->y);
+		horizontal_scroll(ses, doc_view, sx - doc_view->vs->x);
+
+		return true;
+	}
+
+	if (argc == 1) {
+		int top = 0;
+		int left = 0;
+		JS::RootedValue v(ctx);
+		JS::RootedObject v_obj(ctx, &args[0].toObject());
+
+		if (JS_GetProperty(ctx, v_obj, "top", &v)) {
+			top = v.toInt32();
+		}
+		if (JS_GetProperty(ctx, v_obj, "left", &v)) {
+			left = v.toInt32();
+		}
+		int sx = (left + term->cell_width - 1) / term->cell_width;
+		int sy = (top + term->cell_height - 1) / term->cell_height;
+
+		vertical_scroll(ses, doc_view, sy - doc_view->vs->y);
+		horizontal_scroll(ses, doc_view, sx - doc_view->vs->x);
+	}
+
+	return true;
+}
+
 /* @window_funcs{"setInterval"} */
 static bool
 window_setInterval(JSContext *ctx, unsigned int argc, JS::Value *rval)
@@ -623,6 +911,33 @@ window_setInterval(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	return true;
 }
 
+/* @window_funcs{"requestAnimationFrame"} */
+static bool
+window_requestAnimationFrame(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
+
+	if (argc != 1) {
+		return true;
+	}
+	int id = ecmascript_set_request2(ctx, args[0]);
+	args.rval().setInt32(id);
+
+	return true;
+}
+
+
 /* @window_funcs{"setTimeout"} */
 static bool
 window_setTimeout(JSContext *ctx, unsigned int argc, JS::Value *rval)
@@ -663,12 +978,12 @@ window_setTimeout(JSContext *ctx, unsigned int argc, JS::Value *rval)
 			return true;
 		}
 
-		struct ecmascript_timeout *id = ecmascript_set_timeout(ctx, code, timeout, -1);
+		struct ecmascript_timeout *id = ecmascript_set_timeout(ctx, code, timeout, 0);
 		JS::BigInt *bi = JS::NumberToBigInt(ctx, reinterpret_cast<int64_t>(id));
 		args.rval().setBigInt(bi);
 		return true;
 	}
-	struct ecmascript_timeout *id = ecmascript_set_timeout2(ctx, args[0], timeout, -1);
+	struct ecmascript_timeout *id = ecmascript_set_timeout2(ctx, args[0], timeout, 0);
 	JS::BigInt *bi = JS::NumberToBigInt(ctx, reinterpret_cast<int64_t>(id));
 	args.rval().setBigInt(bi);
 
@@ -692,18 +1007,51 @@ window_clearInterval(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
 
-	if (argc != 1) {
+	if (argc != 1 || args[0].isNull()) {
 		return true;
 	}
 	JS::BigInt *bi = JS::ToBigInt(ctx, args[0]);
 	int64_t number = JS::ToBigInt64(bi);
 	struct ecmascript_timeout *t = reinterpret_cast<struct ecmascript_timeout *>(number);
 
-	if (t && found_in_map_timer(t->tid)) {
-		kill_timer(&t->tid);
-		done_string(&t->code);
-		del_from_list(t);
-		mem_free(t);
+	if (found_in_map_timer(t)) {
+		t->timeout_next = -1;
+	}
+	return true;
+}
+
+
+/* @window_funcs{"clearRequestAnimationFrame"} */
+static bool
+window_clearRequestAnimationFrame(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
+
+	if (argc != 1 || args[0].isNull()) {
+		return true;
+	}
+	int id = args[0].toInt32();
+
+	if (!id) {
+		return true;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+
+	if (interpreter->request == id) {
+		interpreter->request = 0;
+		delete interpreter->request_func;
+		interpreter->request_func = NULL;
 	}
 	return true;
 }
@@ -726,18 +1074,15 @@ window_clearTimeout(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
 
-	if (argc != 1) {
+	if (argc != 1 || args[0].isNull()) {
 		return true;
 	}
 	JS::BigInt *bi = JS::ToBigInt(ctx, args[0]);
 	int64_t number = JS::ToBigInt64(bi);
 	struct ecmascript_timeout *t = reinterpret_cast<struct ecmascript_timeout *>(number);
 
-	if (t && found_in_map_timer(t->tid)) {
-		kill_timer(&t->tid);
-		done_string(&t->code);
-		del_from_list(t);
-		mem_free(t);
+	if (found_in_map_timer(t)) {
+		t->timeout_next = -1;
 	}
 	return true;
 }
@@ -954,9 +1299,16 @@ window_get_property_location(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+	struct el_window *elwin = JS::GetMaybePtrFromReservedSlot<struct el_window>(hobj, 0);
+
+	if (elwin && elwin->vs) {
+		JSObject *loc = getLocation(ctx, elwin->vs);
+		args.rval().setObject(*loc);
+		return true;
+	}
 
 	if (!interpreter->location_obj) {
-		interpreter->location_obj = getLocation(ctx);
+		interpreter->location_obj = getLocation(ctx, NULL);
 	}
 	args.rval().setObject(*(JSObject *)(interpreter->location_obj));
 	return true;
@@ -1010,8 +1362,6 @@ window_get_property_parent(JSContext *ctx, unsigned int argc, JS::Value *vp)
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
-	JS::CallArgs args = CallArgsFromVp(argc, vp);
-
 	/* XXX: It would be nice if the following worked, yes.
 	 * The problem is that we get called at the point where
 	 * document.frame properties are going to be mostly NULL.
@@ -1022,7 +1372,95 @@ window_get_property_parent(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	/* FIXME: So now we alias window.parent to window.top, which is
 	 * INCORRECT but works for the most common cases of just two
 	 * frames. Better something than nothing. */
-	args.rval().setUndefined();
+
+	return window_get_property_top(ctx, argc, vp);
+}
+
+static bool
+window_get_property_performance(JSContext *ctx, unsigned int argc, JS::Value *vp)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	JSObject *performance = getPerformance(ctx);
+	args.rval().setObject(*performance);
+	return true;
+}
+
+static bool
+window_get_property_scrollX(JSContext *ctx, unsigned int argc, JS::Value *vp)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+
+	if (!JS_InstanceOf(ctx, hobj, &window_class, &args)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	args.rval().setInt32(doc_view->vs->x * term->cell_width);
+
+	return true;
+}
+
+static bool
+window_get_property_scrollY(JSContext *ctx, unsigned int argc, JS::Value *vp)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
+	JS::RootedObject hobj(ctx, &args.thisv().toObject());
+	JS::Realm *comp = js::GetContextRealm(ctx);
+
+	if (!comp) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+
+	if (!JS_InstanceOf(ctx, hobj, &window_class, &args)) {
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
+#endif
+		return false;
+	}
+	struct view_state *vs = interpreter->vs;
+	struct document_view *doc_view = vs->doc_view;
+	struct session *ses = doc_view->session;
+	struct terminal *term = ses->tab->term;
+
+	args.rval().setInt32(doc_view->vs->y * term->cell_height);
 
 	return true;
 }
@@ -1151,3 +1589,50 @@ window_get_property_top(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	args.rval().setUndefined();
 	return true;
 }
+
+static bool
+window_toString(JSContext *ctx, unsigned int argc, JS::Value *rval)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JS::CallArgs args = JS::CallArgsFromVp(argc, rval);
+
+	args.rval().setString(JS_NewStringCopyZ(ctx, "Window object"));
+	return true;
+}
+
+JSObject *
+getWindow(JSContext *ctx, struct view_state *vs)
+{
+#ifdef ECMASCRIPT_DEBUG
+	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
+#endif
+	JSObject *el = JS_NewObject(ctx, &content_window_class);
+
+	if (!el) {
+		return NULL;
+	}
+	struct el_window *elwin = (struct el_window *)mem_calloc(1, sizeof(*elwin));
+
+	if (!elwin) {
+		return NULL;
+	}
+	JS::RootedObject r_el(ctx, el);
+	JS::Realm *comp = js::GetContextRealm(ctx);
+	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS::GetRealmPrivate(comp);
+	init_list(elwin->listeners);
+	elwin->interpreter = interpreter;
+	elwin->vs = vs;
+	elwin->thisval = r_el;
+
+	if (vs) {
+		vs->winobject = el;
+	}
+	JS::SetReservedSlot(el, 0, JS::PrivateValue(elwin));
+
+	JS_DefineProperties(ctx, r_el, (JSPropertySpec *) window_props);
+	spidermonkey_DefineFunctions(ctx, el, window_funcs);
+
+	return el;
+}
