@@ -46,6 +46,12 @@
 #define USE_LIBEVENT
 #endif
 
+#ifdef CONFIG_LIBUV
+#ifdef HAVE_UV_H
+#include <uv.h>
+#endif
+#endif
+
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
@@ -63,6 +69,8 @@
 #include "osdep/osdep.h"
 #include "osdep/signals.h"
 #include "session/download.h"
+#include "terminal/event.h"
+#include "terminal/kbd.h"
 #include "terminal/terminal.h"
 #include "util/error.h"
 #include "util/memcount.h"
@@ -78,14 +86,167 @@
 #define __STRING(x) #x
 #endif
 
-#if defined(CONFIG_LIBEVENT) && defined(CONFIG_LIBCURL)
-
-#ifdef CONFIG_OS_WIN32
+#undef SOCK_SHIFT
+#if defined(CONFIG_OS_WIN32)
 #define SOCK_SHIFT 1024
 #else
 #define SOCK_SHIFT 0
 #endif
 
+#if defined(CONFIG_LIBUV) && defined(CONFIG_LIBCURL)
+struct datauv g;
+
+#define mycase(code) \
+	case code: s = __STRING(code)
+
+/* Die if we get a bad CURLMcode somewhere */
+void
+mcode_or_die(const char *where, CURLMcode code)
+{
+	ELOG
+	if (CURLM_OK != code) {
+		const char *s;
+
+		switch(code) {
+		mycase(CURLM_BAD_HANDLE); break;
+		mycase(CURLM_BAD_EASY_HANDLE); break;
+		mycase(CURLM_OUT_OF_MEMORY); break;
+		mycase(CURLM_INTERNAL_ERROR); break;
+		mycase(CURLM_UNKNOWN_OPTION); break;
+		mycase(CURLM_LAST); break;
+		default: s = "CURLM_unknown"; break;
+		mycase(CURLM_BAD_SOCKET);
+			fprintf(stderr, "ERROR: %s returns %s\n", where, s);
+			/* ignore this error */
+			return;
+		}
+		fprintf(stderr, "ERROR: %s returns %s\n", where, s);
+		//exit(code);
+	}
+}
+
+static struct curl_context *
+create_curl_context(curl_socket_t sockfd, struct datauv *uv)
+{
+	ELOG
+	struct curl_context *context = (struct curl_context *)mem_alloc(sizeof(*context));
+	context->sockfd = sockfd;
+	context->uv = uv;
+
+	uv_poll_init_socket(uv->loop, &context->poll_handle, sockfd);
+	context->poll_handle.data = context;
+
+	return context;
+}
+
+static void
+curl_close_cb(uv_handle_t *handle)
+{
+	ELOG
+	struct curl_context *context = (struct curl_context *) handle->data;
+	mem_free(context);
+}
+
+static void
+destroy_curl_context(struct curl_context *context)
+{
+	ELOG
+	uv_close((uv_handle_t *) &context->poll_handle, curl_close_cb);
+}
+
+/* callback from libuv on socket activity */
+static void
+on_uv_socket(uv_poll_t *req, int status, int events)
+{
+	ELOG
+	int running_handles;
+	int flags = 0;
+	struct curl_context *context = (struct curl_context *) req->data;
+	(void)status;
+
+	if (events & UV_READABLE) {
+		flags |= CURL_CSELECT_IN;
+	}
+
+	if (events & UV_WRITABLE) {
+		flags |= CURL_CSELECT_OUT;
+	}
+	curl_multi_socket_action(context->uv->multi, context->sockfd, flags, &running_handles);
+	check_multi_info(&g);
+}
+
+/* callback from libuv when timeout expires */
+static void
+on_uv_timeout(uv_timer_t *req)
+{
+	ELOG
+
+	if (1) {
+		int running_handles;
+		curl_multi_socket_action(g.multi, CURL_SOCKET_TIMEOUT, 0, &running_handles);
+	}
+	check_multi_info(&g);
+}
+
+/* callback from libcurl to update the timeout expiry */
+static int
+cb_timeout(CURLM *multi, long timeout_ms, struct datauv *uv)
+{
+	ELOG
+	(void)multi;
+
+	if (timeout_ms < 0) {
+		uv_timer_stop(&uv->timeout);
+	} else {
+		if (timeout_ms == 0) {
+			timeout_ms = 1; /* 0 means call curl_multi_socket_action asap but NOT within the callback itself */
+		}
+		uv_timer_start(&uv->timeout, on_uv_timeout, (uint64_t)timeout_ms, 0); /* do not repeat */
+	}
+	return 0;
+}
+
+/* callback from libcurl to update socket activity to wait for */
+static int
+cb_socket(CURL *easy, curl_socket_t s, int action, struct datauv *uv, void *socketp)
+{
+	ELOG
+	struct curl_context *curl_context;
+	int events = 0;
+	(void)easy;
+
+	switch(action) {
+	case CURL_POLL_IN:
+	case CURL_POLL_OUT:
+	case CURL_POLL_INOUT:
+		curl_context = socketp ? (struct curl_context *) socketp : create_curl_context(s, uv);
+		curl_multi_assign(uv->multi, s, (void *) curl_context);
+
+		if (action != CURL_POLL_IN) {
+			events |= UV_WRITABLE;
+		}
+
+		if (action != CURL_POLL_OUT) {
+			events |= UV_READABLE;
+		}
+		uv_poll_start(&curl_context->poll_handle, events, on_uv_socket);
+		break;
+	case CURL_POLL_REMOVE:
+		if (socketp) {
+			uv_poll_stop(&((struct curl_context*)socketp)->poll_handle);
+			destroy_curl_context((struct curl_context*) socketp);
+			curl_multi_assign(uv->multi, s, NULL);
+		}
+		break;
+	default:
+		fprintf(stderr, "Something went bad");
+	}
+	return 0;
+}
+#endif
+
+#if defined(CONFIG_LIBEVENT) && defined(CONFIG_LIBCURL)
+
 /* Information associated with a specific easy handle */
 typedef struct _ConnInfo
 {
@@ -115,6 +276,7 @@ GlobalInfo g;
 void
 mcode_or_die(const char *where, CURLMcode code)
 {
+	ELOG
 	if (CURLM_OK != code) {
 		const char *s;
 
@@ -140,6 +302,7 @@ mcode_or_die(const char *where, CURLMcode code)
 static int
 multi_timer_cb(CURLM *multi, long timeout_ms, GlobalInfo *g)
 {
+	ELOG
 	struct timeval timeout;
 	(void)multi;
 
@@ -167,6 +330,7 @@ multi_timer_cb(CURLM *multi, long timeout_ms, GlobalInfo *g)
 static void
 event_cb(int fd, short kind, void *userp)
 {
+	ELOG
 	GlobalInfo *g = (GlobalInfo*) userp;
 	CURLMcode rc;
 
@@ -190,6 +354,7 @@ event_cb(int fd, short kind, void *userp)
 static void
 timer_cb(int fd, short kind, void *userp)
 {
+	ELOG
 	GlobalInfo *g = (GlobalInfo *)userp;
 	CURLMcode rc;
 	(void)fd;
@@ -204,6 +369,7 @@ timer_cb(int fd, short kind, void *userp)
 static void
 remsock(SockInfo *f)
 {
+	ELOG
 	//fprintf(stderr, "remsock f=%p\n", f);
 
 	if (f) {
@@ -218,6 +384,7 @@ remsock(SockInfo *f)
 static void
 setsock(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 {
+	ELOG
 	int kind = ((act & CURL_POLL_IN) ? EV_READ : 0) | ((act & CURL_POLL_OUT) ? EV_WRITE : 0) | EV_PERSIST;
 
 	f->sockfd = s;
@@ -238,6 +405,7 @@ setsock(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 static void
 addsock(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
 {
+	ELOG
 	//fprintf(stderr, "addsock easy=%p\n", easy);
 
 	SockInfo *fdp = mem_calloc(1, sizeof(SockInfo));
@@ -250,6 +418,7 @@ addsock(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
 static int
 sock_cb(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 {
+	ELOG
 	GlobalInfo *g = (GlobalInfo*) cbp;
 	SockInfo *fdp = (SockInfo*) sockp;
 //	const char *whatstr[]={ "none", "IN", "OUT", "INOUT", "REMOVE" };
@@ -277,6 +446,7 @@ sock_cb(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 /* Information associated with a specific easy handle */
 typedef struct _ConnInfo
 {
+	ELOG
 	CURL *easy;
 	char *url;
 	GlobalInfo *global;
@@ -286,6 +456,7 @@ typedef struct _ConnInfo
 /* Information associated with a specific socket */
 typedef struct _SockInfo
 {
+	ELOG
 	curl_socket_t sockfd;
 	CURL *easy;
 	int action;
@@ -303,6 +474,7 @@ static void timer_cb(EV_P_ struct ev_timer *w, int revents);
 static int
 multi_timer_cb(CURLM *multi, long timeout_ms, GlobalInfo *g)
 {
+	ELOG
 	//DPRINT("%s %li\n", __PRETTY_FUNCTION__,  timeout_ms);
 	ev_timer_stop(g->loop, &g->timer_event);
 
@@ -321,6 +493,7 @@ multi_timer_cb(CURLM *multi, long timeout_ms, GlobalInfo *g)
 void
 mcode_or_die(const char *where, CURLMcode code)
 {
+	ELOG
 	if (CURLM_OK != code) {
 		const char *s;
 		switch(code) {
@@ -360,6 +533,7 @@ mcode_or_die(const char *where, CURLMcode code)
 static void
 event_cb(EV_P_ struct ev_io *w, int revents)
 {
+	ELOG
 	//DPRINT("%s  w %p revents %i\n", __PRETTY_FUNCTION__, w, revents);
 	GlobalInfo *g = (GlobalInfo*) w->data;
 	CURLMcode rc;
@@ -381,6 +555,7 @@ event_cb(EV_P_ struct ev_io *w, int revents)
 static void
 timer_cb(EV_P_ struct ev_timer *w, int revents)
 {
+	ELOG
 	//DPRINT("%s  w %p revents %i\n", __PRETTY_FUNCTION__, w, revents);
 	GlobalInfo *g = (GlobalInfo *)w->data;
 	CURLMcode rc;
@@ -394,6 +569,7 @@ timer_cb(EV_P_ struct ev_timer *w, int revents)
 static void
 remsock(SockInfo *f, GlobalInfo *g)
 {
+	ELOG
 	//printf("%s  \n", __PRETTY_FUNCTION__);
 
 	if (f) {
@@ -408,6 +584,7 @@ remsock(SockInfo *f, GlobalInfo *g)
 static void
 setsock(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 {
+	ELOG
 	//printf("%s  \n", __PRETTY_FUNCTION__);
 
 	int kind = ((act & CURL_POLL_IN) ? EV_READ : 0) | ((act & CURL_POLL_OUT) ? EV_WRITE : 0);
@@ -429,6 +606,7 @@ setsock(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 static void
 addsock(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
 {
+	ELOG
 	SockInfo *fdp = calloc(1, sizeof(SockInfo));
 
 	fdp->global = g;
@@ -440,6 +618,7 @@ addsock(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
 static int
 sock_cb(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 {
+	ELOG
 	//DPRINT("%s e %p s %i what %i cbp %p sockp %p\n", __PRETTY_FUNCTION__, e, s, what, cbp, sockp);
 	GlobalInfo *g = (GlobalInfo*) cbp;
 	SockInfo *fdp = (SockInfo*) sockp;
@@ -464,16 +643,12 @@ sock_cb(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 }
 #endif
 
-#if !defined(CONFIG_LIBEVENT) && !defined(CONFIG_LIBEV) && defined(CONFIG_LIBCURL)
+#if !defined(CONFIG_LIBEVENT) && !defined(CONFIG_LIBEV) && !defined(CONFIG_LIBUV) && defined(CONFIG_LIBCURL)
 
-#ifdef CONFIG_OS_WIN32
-#define SOCK_SHIFT 1024
-#else
-#define SOCK_SHIFT 0
-#endif
 /* Information associated with a specific easy handle */
 typedef struct _ConnInfo
 {
+	ELOG
 	CURL *easy;
 	char *url;
 	GlobalInfo *global;
@@ -483,6 +658,7 @@ typedef struct _ConnInfo
 /* Information associated with a specific socket */
 typedef struct _SockInfo
 {
+	ELOG
 	curl_socket_t sockfd;
 	CURL *easy;
 	int action;
@@ -499,6 +675,7 @@ GlobalInfo g;
 void
 mcode_or_die(const char *where, CURLMcode code)
 {
+	ELOG
 	if (CURLM_OK != code) {
 		const char *s;
 
@@ -522,11 +699,13 @@ mcode_or_die(const char *where, CURLMcode code)
 
 #endif
 
-#ifdef CONFIG_LIBCURL
+#if defined(CONFIG_LIBCURL) && !defined(CONFIG_LIBEV) && !defined(CONFIG_LIBEVENT) && !defined(CONFIG_LIBUV)
+
 /* Called by libevent when our timeout expires */
 static void
 timer_cb_select(void *userp)
 {
+	ELOG
 	GlobalInfo *g = (GlobalInfo *)userp;
 	CURLMcode rc;
 
@@ -539,6 +718,7 @@ timer_cb_select(void *userp)
 static int
 multi_timer_cb_select(CURLM *multi, long timeout_ms, GlobalInfo *g)
 {
+	ELOG
 	(void)multi;
 	//fprintf(stderr, "multi_timer_cb: Setting timeout to %ld ms\n", timeout_ms);
 
@@ -559,6 +739,7 @@ multi_timer_cb_select(CURLM *multi, long timeout_ms, GlobalInfo *g)
 static void
 event_read_cb_select(void *userp)
 {
+	ELOG
 	SockInfo *f = (SockInfo *)userp;
 	GlobalInfo *g = (GlobalInfo*)f->global;
 	CURLMcode rc;
@@ -573,6 +754,7 @@ event_read_cb_select(void *userp)
 static void
 event_write_cb_select(void *userp)
 {
+	ELOG
 	SockInfo *f = (SockInfo *)userp;
 	GlobalInfo *g = (GlobalInfo*)f->global;
 	CURLMcode rc;
@@ -588,10 +770,11 @@ event_write_cb_select(void *userp)
 static void
 remsock_select(SockInfo *f)
 {
+	ELOG
 	//fprintf(stderr, "remsock f=%p\n", f);
 	if (f) {
 		if (f->sockfd) {
-			set_handlers(f->sockfd + SOCK_SHIFT, NULL, NULL, NULL, NULL);
+			set_handlers(f->sockfd + SOCK_SHIFT, NULL, NULL, NULL, NULL, EL_TYPE_TCP);
 		}
 		mem_free(f);
 	}
@@ -601,6 +784,7 @@ remsock_select(SockInfo *f)
 static void
 setsock_select(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 {
+	ELOG
 	int in = act & CURL_POLL_IN;
 	int out = act & CURL_POLL_OUT;
 
@@ -608,13 +792,14 @@ setsock_select(SockInfo *f, curl_socket_t s, CURL *e, int act, GlobalInfo *g)
 	f->action = act;
 	f->easy = e;
 
-	set_handlers(s + SOCK_SHIFT, in ? event_read_cb_select : NULL, out ? event_write_cb_select : NULL, NULL, f);
+	set_handlers(s + SOCK_SHIFT, in ? event_read_cb_select : NULL, out ? event_write_cb_select : NULL, NULL, f, EL_TYPE_TCP);
 }
 
 /* Initialize a new SockInfo structure */
 static void
 addsock_select(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
 {
+	ELOG
 	//fprintf(stderr, "addsock easy=%p\n", easy);
 
 	SockInfo *fdp = mem_calloc(1, sizeof(SockInfo));
@@ -627,6 +812,7 @@ addsock_select(curl_socket_t s, CURL *easy, int action, GlobalInfo *g)
 static int
 sock_cb_select(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 {
+	ELOG
 	GlobalInfo *g = (GlobalInfo*) cbp;
 	SockInfo *fdp = (SockInfo*) sockp;
 	//const char *whatstr[]={ "none", "IN", "OUT", "INOUT", "REMOVE" };
@@ -654,12 +840,30 @@ sock_cb_select(CURL *e, curl_socket_t s, int what, void *cbp, void *sockp)
 const char *
 get_libevent_version(void)
 {
+	ELOG
 	return event_get_version();
 }
 #else
 const char *
 get_libevent_version(void)
 {
+	ELOG
+	return "";
+}
+#endif
+
+#ifdef CONFIG_LIBUV
+const char *
+get_libuv_version(void)
+{
+	ELOG
+	return uv_version_string();
+}
+#else
+const char *
+get_libuv_version(void)
+{
+	ELOG
 	return "";
 }
 #endif
@@ -670,24 +874,13 @@ get_libevent_version(void)
 
 static int n_threads = 0;
 
-struct thread {
-	select_handler_T read_func;
-	select_handler_T write_func;
-	select_handler_T error_func;
-	void *data;
-#ifdef USE_LIBEVENT
-	struct event *read_event;
-	struct event *write_event;
-#endif
-};
-
 #ifdef CONFIG_OS_WIN32
 /* CreatePipe produces big numbers for handles */
 #undef FD_SETSIZE
 #define FD_SETSIZE 4096
 #endif
 
-static struct thread *threads = NULL;
+struct thread *threads = NULL;
 
 static fd_set w_read;
 static fd_set w_write;
@@ -702,6 +895,7 @@ static int w_max;
 int
 get_file_handles_count(void)
 {
+	ELOG
 	int i = 0, j;
 
 	for (j = 0; j < w_max; j++)
@@ -724,6 +918,7 @@ static INIT_LIST_OF(struct bottom_half, bottom_halves);
 int
 register_bottom_half_do(select_handler_T fn, void *data)
 {
+	ELOG
 	struct bottom_half *bh;
 
 	foreach (bh, bottom_halves)
@@ -742,6 +937,7 @@ register_bottom_half_do(select_handler_T fn, void *data)
 void
 check_bottom_halves(void)
 {
+	ELOG
 	while (!list_empty(bottom_halves)) {
 		struct bottom_half *bh = (struct bottom_half *)bottom_halves.prev;
 		select_handler_T fn = bh->fn;
@@ -753,13 +949,244 @@ check_bottom_halves(void)
 	}
 }
 
-#ifdef USE_LIBEVENT
+#ifdef CONFIG_LIBUV
+int event_enabled = 0;
 
+static void
+one_cb(uv_poll_t *handle, int status, int events)
+{
+	ELOG
+
+	if (!handle) {
+		return;
+	}
+
+	struct libuv_priv *priv = (struct libuv_priv *)uv_handle_get_data((uv_handle_t *)handle);
+
+	if (!priv) {
+		return;
+	}
+	int fd = priv->fd;
+
+	if (events & UV_READABLE) {
+		select_handler_T hr = get_handler(fd, SELECT_HANDLER_READ);
+
+		if (hr) {
+			hr(get_handler_data(fd));
+		}
+	}
+
+	if (events & UV_WRITABLE) {
+		select_handler_T hw = get_handler(fd, SELECT_HANDLER_WRITE);
+
+		if (hw) {
+			hw(get_handler_data(fd));
+		}
+	}
+	check_bottom_halves();
+}
+
+static void
+alloc_one_cb(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf)
+{
+	ELOG
+	buf->base = mem_alloc(suggested_size);
+	buf->len = suggested_size;
+}
+
+static void
+read_one_cb(uv_stream_t *stream, ssize_t r, const uv_buf_t *buf)
+{
+	ELOG
+	mem_free_if(buf->base);
+
+	if (!stream) {
+		return;
+	}
+	struct libuv_priv *priv = (struct libuv_priv *)uv_handle_get_data((uv_handle_t *)stream);
+
+	if (!priv) {
+		return;
+	}
+	int fd = priv->fd;
+
+	if (r >= 0) {
+		select_handler_T hr = get_handler(fd, SELECT_HANDLER_READ);
+
+		if (hr) {
+			hr(get_handler_data(fd));
+		}
+	}
+	check_bottom_halves();
+}
+
+static void
+set_events_for_handle(int h)
+{
+	ELOG
+	struct libuv_priv *priv = NULL;
+	uv_handle_t *handle = NULL;
+	int res;
+	enum el_type_hint type_hint;
+
+	if (h == 0) {
+		type_hint = EL_TYPE_TTY;
+	} else {
+		type_hint = threads[h].type_hint;
+	}
+
+	if (type_hint == EL_TYPE_NONE) {
+		return;
+	}
+
+	if (!threads[h].read_func && !threads[h].write_func) {
+		if (!threads[h].handle) {
+			return;
+		}
+		handle = threads[h].handle;
+
+		if (type_hint == EL_TYPE_TTY || type_hint == EL_TYPE_PIPE) {
+			uv_read_stop((uv_stream_t *)handle);
+		} else {
+			if (type_hint != EL_TYPE_NONE) {
+				uv_poll_stop((uv_poll_t *)handle);
+			}
+		}
+		mem_free_set(&threads[h].handle, NULL);
+		return;
+	}
+	if (!threads[h].handle) {
+		priv = mem_calloc(1, sizeof(*priv));
+
+		if (!priv) {
+			return;
+		}
+		switch (type_hint) {
+		case EL_TYPE_TCP:
+		case EL_TYPE_UDP:
+			handle = mem_calloc(1, sizeof(uv_poll_t));
+			if (!handle) {
+				mem_free(priv);
+				return;
+			}
+			res = uv_poll_init_socket(uv_default_loop(), (uv_poll_t *)handle, h - SOCK_SHIFT);
+
+			if (res) {
+				fprintf(stderr, "Something went bad: res=%d h=%d %s:%d\n", res, h, __FILE__, __LINE__);
+			}
+
+			break;
+		case EL_TYPE_TTY:
+			handle = mem_calloc(1, sizeof(uv_tty_t));
+			if (!handle) {
+				mem_free(priv);
+				return;
+			}
+			res = uv_tty_init(uv_default_loop(), (uv_tty_t *)handle, h, 0);
+
+			if (res) {
+				fprintf(stderr, "Something went bad: res=%d %s:%d\n", res, __FILE__, __LINE__);
+			} else {
+				uv_tty_set_mode((uv_tty_t *)handle, UV_TTY_MODE_RAW);
+			}
+			break;
+		case EL_TYPE_PIPE:
+			handle = mem_calloc(1, sizeof(uv_pipe_t));
+			if (!handle) {
+				mem_free(priv);
+				return;
+			}
+			res = uv_pipe_init(uv_default_loop(), (uv_pipe_t *)handle, 0);
+
+			if (res) {
+				fprintf(stderr, "Something went bad: res=%d %s:%d\n", res, __FILE__, __LINE__);
+			}
+			res = uv_pipe_open((uv_pipe_t *)handle, h);
+
+			if (res) {
+				fprintf(stderr, "Something went bad: res=%d %s:%d\n", res, __FILE__, __LINE__);
+			}
+			break;
+		default:
+			handle = mem_calloc(1, sizeof(uv_poll_t));
+			if (!handle) {
+				mem_free(priv);
+				return;
+			}
+			res = uv_poll_init(uv_default_loop(), (uv_poll_t *)handle, h);
+
+			if (res) {
+				fprintf(stderr, "Something went bad: res=%d %s:%d\n", res, __FILE__, __LINE__);
+			}
+			break;
+		}
+		threads[h].handle = handle;
+	} else {
+		handle = threads[h].handle;
+		priv = (struct libuv_priv *)uv_handle_get_data((uv_handle_t *)handle);
+	}
+	priv->fd = h;
+	uv_handle_set_data((uv_handle_t *)handle, priv);
+
+	if (threads[h].read_func) {
+		if (type_hint == EL_TYPE_TTY || type_hint == EL_TYPE_PIPE) {
+			if (threads[h].read_func == (select_handler_T)in_kbd) {
+				uv_read_start((uv_stream_t *)handle, alloc_kbd_cb, read_kbd_cb);
+			} else if (threads[h].read_func == (select_handler_T)in_term) {
+				uv_read_start((uv_stream_t *)handle, alloc_interm_cb, read_interm_cb);
+			} else {
+				uv_read_start((uv_stream_t *)handle, alloc_one_cb, read_one_cb);
+			}
+		} else {
+			uv_poll_start((uv_poll_t *)handle, UV_READABLE, one_cb);
+		}
+	}
+
+	if (threads[h].write_func) {
+		uv_poll_start((uv_poll_t *)handle, UV_WRITABLE, one_cb);
+	}
+}
+
+static void
+enable_libevent(void)
+{
+	ELOG
+	int i;
+
+	if (get_cmd_opt_bool("no-libevent"))
+		return;
+
+#ifdef CONFIG_MEMCOUNT
+	uv_replace_allocator(el_libuv_malloc, el_libuv_realloc, el_libuv_calloc, el_libuv_free);
+#endif
+
+	event_enabled = 1;
+
+	for (i = 0; i < w_max; i++) {
+		set_events_for_handle(i);
+	}
+	set_events_for_timer();
+}
+
+#undef EVLOOP_ONCE
+#undef EVLOOP_NONBLOCK
+#define EVLOOP_ONCE UV_RUN_ONCE
+#define EVLOOP_NONBLOCK UV_RUN_NOWAIT
+
+static void
+do_event_loop(int flags)
+{
+	ELOG
+	uv_run(uv_default_loop(), flags);
+}
+#endif
+
+#if defined(USE_LIBEVENT) || defined(CONFIG_LIBUV)
 #if defined(USE_POLL)
-
 static void
 restrict_fds(void)
 {
+	ELOG
 #if defined(RLIMIT_OFILE) && !defined(RLIMIT_NOFILE)
 #define RLIMIT_NOFILE RLIMIT_OFILE
 #endif
@@ -777,6 +1204,9 @@ skip_limit:;
 #endif
 }
 #endif /* USE_POLL */
+#endif /* USE_LIBEVENT || CONFIG_LIBUV */
+
+#ifdef USE_LIBEVENT
 
 int event_enabled = 0;
 
@@ -793,6 +1223,7 @@ struct event_base *event_base;
 static void
 event_callback(int h, short ev, void *data)
 {
+	ELOG
 #ifndef EV_PERSIST
 	if (event_add((struct event *)data, NULL) == -1)
 		elinks_internal("ERROR: event_add failed: %s", strerror(errno));
@@ -820,6 +1251,7 @@ event_callback(int h, short ev, void *data)
 static void
 set_event_for_action(int h, void (*func)(void *), struct event **evptr, short evtype)
 {
+	ELOG
 	if (func) {
 		if (!*evptr) {
 #ifdef EV_PERSIST
@@ -845,6 +1277,7 @@ set_event_for_action(int h, void (*func)(void *), struct event **evptr, short ev
 static void
 set_events_for_handle(int h)
 {
+	ELOG
 	set_event_for_action(h, threads[h].read_func, &threads[h].read_event, EV_READ);
 	set_event_for_action(h, threads[h].write_func, &threads[h].write_event, EV_WRITE);
 }
@@ -852,6 +1285,7 @@ set_events_for_handle(int h)
 static void
 enable_libevent(void)
 {
+	ELOG
 	int i;
 
 	if (get_cmd_opt_bool("no-libevent"))
@@ -863,10 +1297,11 @@ enable_libevent(void)
 		return;
 #endif
 
-#if !defined(EVENT__DISABLE_MM_REPLACEMENT) && defined(CONFIG_DEBUG)
+#if !defined(EVENT__DISABLE_MM_REPLACEMENT) && defined(CONFIG_MEMCOUNT)
 	event_set_mem_functions(el_libevent_malloc, el_libevent_realloc, el_libevent_free);
 #endif
 
+
 #if defined(HAVE_EVENT_CONFIG_SET_FLAG)
 	{
 		struct event_config *cfg;
@@ -909,6 +1344,7 @@ enable_libevent(void)
 static void
 terminate_libevent(void)
 {
+	ELOG
 	int i;
 	if (event_enabled) {
 		for (i = 0; i < n_threads; i++) {
@@ -929,6 +1365,7 @@ terminate_libevent(void)
 static void
 do_event_loop(int flags)
 {
+	ELOG
 	int e;
 #ifdef HAVE_EVENT_BASE_SET
 	e = event_base_loop(event_base, flags);
@@ -944,6 +1381,7 @@ do_event_loop(int flags)
 select_handler_T
 get_handler(int fd, enum select_handler_type tp)
 {
+	ELOG
 	if (fd >= w_max || fd < 0) {
 		return NULL;
 	}
@@ -961,6 +1399,7 @@ get_handler(int fd, enum select_handler_type tp)
 void *
 get_handler_data(int fd)
 {
+	ELOG
 	if (fd >= w_max || fd < 0) {
 		return NULL;
 	}
@@ -970,8 +1409,9 @@ get_handler_data(int fd)
 
 void
 set_handlers(int fd, select_handler_T read_func, select_handler_T write_func,
-	     select_handler_T error_func, void *data)
+	     select_handler_T error_func, void *data, enum el_type_hint type_hint)
 {
+	ELOG
 	if (fd < 0) {
 		return;
 	}
@@ -997,7 +1437,7 @@ set_handlers(int fd, select_handler_T read_func, select_handler_T write_func,
 	}
 #endif /* __GNU__ */
 
-#if defined(USE_POLL) && defined(USE_LIBEVENT)
+#if defined(USE_POLL) && (defined(USE_LIBEVENT) || defined(CONFIG_LIBUV))
 	if (!event_enabled)
 #endif
 		if (fd >= (int)FD_SETSIZE) {
@@ -1038,7 +1478,12 @@ set_handlers(int fd, select_handler_T read_func, select_handler_T write_func,
 		w_max = i + 1;
 	}
 
-#ifdef USE_LIBEVENT
+#if defined(USE_LIBEVENT) || defined(CONFIG_LIBUV)
+#ifdef CONFIG_LIBUV
+	if (read_func || write_func) {
+		threads[fd].type_hint = type_hint;
+	}
+#endif
 	if (event_enabled) {
 		set_events_for_handle(fd);
 		return;
@@ -1072,6 +1517,7 @@ static int was_installed_timer = 0;
 static void
 periodic_redraw_all_terminals(void *data)
 {
+	ELOG
 	redraw_all_terminals();
 	was_installed_timer = 0;
 }
@@ -1079,6 +1525,7 @@ periodic_redraw_all_terminals(void *data)
 static void
 try_redraw_all_terminals(void)
 {
+	ELOG
 	if (was_installed_timer) {
 		return;
 	}
@@ -1094,6 +1541,14 @@ try_redraw_all_terminals(void)
 static void
 clear_events(int h, int blocking)
 {
+	ELOG
+#ifdef HAVE_SYS_EVENTFD_H
+	if (1) {
+		int rd;
+		uint64_t t;
+		EINTRLOOP(rd, (int)read(h, &t, sizeof(t)));
+	}
+#else
 #if !defined(O_NONBLOCK) && !defined(FIONBIO)
 	blocking = 1;
 #endif
@@ -1103,22 +1558,28 @@ clear_events(int h, int blocking)
 		EINTRLOOP(rd, (int)read(h, c, sizeof c));
 		if (rd != sizeof c) break;
 	}
+#endif
 }
 
 pid_t signal_pid;
+#if defined(HAVE_SYS_EVENTFD_H)
+int signal_efd = -1;
+#else
 int signal_pipe[2] = { -1, -1 };
+#endif
 
 static void
 clear_events_ptr(void *handle)
 {
+	ELOG
 	clear_events((int)(intptr_t)handle, 0);
 }
-
 #endif
 
 void
 select_loop(void (*init)(void))
 {
+	ELOG
 	timeval_T last_time;
 	int select_errors = 0;
 
@@ -1134,16 +1595,23 @@ select_loop(void (*init)(void))
 #if !defined(NO_SIGNAL_HANDLERS)
 	signal_pid = getpid();
 
+#ifdef HAVE_SYS_EVENTFD_H
+	if ((signal_efd = eventfd(0, EFD_NONBLOCK)) < 0) {
+		elinks_internal("ERROR: can't create eventfd for signal handling");
+	}
+	set_handlers(signal_efd, clear_events_ptr, NULL, NULL, (void *)(intptr_t)signal_efd, EL_TYPE_FD);
+#else
 	if (c_pipe(signal_pipe)) {
 		elinks_internal("ERROR: can't create pipe for signal handling");
 	}
 	set_nonblocking_fd(signal_pipe[0]);
 	set_nonblocking_fd(signal_pipe[1]);
-	set_handlers(signal_pipe[0], clear_events_ptr, NULL, NULL, (void *)(intptr_t)signal_pipe[0]);
+	set_handlers(signal_pipe[0], clear_events_ptr, NULL, NULL, (void *)(intptr_t)signal_pipe[0], EL_TYPE_FD);
+#endif
 #endif
 	init();
 
-#ifdef USE_LIBEVENT
+#if defined(USE_LIBEVENT) || defined(CONFIG_LIBUV)
 	enable_libevent();
 #if defined(USE_POLL)
 	if (!event_enabled) {
@@ -1151,12 +1619,12 @@ select_loop(void (*init)(void))
 	}
 #endif
 #endif
-#ifdef USE_LIBEVENT
+#if defined(USE_LIBEVENT) || defined(CONFIG_LIBUV)
 	if (event_enabled) {
 #if defined(CONFIG_LIBCURL) && defined(CONFIG_LIBEVENT)
 		memset(&g, 0, sizeof(GlobalInfo));
 		g.evbase = event_base;
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_MEMCOUNT
 		curl_global_init_mem(CURL_GLOBAL_DEFAULT, el_curl_malloc, el_curl_free, el_curl_realloc, el_curl_strdup, el_curl_calloc);
 #else
 		curl_global_init(CURL_GLOBAL_DEFAULT);
@@ -1179,7 +1647,7 @@ select_loop(void (*init)(void))
 #if defined(CONFIG_LIBCURL) && defined(CONFIG_LIBEV)
 		memset(&g, 0, sizeof(GlobalInfo));
 		g.loop = ev_default_loop(0);
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_MEMCOUNT
 		curl_global_init_mem(CURL_GLOBAL_DEFAULT, el_curl_malloc, el_curl_free, el_curl_realloc, el_curl_strdup, el_curl_calloc);
 #else
 		curl_global_init(CURL_GLOBAL_DEFAULT);
@@ -1198,6 +1666,26 @@ select_loop(void (*init)(void))
 
 		/* we do not call any curl_multi_socket*() function yet as we have no handles added! */
 #endif
+
+#if defined(CONFIG_LIBCURL) && defined(CONFIG_LIBUV)
+		memset(&g, 0, sizeof(struct datauv));
+		g.loop = uv_default_loop();
+		uv_timer_init(g.loop, &g.timeout);
+#ifdef CONFIG_MEMCOUNT
+		curl_global_init_mem(CURL_GLOBAL_DEFAULT, el_curl_malloc, el_curl_free, el_curl_realloc, el_curl_strdup, el_curl_calloc);
+#else
+		curl_global_init(CURL_GLOBAL_DEFAULT);
+#endif
+		g.multi = curl_multi_init();
+
+		/* setup the generic multi interface options we want */
+		curl_multi_setopt(g.multi, CURLMOPT_SOCKETFUNCTION, cb_socket);
+		curl_multi_setopt(g.multi, CURLMOPT_SOCKETDATA, &g);
+		curl_multi_setopt(g.multi, CURLMOPT_TIMERFUNCTION, cb_timeout);
+		curl_multi_setopt(g.multi, CURLMOPT_TIMERDATA, &g);
+
+		/* we do not call any curl_multi_socket*() function yet as we have no handles added! */
+#endif
 		check_bottom_halves();
 		while (!program.terminate) {
 			check_signals();
@@ -1205,8 +1693,11 @@ select_loop(void (*init)(void))
 				do_event_loop(EVLOOP_NONBLOCK);
 				check_signals();
 				try_redraw_all_terminals();
+				check_bottom_halves();
+			}
+			if (program.terminate) {
+				break;
 			}
-			if (program.terminate) break;
 			do_event_loop(EVLOOP_ONCE);
 		}
 		if (was_installed_timer) {
@@ -1225,14 +1716,25 @@ select_loop(void (*init)(void))
 		//event_base_free(g.evbase);
 		curl_multi_cleanup(g.multi);
 		curl_global_cleanup();
+#endif
+
+#if defined(CONFIG_LIBCURL) && defined(CONFIG_LIBUV)
+		uv_timer_stop(&g.timeout);
+		curl_multi_cleanup(g.multi);
+		curl_global_cleanup();
+#endif
+#ifdef CONFIG_LIBUV
+#ifdef CONFIG_OS_WIN32
+		uv_tty_reset_mode();
+#endif
 #endif
 		return;
 	} else
 #endif
 	{
-#if defined(CONFIG_LIBCURL)
+#if defined(CONFIG_LIBCURL) && !defined(CONFIG_LIBUV) && !defined(CONFIG_LIBEV) && !defined(CONFIG_LIBEVENT)
 		memset(&g, 0, sizeof(GlobalInfo));
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_MEMCOUNT
 		curl_global_init_mem(CURL_GLOBAL_DEFAULT, el_curl_malloc, el_curl_free, el_curl_realloc, el_curl_strdup, el_curl_calloc);
 #else
 		curl_global_init(CURL_GLOBAL_DEFAULT);
@@ -1249,8 +1751,12 @@ select_loop(void (*init)(void))
 		check_bottom_halves();
 
 	while (!program.terminate) {
+#ifdef CONFIG_OS_DOS
+		timeval_T *timeout_ptr = NULL;
+#else
 		struct timeval timeout = { 0, 0 };
 		struct timeval *timeout_ptr = NULL;
+#endif
 		int n, i, has_timer;
 		timeval_T t;
 
@@ -1290,9 +1796,13 @@ select_loop(void (*init)(void))
 		if (has_timer) {
 			/* Be sure timeout is not negative. */
 			timeval_limit_to_zero_or_one(&t);
+#ifdef CONFIG_OS_DOS
+			timeout_ptr = &t;
+#else
 			timeout.tv_sec = t.sec;
 			timeout.tv_usec = t.usec;
 			timeout_ptr = &timeout;
+#endif
 		}
 
 		n = loop_select(w_max, &x_read, &x_write, &x_error, timeout_ptr);
@@ -1302,7 +1812,7 @@ select_loop(void (*init)(void))
 			const int errno_from_select = errno;
 
 			if (errno_from_select != EINTR) {
-				ERROR(gettext("The call to %s failed: %d (%s)"),
+				EL_ERROR(gettext("The call to %s failed: %d (%s)"),
 				      "select()", errno_from_select, (char *) strerror(errno_from_select));
 				if (++select_errors > 10) /* Infinite loop prevention. */
 					INTERNAL(gettext("%d select() failures."),
@@ -1366,6 +1876,7 @@ select_loop(void (*init)(void))
 static int
 can_read_or_write(int fd, int write)
 {
+	ELOG
 #if defined(USE_POLL)
 	struct pollfd p;
 	int rs;
@@ -1377,7 +1888,6 @@ can_read_or_write(int fd, int write)
 	if (p.revents & POLLNVAL) elinks_internal("ERROR: poll for %s (%d) failed: %s", !write ? "read" : "write", fd, strerror(errno));
 	return 1;
 #else
-	struct timeval tv = {0, 0};
 	fd_set fds;
 	fd_set *rfds = NULL;
 	fd_set *wfds = NULL;
@@ -1389,26 +1899,34 @@ can_read_or_write(int fd, int write)
 		wfds = &fds;
 	else
 		rfds = &fds;
-
+#ifdef CONFIG_OS_DOS
+	timeval_T tt = {0,0,0};
+	return select2(fd + 1, rfds, wfds, NULL, &tt);
+#else
+	struct timeval tv = {0, 0};
 	return select2(fd + 1, rfds, wfds, NULL, &tv);
 #endif
+#endif
 }
 
 int
 can_read(int fd)
 {
+	ELOG
 	return can_read_or_write(fd, 0);
 }
 
 int
 can_write(int fd)
 {
+	ELOG
 	return can_read_or_write(fd, 1);
 }
 
 void
 terminate_select(void)
 {
+	ELOG
 #ifdef USE_LIBEVENT
 	terminate_libevent();
 #endif
