@@ -1,4 +1,3 @@
-
 /* Elinks overrides for Win32 (MSVC + MingW)
  * Loosely based on osdep/beos/overrides.c by Mikulas Patocka.
  *
@@ -25,14 +24,15 @@
 #include "intl/libintl.h"
 #include "osdep/osdep.h"
 #include "osdep/win32/overrides.h"
+#ifndef CONFIG_LIBUV
 #include "osdep/win32/vt100.h"
+#endif
 #include "osdep/win32/win32.h"
 #include "terminal/mouse.h"
 #include "terminal/terminal.h"
 
 #define SOCK_SHIFT  1024
 
-
 enum fd_types {
 	FDT_FILE = 1,
 	FDT_SOCKET,
@@ -75,15 +75,21 @@ static const char *keymap_2[] = {
 	"\E[24~"   /* VK_F12 */
 };
 
+
+#ifdef CONFIG_FASTMEM
+#define TRACE(m...)
+#else
 #define TRACE(m...)					\
 	do {						\
 		if (cmdline_options && (get_cmd_opt_int("verbose") == 2))	\
 			DBG(m);				\
 	} while (0)
+#endif
 
 static enum fd_types
 what_fd_type(int *fd)
 {
+	ELOG
 	if (*fd == (int)(intptr_t)GetStdHandle(STD_INPUT_HANDLE)
 	    || *fd == (int)(intptr_t)GetStdHandle(STD_OUTPUT_HANDLE)
 	    || *fd == (int)(intptr_t)GetStdHandle(STD_ERROR_HANDLE))
@@ -103,6 +109,7 @@ what_fd_type(int *fd)
 static int
 console_key_read(const KEY_EVENT_RECORD *ker, char *buf, int max)
 {
+	ELOG
 	char c;
 	int  vkey;
 
@@ -136,9 +143,11 @@ struct elwin_mouse {
 	void *data;
 } elm;
 
+#ifndef CONFIG_WIN32_VT100_NATIVE
 static int
 console_mouse_read(const MOUSE_EVENT_RECORD *mer, char *buf, int max)
 {
+	ELOG
 	static struct interlink_event_mouse prev_mouse;
 	static int prev_buttons;
 	struct interlink_event ev;
@@ -188,32 +197,43 @@ console_mouse_read(const MOUSE_EVENT_RECORD *mer, char *buf, int max)
 
 	return 0;
 }
+#endif
 
 void *
 handle_mouse(int cons, void (*fn)(void *, char *, int),
 	     void *data)
 {
+	ELOG
+#ifdef CONFIG_WIN32_VT100_NATIVE
+	return NULL;
+#else
 	elm.cons = cons;
 	elm.fn = fn;
 	elm.data = data;
 
 	return &elm;
+#endif
 }
 
 void
 unhandle_mouse(void *h)
 {
+	ELOG
+#ifndef CONFIG_WIN32_VT100_NATIVE
 	memset(&elm, 0, sizeof(elm));
+#endif
 }
 
 void
 suspend_mouse(void *h)
 {
+	ELOG
 }
 
 void
 resume_mouse(void *h)
 {
+	ELOG
 }
 
 #endif
@@ -221,9 +241,12 @@ resume_mouse(void *h)
 static int
 console_read(HANDLE hnd, void *buf, int max, INPUT_RECORD *irp)
 {
+	ELOG
 	INPUT_RECORD ir;
 	const KEY_EVENT_RECORD   *ker = &ir.Event.KeyEvent;
+#ifndef CONFIG_WIN32_VT100_NATIVE
 	const MOUSE_EVENT_RECORD *mer = &ir.Event.MouseEvent;
+#endif
 	DWORD num_events;
 
 	ReadConsoleInput (hnd, &ir, 1, &num_events);
@@ -238,10 +261,14 @@ console_read(HANDLE hnd, void *buf, int max, INPUT_RECORD *irp)
 
 #ifdef CONFIG_MOUSE
 	case MOUSE_EVENT:
+#ifdef CONFIG_WIN32_VT100_NATIVE
+		break;
+#else
 		TRACE("MOUSE_EVENT: flg %d, button-state %d, x %lu, y %lu",
 		      mer->dwEventFlags, mer->dwButtonState,
 		      mer->dwMousePosition.X, mer->dwMousePosition.Y);
 		return console_mouse_read(mer, buf, max);
+#endif
 #endif
 
 	case WINDOW_BUFFER_SIZE_EVENT:
@@ -255,6 +282,7 @@ console_read(HANDLE hnd, void *buf, int max, INPUT_RECORD *irp)
 static int
 console_peek(HANDLE hnd)
 {
+	ELOG
 	DWORD num = 0;
 	char  buf[32];
 	int   rc = 0;
@@ -279,10 +307,12 @@ console_peek(HANDLE hnd)
 int
 win32_write(int fd, const void *buf, unsigned len)
 {
+	ELOG
 	DWORD written = 0;
 	int   rc = -1;
+#ifndef CONFIG_FASTMEM
 	int   orig_fd = fd;
-
+#endif
 	switch (what_fd_type(&fd)) {
 	case FDT_FILE:
 		rc = write(fd, buf, len);
@@ -296,14 +326,20 @@ win32_write(int fd, const void *buf, unsigned len)
 		break;
 
 	case FDT_TERMINAL:
-		if (isatty(STDOUT_FILENO) > 0) {
-#if 0
-			WriteConsole ((HANDLE)(intptr_t)fd, buf, len, &written, NULL);
-			rc = written;
+#ifdef CONFIG_LIBUV
+		if (0) {}
 #else
-			rc = VT100_decode((HANDLE)(intptr_t)fd, buf, len);
+		if (isatty(STDOUT_FILENO) > 0) {
+			if (is_xterm()) {
+				rc = write(STDOUT_FILENO, buf, len);
+			} else {
+#ifndef CONFIG_WIN32_VT100_NATIVE
+				rc = VT100_decode((HANDLE)(intptr_t)fd, buf, len);
 #endif
-		} else {
+			}
+		}
+#endif
+		else {
 			/* stdout redirected */
 			rc = write(STDOUT_FILENO, buf, len);
 		}
@@ -316,17 +352,21 @@ win32_write(int fd, const void *buf, unsigned len)
 		break;
 	}
 
+#ifndef CONFIG_LIBUV
 	TRACE("fd %d, buf 0x%p, len %u -> rc %d; %s",
 	      orig_fd, buf, len, rc, rc < 0 ? win32_strerror(errno) : "okay");
-
+#endif
 	return rc;
 }
 
 int
 win32_read(int fd, void *buf, unsigned len)
 {
+	ELOG
 	int   rc = -1;
+#ifndef CONFIG_FASTMEM
 	int   orig_fd = fd;
+#endif
 	DWORD Read;
 
 	switch (what_fd_type(&fd)) {
@@ -342,11 +382,18 @@ win32_read(int fd, void *buf, unsigned len)
 		break;
 
 	case FDT_TERMINAL:
+#ifdef CONFIG_WIN32_VT100_NATIVE
+		if (!ReadFile((HANDLE)(intptr_t)fd, buf, len, &Read, NULL))
+			rc = -1;
+		else
+			rc = Read;
+		break;
+#else
 		rc = console_read((HANDLE)(intptr_t)fd, buf, len, NULL);
 		if (rc == 0)
 			TRACE("read 0 !!");
 		break;
-
+#endif
 	case FDT_SOCKET:
 		rc = recv(fd, buf, len, 0);
 		if (rc < 0)
@@ -363,9 +410,11 @@ win32_read(int fd, void *buf, unsigned len)
 int
 win32_close(int fd)
 {
+	ELOG
 	int rc = -1;
+#ifndef CONFIG_FASTMEM
 	int orig_fd = fd;
-
+#endif
 	switch (what_fd_type(&fd)) {
 	case FDT_FILE:
 		rc = close(fd);
@@ -396,8 +445,12 @@ win32_close(int fd)
 int
 win32_ioctl(int fd, long option, int *flag)
 {
+	ELOG
 	char  cmd[20];
-	int  rc = 0, orig_fd = fd, flg = *flag;
+	int  rc = 0, flg = *flag;
+#ifndef CONFIG_FASTMEM
+	int orig_fd = fd;
+#endif
 	DWORD mode;
 
 	switch (what_fd_type(&fd)) {
@@ -440,6 +493,7 @@ win32_ioctl(int fd, long option, int *flag)
 int
 win32_socket(int pf, int type, int protocol)
 {
+	ELOG
 	int     s;
 	int    rc;
 
@@ -460,6 +514,7 @@ win32_socket(int pf, int type, int protocol)
 int
 win32_connect(int fd, struct sockaddr *addr, int addr_len)
 {
+	ELOG
 	int rc = connect(fd - SOCK_SHIFT, addr, addr_len);
 
 	if (rc < 0)
@@ -473,6 +528,7 @@ win32_connect(int fd, struct sockaddr *addr, int addr_len)
 int
 win32_getpeername (int fd, struct sockaddr *addr, int *addr_len)
 {
+	ELOG
 	int rc = getpeername(fd - SOCK_SHIFT, addr, addr_len);
 
 	if (rc < 0)
@@ -486,6 +542,7 @@ win32_getpeername (int fd, struct sockaddr *addr, int *addr_len)
 int
 win32_getsockname(int fd, struct sockaddr *addr, int *addr_len)
 {
+	ELOG
 	int rc = getsockname(fd - SOCK_SHIFT, addr, addr_len);
 
 	if (rc < 0)
@@ -499,6 +556,7 @@ win32_getsockname(int fd, struct sockaddr *addr, int *addr_len)
 int
 win32_accept(int fd, struct sockaddr *addr, int *addr_len)
 {
+	ELOG
 	int rc = accept(fd - SOCK_SHIFT, addr, addr_len);
 
 	if (rc < 0)
@@ -512,6 +570,7 @@ win32_accept(int fd, struct sockaddr *addr, int *addr_len)
 int
 win32_listen(int fd, int backlog)
 {
+	ELOG
 	int rc = listen(fd - SOCK_SHIFT, backlog);
 
 	if (rc < 0)
@@ -526,6 +585,7 @@ win32_listen(int fd, int backlog)
 int
 win32_bind(int fd, struct sockaddr *addr, int addr_len)
 {
+	ELOG
 	int rc = bind(fd-SOCK_SHIFT, addr, addr_len);
 
 	if (rc < 0)
@@ -539,6 +599,7 @@ win32_bind(int fd, struct sockaddr *addr, int addr_len)
 int
 win32_getsockopt(int fd, int level, int option, void *optval, int *optlen)
 {
+	ELOG
 	int rc = getsockopt(fd - SOCK_SHIFT, level, option, optval, optlen);
 
 	if (rc < 0)
@@ -550,9 +611,26 @@ win32_getsockopt(int fd, int level, int option, void *optval, int *optlen)
 	return rc;
 }
 
+int
+win32_setsockopt(int fd, int level, int optname, const char *optval, int optlen)
+{
+	ELOG
+	int rc = setsockopt(fd - SOCK_SHIFT, level, optname, optval, optlen);
+
+	if (rc < 0) {
+		errno = WSAGetLastError();
+	}
+
+	TRACE("fd %d, level %d, optname %d -> rc %d; %s",
+	      fd, level, optname, rc, rc < 0 ? win32_strerror(errno) : "okay");
+
+	return rc;
+}
+
 int
 win32_pipe(int *fds)
 {
+	ELOG
 	HANDLE rd_pipe, wr_pipe;
 
 	if (!CreatePipe(&rd_pipe,&wr_pipe,NULL,0)) {
@@ -566,9 +644,11 @@ win32_pipe(int *fds)
 	return 0;
 }
 
+#ifndef CONFIG_FASTMEM
 static const char *
 timeval_str(const struct timeval *tv)
 {
+	ELOG
 	static char buf[30];
 
 	snprintf(buf, sizeof(buf), "%ld.%03ld", tv->tv_sec, tv->tv_usec/1000);
@@ -579,6 +659,7 @@ timeval_str(const struct timeval *tv)
 static char *
 fd_set_str(char *buf, size_t len, const fd_set *fd, int num_fds)
 {
+	ELOG
 	char *p   = buf;
 	char *end = buf + len;
 	int   i, num;
@@ -604,6 +685,7 @@ fd_set_str(char *buf, size_t len, const fd_set *fd, int num_fds)
 static void
 select_dump(int num_fds, const fd_set *rd, const fd_set *wr, const fd_set *ex)
 {
+	ELOG
 	char buf_rd[512], buf_wr[512], buf_ex[512];
 
 	TRACE("\tread-fds:   %s\n"
@@ -613,10 +695,12 @@ select_dump(int num_fds, const fd_set *rd, const fd_set *wr, const fd_set *ex)
 	      wr ? fd_set_str(buf_wr,sizeof(buf_wr),wr,num_fds) : "<none>",
 	      ex ? fd_set_str(buf_ex,sizeof(buf_ex),ex,num_fds) : "<none>");
 }
+#endif
 
 static int
 select_read(int fd, struct fd_set *rd)
 {
+	ELOG
 	int rc = 0;
 	HANDLE hnd = (HANDLE)(intptr_t)fd;
 
@@ -647,6 +731,7 @@ static int
 select_one_loop(int num_fds, struct fd_set *rd, struct fd_set *wr,
 		struct fd_set *ex)
 {
+	ELOG
 	struct timeval tv = { 0, 100 };
 	int    rc, fd;
 
@@ -704,6 +789,7 @@ select_one_loop(int num_fds, struct fd_set *rd, struct fd_set *wr,
 int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 		struct fd_set *ex, struct timeval *tv)
 {
+	ELOG
 	struct fd_set tmp_rd, tmp_ex;
 	struct timeval expiry = {0};
 	int    fd, rc;
@@ -713,8 +799,10 @@ int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 	      num_fds, rd ? "r" : "-", wr ? "w" : "-",
 	      ex ? "x" : "-", tv ? timeval_str(tv) : "NULL");
 
+#ifndef CONFIG_FASTMEM
 	if (get_cmd_opt_int("verbose") == 2)
 		select_dump(num_fds, rd, wr, ex);
+#endif
 
 	FD_ZERO(&tmp_rd);
 	FD_ZERO(&tmp_ex);
@@ -732,9 +820,9 @@ int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 			expiry.tv_sec++;
 		}
 	}
-
+#ifndef CONFIG_FASTMEM
 	int errnol = 0;
-
+#endif
 	for (rc = 0; !expired; ) {
 		rc += select_one_loop (num_fds, &tmp_rd, wr, &tmp_ex);
 
@@ -770,6 +858,7 @@ int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 
 	TRACE("-> rc %d, err %d", rc, rc < 0 ? errnol : 0);
 
+#ifndef CONFIG_FASTMEM
 	if (get_cmd_opt_int("verbose") == 2)
 		select_dump(num_fds, rd, wr, ex);
 
@@ -777,7 +866,7 @@ int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 		Sleep (400);
 	else
 		Sleep (0);
-
+#endif
 	return rc;
 }
 
@@ -785,6 +874,7 @@ int win32_select (int num_fds, struct fd_set *rd, struct fd_set *wr,
 static char *
 get_winsock_error(int err, char *buf, size_t len)
 {
+	ELOG
 	char *p;
 
 	switch (err) {
@@ -965,6 +1055,7 @@ get_winsock_error(int err, char *buf, size_t len)
 char *
 win32_strerror(int err)
 {
+	ELOG
 	static char buf[512];
 	char  *p;
 
@@ -991,6 +1082,7 @@ win32_strerror(int err)
 int
 win32_send(int sockfd, const void *buf, unsigned len, int flags)
 {
+	ELOG
 	int rc;
 
 	if (sockfd >= SOCK_SHIFT) {
@@ -1005,9 +1097,24 @@ win32_send(int sockfd, const void *buf, unsigned len, int flags)
 	return rc;
 }
 
+int
+win32_sendto(int sockfd, const char *buf, int len, int flags, const void *to, int tolen)
+{
+	ELOG
+
+	int rc = sendto(sockfd - SOCK_SHIFT, buf, len, flags, to, tolen);
+
+	if (rc < 0) {
+		errno = WSAGetLastError();
+	}
+
+	return rc;
+}
+
 int
 win32_recv(int sockfd, void *buf, unsigned len, int flags)
 {
+	ELOG
 	int rc;
 
 	if (sockfd >= SOCK_SHIFT) {
@@ -1021,3 +1128,4 @@ win32_recv(int sockfd, void *buf, unsigned len, int flags)
 
 	return rc;
 }
+
