@@ -7,6 +7,11 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
+
+#ifdef HAVE_SYS_CYGWIN_H
+#include <sys/cygwin.h>
+#endif
+
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h> /* OS/2 needs this after sys/types.h */
@@ -57,6 +62,7 @@ struct module dgi_protocol_module = struct_module(
 static struct mime_handler *
 find_dgi(const char *name)
 {
+	ELOG
 	const char *last = strrchr(name, '/');
 	struct mime_handler *handler;
 
@@ -81,6 +87,7 @@ find_dgi(const char *name)
 static void
 write_request_to_file(struct connection *conn, const char *filename)
 {
+	ELOG
 	if (!conn->uri->post) {
 		return;
 	}
@@ -108,6 +115,7 @@ enum dgi_state {
 static void
 prepare_command(char *program, const char *filename, const char *query, char *inpext, char *outext, struct string *cmd, char **inp, char **out, char **queryfile)
 {
+	ELOG
 	const char *ch;
 	enum dgi_state state = NORMAL;
 
@@ -194,7 +202,7 @@ prepare_command(char *program, const char *filename, const char *query, char *in
 					state = NORMAL;
 					break;
 				case 'q':
-					*queryfile = tempname(NULL, "elinks", inpext);
+					*queryfile = tempname(NULL, ELINKS_TEMPNAME_PREFIX, inpext);
 					if (*queryfile) {
 						add_to_string(cmd, *queryfile);
 					}
@@ -228,9 +236,19 @@ prepare_command(char *program, const char *filename, const char *query, char *in
 					break;
 				case '1':
 					if (filename) {
-						add_to_string(cmd, filename);
+#if defined(HAVE_CYGWIN_CONV_TO_FULL_WIN32_PATH)
+#ifdef MAX_PATH
+						char new_path[MAX_PATH];
+#else
+						char new_path[1024];
+#endif
+						cygwin_conv_to_full_win32_path(filename, new_path);
+						add_to_string(cmd, new_path);
+#else
+						add_shell_quoted_to_string(cmd, filename, strlen(filename));
+#endif
 					} else {
-						*inp = tempname(NULL, "elinks", inpext);
+						*inp = tempname(NULL, ELINKS_TEMPNAME_PREFIX, inpext);
 						if (*inp) {
 							add_to_string(cmd, *inp);
 						}
@@ -238,7 +256,7 @@ prepare_command(char *program, const char *filename, const char *query, char *in
 					state = NORMAL;
 					break;
 				case '2':
-					*out = tempname(NULL, "elinks", outext);
+					*out = tempname(NULL, ELINKS_TEMPNAME_PREFIX, outext);
 					if (*out) {
 						add_to_string(cmd, *out);
 					}
@@ -257,6 +275,7 @@ prepare_command(char *program, const char *filename, const char *query, char *in
 void
 dgi_protocol_handler(struct connection *conn)
 {
+	ELOG
 #define NUMKVPAIRS 16
 	char *query;
 	struct connection_state state = connection_state(S_OK);
@@ -297,7 +316,7 @@ dgi_protocol_handler(struct connection *conn)
 	}
 	prepare_command(command, filename, NULL, inpext, outext, &command_str, &tempfilename, &outputfilename, NULL);
 
-	(void)!system(command_str.source);
+	(void)system(command_str.source);
 	done_string(&command_str);
 
 	if (del) {
@@ -307,6 +326,7 @@ dgi_protocol_handler(struct connection *conn)
 	if (tempfilename) {
 		unlink(tempfilename);
 	}
+	cls_redraw_all_terminals();
 
 	if (!outputfilename) {
 		state = connection_state(S_OK);
@@ -379,6 +399,7 @@ dgi_protocol_handler(struct connection *conn)
 int
 execute_dgi(struct connection *conn)
 {
+	ELOG
 	char *script;
 	struct mime_handler *handler;
 	struct string command;
@@ -439,14 +460,12 @@ execute_dgi(struct connection *conn)
 		unlink(queryfile);
 	}
 
-
 	if (!outputfilename) {
 		mem_free(handler);
 		state = connection_state(S_OK);
 		abort_connection(conn, state);
 		return 0;
 	}
-
 	struct string page;
 	struct string name;
 
@@ -460,50 +479,51 @@ execute_dgi(struct connection *conn)
 	unlink(outputfilename);
 	done_string(&name);
 
-	if (is_in_state(state, S_OK)) {
-		struct cache_entry *cached;
-
-		/* Try to add fragment data to the connection cache if either
-		 * file reading or directory listing worked out ok. */
-		cached = conn->cached = get_cache_entry(conn->uri);
-		if (!conn->cached) {
-			state = connection_state(S_OUT_OF_MEM);
-		} else {
-			add_fragment(cached, 0, page.source, page.length);
-			conn->from += page.length;
-
-			if (1) {
-				char *head;
-				char *otype = NULL;
-
-				if (handler->outext) {
-					otype = get_extension_content_type(handler->outext);
-				}
-				if (!otype) {
-					otype = stracpy("text/html");
-				}
-
-				/* If the system charset somehow
-				 * changes after the directory listing
-				 * has been generated, it should be
-				 * parsed with the original charset.  */
-				head = straconcat("\r\nContent-Type: ", otype, "; charset=",
-						  get_cp_mime_name(get_cp_index("System")),
-						  "\r\n", (char *) NULL);
-
-				mem_free_if(otype);
-
-				/* Not so gracefully handle failed memory
-				 * allocation. */
-				if (!head)
-					state = connection_state(S_OUT_OF_MEM);
-
-				/* Setup directory listing for viewing. */
-				mem_free_set(&cached->head, head);
-			}
-			done_string(&page);
-		}
+	if (!is_in_state(state, S_OK)) {
+		goto out;
 	}
+	struct cache_entry *cached;
+
+	/* Try to add fragment data to the connection cache if either
+	 * file reading or directory listing worked out ok. */
+	cached = conn->cached = get_cache_entry(conn->uri);
+	if (!conn->cached) {
+		state = connection_state(S_OUT_OF_MEM);
+	} else {
+		add_fragment(cached, 0, page.source, page.length);
+		conn->from += page.length;
+
+		char *head;
+		char *otype = NULL;
+
+		if (handler->outext) {
+			otype = get_extension_content_type(handler->outext);
+		}
+		if (!otype) {
+			otype = stracpy("text/html");
+		}
+
+		/* If the system charset somehow
+		 * changes after the directory listing
+		 * has been generated, it should be
+		 * parsed with the original charset.  */
+		head = straconcat("\r\nContent-Type: ", otype, "; charset=",
+				  get_cp_mime_name(get_cp_index("System")),
+				  "\r\n", (char *) NULL);
+
+		mem_free_if(otype);
+
+		/* Not so gracefully handle failed memory
+		 * allocation. */
+		if (!head) {
+			state = connection_state(S_OUT_OF_MEM);
+		}
+
+		/* Setup directory listing for viewing. */
+		mem_free_set(&cached->head, head);
+	}
+	done_string(&page);
+out:
 	mem_free(handler);
 	abort_connection(conn, state);
 	return 0;
