@@ -39,8 +39,35 @@
 #include "util/env.h"
 #include "util/string.h"
 
+static union option_info mailcap_options[] = {
+	INIT_OPT_TREE("protocol", N_("Mailcap"),
+		"mailcap", OPT_ZERO,
+		N_("Options specific to mailcap.")),
+
+	INIT_OPT_BOOL("protocol.mailcap", N_("Allow empty referrer"),
+		"allow_empty_referrer", OPT_ZERO, 0,
+		N_("Whether to allow empty referrer for mailcap protocol. "
+		"It can be helpful for restoring sessions, but it is a bit dangerous. "
+		"When allowed you can execute any command in goto url dialog. "
+		"For example mailcap:ls")),
+
+	NULL_OPTION_INFO,
+};
+
+
 struct module mailcap_protocol_module = struct_module(
 	/* name: */		N_("Mailcap"),
+	/* options: */		mailcap_options,
+	/* hooks: */		NULL,
+	/* submodules: */	NULL,
+	/* data: */		NULL,
+	/* init: */		NULL,
+	/* done: */		NULL,
+	/* getname: */	NULL
+);
+
+struct module mailcap_html_protocol_module = struct_module(
+	/* name: */		N_("Mailcap html"),
 	/* options: */		NULL,
 	/* hooks: */		NULL,
 	/* submodules: */	NULL,
@@ -52,15 +79,22 @@ struct module mailcap_protocol_module = struct_module(
 
 #ifdef HAVE_FORK
 static void
-get_request(struct connection *conn)
+get_request(struct connection *conn, int x_htmloutput)
 {
+	ELOG
 	struct read_buffer *rb = alloc_read_buffer(conn->socket);
 
 	if (!rb) return;
 
-	memcpy(rb->data, "HTTP/1.0 200 OK\r\nContent-Type: text/plain\r\n\r\n", 45);
-	rb->length = 45;
-	rb->freespace -= 45;
+	if (x_htmloutput) {
+		memcpy(rb->data, "HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n", 44);
+		rb->length = 44;
+		rb->freespace -= 44;
+	} else {
+		memcpy(rb->data, "HTTP/1.0 200 OK\r\nContent-Type: text/plain\r\n\r\n", 45);
+		rb->length = 45;
+		rb->freespace -= 45;
+	}
 
 	conn->unrestartable = 1;
 
@@ -70,28 +104,27 @@ get_request(struct connection *conn)
 }
 #endif
 
-void
-mailcap_protocol_handler(struct connection *conn)
+static void
+mailcap_protocol_handler_common(struct connection *conn, int html)
 {
+	ELOG
 #ifdef HAVE_FORK
-	char *script, *ref;
+	char *script;
 	pid_t pid;
 	struct connection_state state = connection_state(S_OK);
-	int pipe_read[2], check;
+	int pipe_read[2];
 
 	/* security checks */
-	if (!conn->referrer || conn->referrer->protocol != PROTOCOL_MAILCAP) {
+
+	if (!conn->referrer && !get_opt_bool("protocol.mailcap.allow_empty_referrer", NULL)) {
 		goto bad;
 	}
-	ref = get_uri_string(conn->referrer, URI_DATA);
-	if (!ref) {
+
+	if (conn->referrer && (conn->referrer->protocol != PROTOCOL_MAILCAP && conn->referrer->protocol != PROTOCOL_MAILCAP_HTML)) {
 		goto bad;
 	}
-	check = strcmp(ref, "elmailcap");
-	mem_free(ref);
-	if (check) goto bad;
-	
 	script = get_uri_string(conn->uri, URI_DATA);
+
 	if (!script) {
 		state = connection_state(S_OUT_OF_MEM);
 		goto end2;
@@ -133,7 +166,7 @@ mailcap_protocol_handler(struct connection *conn)
 		conn->cgi = 1;
 		set_nonblocking_fd(conn->socket->fd);
 
-		get_request(conn);
+		get_request(conn, html);
 		return;
 	}
 
@@ -148,3 +181,17 @@ bad:
 #endif
 	abort_connection(conn, connection_state(S_BAD_URL));
 }
+
+void
+mailcap_protocol_handler(struct connection *conn)
+{
+	ELOG
+	mailcap_protocol_handler_common(conn, 0);
+}
+
+void
+mailcap_html_protocol_handler(struct connection *conn)
+{
+	ELOG
+	mailcap_protocol_handler_common(conn, 1);
+}
