@@ -40,6 +40,9 @@ use of this software.
 #include <netdb.h>
 #endif
 
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
@@ -70,6 +73,7 @@ use of this software.
 long
 random(void)
 {
+	ELOG
 	unsigned int number = 0;
 	(void)rand_s(&number);
 	return (long)number;
@@ -119,6 +123,7 @@ struct module fsp_protocol_module = struct_module(
 static int
 buildfilename(const FSP_SESSION *s, FSP_PKT *out, const char *dirname)
 {
+	ELOG
 	int len;
 
 	len = strlen(dirname);
@@ -159,6 +164,7 @@ buildfilename(const FSP_SESSION *s, FSP_PKT *out, const char *dirname)
 static size_t
 fsp_pkt_write(const FSP_PKT *p, void *space)
 {
+	ELOG
 	size_t used;
 	unsigned char *ptr;
 	int checksum;
@@ -201,6 +207,7 @@ fsp_pkt_write(const FSP_PKT *p, void *space)
 static int
 fsp_pkt_read(FSP_PKT *p, const void *space, size_t recv_len)
 {
+	ELOG
 	int mysum;
 	size_t i;
 	const unsigned char *ptr;
@@ -260,6 +267,7 @@ static void fsp_transaction_send_loop(void *data);
 static void
 try_fsp_transaction(void *data)
 {
+	ELOG
 	struct connection *conn = (struct connection *)data;
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 	FSP_SESSION *s = fsp->ses;
@@ -290,6 +298,7 @@ static void fsp_transaction_continue(void *data);
 static void
 fsp_transaction_send_loop(void *data)
 {
+	ELOG
 	struct connection *conn = (struct connection *)data;
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 
@@ -322,9 +331,9 @@ fsp_transaction_send_loop(void *data)
 	fsp->l_delay = fsp->w_delay;
 
 	/* send packet */
-	if (send(s->fd, fsp->buf, l, 0) < 0) {
-#if 1
-		fprintf(stderr, "Send failed.\n");
+	if (l <= 0 || sendto(s->fd, fsp->buf, l, 0, (struct sockaddr *)&s->host_addr, sizeof(s->host_addr)) < 0) {
+#if 0
+		fprintf(stderr, "Send failed errno=%d\n", errno);
 #endif
 		if (errno == EBADF || errno == ENOTSOCK) {
 			client_set_key((FSP_LOCK *)s->lock, fsp->out.key);
@@ -347,12 +356,13 @@ fsp_transaction_send_loop(void *data)
 		fsp->w_delay = 1000;
 	}
 	fsp->t_delay += fsp->w_delay;
-	set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn);
+	set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn, EL_TYPE_UDP);
 }
 
 static void
 fsp_transaction_continue(void *data)
 {
+	ELOG
 	struct connection *conn = (struct connection *)data;
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 
@@ -387,7 +397,7 @@ fsp_transaction_continue(void *data)
 
 	/* process received packet */
 	if (fsp_pkt_read(&fsp->in, fsp->buf, r) < 0) {
-		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn);
+		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn, EL_TYPE_UDP);
 		/* unpack failed */
 		return;
 	}
@@ -396,14 +406,14 @@ fsp_transaction_continue(void *data)
 	if ((fsp->in.seq & 0xfff8) != s->seq) {
 		/* duplicate */
 		fsp->dupes++;
-		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn);
+		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn, EL_TYPE_UDP);
 		return;
 	}
 
 	/* check command code */
 	if ((fsp->in.cmd != fsp->out.cmd) && (fsp->in.cmd != FSP_CC_ERR)) {
 		fsp->dupes++;
-		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn);
+		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn, EL_TYPE_UDP);
 		return;
 	}
 
@@ -412,7 +422,7 @@ fsp_transaction_continue(void *data)
 		fsp->out.cmd == FSP_CC_GET_FILE || fsp->out.cmd == FSP_CC_UP_LOAD ||
 		fsp->out.cmd == FSP_CC_GRAB_FILE || fsp->out.cmd == FSP_CC_INFO) ) {
 		fsp->dupes++;
-		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn);
+		set_handlers(s->fd, fsp_transaction_continue, NULL, NULL, conn, EL_TYPE_UDP);
 		return;
 	}
 
@@ -440,98 +450,132 @@ fsp_transaction_continue(void *data)
 	}
 }
 
+static int
+_x_adr(const char *xhost, unsigned short port, struct sockaddr_in *his)
+{
+	struct hostent *H;
+	int    i;
+	char *s, *d;
+
+	memset(his, 0, sizeof(*his));
+
+#ifdef HAVE_INET_PTON
+	if (inet_pton(AF_INET, xhost, &his->sin_addr)) {
+#elif defined(HAVE_INET_ATON)
+	if (inet_aton(xhost, &his->sin_addr)) {
+#else
+	his->sin_addr.s_addr = inet_addr(xhost);
+	if (his->sin_addr.s_addr != INADDR_NONE) {
+#endif
+		his->sin_family = AF_INET;
+	} else if ((H = gethostbyname(xhost))) {
+		for (s = (char *)H->h_addr, d = (char *)&his->sin_addr, i = H->h_length; i--; *d++ = *s++);
+		his->sin_family = H->h_addrtype;
+	} else return -1;
+	his->sin_port = htons((unsigned short) port);
+
+	return 0;
+}
+
+static int
+_x_udp(unsigned short *port)
+{
+	int f, zz = 1;
+	struct sockaddr_in me ;
+	struct sockaddr_in sin;
+	socklen_t len;
+
+	memset(&me, 0, sizeof(me));
+	me.sin_port = htons(*port);
+	me.sin_family = AF_INET;
+
+	if ((f = socket(AF_INET,SOCK_DGRAM,0)) == -1) {
+		return -1;
+	}
+
+	if (setsockopt(f, SOL_SOCKET, SO_REUSEADDR, (const char *)&zz, sizeof(zz)) < 0) {
+		int tmp = errno;
+		close(f);
+		errno = tmp;
+		return -1;
+	}
+
+	if (bind(f, (struct sockaddr *)&me, (len = sizeof(me))) < 0) {
+		int tmp = errno;
+		close(f);
+		errno = tmp;
+		return -1;
+	}
+
+	if (getsockname(f, (struct sockaddr*)&sin, &len) < 0) {
+		int tmp = errno;
+		close(f);
+		errno = tmp;
+		return -1;
+	}
+	if (!*port) *port = ntohs((unsigned short)sin.sin_port);
+	return f;
+}
+
+static int
+init_client(const char *myhost, unsigned short port, unsigned short myport, struct sockaddr_in *server_addr)
+{
+	int myfd;
+
+	if ((myfd = _x_udp(&myport)) == -1) {
+		return -1;
+	}
+
+	if (_x_adr(myhost, port, server_addr) == -1) {
+		close(myfd);
+		return -1;
+	}
+
+	return myfd;
+}
+
 /* ******************* Session management functions ************ */
 
 /* initializes a session */
 static FSP_SESSION *
 fsp_open_session(const char *host, unsigned short port, const char *password)
 {
+	ELOG
 	FSP_SESSION *s;
 	int fd;
-	struct addrinfo hints,*res;
-	char port_s[6];
-	struct sockaddr_in *addrin;
+	struct sockaddr_in addrin;
+
 	FSP_LOCK *lock;
 
-	memset(&hints, 0, sizeof (hints));
-	/* fspd do not supports inet6 */
-	hints.ai_family = PF_INET;
-	hints.ai_socktype = SOCK_DGRAM;
-
-	if (port == 0) {
-		strcpy(port_s, "fsp");
-	} else {
-		sprintf(port_s,"%hu", port);
-	}
-
-	if ((fd = getaddrinfo(host, port_s, &hints, &res)) != 0) {
-
-#ifdef CONFIG_OS_WIN32
-		if (1) {
-#else
-		if (fd != EAI_SYSTEM) {
-#endif
-			/* We need to set errno ourself */
-			switch (fd) {
-			case EAI_SOCKTYPE:
-			case EAI_SERVICE:
-			case EAI_FAMILY:
-				errno = EOPNOTSUPP;
-				break;
-			case EAI_AGAIN:
-				errno = EAGAIN;
-				break;
-			case EAI_FAIL:
-				errno = ECONNRESET;
-				break;
-			case EAI_BADFLAGS:
-				errno = EINVAL;
-				break;
-			case EAI_MEMORY:
-				errno = ENOMEM;
-				break;
-			default:
-				errno = EFAULT;
-			}
-		}
-		return NULL; /* host not found */
-	}
-
-	/* create socket */
-	fd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+	fd = init_client(host, port, 0, &addrin);
 
 	if (fd < 0) {
 		return NULL;
 	}
-	/* connect socket */
-	if (connect(fd, res->ai_addr, res->ai_addrlen)) {
-		close(fd);
-		return NULL;
-	}
+
 	/* allocate memory */
-	s = calloc(1, sizeof(FSP_SESSION));
+	s = mem_calloc(1, sizeof(*s));
 
 	if (!s) {
 		close(fd);
 		errno = ENOMEM;
 		return NULL;
 	}
-	lock = malloc(sizeof(FSP_LOCK));
+	lock = mem_calloc(1, sizeof(*lock));
 
 	if (!lock) {
 		close(fd);
-		free(s);
+		mem_free(s);
 		errno = ENOMEM;
 		return NULL;
 	}
 	s->lock = lock;
 	/* init locking subsystem */
-	addrin = (struct sockaddr_in *)res->ai_addr;
 
-	if (client_init_key((FSP_LOCK *)s->lock, addrin->sin_addr.s_addr, ntohs(addrin->sin_port))) {
-		free(s);
+	if (client_init_key((FSP_LOCK *)s->lock, addrin.sin_addr.s_addr, ntohs(addrin.sin_port))) {
+		mem_free(s);
 		close(fd);
-		free(lock);
+		mem_free(lock);
 		return NULL;
 	}
 	s->fd = fd;
@@ -540,8 +584,9 @@ fsp_open_session(const char *host, unsigned short port, const char *password)
 	s->seq = random() & 0xfff8;
 
 	if (password) {
-		s->password = strdup(password);
+		s->password = stracpy(password);
 	}
+	s->host_addr = addrin;
 
 	return s;
 }
@@ -552,6 +597,7 @@ static void bye_bye(void *data);
 static void
 try_fsp_close_session(struct connection *conn)
 {
+	ELOG
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 
 	if (!fsp->ses || fsp->ses->fd == -1) {
@@ -571,18 +617,19 @@ try_fsp_close_session(struct connection *conn)
 static void
 bye_bye(void *data)
 {
+	ELOG
 	struct connection *conn = (struct connection *)data;
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 	FSP_SESSION *s = fsp->ses;
 
 	kill_timer(&fsp->send_timer);
 	close(s->fd);
-	if (s->password) free(s->password);
+	mem_free_if(s->password);
 	client_destroy_key((FSP_LOCK *)s->lock);
-	free(s->lock);
+	mem_free(s->lock);
 	memset(s, 0, sizeof(FSP_SESSION));
 	s->fd = -1;
-	free(s);
+	mem_free(s);
 
 	fsp->return_func = NULL;
 }
@@ -593,6 +640,7 @@ bye_bye(void *data)
 static int
 fsp_readdir_native(FSP_DIR *dir, FSP_RDENTRY *entry, FSP_RDENTRY **result)
 {
+	ELOG
 	unsigned char ftype;
 	int namelen;
 
@@ -674,15 +722,14 @@ fsp_readdir_native(FSP_DIR *dir, FSP_RDENTRY *entry, FSP_RDENTRY **result)
 static int
 fsp_closedir(FSP_DIR *dirp)
 {
+	ELOG
 	if (dirp == NULL) {
 		return -1;
 	}
 
-	if (dirp->dirname) {
-		free(dirp->dirname);
-	}
-	free(dirp->data);
-	free(dirp);
+	mem_free_if(dirp->dirname);
+	mem_free(dirp->data);
+	mem_free(dirp);
 
 	return 0;
 }
@@ -691,6 +738,7 @@ fsp_closedir(FSP_DIR *dirp)
 static FSP_FILE *
 fsp_fopen(struct connection *conn, const char *path, const char *modeflags)
 {
+	ELOG
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 	FSP_SESSION *session = fsp->ses;
 	FSP_FILE   *f;
@@ -700,7 +748,7 @@ fsp_fopen(struct connection *conn, const char *path, const char *modeflags)
 		return NULL;
 	}
 
-	f = calloc(1, sizeof(FSP_FILE));
+	f = mem_calloc(1, sizeof(*f));
 
 	if (f == NULL) {
 		return NULL;
@@ -711,13 +759,13 @@ fsp_fopen(struct connection *conn, const char *path, const char *modeflags)
 	case 'r':
 		break;
 	default:
-		free(f);
+		mem_free(f);
 		errno = EINVAL;
 		 return NULL;
 	}
 
 	if (*modeflags == '+' || ( *modeflags == 'b' && modeflags[1] == '+')) {
-		free(f);
+		mem_free(f);
 		return NULL;
 	}
 
@@ -726,7 +774,7 @@ fsp_fopen(struct connection *conn, const char *path, const char *modeflags)
 		f->out.cmd = FSP_CC_UP_LOAD;
 	} else {
 		if (buildfilename(session, &fsp->out, path)) {
-			free(f);
+			mem_free(f);
 			return NULL;
 		}
 		f->bufpos = FSP_SPACE;
@@ -735,10 +783,10 @@ fsp_fopen(struct connection *conn, const char *path, const char *modeflags)
 	fsp->out.xlen = 0;
 	/* setup control variables */
 	f->s = session;
-	f->name = strdup(path);
+	f->name = stracpy(path);
 
 	if (f->name == NULL) {
-		free(f);
+		mem_free(f);
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -749,12 +797,13 @@ fsp_fopen(struct connection *conn, const char *path, const char *modeflags)
 static int
 fsp_fclose(FSP_FILE *file)
 {
+	ELOG
 	int rc;
 
 	rc = 0;
 	errno = 0;
-	free(file->name);
-	free(file);
+	mem_free(file->name);
+	mem_free(file);
 
 	return rc;
 }
@@ -763,13 +812,14 @@ fsp_fclose(FSP_FILE *file)
 static void
 try_fsp_stat(struct connection *conn, const char *path)
 {
+	ELOG
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 
 	if (buildfilename(fsp->ses, &fsp->out, path)) {
 		abort_connection(conn, connection_state(S_OUT_OF_MEM));
 		return;
 	}
-	fsp->path = strdup(path);
+	fsp->path = stracpy(path);
 
 	if (!fsp->path) {
 		abort_connection(conn, connection_state(S_OUT_OF_MEM));
@@ -792,6 +842,7 @@ static void try_fsp_opendir(void *data);
 static void
 fsp_stat_continue(void *data)
 {
+	ELOG
 	struct connection *conn = (struct connection *)data;
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 	unsigned char ftype;
@@ -881,6 +932,7 @@ fsp_stat_continue(void *data)
 static void
 display_entry(struct connection *conn, const FSP_RDENTRY *fentry, const char dircolor[])
 {
+	ELOG
 	struct string string;
 
 	/* fentry->name is a fixed-size array and is followed by other
@@ -949,6 +1001,7 @@ display_entry(struct connection *conn, const FSP_RDENTRY *fentry, const char dir
 static int
 compare(const void *av, const void *bv)
 {
+	ELOG
 	const FSP_RDENTRY *a = (const FSP_RDENTRY *)av, *b = (const FSP_RDENTRY *)bv;
 	int res = ((b->type == FSP_RDTYPE_DIR) - (a->type == FSP_RDTYPE_DIR));
 
@@ -960,6 +1013,7 @@ compare(const void *av, const void *bv)
 static void
 sort_and_display_entries(struct connection *conn, FSP_DIR *dir, const char dircolor[])
 {
+	ELOG
 	/* fsp_readdir_native in fsplib 0.9 and earlier requires
 	 * the third parameter to point to a non-null pointer
 	 * even though it does not dereference that pointer
@@ -998,12 +1052,13 @@ sort_and_display_entries(struct connection *conn, FSP_DIR *dir, const char dirco
 static void
 show_fsp_directory(struct connection *conn)
 {
+	ELOG
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 	struct string buf;
 
 	fsp->dir->inuse = 1;
 	fsp->dir->blocksize = fsp->blocksize;
-	fsp->dir->dirname = strdup(fsp->path);
+	fsp->dir->dirname = stracpy(fsp->path);
 	fsp->dir->datasize = fsp->pos;
 
 	errno = 0;
@@ -1050,6 +1105,7 @@ show_fsp_directory(struct connection *conn)
 static void
 try_fsp_opendir(void *data)
 {
+	ELOG
 	struct connection *conn = (struct connection *)data;
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 	unsigned char *tmp;
@@ -1079,14 +1135,14 @@ try_fsp_opendir(void *data)
 	}
 
 	if (fsp->dir == NULL) {
-		fsp->dir = calloc(1, sizeof(FSP_DIR));
+		fsp->dir = mem_calloc(1, sizeof(*fsp->dir));
 
 		if (!fsp->dir) {
 			abort_connection(conn, connection_state(S_OUT_OF_MEM));
 			return;
 		}
 	}
-	tmp = realloc(fsp->dir->data, fsp->pos + fsp->in.len);
+	tmp = mem_realloc(fsp->dir->data, fsp->pos + fsp->in.len);
 
 	if (tmp == NULL) {
 		abort_connection(conn, connection_state(S_OUT_OF_MEM));
@@ -1107,6 +1163,7 @@ try_fsp_opendir(void *data)
 static void
 try_fsp_read_file(void *data)
 {
+	ELOG
 	struct connection *conn = (struct connection *)data;
 	struct fsp_connection_info *fsp = (struct fsp_connection_info *)conn->info;
 
@@ -1149,6 +1206,7 @@ try_fsp_read_file(void *data)
 static void
 done_fsp_connection(struct connection *conn)
 {
+	ELOG
 	struct fsp_connection_info *fsp;
 
 	if (!conn || !conn->info) {
@@ -1167,15 +1225,13 @@ done_fsp_connection(struct connection *conn)
 	if (fsp->ses) {
 		try_fsp_close_session(conn);
 	}
-
-	if (fsp->path) {
-		free(fsp->path);
-	}
+	mem_free_if(fsp->path);
 }
 
 static void
 do_fsp(struct connection *conn)
 {
+	ELOG
 	FSP_SESSION *ses;
 	struct uri *uri = conn->uri;
 	struct auth_entry *auth;
@@ -1225,6 +1281,7 @@ do_fsp(struct connection *conn)
 void
 fsp_protocol_handler(struct connection *conn)
 {
+	ELOG
 	conn->from = 0;
 	conn->unrestartable = 1;
 	do_fsp(conn);
