@@ -41,6 +41,9 @@
 #include "network/connection.h"
 #include "network/socket.h"
 #include "protocol/common.h"
+#ifdef CONFIG_LIBCURL
+#include "protocol/curl/gopher.h"
+#endif
 #include "protocol/gopher/gopher.h"
 #include "protocol/protocol.h"
 #include "protocol/uri.h"
@@ -48,9 +51,25 @@
 #include "util/memory.h"
 #include "util/string.h"
 
+union option_info gopher_options[] = {
+	INIT_OPT_TREE("protocol", N_("Gopher"),
+		"gopher", OPT_ZERO,
+		N_("Gopher specific options.")),
+#if defined(CONFIG_LIBCURL)
+	INIT_OPT_BOOL("protocol.gopher", N_("Use libcurl"),
+		"use_curl", OPT_ZERO, 0,
+		N_("Use libcurl implementation of gopher.")),
+
+	INIT_OPT_STRING("protocol.gopher", N_("Curl TLS13 ciphers"),
+		"curl_tls13_ciphers", OPT_ZERO, "",
+		N_("Colon separated list of TLS13 ciphers. See https://curl.se/docs/ssl-ciphers.html")),
+#endif
+	NULL_OPTION_INFO,
+};
+
 struct module gopher_protocol_module = struct_module(
 	/* name: */		N_("Gopher"),
-	/* options: */		NULL,
+	/* options: */		gopher_options,
 	/* hooks: */		NULL,
 	/* submodules: */	NULL,
 	/* data: */		NULL,
@@ -147,6 +166,7 @@ static struct gopher_entity_info gopher_entity_info[] = {
 static struct gopher_entity_info *
 get_gopher_entity_info(gopher_entity_T type)
 {
+	ELOG
 	int entry;
 
 	for (entry = 0; entry < sizeof_array(gopher_entity_info) - 1; entry++)
@@ -161,6 +181,7 @@ get_gopher_entity_info(gopher_entity_T type)
 static const char *
 get_gopher_entity_description(gopher_entity_T type)
 {
+	ELOG
 	struct gopher_entity_info *info = get_gopher_entity_info(type);
 
 	return info ? info->description : NULL;
@@ -180,6 +201,7 @@ static void
 add_uri_decoded(struct string *command, char *string, int length,
 		int replace_plus)
 {
+	ELOG
 	int oldlen = command->length;
 
 	assert(string);
@@ -212,6 +234,7 @@ add_gopher_command(struct connection *conn, struct string *command,
 		   gopher_entity_T entity,
 		   char *selector, int selectorlen)
 {
+	ELOG
 	char *query;
 	int querylen;
 
@@ -280,6 +303,7 @@ add_gopher_command(struct connection *conn, struct string *command,
 static struct connection_state
 init_gopher_connection_info(struct connection *conn)
 {
+	ELOG
 	struct gopher_connection_info *gopher;
 	struct connection_state state;
 	struct string command;
@@ -361,6 +385,7 @@ static void
 add_gopher_link(struct string *buffer, const char *text,
 		const char *addr)
 {
+	ELOG
 	add_to_string(buffer, "<a href=\"");
 	add_html_to_string(buffer, addr, strlen(addr));
 	add_to_string(buffer, "\">");
@@ -372,6 +397,7 @@ static void
 add_gopher_search_field(struct string *buffer, const char *text,
 		const char *addr)
 {
+	ELOG
 	add_to_string(buffer, "<form action=\"");
 	add_html_to_string(buffer, addr, strlen(addr));
 	add_to_string(buffer, "\">"
@@ -389,6 +415,7 @@ add_gopher_search_field(struct string *buffer, const char *text,
 static void
 add_gopher_description(struct string *buffer, gopher_entity_T entity)
 {
+	ELOG
 	const char *description = get_gopher_entity_description(entity);
 
 	if (!description)
@@ -402,6 +429,7 @@ add_gopher_description(struct string *buffer, gopher_entity_T entity)
 static void
 encode_selector_string(struct string *buffer, char *selector)
 {
+	ELOG
 	char *slashes;
 
 	/* Rather hackishly only convert slashes if there are
@@ -417,9 +445,10 @@ encode_selector_string(struct string *buffer, char *selector)
 	encode_uri_string(buffer, selector, -1, 0);
 }
 
-static void
+void
 add_gopher_menu_line(struct string *buffer, char *line)
 {
+	ELOG
 	/* Gopher menu fields */
 	char *name = line;
 	char *selector = NULL;
@@ -456,13 +485,16 @@ add_gopher_menu_line(struct string *buffer, char *line)
 		port = host ? strchr(host, ASCII_TAB) : NULL;
 		if (port) {
 			char *end;
-			int portno;
-
+			int portno = -1;
 			errno = 0;
-			portno = strtol(port + 1, (char **) &end, 10);
+
+			if (!isdigit(port[1])) {
+				port = NULL;
+			} else {
+				portno = strtol(port + 1, (char **) &end, 10);
+			}
 			if (errno || !uri_port_is_valid(portno)) {
 				port = NULL;
-
 			} else {
 				/* Try to wipe out the default gopher port
 				 * number from being appended to links. */
@@ -521,9 +553,7 @@ add_gopher_menu_line(struct string *buffer, char *line)
 	default:
 	{
 		struct string address;
-		const char *format = selector && *selector
-				      ? "%s://%s@%s/" : "%s://%s%s/";
-
+		const char *format = "%s://%s%s%s/";
 		/* If port is defined it means that both @selector and @host
 		 * was correctly parsed. */
 		if (!port || !init_string(&address)) {
@@ -535,21 +565,23 @@ add_gopher_menu_line(struct string *buffer, char *line)
 		assert(selector && host);
 
 		if (entity == GOPHER_TELNET) {
+			if (*selector == '/') {
+				++selector;
+			}
 			add_format_to_string(&address, format,
-					     "telnet", selector, host);
+					     "telnet", selector, (*selector ? "@" : ""), host);
 
 		} else if (entity == GOPHER_TN3270) {
+			if (*selector == '/') {
+				++selector;
+			}
 			add_format_to_string(&address, format,
-					     "tn3270", selector, host);
+					     "tn3270", selector, (*selector ? "@" : ""), host);
 
 		} else {
 			add_format_to_string(&address, "gopher://%s/%c",
 				host, entity);
 
-			/* Ensure we put a slash after entity type */
-			if (selector[0] != '/')
-				add_char_to_string(&address, '/');
-
 			/* Encode selector string */
 			encode_selector_string(&address, selector);
 		}
@@ -577,11 +609,12 @@ add_gopher_menu_line(struct string *buffer, char *line)
 
 
 /* Search for line ending \r\n pair */
-static char *
+char *
 get_gopher_line_end(char *data, int datalen)
 {
+	ELOG
 	for (; datalen >= 1; data++, datalen--) {
-		if (data[0] == ASCII_CR && data[1] == ASCII_LF)
+		if (data[0] == ASCII_CR && datalen > 1 && data[1] == ASCII_LF)
 			return data + 2;
 
 		if (data[0] == ASCII_LF)
@@ -591,9 +624,10 @@ get_gopher_line_end(char *data, int datalen)
 	return NULL;
 }
 
-static inline char *
+static char *
 check_gopher_last_line(char *line, char *end)
 {
+	ELOG
 	assert(line < end);
 
 	return line[0] == '.' && !line[1] ? NULL : line;
@@ -603,6 +637,7 @@ check_gopher_last_line(char *line, char *end)
 static struct connection_state
 read_gopher_directory_data(struct connection *conn, struct read_buffer *rb)
 {
+	ELOG
 	struct connection_state state = connection_state(S_TRANS);
 	struct string buffer;
 	char *end;
@@ -630,6 +665,7 @@ read_gopher_directory_data(struct connection *conn, struct read_buffer *rb)
 		add_gopher_menu_line(&buffer, line);
 		conn->received += end - rb->data;
 		kill_buffer_data(rb, end - rb->data);
+		rb->data[rb->length] = '\0';
 	}
 
 	if (!is_in_state(state, S_TRANS)
@@ -651,6 +687,7 @@ read_gopher_directory_data(struct connection *conn, struct read_buffer *rb)
 static struct cache_entry *
 init_gopher_cache_entry(struct connection *conn)
 {
+	ELOG
 	struct gopher_connection_info *gopher = (struct gopher_connection_info *)conn->info;
 	struct cache_entry *cached = get_cache_entry(conn->uri);
 
@@ -673,6 +710,7 @@ init_gopher_cache_entry(struct connection *conn)
 static struct connection_state
 init_gopher_index_cache_entry(struct connection *conn)
 {
+	ELOG
 	char *where;
 	struct string buffer;
 
@@ -720,6 +758,7 @@ init_gopher_index_cache_entry(struct connection *conn)
 static void
 read_gopher_response_data(struct socket *socket, struct read_buffer *rb)
 {
+	ELOG
 	struct connection *conn = (struct connection *)socket->conn;
 	struct gopher_connection_info *gopher = (struct gopher_connection_info *)conn->info;
 	struct connection_state state = connection_state(S_TRANS);
@@ -789,6 +828,7 @@ read_gopher_response_data(struct socket *socket, struct read_buffer *rb)
 static void
 send_gopher_command(struct socket *socket)
 {
+	ELOG
 	struct connection *conn = (struct connection *)socket->conn;
 	struct gopher_connection_info *gopher = (struct gopher_connection_info *)conn->info;
 
@@ -802,6 +842,13 @@ send_gopher_command(struct socket *socket)
 void
 gopher_protocol_handler(struct connection *conn)
 {
+	ELOG
+#if defined(CONFIG_LIBCURL)
+	if (get_opt_bool("protocol.gopher.use_curl", NULL)) {
+		gophers_protocol_handler(conn);
+		return;
+	}
+#endif
 	struct uri *uri = conn->uri;
 	struct connection_state state = connection_state(S_CONN);
 
