@@ -29,6 +29,7 @@ enum protocol {
 	PROTOCOL_HTTPS,
 	PROTOCOL_JAVASCRIPT,
 	PROTOCOL_MAILCAP,
+	PROTOCOL_MAILCAP_HTML,
 	PROTOCOL_NEWS,
 	PROTOCOL_NNTP,
 	PROTOCOL_NNTPS,
@@ -36,6 +37,7 @@ enum protocol {
 	PROTOCOL_SFTP,
 	PROTOCOL_SMB,
 	PROTOCOL_SNEWS,
+	PROTOCOL_SPARTAN,
 
 	/* Keep these last! */
 	PROTOCOL_UNKNOWN,
