@@ -13,6 +13,8 @@
 #include "elinks.h"
 
 #include "config/home.h"
+#include "config/options.h"
+#include "config/opttypes.h"
 #include "main/main.h"
 #include "main/module.h"
 #include "scripting/python/core.h"
@@ -38,6 +40,7 @@ PyObject *python_hooks = NULL;
 void
 alert_python_error(void)
 {
+	ELOG
 	const char *msg = "(no traceback available)";
 	PyObject *err_type = NULL, *err_value = NULL, *err_traceback = NULL;
 	PyObject *tb_module = NULL;
@@ -98,6 +101,7 @@ end:
 static int
 set_python_search_path(void)
 {
+	ELOG
 	struct string new_python_path;
 	char *old_python_path;
 	int result = -1;
@@ -129,6 +133,7 @@ end:
 static int
 hooks_module_exists(void)
 {
+	ELOG
 	PyObject *imp_module = NULL, *result = NULL;
 	int found_hooks = 0;
 
@@ -179,6 +184,7 @@ All other arguments are ignored.\n");
 static PyCFunction *
 python_showwarning(PyObject *self, PyObject *args, PyObject *kwargs)
 {
+	ELOG
 	PyObject *warning;
 
 	if (!PyTuple_Check(args)) {
@@ -204,6 +210,7 @@ static PyMethodDef warning_methods[] = {
 static int
 replace_showwarning(void)
 {
+	ELOG
 	PyObject *warnings_module = NULL, *module_name = NULL, *module_dict;
 	int result = -1;
 
@@ -225,8 +232,142 @@ end:
 	return result;
 }
 
+static PyObject *
+python_get_option(PyObject *self, PyObject *args)
+{
+	const char *name;
+
+	if (!PyArg_ParseTuple(args, "s", &name)) {
+		return NULL;
+	}
+	struct option *opt = get_opt_rec(config_options, (char *)name);
+
+	if (!opt) {
+		return NULL;
+	}
+	/* Convert to an appropriate Python type */
+	switch (opt->type) {
+	case OPT_BOOL:
+		return PyBool_FromLong(opt->value.number);
+	case OPT_INT:
+		return PyLong_FromLong(opt->value.number);
+	case OPT_LONG:
+		return PyLong_FromLongLong(opt->value.big_number);
+	case OPT_STRING:
+		return PyUnicode_FromString(opt->value.string);
+	case OPT_CODEPAGE:
+	{
+		const char *cp_name;
+
+		cp_name = get_cp_config_name(opt->value.number);
+		return PyUnicode_FromString(cp_name);
+	}
+	case OPT_LANGUAGE:
+	{
+		const char *lang;
+
+#ifdef ENABLE_NLS
+		lang = language_to_name(current_language);
+#else
+		lang = "System";
+#endif
+		return PyUnicode_FromString(lang);
+	}
+	case OPT_COLOR:
+	{
+		color_T color;
+		char hexcolor[8];
+		const char *strcolor;
+
+		color = opt->value.color;
+		strcolor = get_color_string(color, hexcolor);
+
+		return PyUnicode_FromString(strcolor);
+	}
+	case OPT_COMMAND:
+	default:
+		Py_RETURN_NONE;
+	}
+}
+
+static PyObject *
+python_set_option(PyObject *self, PyObject *args)
+{
+	const char *name;
+	PyObject *v;
+
+	if (!PyArg_ParseTuple(args, "sO", &name, &v)) {
+		return NULL;
+	}
+	/* Get option record */
+	struct option *opt = get_opt_rec(config_options, (char *)name);
+	if (opt == NULL) {
+		Py_RETURN_NONE;
+	}
+
+	/* Set option */
+	switch (opt->type) {
+	case OPT_BOOL:
+	{
+		/* option_types[OPT_BOOL].set expects a long even though it
+		 * saves the value to opt->value.number, which is an int.  */
+		long value = PyLong_AsLong(v);
+		option_types[opt->type].set(opt, (char *)(&value));
+		break;
+	}
+	case OPT_INT:
+	case OPT_LONG:
+	{
+		/* option_types[OPT_INT].set expects a long even though it
+		 * saves the value to opt->value.number, which is an int.
+		 * option_types[OPT_LONG].set of course wants a long too.  */
+		long value = PyLong_AsLong(v);
+		option_types[opt->type].set(opt, (char *)(&value));
+		break;
+	}
+	case OPT_STRING:
+	case OPT_CODEPAGE:
+	case OPT_LANGUAGE:
+	case OPT_COLOR:
+		if (!PyUnicode_Check(v)) {
+			Py_RETURN_NONE;
+		}
+		PyObject *utf8 = PyUnicode_AsEncodedString(v, "utf-8", NULL);
+		const char *text = PyBytes_AsString(utf8);
+		if (text) {
+			option_types[opt->type].set(opt, (char *)text);
+		}
+		Py_XDECREF(utf8);
+		break;
+	default:
+		Py_RETURN_NONE;
+	}
+	/* Call hook */
+	option_changed(python_ses, opt);
+	Py_RETURN_NONE;
+}
+
 /* Module-level documentation for the Python interpreter's elinks module. */
 
+char python_get_option_doc[] =
+PYTHON_DOCSTRING("get_option(option)\n\
+\n\
+Return value of given option.\n\
+\n\
+Arguments:\n\
+\n\
+option -- A string containing option name.\n");
+
+char python_set_option_doc[] =
+PYTHON_DOCSTRING("set_option(option, value) -> None\n\
+\n\
+Set value for given option.\n\
+\n\
+Arguments:\n\
+\n\
+option -- A string containing option name.\n\
+value -- A value\n");
+
 static char module_doc[] =
 PYTHON_DOCSTRING("Interface to the ELinks web browser.\n\
 \n\
@@ -286,6 +427,10 @@ static PyMethodDef python_methods[] = {
 				METH_VARARGS | METH_KEYWORDS,
 				python_bind_key_doc},
 
+	{"get_option",		python_get_option,
+				METH_VARARGS,
+				python_get_option_doc},
+
 	{"load",		python_load,
 				METH_VARARGS,
 				python_load_doc},
@@ -298,6 +443,10 @@ static PyMethodDef python_methods[] = {
 				METH_VARARGS | METH_KEYWORDS,
 				python_open_doc},
 
+	{"set_option",		python_set_option,
+				METH_VARARGS,
+				python_set_option_doc},
+
 	{NULL,			NULL, 0, NULL}
 };
 
@@ -316,6 +465,7 @@ static struct PyModuleDef moduledef = {
 static int
 add_constant(PyObject *dict, const char *key, int value)
 {
+	ELOG
 	PyObject *constant = PyLong_FromLong(value);
 	int result;
 
@@ -329,6 +479,7 @@ add_constant(PyObject *dict, const char *key, int value)
 PyMODINIT_FUNC
 PyInit_elinks(void)
 {
+	ELOG
 	PyObject *elinks_module, *module_dict, *module_name;
 	char *xdg_config_home = get_xdg_config_home();
 
@@ -384,6 +535,7 @@ python_error:
 void
 init_python(struct module *module)
 {
+	ELOG
 	if (set_python_search_path() != 0) {
 		return;
 	}
@@ -408,6 +560,7 @@ python_error:
 void
 cleanup_python(struct module *module)
 {
+	ELOG
 	if (Py_IsInitialized()) {
 		PyObject *temp;
 
@@ -428,6 +581,7 @@ cleanup_python(struct module *module)
 int
 add_python_methods(PyObject *dict, PyObject *name, PyMethodDef *methods)
 {
+	ELOG
 	PyMethodDef *method;
 
 	for (method = methods; method && method->ml_name; method++) {
