@@ -10,8 +10,8 @@
 
 #include "bfu/msgbox.h"
 #include "config/home.h"
-#include "ecmascript/spidermonkey-shared.h"
-#include "ecmascript/spidermonkey/util.h"
+#include "js/spidermonkey-shared.h"
+#include "js/spidermonkey/util.h"
 #include "intl/libintl.h"
 #include "protocol/uri.h"
 #include "scripting/scripting.h"
@@ -143,7 +143,7 @@ smjs_get_elinks_object(void)
 	return spidermonkey_InitClass(smjs_ctx, obj, NULL,
 	                              (JSClass *) &elinks_class, NULL, 0,
 	                              (JSPropertySpec *) elinks_props,
-	                              elinks_funcs, NULL, NULL);
+	                              elinks_funcs, NULL, NULL, "elinks");
 }
 
 void
