@@ -10,7 +10,7 @@
 
 #include "elinks.h"
 
-#include "ecmascript/spidermonkey/util.h"
+#include "js/spidermonkey/util.h"
 #include "main/select.h"
 #include "protocol/uri.h"
 #include "scripting/smjs/core.h"
@@ -1134,5 +1134,5 @@ smjs_init_session_interface(void)
 	 spidermonkey_InitClass(smjs_ctx, obj, NULL,
 	                        (JSClass *) &session_class, session_construct,
 	                        0, (JSPropertySpec *) session_props,
-	                        session_funcs, NULL, NULL);
+	                        session_funcs, NULL, NULL, "session");
 }
