@@ -132,6 +132,7 @@ struct session_status {
 	struct led *ecmascript_led;
 	struct led *popup_led;
 	struct led *download_led;
+	struct led *onion_led;
 #endif
 	/** Has the tab been visited yet. */
 	unsigned int visited:1;
@@ -315,6 +316,7 @@ void free_files(struct session *);
 void display_timer(struct session *ses);
 void load_common(struct session *ses);
 
+struct uri *get_open_onion_location(struct session *ses);
 
 /** session_is_loading() is like !!get_current_download() but doesn't take
  * session.req_sent into account.
@@ -324,6 +326,7 @@ struct download *get_current_download(struct session *ses);
 
 /** Information about the current document */
 char *get_current_url(struct session *, char *, size_t);
+char *get_current_url_with_pos(struct session *, char *, size_t);
 char *get_current_title(struct session *, char *, size_t);
 
 struct link *get_current_session_link(struct session *ses);
