@@ -16,11 +16,16 @@
 #include <sys/socket.h>
 #endif
 
+#ifdef CONFIG_LIBUV
+#include <uv.h>
+#endif
+
 #include "elinks.h"
 
 #include "intl/libintl.h"
 #include "main/main.h"			/* terminate */
 #include "main/object.h"
+#include "main/select.h"
 #include "session/session.h"
 #include "terminal/draw.h"
 #include "terminal/event.h"
@@ -68,6 +73,7 @@ struct terminal_interlink {
 void
 term_send_event(struct terminal *term, struct term_event *ev)
 {
+	ELOG
 	struct window *win;
 
 	assert(ev && term);
@@ -81,16 +87,14 @@ term_send_event(struct terminal *term, struct term_event *ev)
 		int height = ev->info.size.height;
 
 		if (width < 0 || height < 0) {
-			ERROR(gettext("Bad terminal size: %d, %d"),
+			EL_ERROR(gettext("Bad terminal size: %d, %d"),
 			      width, height);
 			break;
 		}
 
 		resize_screen(term, width, height);
-#ifdef CONFIG_LIBSIXEL
 		term->cell_width = ev->info.size.cell_width;
 		term->cell_height = ev->info.size.cell_height;
-#endif
 		erase_screen(term);
 		/* Fall through */
 	}
@@ -147,6 +151,7 @@ static void
 term_send_ucs(struct terminal *term, unicode_val_T u,
 	      term_event_modifier_T modifier)
 {
+	ELOG
 #ifdef CONFIG_UTF8
 	struct term_event ev;
 
@@ -170,6 +175,7 @@ term_send_ucs(struct terminal *term, unicode_val_T u,
 static void
 check_terminal_name(struct terminal *term, struct terminal_info *info)
 {
+	ELOG
 	char name[MAX_TERM_LEN + 10];
 	int i;
 
@@ -206,6 +212,7 @@ check_terminal_name(struct terminal *term, struct terminal_info *info)
 static int
 ignore_mouse_event(struct terminal *term, struct term_event *ev)
 {
+	ELOG
 	struct term_event_mouse *prev = &term->prev_mouse_event;
 	struct term_event_mouse *current = &ev->info.mouse;
 
@@ -226,6 +233,7 @@ ignore_mouse_event(struct terminal *term, struct term_event *ev)
 static int
 handle_interlink_event(struct terminal *term, struct interlink_event *ilev)
 {
+	ELOG
 	struct terminal_info *info = NULL;
 	struct terminal_interlink *interlink = term->interlink;
 	struct term_event tev;
@@ -456,7 +464,7 @@ invalid_utf8_start_byte:
 		return 0;
 
 	default:
-		ERROR(gettext("Bad event %d"), ilev->ev);
+		EL_ERROR(gettext("Bad event %d"), ilev->ev);
 	}
 
 	/* For EVENT_INIT we read a liitle more */
@@ -467,6 +475,7 @@ invalid_utf8_start_byte:
 void
 in_term(struct terminal *term)
 {
+	ELOG
 	struct terminal_interlink *interlink = term->interlink;
 	ssize_t r;
 	char *iq;
@@ -499,7 +508,7 @@ in_term(struct terminal *term)
 	r = safe_read(term->fdin, iq + interlink->qlen, interlink->qfreespace);
 	if (r <= 0) {
 		if (r == -1 && errno != ECONNRESET)
-			ERROR(gettext("Could not read event: %d (%s)"),
+			EL_ERROR(gettext("Could not read event: %d (%s)"),
 			      errno, (char *) strerror(errno));
 
 		destroy_terminal(term);
@@ -527,3 +536,107 @@ in_term(struct terminal *term)
 		memmove(iq, iq + event_size, interlink->qlen);
 	}
 }
+
+#ifdef CONFIG_LIBUV
+void
+read_interm_cb(uv_stream_t *stream, ssize_t r, const uv_buf_t *buf)
+{
+	ELOG
+	struct libuv_priv *priv = (struct libuv_priv *)uv_handle_get_data((uv_handle_t *)stream);
+
+	if (!priv) {
+		return;
+	}
+	int fd = priv->fd;
+	struct terminal *term = (struct terminal *)get_handler_data(fd);
+
+	if (!term) {
+		return;
+	}
+
+	if (r == UV_EOF) {
+		mem_free_set(&threads[fd].handle, NULL);
+		mem_free(priv);
+		mem_free_if(buf->base);
+		check_bottom_halves();
+		return;
+	}
+
+	if (r <= 0) {
+		if (r == -1 && errno != ECONNRESET)
+			EL_ERROR(gettext("Could not read event: %d (%s)"),
+			      errno, (char *) strerror(errno));
+
+		destroy_terminal(term);
+		check_bottom_halves();
+		return;
+	}
+	char *iq;
+
+	/* Mark this as the most recently used terminal.  */
+	move_to_top_of_list(terminals, term);
+	struct terminal_interlink *interlink = term->interlink;
+
+	if (!interlink
+	    || !interlink->qfreespace
+	    || interlink->qfreespace < r
+	    || interlink->qfreespace - interlink->qlen > ALLOC_GR) {
+		int qlen = interlink ? interlink->qlen : 0;
+		int queuesize = ((qlen + ALLOC_GR) & ~(ALLOC_GR - 1));
+
+		if (queuesize < r) {
+			queuesize = r;
+		}
+		int newsize = sizeof(*interlink) + queuesize;
+
+		interlink = (struct terminal_interlink *)mem_realloc(interlink, newsize);
+		if (!interlink) {
+			mem_free(buf->base);
+			destroy_terminal(term);
+			check_bottom_halves();
+			return;
+		}
+
+		/* Blank the members for the first allocation */
+		if (!term->interlink)
+			memset(interlink, 0, sizeof(*interlink));
+
+		term->interlink = interlink;
+		interlink->qfreespace = queuesize - interlink->qlen;
+	}
+	iq = interlink->input_queue;
+	memcpy(iq + interlink->qlen, buf->base, r);
+	mem_free(buf->base);
+
+	interlink->qlen += r;
+	interlink->qfreespace -= r;
+
+//eof:
+	while (interlink->qlen >= sizeof(struct interlink_event)) {
+		struct interlink_event *ev = (struct interlink_event *) iq;
+		int event_size = handle_interlink_event(term, ev);
+
+		/* If the event was not handled save the bytes in the queue for
+		 * later in case more stuff is read later. */
+		if (!event_size) break;
+
+		/* Acount for the handled bytes */
+		interlink->qlen -= event_size;
+		interlink->qfreespace += event_size;
+
+		/* If there are no more bytes to handle stop else move next
+		 * event bytes to the front of the queue. */
+		if (!interlink->qlen) break;
+		memmove(iq, iq + event_size, interlink->qlen);
+	}
+	check_bottom_halves();
+}
+
+void
+alloc_interm_cb(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf)
+{
+	ELOG
+	buf->base = mem_alloc(suggested_size);
+	buf->len = suggested_size;
+}
+#endif
