@@ -26,6 +26,10 @@
 #define HPUX_PIPE	1
 #endif
 
+#ifdef CONFIG_LIBUV
+#include <uv.h>
+#endif
+
 #include "elinks.h"
 
 #include "config/options.h"
@@ -38,6 +42,9 @@
 #include "terminal/hardio.h"
 #include "terminal/itrm.h"
 #include "terminal/kbd.h"
+#ifdef CONFIG_KITTY
+#include "terminal/map.h"
+#endif
 #include "terminal/mouse.h"
 #include "terminal/terminal.h"
 #include "util/error.h"
@@ -48,7 +55,6 @@
 struct itrm *ditrm = NULL;
 
 static void free_itrm(struct itrm *);
-static void in_kbd(struct itrm *);
 static void in_sock(struct itrm *);
 static int process_queue(struct itrm *);
 static void handle_itrm_stdin(struct itrm *);
@@ -65,6 +71,7 @@ static const enum term_event_special_key dummy_term_event_special_key;
 int
 is_blocked(void)
 {
+	ELOG
 	return ditrm && ditrm->blocked;
 }
 
@@ -72,6 +79,7 @@ is_blocked(void)
 void
 free_all_itrms(void)
 {
+	ELOG
 	if (ditrm) free_itrm(ditrm);
 }
 
@@ -83,6 +91,7 @@ free_all_itrms(void)
 static void
 itrm_queue_write(struct itrm *itrm)
 {
+	ELOG
 	int written;
 	int qlen = int_min(itrm->out.queue.len, 128);
 
@@ -102,7 +111,7 @@ itrm_queue_write(struct itrm *itrm)
 			     get_handler(itrm->out.sock, SELECT_HANDLER_READ),
 			     NULL,
 			     get_handler(itrm->out.sock, SELECT_HANDLER_ERROR),
-			     get_handler_data(itrm->out.sock));
+			     get_handler_data(itrm->out.sock), EL_TYPE_PIPE);
 	} else {
 		assert(itrm->out.queue.len > 0);
 		memmove(itrm->out.queue.data, itrm->out.queue.data + written, itrm->out.queue.len);
@@ -113,6 +122,7 @@ itrm_queue_write(struct itrm *itrm)
 void
 itrm_queue_event(struct itrm *itrm, char *data, int len)
 {
+	ELOG
 	int w = 0;
 
 	if (!len) return;
@@ -141,7 +151,7 @@ itrm_queue_event(struct itrm *itrm, char *data, int len)
 		set_handlers(itrm->out.sock,
 			     get_handler(itrm->out.sock, SELECT_HANDLER_READ),
 			     (select_handler_T) itrm_queue_write,
-			     (select_handler_T) free_itrm, itrm);
+			     (select_handler_T) free_itrm, itrm, EL_TYPE_PIPE);
 	}
 }
 
@@ -149,6 +159,7 @@ itrm_queue_event(struct itrm *itrm, char *data, int len)
 void
 kbd_ctrl_c(void)
 {
+	ELOG
 	struct interlink_event ev;
 
 	if (!ditrm) return;
@@ -170,6 +181,7 @@ kbd_ctrl_c(void)
 static void
 send_init_sequence(int h, int altscreen)
 {
+	ELOG
 #ifdef CONFIG_OS_DOS
 	save_terminal();
 #endif
@@ -199,6 +211,7 @@ send_init_sequence(int h, int altscreen)
 static void
 send_done_sequence(int h, int altscreen)
 {
+	ELOG
 	want_draw();
 #ifndef CONFIG_OS_DOS
 	write_sequence(h, DONE_BRACKETED_PASTE_SEQ);
@@ -227,6 +240,7 @@ send_done_sequence(int h, int altscreen)
 void
 resize_terminal(void)
 {
+	ELOG
 	struct interlink_event ev;
 	int width, height;
 	int cell_width, cell_height;
@@ -239,6 +253,7 @@ resize_terminal(void)
 void
 get_terminal_name(char name[MAX_TERM_LEN])
 {
+	ELOG
 	char *term = getenv("TERM");
 	int i;
 
@@ -254,6 +269,7 @@ get_terminal_name(char name[MAX_TERM_LEN])
 static int
 setraw(struct itrm *itrm, int save_orig)
 {
+	ELOG
 	struct termios t;
 
 	memset(&t, 0, sizeof(t));
@@ -318,6 +334,7 @@ void
 handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 	   void *init_string, int init_len, int remote)
 {
+	ELOG
 	struct itrm *itrm;
 	struct terminal_info info;
 	struct interlink_event_size *size = &info.event.info.size;
@@ -350,6 +367,7 @@ handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 		mem_free(itrm);
 		return;
 	}
+	itrm->in.queue.size = ITRM_IN_QUEUE_SIZE;
 
 	ditrm = itrm;
 	itrm->in.std = std_in;
@@ -383,9 +401,10 @@ handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 		if (std_in >= 0) handle_itrm_stdin(itrm);
 	}
 
-	if (sock_in != std_out)
+	if (sock_in != std_out) {
 		set_handlers(sock_in, (select_handler_T) in_sock,
-			     NULL, (select_handler_T) free_itrm, itrm);
+			     NULL, (select_handler_T) free_itrm, itrm, EL_TYPE_TCP);
+	}
 
 	get_terminal_name(info.name);
 
@@ -406,6 +425,7 @@ handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 static void
 unblock_itrm_x(void *h)
 {
+	ELOG
 	close_handle(h);
 	if (!ditrm) return;
 	unblock_itrm();
@@ -418,6 +438,7 @@ unblock_itrm_x(void *h)
 int
 unblock_itrm(void)
 {
+	ELOG
 	if (!ditrm) return -1;
 
 	if (ditrm->in.ctl >= 0 && setraw(ditrm, 0)) return -1;
@@ -437,6 +458,7 @@ unblock_itrm(void)
 void
 block_itrm(void)
 {
+	ELOG
 	if (!ditrm) return;
 
 	ditrm->blocked = 1;
@@ -450,10 +472,10 @@ block_itrm(void)
 	suspend_mouse(ditrm->mouse_h);
 }
 
-
 static void
 free_itrm(struct itrm *itrm)
 {
+	ELOG
 	if (!itrm) return;
 
 	if (!itrm->remote) {
@@ -505,6 +527,7 @@ free_itrm(struct itrm *itrm)
 static inline void
 resize_terminal_from_str(const char *text_)
 {
+	ELOG
 	enum { NEW_WIDTH = 0, NEW_HEIGHT, OLD_WIDTH, OLD_HEIGHT, NUMBERS };
 	int numbers[NUMBERS];
 	int i;
@@ -544,6 +567,7 @@ resize_terminal_from_str(const char *text_)
 void
 dispatch_special(const char *text)
 {
+	ELOG
 	switch (text[0]) {
 		case TERM_FN_TITLE:
 			if (ditrm) {
@@ -589,6 +613,7 @@ dispatch_special(const char *text)
 static void inline
 safe_hard_write(int fd, const char *buf, int len)
 {
+	ELOG
 	if (is_blocked()) return;
 
 	want_draw();
@@ -602,6 +627,7 @@ safe_hard_write(int fd, const char *buf, int len)
 static void
 in_sock(struct itrm *itrm)
 {
+	ELOG
 	struct string path;
 	struct string delete_;
 	char ch;
@@ -710,11 +736,11 @@ has_nul_byte:
 		if (fg == TERM_EXEC_FG) {
 			set_handlers(blockh, (select_handler_T) unblock_itrm_x,
 				     NULL, (select_handler_T) unblock_itrm_x,
-				     (void *) (intptr_t) blockh);
+				     (void *) (intptr_t) blockh, EL_TYPE_PIPE);
 
 		} else {
 			set_handlers(blockh, close_handle, NULL, close_handle,
-				     (void *) (intptr_t) blockh);
+				     (void *) (intptr_t) blockh, EL_TYPE_PIPE);
 		}
 	}
 
@@ -734,6 +760,7 @@ free_and_return:
 static int
 get_esc_code(unsigned char *str, int len, unsigned char *code, int *num, int *el)
 {
+	ELOG
 	int pos;
 	*num = 0;
 	for (pos = 2; pos < len; pos++) {
@@ -761,9 +788,78 @@ int ui_double_esc;
 static inline int
 get_ui_double_esc(void)
 {
+	ELOG
 	return ui_double_esc;
 }
 
+#ifdef CONFIG_KITTY
+static int
+decode_kitty_escape_sequence(struct itrm *itrm)
+{
+	ELOG
+	int i = 3;
+	int was_esc = 0;
+
+	if (itrm->in.queue.len < 3) {
+		return -1;
+	}
+
+	if (itrm->in.queue.data[2] != 'G') {
+		return -1;
+	}
+
+	for (i = 3; i < itrm->in.queue.len; i++) {
+		if (itrm->in.queue.data[i] == ASCII_ESC) {
+			was_esc = i + 1;
+			break;
+		}
+	}
+	if (!was_esc) {
+		return -1;
+	}
+
+	if (was_esc >= itrm->in.queue.len || itrm->in.queue.data[was_esc] != '\\') {
+		return -1;
+	}
+	itrm->in.queue.data[was_esc] = '\0';
+	const char *data = (const char *)(itrm->in.queue.data + 3);
+	char *ok = strstr(data, "OK");
+	int id = -1;
+	int ID = -1;
+
+	if (ok) {
+		char *ieq = strstr(data, "i=");
+
+		if (ieq) {
+			id = atoi(ieq + 2);
+		}
+		char *Ieq = strstr(data, "I=");
+		if (Ieq) {
+			ID = atoi(Ieq + 2);
+		}
+		if (id >= 0 && ID >= 0) {
+			set_id_ID(id, ID);
+		}
+	} else {
+		char *enoent = strstr(data, "ENOENT");
+
+		if (enoent) {
+			char *ieq = strstr(data, "i=");
+
+			if (ieq) {
+				id = atoi(ieq + 2);
+			}
+			if (id >= 0) {
+				remove_id(id);
+			}
+		}
+	}
+	itrm->in.queue.data[was_esc] = '\\';
+
+	return was_esc + 1;
+}
+#endif
+
 /** Decode a control sequence that begins with CSI (CONTROL SEQUENCE
  * INTRODUCER) encoded as ESC [, and set @a *ev accordingly.
  * (ECMA-48 also allows 0x9B as a single-byte CSI, but we don't
@@ -777,6 +873,7 @@ get_ui_double_esc(void)
 static int
 decode_terminal_escape_sequence(struct itrm *itrm, struct interlink_event *ev)
 {
+	ELOG
 	struct term_event_keyboard kbd = { KBD_UNDEF, KBD_MOD_NONE };
 	unsigned char c;
 	int v;
@@ -814,7 +911,7 @@ decode_terminal_escape_sequence(struct itrm *itrm, struct interlink_event *ev)
 		 * kbd_timeout might call in_kbd and thus process_input
 		 * and come right back here.  Better just reject the whole
 		 * thing and let the initial CSI be handled as Alt-[.  */
-		if (itrm->in.queue.len == ITRM_IN_QUEUE_SIZE)
+		if (itrm->in.queue.len == itrm->in.queue.size)
 			return 0;
 		else
 			return -1;
@@ -959,6 +1056,7 @@ decode_terminal_escape_sequence(struct itrm *itrm, struct interlink_event *ev)
 static int
 decode_terminal_application_key(struct itrm *itrm, struct interlink_event *ev)
 {
+	ELOG
 	unsigned char c;
 	struct interlink_event_keyboard kbd = { KBD_UNDEF, KBD_MOD_NONE };
 
@@ -1011,6 +1109,7 @@ static void
 set_kbd_event(const struct itrm *itrm, struct interlink_event *ev,
 	      int key, term_event_modifier_T modifier)
 {
+	ELOG
 	if (key == itrm->verase)
 		key = KBD_BS;
 	else switch (key) {
@@ -1045,6 +1144,7 @@ set_kbd_event(const struct itrm *itrm, struct interlink_event *ev,
 static void
 kbd_timeout(struct itrm *itrm)
 {
+	ELOG
 	struct interlink_event ev;
 	int el;
 
@@ -1087,6 +1187,7 @@ kbd_timeout(struct itrm *itrm)
 static int
 process_queue(struct itrm *itrm)
 {
+	ELOG
 	struct interlink_event ev;
 	int el = 0;
 
@@ -1137,6 +1238,10 @@ process_queue(struct itrm *itrm)
 			el = decode_terminal_escape_sequence(itrm, &ev);
 		} else if (itrm->in.queue.data[1] == 0x4F /* SS3 */) {
 			el = decode_terminal_application_key(itrm, &ev);
+#ifdef CONFIG_KITTY
+		} else if (itrm->in.queue.data[1] == '_') {
+			el = decode_kitty_escape_sequence(itrm);
+#endif
 		} else if (itrm->in.queue.data[1] == ASCII_ESC) {
 			/* ESC ESC can be either Alt-Esc or the
 			 * beginning of e.g. ESC ESC 0x5B 0x41,
@@ -1216,7 +1321,7 @@ return_without_event:
 		if (itrm->in.queue.len)
 			memmove(itrm->in.queue.data, itrm->in.queue.data + el, itrm->in.queue.len);
 
-		if (itrm->in.queue.len < ITRM_IN_QUEUE_SIZE)
+		if (itrm->in.queue.len < itrm->in.queue.size)
 			handle_itrm_stdin(itrm);
 
 		return el;
@@ -1226,37 +1331,101 @@ return_without_event:
 
 /** A select_handler_T read_func for itrm_in.std.  This is called when
  * characters typed by the user arrive from the terminal. */
-static void
+void
 in_kbd(struct itrm *itrm)
 {
+	ELOG
 	int r;
 
 	if (!can_read(itrm->in.std)) return;
 
 	kill_timer(&itrm->timer);
 
-	if (itrm->in.queue.len >= ITRM_IN_QUEUE_SIZE) {
+	if (itrm->in.queue.len >= itrm->in.queue.size) {
 		unhandle_itrm_stdin(itrm);
 		while (process_queue(itrm));
 		return;
 	}
 
 	r = safe_read(itrm->in.std, itrm->in.queue.data + itrm->in.queue.len,
-		      ITRM_IN_QUEUE_SIZE - itrm->in.queue.len);
+		      itrm->in.queue.size - itrm->in.queue.len);
 	if (r < 0) {
 		free_itrm(itrm);
 		return;
 	}
 
 	itrm->in.queue.len += r;
-	if (itrm->in.queue.len > ITRM_IN_QUEUE_SIZE) {
-		ERROR(gettext("Too many bytes read from the itrm!"));
-		itrm->in.queue.len = ITRM_IN_QUEUE_SIZE;
+	if (itrm->in.queue.len > itrm->in.queue.size) {
+		EL_ERROR(gettext("Too many bytes read from the itrm!"));
+		itrm->in.queue.len = itrm->in.queue.size;
 	}
 
 	while (process_queue(itrm));
 }
 
+#ifdef CONFIG_LIBUV
+void
+read_kbd_cb(uv_stream_t *stream, ssize_t nread, const uv_buf_t *buf)
+{
+	ELOG
+	struct libuv_priv *priv = (struct libuv_priv *)uv_handle_get_data((uv_handle_t *)stream);
+
+	if (!priv) {
+		return;
+	}
+	int fd = priv->fd;
+	struct itrm *itrm = (struct itrm *)get_handler_data(fd);
+
+	if (!itrm) {
+		return;
+	}
+	kill_timer(&itrm->timer);
+
+	if (nread < 0) {
+		uv_read_stop(stream);
+		unhandle_itrm_stdin(itrm);
+		while (process_queue(itrm));
+
+		if (nread == UV_EOF) {
+			mem_free_set(&threads[fd].handle, NULL);
+			mem_free(priv);
+			mem_free_if(buf->base);
+		}
+		check_bottom_halves();
+		return;
+	}
+
+	if (itrm->in.queue.len + nread > itrm->in.queue.size) {
+		unsigned char *tmp = mem_realloc(itrm->in.queue.data, itrm->in.queue.len + nread);
+
+		if (!tmp) {
+			mem_free_if(buf->base);
+			uv_read_stop(stream);
+			unhandle_itrm_stdin(itrm);
+			while (process_queue(itrm));
+			check_bottom_halves();
+			return;
+		}
+		itrm->in.queue.data = tmp;
+		itrm->in.queue.size = itrm->in.queue.len + nread;
+	}
+	memcpy(itrm->in.queue.data + itrm->in.queue.len, buf->base, nread);
+	itrm->in.queue.len += nread;
+	mem_free_if(buf->base);
+
+	while (process_queue(itrm));
+	check_bottom_halves();
+}
+
+void
+alloc_kbd_cb(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf)
+{
+	ELOG
+	buf->base = mem_alloc(ITRM_IN_QUEUE_SIZE);
+	buf->len = ITRM_IN_QUEUE_SIZE;
+}
+#endif
+
 /** Enable reading from itrm_in.std.  ELinks will read any available
  * bytes from the tty into itrm->in.queue and then parse them.
  * Reading should be enabled whenever itrm->in.queue is not full and
@@ -1264,11 +1433,12 @@ in_kbd(struct itrm *itrm)
 static void
 handle_itrm_stdin(struct itrm *itrm)
 {
-	assert(itrm->in.std >= 0);
+	ELOG
+	assert(program.testjs || itrm->in.std >= 0);
 	if_assert_failed return;
 
 	set_handlers(itrm->in.std, (select_handler_T) in_kbd, NULL,
-		     (select_handler_T) free_itrm, itrm);
+		     (select_handler_T) free_itrm, itrm, EL_TYPE_TTY);
 }
 
 /** Disable reading from itrm_in.std.  Reading should be disabled
@@ -1277,9 +1447,10 @@ handle_itrm_stdin(struct itrm *itrm)
 static void
 unhandle_itrm_stdin(struct itrm *itrm)
 {
+	ELOG
 	assert(itrm->in.std >= 0);
 	if_assert_failed return;
 
 	set_handlers(itrm->in.std, (select_handler_T) NULL, NULL,
-		     (select_handler_T) free_itrm, itrm);
+		     (select_handler_T) free_itrm, itrm, EL_TYPE_TTY);
 }
