@@ -11,27 +11,28 @@ extern "C" {
 
 #ifdef CONFIG_LIBSIXEL
 struct document;
+struct el_box;
 struct terminal;
 
 
 struct image {
 	LIST_HEAD_EL(struct image);
 	struct string pixels;
-	int x;
-	int y;
+	int cx;
+	int cy;
 	int width;
 	int height;
+	int image_number;
 };
 
 void delete_image(struct image *im);
 
-void try_to_draw_images(struct terminal *term);
+void try_to_draw_images(struct terminal *term, struct string *text);
 
 /* return height of image in terminal rows */
-int add_image_to_document(struct document *doc, struct string *pixels, int lineno);
-
-struct image *copy_frame(struct image *src, int box_width, int box_height, int cell_width, int cell_height, int dx, int dy);
+int add_image_to_document(struct document *doc, char *data, int datalen, int lineno, struct image **imagine);
 
+struct image *copy_frame(struct image *src, struct el_box *box, int cell_width, int cell_height, int dx, int dy);
 
 #endif
 
