@@ -18,7 +18,8 @@ static unsigned char base64_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmno
 unsigned char *
 base64_encode(unsigned char *in)
 {
-	assert(in && *in);
+	ELOG
+	assert(in);
 	if_assert_failed return NULL;
 
 	return base64_encode_bin(in, strlen((const char*)in), NULL);
@@ -27,10 +28,11 @@ base64_encode(unsigned char *in)
 unsigned char *
 base64_encode_bin(unsigned char *in, int inlen, int *outlen)
 {
+	ELOG
 	unsigned char *out;
 	unsigned char *outstr;
 
-	assert(in && *in);
+	assert(in);
 	if_assert_failed return NULL;
 
 	out = outstr = (unsigned char *)mem_alloc((inlen / 3) * 4 + 4 + 1);
@@ -66,6 +68,7 @@ base64_encode_bin(unsigned char *in, int inlen, int *outlen)
 unsigned char *
 base64_decode(const unsigned char *in)
 {
+	ELOG
 	assert(in && *in);
 	if_assert_failed return NULL;
 
@@ -82,6 +85,7 @@ base64_decode(const unsigned char *in)
 unsigned char *
 base64_decode_bin(const unsigned char *in, int inlen, int *outlen)
 {
+	ELOG
 	static unsigned char is_base64_char[256]; /* static to force initialization at zero */
 	static unsigned char decode[256];
 	unsigned char *out;
