@@ -1,7 +1,7 @@
 #ifndef EL__UTIL_MEMCOUNT_H
 #define EL__UTIL_MEMCOUNT_H
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_MEMCOUNT
 
 #ifdef CONFIG_LIBSIXEL
 #include <sixel.h>
@@ -36,6 +36,15 @@ uint64_t get_gzip_size(void);
 uint64_t get_gzip_active(void);
 #endif
 
+#if defined(CONFIG_KITTY) || defined(CONFIG_LIBSIXEL)
+void *el_stbi_malloc(size_t size);
+void *el_stbi_realloc(void *p, size_t n);
+void el_stbi_free(void *p);
+uint64_t get_stbi_total_allocs(void);
+uint64_t get_stbi_size(void);
+uint64_t get_stbi_active(void);
+#endif
+
 #ifdef CONFIG_LIBCURL
 void *el_curl_malloc(size_t size);
 void *el_curl_calloc(size_t nelm, size_t elsize);
@@ -66,6 +75,16 @@ uint64_t get_sixel_size(void);
 uint64_t get_sixel_active(void);
 #endif
 
+#ifdef CONFIG_LIBUV
+void *el_libuv_malloc(size_t size);
+void *el_libuv_calloc(size_t nelm, size_t elsize);
+void *el_libuv_realloc(void *p, size_t n);
+void el_libuv_free(void *p);
+uint64_t get_libuv_total_allocs(void);
+uint64_t get_libuv_size(void);
+uint64_t get_libuv_active(void);
+#endif
+
 #ifdef CONFIG_MUJS
 void *el_mujs_alloc(void *memctx, void *ptr, int size);
 uint64_t get_mujs_total_allocs(void);
