@@ -23,6 +23,7 @@
 #include "dialogs/menu.h"
 #include "dialogs/options.h"
 #include "dialogs/status.h"
+#include "dialogs/tabs.h"
 #include "document/document.h"
 #include "document/view.h"
 #ifdef CONFIG_ECMASCRIPT
@@ -34,6 +35,7 @@
 #include "network/connection.h"
 #include "protocol/auth/auth.h"
 #include "protocol/auth/dialogs.h"
+#include "protocol/uri.h"
 #include "terminal/tab.h"
 #include "terminal/terminal.h"
 #include "terminal/window.h"
@@ -51,6 +53,7 @@ static void
 goto_url_action(struct session *ses,
 		char *(*get_url)(struct session *, char *, size_t))
 {
+	ELOG
 	char url[MAX_STR_LEN];
 
 	if (!get_url || !get_url(ses, url, sizeof(url)))
@@ -67,10 +70,12 @@ goto_url_action(struct session *ses,
 enum frame_event_status
 do_action(struct session *ses, main_action_T action_id, int verbose)
 {
+	ELOG
 	enum frame_event_status status = FRAME_EVENT_OK;
 	struct terminal *term = ses->tab->term;
 	struct document_view *doc_view = current_frame(ses);
 	struct link *link = NULL;
+	struct uri *onion = NULL;
 
 	if (action_id == -1) goto unknown_action;
 
@@ -571,6 +576,15 @@ do_action(struct session *ses, main_action_T action_id, int verbose)
 			open_in_new_window(term, (void *)send_open_new_window, ses);
 			break;
 
+		case ACT_MAIN_OPEN_ONION_LOCATION:
+			onion = get_open_onion_location(ses);
+
+			if (onion) {
+				goto_uri(ses, onion);
+				done_uri(onion);
+			}
+			break;
+
 		case ACT_MAIN_OPEN_OS_SHELL:
 			exec_shell(term);
 			break;
@@ -717,6 +731,10 @@ do_action(struct session *ses, main_action_T action_id, int verbose)
 			switch_current_tab(ses, -1);
 			break;
 
+		case ACT_MAIN_TAB_MANAGER:
+			tab_manager(ses);
+			break;
+
 		case ACT_MAIN_TERMINAL_RESIZE:
 			resize_terminal_dialog(term);
 			break;
@@ -731,6 +749,20 @@ do_action(struct session *ses, main_action_T action_id, int verbose)
 			toggle_document_option(ses, "document.browse.images.show_as_links");
 			break;
 
+		case ACT_MAIN_TOGGLE_DISPLAY_KITTY:
+#ifdef CONFIG_KITTY
+			toggle_document_option(ses, "document.html.kitty");
+			reload(ses, CACHE_MODE_INCREMENT);
+#endif
+			break;
+
+		case ACT_MAIN_TOGGLE_DISPLAY_SIXEL:
+#ifdef CONFIG_LIBSIXEL
+			toggle_document_option(ses, "document.html.sixel");
+			reload(ses, CACHE_MODE_INCREMENT);
+#endif
+			break;
+
 		case ACT_MAIN_TOGGLE_DISPLAY_TABLES:
 			toggle_document_option(ses, "document.html.display_tables");
 			break;
@@ -750,6 +782,13 @@ do_action(struct session *ses, main_action_T action_id, int verbose)
 #endif
 			break;
 
+		case ACT_MAIN_TOGGLE_ECMASCRIPT_KEYS:
+#if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
+			program.ecmascript_keys = !program.ecmascript_keys;
+			set_led_value(ses->status.ecmascript_led, program.ecmascript_keys ? 'A' : '-');
+#endif
+			break;
+
 		case ACT_MAIN_TOGGLE_HTML_PLAIN:
 			toggle_plain_html(ses, ses->doc_view, 0);
 			break;
