@@ -31,6 +31,10 @@
 #include "session/location.h"
 #include "session/session.h"
 #include "terminal/draw.h"
+#ifdef CONFIG_KITTY
+#include "terminal/kitty.h"
+#endif
+#include "terminal/screen.h"
 #ifdef CONFIG_LIBSIXEL
 #include "terminal/sixel.h"
 #endif
@@ -52,6 +56,7 @@
 static inline int
 check_document_fragment(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct document *document = doc_view->document;
 	struct uri *uri = doc_view->vs->uri;
 	int vy;
@@ -100,6 +105,7 @@ static void
 draw_frame_lines(struct terminal *term, struct frameset_desc *frameset_desc,
 		 int xp, int yp, struct color_pair *colors)
 {
+	ELOG
 	int y, j;
 
 	assert(term && frameset_desc && frameset_desc->frame_desc);
@@ -172,49 +178,10 @@ draw_frame_lines(struct terminal *term, struct frameset_desc *frameset_desc,
 	}
 }
 
-static void
-draw_iframe_lines(struct terminal *term, struct iframeset_desc *iframe_desc,
-		 int xp, int yp, struct color_pair *colors)
-{
-	int j;
-
-	assert(term && iframe_desc && iframe_desc->iframe_desc);
-	if_assert_failed return;
-
-	for (j = 0; j < iframe_desc->n; j++) {
-		struct el_box box;
-
-		int y = yp + iframe_desc->iframe_desc[j].box.y - 1;
-		int x = xp + iframe_desc->iframe_desc[j].box.x - 1;
-
-		int height = iframe_desc->iframe_desc[j].box.height + 1;
-		int width = iframe_desc->iframe_desc[j].box.width + 1;
-
-		set_box(&box, x, y + 1, 1, height - 1);
-		draw_box(term, &box, BORDER_SVLINE, SCREEN_ATTR_FRAME, colors);
-
-		set_box(&box, x + width, y + 1, 1, height - 1);
-		draw_box(term, &box, BORDER_SVLINE, SCREEN_ATTR_FRAME, colors);
-
-		set_box(&box, x + 1, y, width -1 , 1);
-		draw_box(term, &box, BORDER_SHLINE, SCREEN_ATTR_FRAME, colors);
-
-		set_box(&box, x + 1, y + height, width - 1, 1);
-		draw_box(term, &box, BORDER_SHLINE, SCREEN_ATTR_FRAME, colors);
-
-		draw_border_char(term, x, y, BORDER_SULCORNER, colors);
-		draw_border_char(term, x, y + height, BORDER_SDLCORNER, colors);
-
-		draw_border_char(term, x + width, y, BORDER_SURCORNER, colors);
-		draw_border_char(term, x + width, y + height, BORDER_SDRCORNER, colors);
-		draw_border_cross(term, x, y, BORDER_X_DOWN, colors);
-	}
-}
-
-
 static void
 draw_clipboard(struct terminal *term, struct document_view *doc_view)
 {
+	ELOG
 	struct document *document = doc_view->document;
 	struct color_pair *color;
 	int starty, startx, endy, endx, x, y, xoffset, yoffset;
@@ -263,6 +230,7 @@ draw_clipboard(struct terminal *term, struct document_view *doc_view)
 static void
 draw_view_status(struct session *ses, struct document_view *doc_view, int active)
 {
+	ELOG
 	struct terminal *term = ses->tab->term;
 
 	draw_forms(term, doc_view);
@@ -278,6 +246,7 @@ draw_view_status(struct session *ses, struct document_view *doc_view, int active
 static void
 check_link_under_cursor(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int x = ses->tab->x;
 	int y = ses->tab->y;
 	struct el_box *box = &doc_view->box;
@@ -295,6 +264,7 @@ check_link_under_cursor(struct session *ses, struct document_view *doc_view)
 static void
 draw_doc(struct session *ses, struct document_view *doc_view, int active)
 {
+	ELOG
 	struct color_pair color;
 	struct view_state *vs;
 	struct terminal *term;
@@ -356,7 +326,6 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 		return;
 	}
 
-
 	if (ses->navigate_mode == NAVIGATE_LINKWISE) {
 		check_vs(doc_view);
 	} else {
@@ -393,10 +362,10 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 	if (doc_view->last_x != -1
 	    && doc_view->last_x == vx
 	    && doc_view->last_y == vy
-	    && !has_search_word(doc_view)
-            && !document_has_iframes(doc_view->document)) {
+	    && !has_search_word(doc_view)) {
 		clear_link(term, doc_view);
 		draw_view_status(ses, doc_view, active);
+
 		return;
 	}
 	doc_view->last_x = vx;
@@ -429,15 +398,17 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 		int st = int_max(vx, 0);
 		int en = int_min(doc_view->document->data[y].length,
 				 box->width + vx);
-		int max = int_min(en, st + 200);
+		int max = int_min(en, doc_view->document->width);
 
 		if (en - st > 0) {
-			draw_line(term, box->x + st - vx, box->y + y - vy,
+			if (st - vx >= 0) {
+				draw_line(term, box->x + st - vx, box->y + y - vy,
 				  en - st,
 				  &doc_view->document->data[y].ch.chars[st]);
+			}
 
 			for (i = en - 1; i >= 0; --i) {
-				if (doc_view->document->data[y].ch.chars[i].data != ' ') {
+				if (doc_view->document->data[y].ch.chars[i].data != ' ' && doc_view->document->data[y].ch.chars[i].data != UCS_NO_BREAK_SPACE) {
 					last = &doc_view->document->data[y].ch.chars[i];
 					last_index = i + 1;
 					break;
@@ -446,20 +417,18 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 		}
 
 		for (i = st; i < max; i++) {
-			if (doc_view->document->data[y].ch.chars[i].data != ' ') {
+			if (doc_view->document->data[y].ch.chars[i].data != ' '  && doc_view->document->data[y].ch.chars[i].data != UCS_NO_BREAK_SPACE) {
 				first = &doc_view->document->data[y].ch.chars[i];
 				break;
 			}
 		}
 
-		for (j = st; j < i; j++) {
-			draw_space(term, box->x + j - vx, box->y + y - vy,
-				   first);
+		for (j = int_max(st - vx, 0); j < i - vx; j++) {
+			draw_space(term, box->x + j, box->y + y - vy, first);
 		}
 
-		for (i = last_index; i < box->width + vx; i++) {
-			draw_space(term, box->x + i - vx, box->y + y - vy,
-				   last);
+		for (i = int_max(last_index - vx, 0); i < box->width; i++) {
+			draw_space(term, box->x + i, box->y + y - vy, last);
 		}
 	}
 #if 0
@@ -470,49 +439,168 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 	if (has_search_word(doc_view))
 		doc_view->last_x = doc_view->last_y = -1;
 
-	if (document_has_iframes(doc_view->document)) {
-		draw_iframe_lines(term, doc_view->document->iframe_desc, box->x, box->y, &color);
-//		if (vs->current_link == -1)
-//			vs->current_link = 0;
+#ifdef CONFIG_KITTY
+	{
+	struct terminal_screen *screen = term->screen;
+
+	if (!screen || !screen->was_dirty) {
+		return;
 	}
-#ifdef CONFIG_LIBSIXEL
-	while (!list_empty(term->images)) {
-		delete_image((struct image *)term->images.next);
+	int iii;
+
+	for (iii = 0; iii < term->number_of_images; iii++) {
+		struct k_image *im = term->k_images[iii];
+		int yend = im->cy + (im->height + term->cell_height - 1) / term->cell_height;
+
+		set_screen_dirty_image(term->screen, im->cy, yend);
+		mem_free(im);
 	}
+	mem_free_set(&term->k_images, NULL);
+	term->number_of_images = 0;
 
-	if (1) {
-		struct image *im;
+	if (term->kitty) {
+		struct k_image *im2;
+		struct line *data = doc_view->document->data;
 
-		foreach (im, doc_view->document->images) {
-			if (im->y >= vs->y + box->height) {
+		foreach (im2, doc_view->document->k_images) {
+			struct k_image im;
+
+			copy_struct(&im, im2);
+
+			if (im.cy >= doc_view->document->height) {
 				continue;
 			}
 
-			if (im->y + ((im->height + term->cell_height - 1) / term->cell_height) < vs->y) {
+			if (im.cy >= vs->y + box->height) {
 				continue;
 			}
 
-			if (im->x >= vs->x + box->width) {
+			if (im.cy + ((im.height + term->cell_height - 1) / term->cell_height) <= vs->y) {
 				continue;
 			}
 
-			if (im->x + ((im->width + term->cell_width - 1) / term->cell_width) < vs->x) {
+			int cx = 0;
+			int found = vs->plain;
+
+			if (!found) {
+				for (;cx < data[im.cy].length; cx++) {
+					if (im.number == data[im.cy].ch.chars[cx].number) {
+						found = 1;
+						break;
+					}
+				}
+
+				if (!found) {
+					continue;
+				}
+			}
+			im.cx += cx;
+
+			if (im.cx >= vs->x + box->width) {
 				continue;
 			}
 
-			struct image *im_copy = copy_frame(im, box->width, box->height, term->cell_width, term->cell_height, vs->x, vs->y);
+			if (im.cx + ((im.width + term->cell_width - 1) / term->cell_width) <= vs->x) {
+				continue;
+			}
+			struct k_image *im_copy = copy_k_frame(&im, box, term->cell_width, term->cell_height, vs->x, vs->y);
 
 			if (im_copy) {
+				im_copy->cx += box->x;
+				im_copy->cy += box->y;
+
+				struct k_image **tmp = mem_realloc(term->k_images, (1 + term->number_of_images) * (sizeof(struct k_image *)));
+
+				if (tmp) {
+					term->k_images = tmp;
+					term->k_images[term->number_of_images++] = im_copy;
+					im2->sent = 1;
+				} else {
+					mem_free(im_copy);
+				}
+			}
+		}
+	}
+	}
+#endif
+
+#ifdef CONFIG_LIBSIXEL
+	{
+	struct terminal_screen *screen = term->screen;
+
+	if (!screen || !screen->was_dirty) {
+		return;
+	}
+	while (!list_empty(term->images)) {
+		struct image *im = (struct image *)term->images.next;
+		int yend = im->cy + (im->height + term->cell_height - 1) / term->cell_height;
+
+		set_screen_dirty_image(term->screen, im->cy, yend);
+		delete_image(im);
+	}
+
+	if (term->sixel) {
+		struct image *im2;
+		struct line *data = doc_view->document->data;
+
+		foreach (im2, doc_view->document->images) {
+			struct image im;
+
+			copy_struct(&im, im2);
+
+			if (im.cy >= doc_view->document->height) {
+				continue;
+			}
+
+			if (im.cy >= vs->y + box->height) {
+				continue;
+			}
+
+			if (im.cy + ((im.height + term->cell_height - 1) / term->cell_height) <= vs->y) {
+				continue;
+			}
+
+			int x = 0;
+			int found = vs->plain;
+
+			if (!found) {
+				for (;x < data[im.cy].length; x++) {
+					if (im.image_number == data[im.cy].ch.chars[x].number) {
+						found = 1;
+						break;
+					}
+				}
+
+				if (!found) {
+					continue;
+				}
+			}
+			im.cx += x;
+
+			if (im.cx >= vs->x + box->width) {
+				continue;
+			}
+
+			if (im.cx + ((im.width + term->cell_width - 1) / term->cell_width) <= vs->x) {
+				continue;
+			}
+			struct image *im_copy = copy_frame(&im, box, term->cell_width, term->cell_height, vs->x, vs->y);
+
+			if (im_copy) {
+				im_copy->cx += box->x;
+				im_copy->cy += box->y;
 				add_to_list(term->images, im_copy);
 			}
 		}
 	}
+	}
 #endif
 }
 
 static void
 draw_frames(struct session *ses)
 {
+	ELOG
 	struct document_view *doc_view, *current_doc_view;
 	int *l;
 	int n, d;
@@ -554,32 +642,11 @@ draw_frames(struct session *ses)
 	};
 }
 
-static void
-draw_iframes(struct session *ses)
-{
-	struct document_view *doc_view, *current_doc_view;
-
-	assert(ses && ses->doc_view && ses->doc_view->document);
-	if_assert_failed {
-		return;
-	}
-
-	foreach (doc_view, ses->scrn_iframes) {
-	       doc_view->last_x = doc_view->last_y = -1;
-	}
-
-	current_doc_view = current_frame(ses);
-
-	foreach (doc_view, ses->scrn_iframes) {
-		draw_doc(ses, doc_view, doc_view == current_doc_view);
-	}
-}
-
-
 /** @todo @a rerender is ridiciously wound-up. */
 void
 draw_formatted(struct session *ses, int rerender)
 {
+	ELOG
 	assert(ses && ses->tab);
 	if_assert_failed return;
 
@@ -623,6 +690,7 @@ draw_formatted(struct session *ses, int rerender)
 void
 refresh_view(struct session *ses, struct document_view *doc_view, int frames)
 {
+	ELOG
 	/* If refresh_view() is being called because the value of a
 	 * form field has changed, @ses might not be in the current
 	 * tab: consider SELECT pop-ups behind which -remote loads
@@ -640,7 +708,6 @@ refresh_view(struct session *ses, struct document_view *doc_view, int frames)
 			draw_doc(ses, doc_view, 1);
 		}
 		if (frames) draw_frames(ses);
-		draw_iframes(ses);
 	}
 	print_screen_status(ses);
 }
