@@ -42,6 +42,7 @@
 #include "intl/charsets.h"
 #include "intl/libintl.h"
 #include "main/event.h"
+#include "main/main.h"
 #include "osdep/osdep.h"
 #include "protocol/uri.h"
 #include "session/download.h"
@@ -77,6 +78,7 @@ static enum frame_event_status move_clipboard_pos(struct session *ses, struct do
 void
 detach_formatted(struct document_view *doc_view)
 {
+	ELOG
 	assert(doc_view);
 	if_assert_failed return;
 
@@ -100,6 +102,7 @@ detach_formatted(struct document_view *doc_view)
 static void
 move_down(struct session *ses, struct document_view *doc_view, int type, int overlap)
 {
+	ELOG
 	int newpos;
 
 	assert(ses && doc_view && doc_view->vs);
@@ -130,6 +133,7 @@ move_down(struct session *ses, struct document_view *doc_view, int type, int ove
 static enum frame_event_status
 move_part_page_down(struct session *ses, struct document_view *doc_view, int overlap)
 {
+	ELOG
 	int oldy = doc_view->vs->y;
 	int count = eat_kbd_repeat_count(ses);
 
@@ -143,18 +147,21 @@ move_part_page_down(struct session *ses, struct document_view *doc_view, int ove
 enum frame_event_status
 move_page_down(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	return move_part_page_down(ses, doc_view, get_opt_int("document.browse.scrolling.vertical_overlap", ses));
 }
 
 enum frame_event_status
 move_half_page_down(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	return move_part_page_down(ses, doc_view, doc_view->box.height / 2);
 }
 
 enum frame_event_status
 move_current_top(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	return move_part_page_down(ses, doc_view, doc_view->box.height - ses->tab->y + 1);
 }
 
@@ -163,6 +170,7 @@ move_current_top(struct session *ses, struct document_view *doc_view)
 static void
 move_up(struct session *ses, struct document_view *doc_view, int type, int overlap)
 {
+	ELOG
 	assert(ses && doc_view && doc_view->vs);
 	if_assert_failed return;
 
@@ -191,6 +199,7 @@ move_up(struct session *ses, struct document_view *doc_view, int type, int overl
 enum frame_event_status
 move_part_page_up(struct session *ses, struct document_view *doc_view, int overlap)
 {
+	ELOG
 	int oldy = doc_view->vs->y;
 	int count = eat_kbd_repeat_count(ses);
 
@@ -204,12 +213,14 @@ move_part_page_up(struct session *ses, struct document_view *doc_view, int overl
 enum frame_event_status
 move_page_up(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	return move_part_page_up(ses, doc_view, get_opt_int("document.browse.scrolling.vertical_overlap", ses));
 }
 
 enum frame_event_status
 move_half_page_up(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	return move_part_page_up(ses, doc_view, doc_view->box.height / 2);
 }
 
@@ -217,6 +228,7 @@ enum frame_event_status
 move_link(struct session *ses, struct document_view *doc_view, int direction,
 	  int wraparound_bound, int wraparound_link)
 {
+	ELOG
 	int wraparound = 0;
 	int count;
 	int cur = doc_view->vs->current_link;
@@ -294,6 +306,7 @@ move_link(struct session *ses, struct document_view *doc_view, int direction,
 enum frame_event_status
 move_link_dir(struct session *ses, struct document_view *doc_view, int dir_x, int dir_y)
 {
+	ELOG
 	int count;
 
 	assert(ses && doc_view && doc_view->vs && doc_view->document);
@@ -324,24 +337,24 @@ move_link_dir(struct session *ses, struct document_view *doc_view, int dir_x, in
 }
 
 /*! @a steps > 0 -> down */
-enum frame_event_status
-vertical_scroll(struct session *ses, struct document_view *doc_view, int steps)
+static enum frame_event_status
+vertical_scroll_extended(struct session *ses, struct document_view *doc_view, int steps, int extended)
 {
+	ELOG
 	int y;
+	int max_height;
 
 	assert(ses && doc_view && doc_view->vs && doc_view->document);
 	if_assert_failed return FRAME_EVENT_OK;
 
 	y = doc_view->vs->y + steps;
-	if (steps > 0) {
-		/* DOWN */
-		int max_height = doc_view->document->height - doc_view->box.height;
 
-		if (doc_view->vs->y >= max_height) return FRAME_EVENT_OK;
-		int_upper_bound(&y, max_height);
+	if (extended) {
+		max_height = doc_view->document->height - 1;
+	} else {
+		max_height = int_max(doc_view->vs->y, doc_view->document->height - doc_view->box.height);
 	}
-
-	int_lower_bound(&y, 0);
+	int_bounds(&y, 0, max_height);
 
 	if (doc_view->vs->y == y) return FRAME_EVENT_OK;
 
@@ -358,10 +371,23 @@ vertical_scroll(struct session *ses, struct document_view *doc_view, int steps)
 	return FRAME_EVENT_REFRESH;
 }
 
+/*! @a steps > 0 -> down */
+enum frame_event_status
+vertical_scroll(struct session *ses, struct document_view *doc_view, int steps)
+{
+	ELOG
+	assert(ses && doc_view && doc_view->vs && doc_view->document);
+	if_assert_failed return FRAME_EVENT_OK;
+
+	return vertical_scroll_extended(ses, doc_view, steps,
+		get_opt_bool("document.browse.scrolling.vertical_extended", ses));
+}
+
 /*! @a steps > 0 -> right */
 enum frame_event_status
 horizontal_scroll_extended(struct session *ses, struct document_view *doc_view, int steps, int extended)
 {
+	ELOG
 	int x, max;
 
 	assert(ses && doc_view && doc_view->vs && doc_view->document);
@@ -390,9 +416,10 @@ horizontal_scroll_extended(struct session *ses, struct document_view *doc_view,
 }
 
 /*! @a steps > 0 -> right */
-static enum frame_event_status
+enum frame_event_status
 horizontal_scroll(struct session *ses, struct document_view *doc_view, int steps)
 {
+	ELOG
 	assert(ses && doc_view && doc_view->vs && doc_view->document);
 	if_assert_failed return FRAME_EVENT_OK;
 
@@ -403,6 +430,7 @@ horizontal_scroll(struct session *ses, struct document_view *doc_view, int steps
 enum frame_event_status
 scroll_up(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
@@ -414,6 +442,7 @@ scroll_up(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 scroll_down(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
@@ -425,6 +454,7 @@ scroll_down(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 scroll_left(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
@@ -436,6 +466,7 @@ scroll_left(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 scroll_right(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = eat_kbd_repeat_count(ses);
 
 	if (!steps)
@@ -448,6 +479,7 @@ scroll_right(struct session *ses, struct document_view *doc_view)
 static enum frame_event_status
 scroll_mouse_up(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = get_opt_int("document.browse.scrolling.vertical_step", ses);
 
 	return vertical_scroll(ses, doc_view, -steps);
@@ -456,6 +488,7 @@ scroll_mouse_up(struct session *ses, struct document_view *doc_view)
 static enum frame_event_status
 scroll_mouse_down(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = get_opt_int("document.browse.scrolling.vertical_step", ses);
 
 	return vertical_scroll(ses, doc_view, steps);
@@ -464,6 +497,7 @@ scroll_mouse_down(struct session *ses, struct document_view *doc_view)
 static enum frame_event_status
 scroll_mouse_left(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = get_opt_int("document.browse.scrolling.horizontal_step", ses);
 
 	return horizontal_scroll(ses, doc_view, -steps);
@@ -472,6 +506,7 @@ scroll_mouse_left(struct session *ses, struct document_view *doc_view)
 static enum frame_event_status
 scroll_mouse_right(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int steps = get_opt_int("document.browse.scrolling.horizontal_step", ses);
 
 	return horizontal_scroll(ses, doc_view, steps);
@@ -481,6 +516,7 @@ scroll_mouse_right(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 move_document_start(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	assert(ses && doc_view && doc_view->vs);
 	if_assert_failed return FRAME_EVENT_OK;
 
@@ -499,6 +535,7 @@ move_document_start(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 move_document_end(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int max_height;
 
 	assert(ses && doc_view && doc_view->vs && doc_view->document);
@@ -525,6 +562,7 @@ move_document_end(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 set_frame(struct session *ses, struct document_view *doc_view, int xxxx)
 {
+	ELOG
 	assert(ses && ses->doc_view && doc_view && doc_view->vs);
 	if_assert_failed return FRAME_EVENT_OK;
 
@@ -539,6 +577,7 @@ set_frame(struct session *ses, struct document_view *doc_view, int xxxx)
 void
 toggle_plain_html(struct session *ses, struct document_view *doc_view, int xxxx)
 {
+	ELOG
 	assert(ses && doc_view && ses->tab && ses->tab->term);
 	if_assert_failed return;
 
@@ -554,6 +593,7 @@ toggle_plain_html(struct session *ses, struct document_view *doc_view, int xxxx)
 void
 toggle_wrap_text(struct session *ses, struct document_view *doc_view, int xxxx)
 {
+	ELOG
 	assert(ses && doc_view && ses->tab && ses->tab->term);
 	if_assert_failed return;
 
@@ -573,6 +613,7 @@ toggle_wrap_text(struct session *ses, struct document_view *doc_view, int xxxx)
 enum frame_event_status
 move_cursor(struct session *ses, struct document_view *doc_view, int x, int y)
 {
+	ELOG
 	enum frame_event_status status = FRAME_EVENT_REFRESH;
 	struct terminal *term = ses->tab->term;
 	struct el_box *box = &doc_view->box;
@@ -632,6 +673,7 @@ static enum frame_event_status
 move_cursor_rel_count(struct session *ses, struct document_view *view,
 		      int rx, int ry, int count)
 {
+	ELOG
 	enum frame_event_status status;
 	int x, y;
 
@@ -645,6 +687,7 @@ static enum frame_event_status
 move_cursor_rel(struct session *ses, struct document_view *view,
 	        int rx, int ry)
 {
+	ELOG
 	int count = eat_kbd_repeat_count(ses);
 
 	int_lower_bound(&count, 1);
@@ -655,30 +698,35 @@ move_cursor_rel(struct session *ses, struct document_view *view,
 enum frame_event_status
 move_cursor_left(struct session *ses, struct document_view *view)
 {
+	ELOG
 	return move_cursor_rel(ses, view, -1, 0);
 }
 
 enum frame_event_status
 move_cursor_right(struct session *ses, struct document_view *view)
 {
+	ELOG
 	return move_cursor_rel(ses, view, 1, 0);
 }
 
 enum frame_event_status
 move_cursor_up(struct session *ses, struct document_view *view)
 {
+	ELOG
 	return move_cursor_rel(ses, view, 0, -1);
 }
 
 enum frame_event_status
 move_cursor_down(struct session *ses, struct document_view *view)
 {
+	ELOG
 	return move_cursor_rel(ses, view, 0, 1);
 }
 
 enum frame_event_status
 move_link_up_line(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct document *document;
 	struct view_state *vs;
 	struct el_box *box;
@@ -744,6 +792,7 @@ move_link_up_line(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 move_link_down_line(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct document *document;
 	struct view_state *vs;
 	struct el_box *box;
@@ -806,6 +855,7 @@ move_link_down_line(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 move_link_prev_line(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct view_state *vs;
 	struct document *document;
 	struct el_box *box;
@@ -886,6 +936,7 @@ move_link_prev_line(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 move_link_next_line(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct view_state *vs;
 	struct document *document;
 	struct el_box *box;
@@ -964,6 +1015,7 @@ move_link_next_line(struct session *ses, struct document_view *doc_view)
 enum frame_event_status
 move_cursor_line_start(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct view_state *vs;
 	struct el_box *box;
 	int x;
@@ -980,6 +1032,7 @@ move_cursor_line_start(struct session *ses, struct document_view *doc_view)
 static enum frame_event_status
 move_clipboard_pos(struct session *ses, struct document_view *view, enum frame_event_status status)
 {
+	ELOG
 	struct document *document = view->document;
 	int xoffset, yoffset, x, y;
 
@@ -1001,6 +1054,7 @@ move_clipboard_pos(struct session *ses, struct document_view *view, enum frame_e
 static enum frame_event_status
 copy_to_clipboard2(struct document_view *doc_view)
 {
+	ELOG
 	struct document *document = doc_view->document;
 	struct string data;
 	int starty, endy, startx, y, endx;
@@ -1089,6 +1143,7 @@ copy_to_clipboard2(struct document_view *doc_view)
 enum frame_event_status
 mark_clipboard(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	struct document *document = doc_view->document;
 
 	int xoffset = doc_view->vs->x - doc_view->box.x;
@@ -1135,6 +1190,7 @@ copy_current_link_to_clipboard(struct session *ses,
 			       struct document_view *doc_view,
 			       int xxx)
 {
+	ELOG
 	struct link *link;
 	struct uri *uri;
 	char *uristring;
@@ -1159,6 +1215,7 @@ copy_current_link_to_clipboard(struct session *ses,
 enum frame_event_status
 copy_to_clipboard(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	if (doc_view && doc_view->document && doc_view->document->clipboard_status != CLIPBOARD_NONE) {
 		return copy_to_clipboard2(doc_view);
 	}
@@ -1169,6 +1226,7 @@ copy_to_clipboard(struct session *ses, struct document_view *doc_view)
 int
 try_jump_to_link_number(struct session *ses, struct document_view *doc_view)
 {
+	ELOG
 	int link_number = eat_kbd_repeat_count(ses) - 1;
 
 	if (link_number < 0) return 1;
@@ -1189,6 +1247,7 @@ enum frame_event_status
 try_mark_key(struct session *ses, struct document_view *doc_view,
 	     struct term_event *ev)
 {
+	ELOG
 	term_event_key_T key = get_kbd_key(ev);
 	unsigned char mark;
 
@@ -1229,6 +1288,7 @@ try_mark_key(struct session *ses, struct document_view *doc_view,
 void
 open_link_dialog(struct session *ses)
 {
+	ELOG
 	input_dialog(ses->tab->term, NULL,
 		N_("Go to link"), N_("Enter link number"),
 		ses, NULL, MAX_STR_LEN, "", 0, 0, NULL,
@@ -1239,6 +1299,7 @@ static enum frame_event_status
 try_prefix_key(struct session *ses, struct document_view *doc_view,
 	       struct term_event *ev)
 {
+	ELOG
 	struct document *document = doc_view->document;
 	struct document_options *doc_opts = &document->options;
 	int digit = get_kbd_key(ev) - '0';
@@ -1300,6 +1361,7 @@ static enum frame_event_status
 try_form_action(struct session *ses, struct document_view *doc_view,
 		struct link *link, struct term_event *ev)
 {
+	ELOG
 	enum frame_event_status status = FRAME_EVENT_OK;
 
 	assert(link);
@@ -1310,8 +1372,21 @@ try_form_action(struct session *ses, struct document_view *doc_view,
 		memset(&last_event, 0, sizeof(last_event));
 	}
 
-	if (!link_is_textinput(link))
+#if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
+	if (program.ecmascript_keys) {
+		if (ev) {
+			int ret = document_fire_onkeydown(doc_view->vs->ecmascript, ev);
+			int ret2 = document_fire_onkeyup(doc_view->vs->ecmascript, ev);
+			if (ret || ret2) {
+				return FRAME_EVENT_JS;
+			}
+		}
+	}
+#endif
+
+	if (!link_is_textinput(link)) {
 		return FRAME_EVENT_IGNORED;
+	}
 
 #if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
 	if (ses->insert_mode == INSERT_MODE_ON) {
@@ -1354,6 +1429,7 @@ try_form_action(struct session *ses, struct document_view *doc_view,
 static enum frame_event_status
 frame_ev_kbd(struct session *ses, struct document_view *doc_view, struct term_event *ev)
 {
+	ELOG
 	enum frame_event_status status = FRAME_EVENT_IGNORED;
 	int accesskey_priority;
 	struct link *link = get_current_link(doc_view);
@@ -1400,6 +1476,7 @@ frame_ev_kbd(struct session *ses, struct document_view *doc_view, struct term_ev
 static enum frame_event_status
 frame_ev_mouse(struct session *ses, struct document_view *doc_view, struct term_event *ev)
 {
+	ELOG
 	int x = ev->info.mouse.x;
 	int y = ev->info.mouse.y;
 	struct link *link;
@@ -1431,7 +1508,7 @@ frame_ev_mouse(struct session *ses, struct document_view *doc_view, struct term_
 
 			if (check_mouse_button(ev, B_LEFT)
 			     || check_mouse_button(ev, B_MIDDLE)) {
-				if (check_mouse_action(ev, B_DOWN))
+				if (0 && check_mouse_action(ev, B_DOWN))
 					do_not_ignore_next_mouse_event(ses->tab->term);
 				else if (check_mouse_button(ev, B_LEFT))
 					status = enter(ses, doc_view, 0);
@@ -1477,6 +1554,7 @@ frame_ev_mouse(struct session *ses, struct document_view *doc_view, struct term_
 static enum frame_event_status
 frame_ev(struct session *ses, struct document_view *doc_view, struct term_event *ev)
 {
+	ELOG
 	assertm(doc_view && doc_view->document, "document not formatted");
 	if_assert_failed return FRAME_EVENT_IGNORED;
 
@@ -1501,6 +1579,7 @@ frame_ev(struct session *ses, struct document_view *doc_view, struct term_event
 struct document_view *
 current_frame(struct session *ses)
 {
+	ELOG
 	struct document_view *doc_view = NULL;
 	int current_frame_number;
 
@@ -1524,8 +1603,9 @@ current_frame(struct session *ses)
 
 	doc_view = ses->doc_view;
 
-	assert(doc_view && doc_view->document);
-	if_assert_failed return NULL;
+	if (!(doc_view && doc_view->document)) {
+		return NULL;
+	}
 
 	if (document_has_frames(doc_view->document)) return NULL;
 	return doc_view;
@@ -1535,6 +1615,7 @@ static enum frame_event_status
 send_to_frame(struct session *ses, struct document_view *doc_view,
 	      struct term_event *ev)
 {
+	ELOG
 	enum frame_event_status status;
 
 	assert(ses && ses->tab && ses->tab->term && ev);
@@ -1555,6 +1636,7 @@ static int
 do_mouse_event(struct session *ses, struct term_event *ev,
 	       struct document_view *doc_view)
 {
+	ELOG
 	struct term_event evv;
 	struct document_view *matched = NULL, *first = doc_view;
 
@@ -1567,10 +1649,12 @@ do_mouse_event(struct session *ses, struct term_event *ev,
 		assert(doc_view && doc_view->document);
 		if_assert_failed return 0;
 
+#if 0
 		assertm(doc_view->document->options.box.x == doc_view->box.x
 		        && doc_view->document->options.box.y == doc_view->box.y,
 			"Jonas' 1.565 -> 1.566 patch sucks");
 		if_assert_failed return 0;
+#endif
 
 		if (check_mouse_position(ev, &doc_view->box)) {
 			matched = doc_view;
@@ -1597,6 +1681,7 @@ do_mouse_event(struct session *ses, struct term_event *ev,
 static int
 is_mouse_on_tab_bar(struct session *ses, struct term_event_mouse *mouse)
 {
+	ELOG
 	struct terminal *term = ses->tab->term;
 	int y;
 
@@ -1611,10 +1696,12 @@ static struct session *
 send_mouse_event(struct session *ses, struct document_view *doc_view,
 		 struct term_event *ev)
 {
+	ELOG
 	struct terminal *term = ses->tab->term;
 	struct term_event_mouse *mouse = &ev->info.mouse;
 
 	if (mouse->y == 0
+	    && ses->status.show_title_bar
 	    && check_mouse_action(ev, B_DOWN)
 	    && !check_mouse_wheel(ev)) {
 		struct window *m;
@@ -1685,6 +1772,7 @@ static void
 try_typeahead(struct session *ses, struct document_view *doc_view,
               struct term_event *ev, main_action_T action_id)
 {
+	ELOG
 	switch (get_opt_int("document.browse.search.typeahead", ses)) {
 		case 0:
 			return;
@@ -1710,6 +1798,7 @@ try_typeahead(struct session *ses, struct document_view *doc_view,
 static enum frame_event_status
 try_menu(struct session *ses, struct term_event *ev)
 {
+	ELOG
 	struct window *win;
 
 	get_kbd_modifier(ev) &= ~KBD_MOD_ALT;
@@ -1736,14 +1825,24 @@ static struct session *
 send_kbd_event(struct session *ses, struct document_view *doc_view,
 	       struct term_event *ev)
 {
+	ELOG
 	int event;
 	main_action_T action_id;
+	enum frame_event_status status = FRAME_EVENT_IGNORED;
 
-	if (doc_view && send_to_frame(ses, doc_view, ev) != FRAME_EVENT_IGNORED)
-		return NULL;
+	if (doc_view) {
+		status = send_to_frame(ses, doc_view, ev);
 
+		if (status != FRAME_EVENT_IGNORED && status != FRAME_EVENT_JS) {
+			return NULL;
+		}
+	}
 	action_id = kbd_action(KEYMAP_MAIN, ev, &event);
 
+	if (status == FRAME_EVENT_JS && action_id != ACT_MAIN_TOGGLE_ECMASCRIPT_KEYS) {
+		return NULL;
+	}
+
 	if (action_id == ACT_MAIN_QUIT) {
 		if (check_kbd_key(ev, KBD_CTRL_C))
 quit:
@@ -1754,6 +1853,7 @@ quit:
 		case FRAME_EVENT_SESSION_DESTROYED:
 			return NULL;
 		case FRAME_EVENT_IGNORED:
+		case FRAME_EVENT_JS:
 			break;
 		case FRAME_EVENT_OK:
 		case FRAME_EVENT_REFRESH:
@@ -1800,6 +1900,7 @@ quit:
 void
 send_event(struct session *ses, struct term_event *ev)
 {
+	ELOG
 	assert(ses && ev);
 	if_assert_failed return;
 
@@ -1824,6 +1925,7 @@ enum frame_event_status
 download_link(struct session *ses, struct document_view *doc_view,
 	      action_id_T action_id)
 {
+	ELOG
 	struct link *link = get_current_link(doc_view);
 	void (*download)(void *ses, char *file) = start_download;
 
@@ -1862,6 +1964,7 @@ download_link(struct session *ses, struct document_view *doc_view,
 enum frame_event_status
 view_image(struct session *ses, struct document_view *doc_view, int xxxx)
 {
+	ELOG
 	struct link *link = get_current_link(doc_view);
 
 	if (link && link->where_img)
@@ -1873,6 +1976,7 @@ view_image(struct session *ses, struct document_view *doc_view, int xxxx)
 enum frame_event_status
 save_as(struct session *ses, struct document_view *doc_view, int magic)
 {
+	ELOG
 	assert(ses);
 	if_assert_failed return FRAME_EVENT_OK;
 
@@ -1896,6 +2000,7 @@ static void
 save_formatted_finish(struct terminal *term, int h,
 		      void *data, download_flags_T flags)
 {
+	ELOG
 	struct document *document = (struct document *)data;
 
 	assert(term && document);
@@ -1912,6 +2017,7 @@ save_formatted_finish(struct terminal *term, int h,
 static void
 save_formatted(void *data, char *file)
 {
+	ELOG
 	struct session *ses = (struct session *)data;
 	struct document_view *doc_view;
 
@@ -1929,6 +2035,7 @@ save_formatted(void *data, char *file)
 enum frame_event_status
 save_formatted_dlg(struct session *ses, struct document_view *doc_view, int xxxx)
 {
+	ELOG
 	query_file(ses, doc_view->vs->uri, ses, save_formatted, NULL, 1);
 	return FRAME_EVENT_OK;
 }
